@@ -21,16 +21,14 @@ package configs
 import (
 	"fmt"
 	"math"
-	"os"
-	"path/filepath"
 	"regexp"
 	"strings"
 	"time"
 
 	"go.uber.org/zap"
 
+	"github.com/apache/yunikorn-core/pkg/common"
 	"github.com/apache/yunikorn-core/pkg/common/resources"
-	"github.com/apache/yunikorn-core/pkg/common/security"
 	"github.com/apache/yunikorn-core/pkg/log"
 	"github.com/apache/yunikorn-core/pkg/scheduler/placement/types"
 	"github.com/apache/yunikorn-core/pkg/scheduler/policies"
@@ -52,8 +50,16 @@ const (
 	// app sort priority values
 	ApplicationSortPriorityEnabled  = "enabled"
 	ApplicationSortPriorityDisabled = "disabled"
+
+	// placement rule validation
+	placementOK placementPathCheckResult = iota
+	errNonExistingQueue
+	errQueueNotLeaf
+	errLastQueueLeaf
 )
 
+type placementPathCheckResult int
+
 // Priority
 var MinPriority int32 = math.MinInt32
 var MaxPriority int32 = math.MaxInt32
@@ -78,26 +84,19 @@ var SpecialRegExp = regexp.MustCompile(`[\^$*+?()\[{}|]`)
 var RuleNameRegExp = regexp.MustCompile(`^[_a-zA-Z][a-zA-Z0-9_]*$`)
 
 type placementStaticPath struct {
-	path      string
-	ruleChain string
-	create    bool
-	ruleNo    int
+	path           string
+	ruleChain      string
+	create         bool
+	hasDynamicPart bool
+	ruleNo         int
 }
 
-type placementPathCheckResult int
-
-const (
-	checkOK placementPathCheckResult = iota
-	nonExistingQueue
-	queueNotParent
-)
-
 // Check the ACL
 func checkACL(acl string) error {
 	// trim any white space
 	acl = strings.TrimSpace(acl)
 	// handle special cases: deny and wildcard
-	if len(acl) == 0 || acl == security.WildCard {
+	if len(acl) == 0 || acl == common.Wildcard {
 		return nil
 	}
 
@@ -127,7 +126,7 @@ func checkQueueResource(cur QueueConfig, parentM *resources.Resource) (*resource
 		}
 		sumG.AddTo(childG)
 	}
-	if !resources.IsZero(curG) && !resources.FitIn(curG, sumG) {
+	if !curG.FitInMaxUndef(sumG) {
 		return nil, fmt.Errorf("guaranteed resource of parent %s is smaller than sum of guaranteed resources %s of the children for queue %s", curG.String(), sumG.String(), cur.Name)
 	}
 	if !curM.FitInMaxUndef(sumG) {
@@ -139,11 +138,82 @@ func checkQueueResource(cur QueueConfig, parentM *resources.Resource) (*resource
 	return curG, nil
 }
 
+func checkLimitResource(cur QueueConfig, users map[string]map[string]*resources.Resource, groups map[string]map[string]*resources.Resource, queuePath string) error {
+	var curQueuePath string
+	if cur.Name == RootQueue {
+		curQueuePath = RootQueue
+	} else {
+		curQueuePath = queuePath + DOT + cur.Name
+	}
+
+	// Carry forward (populate) the parent limit settings to the next level if limits are not configured for the current queue
+	if len(cur.Limits) == 0 {
+		if _, ok := users[queuePath]; ok {
+			users[curQueuePath] = users[queuePath]
+		}
+		if _, ok := groups[queuePath]; ok {
+			groups[curQueuePath] = groups[queuePath]
+		}
+	} else {
+		// compare user & group limit setting between the current queue and parent queue
+		for _, limit := range cur.Limits {
+			limitMaxResources, err := resources.NewResourceFromConf(limit.MaxResources)
+			if err != nil {
+				return err
+			}
+			for _, user := range limit.Users {
+				// Is user limit setting exists?
+				if userMaxResource, ok := users[queuePath][user]; ok {
+					if !userMaxResource.FitInMaxUndef(limitMaxResources) {
+						return fmt.Errorf("user %s max resource %s of queue %s is greater than immediate or ancestor parent maximum resource %s", user, limitMaxResources.String(), cur.Name, userMaxResource.String())
+					}
+					// Override with min resource
+					if _, ok := users[curQueuePath]; !ok {
+						users[curQueuePath] = make(map[string]*resources.Resource)
+					}
+					users[curQueuePath][user] = resources.ComponentWiseMinPermissive(limitMaxResources, userMaxResource)
+				} else {
+					if _, ok := users[curQueuePath]; !ok {
+						users[curQueuePath] = make(map[string]*resources.Resource)
+					}
+					users[curQueuePath][user] = limitMaxResources
+				}
+			}
+			for _, group := range limit.Groups {
+				// Is group limit setting exists?
+				if groupMaxResource, ok := groups[queuePath][group]; ok {
+					if !groupMaxResource.FitInMaxUndef(limitMaxResources) {
+						return fmt.Errorf("group %s max resource %s of queue %s is greater than immediate or ancestor parent maximum resource %s", group, limitMaxResources.String(), cur.Name, groupMaxResource.String())
+					}
+					// Override with min resource
+					if _, ok := groups[curQueuePath]; !ok {
+						groups[curQueuePath] = make(map[string]*resources.Resource)
+					}
+					groups[curQueuePath][group] = resources.ComponentWiseMinPermissive(limitMaxResources, groupMaxResource)
+				} else {
+					if _, ok := groups[curQueuePath]; !ok {
+						groups[curQueuePath] = make(map[string]*resources.Resource)
+					}
+					groups[curQueuePath][group] = limitMaxResources
+				}
+			}
+		}
+	}
+	// traverse child queues
+	for _, child := range cur.Queues {
+		err := checkLimitResource(child, users, groups, curQueuePath)
+		if err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 func checkQueueMaxApplications(cur QueueConfig) error {
 	var err error
 	for _, child := range cur.Queues {
 		if cur.MaxApplications != 0 && (cur.MaxApplications < child.MaxApplications || child.MaxApplications == 0) {
-			return fmt.Errorf("parent maxRunningApps must be larger than child maxRunningApps")
+			return fmt.Errorf("parent maxApplications must be larger than child maxApplications")
 		}
 		err = checkQueueMaxApplications(child)
 		if err != nil {
@@ -153,6 +223,75 @@ func checkQueueMaxApplications(cur QueueConfig) error {
 	return nil
 }
 
+func checkLimitMaxApplications(cur QueueConfig, users map[string]map[string]uint64, groups map[string]map[string]uint64, queuePath string) error {
+	var curQueuePath string
+	if cur.Name == RootQueue {
+		curQueuePath = RootQueue
+	} else {
+		curQueuePath = queuePath + DOT + cur.Name
+	}
+
+	// Carry forward (populate) the parent limit settings to the next level if limits are not configured for the current queue
+	if len(cur.Limits) == 0 {
+		if _, ok := users[queuePath]; ok {
+			users[curQueuePath] = users[queuePath]
+		}
+		if _, ok := groups[queuePath]; ok {
+			groups[curQueuePath] = groups[queuePath]
+		}
+	} else {
+		// compare user & group limit setting between the current queue and parent queue
+		for _, limit := range cur.Limits {
+			limitMaxApplications := limit.MaxApplications
+			for _, user := range limit.Users {
+				// Is user limit setting exists?
+				if userMaxApplications, ok := users[queuePath][user]; ok {
+					if userMaxApplications != 0 && (userMaxApplications < limitMaxApplications || limitMaxApplications == 0) {
+						return fmt.Errorf("user %s max applications %d of queue %s is greater than immediate or ancestor parent max applications %d", user, limitMaxApplications, cur.Name, userMaxApplications)
+					}
+					// Override with min resource
+					if _, ok := users[curQueuePath]; !ok {
+						users[curQueuePath] = make(map[string]uint64)
+					}
+					users[curQueuePath][user] = common.Min(limitMaxApplications, userMaxApplications)
+				} else {
+					if _, ok := users[curQueuePath]; !ok {
+						users[curQueuePath] = make(map[string]uint64)
+					}
+					users[curQueuePath][user] = limitMaxApplications
+				}
+			}
+
+			for _, group := range limit.Groups {
+				// Is user limit setting exists?
+				if groupMaxApplications, ok := groups[queuePath][group]; ok {
+					if groupMaxApplications != 0 && (groupMaxApplications < limitMaxApplications || limitMaxApplications == 0) {
+						return fmt.Errorf("group %s max applications %d of queue %s is greater than immediate or ancestor parent max applications %d", group, limitMaxApplications, cur.Name, groupMaxApplications)
+					}
+					// Override with min resource
+					if _, ok := groups[curQueuePath]; !ok {
+						groups[curQueuePath] = make(map[string]uint64)
+					}
+					groups[curQueuePath][group] = common.Min(limitMaxApplications, groupMaxApplications)
+				} else {
+					if _, ok := groups[curQueuePath]; !ok {
+						groups[curQueuePath] = make(map[string]uint64)
+					}
+					groups[curQueuePath][group] = limitMaxApplications
+				}
+			}
+		}
+	}
+	// traverse child queues
+	for _, child := range cur.Queues {
+		err := checkLimitMaxApplications(child, users, groups, curQueuePath)
+		if err != nil {
+			return err
+		}
+	}
+	return nil
+}
+
 func checkResourceConfig(cur QueueConfig) (*resources.Resource, *resources.Resource, error) {
 	var g, m *resources.Resource
 	var err error
@@ -177,7 +316,7 @@ func checkPlacementRules(partition *PartitionConfig) error {
 		return nil
 	}
 
-	log.Logger().Debug("checking placement rule config",
+	log.Log(log.Config).Debug("checking placement rule config",
 		zap.String("partitionName", partition.Name))
 	// top level rule checks, parents are called recursively
 	for _, rule := range partition.PlacementRules {
@@ -186,33 +325,52 @@ func checkPlacementRules(partition *PartitionConfig) error {
 		}
 	}
 
-	placementStaticPaths := getLongestPlacementPaths(partition.PlacementRules)
+	placementStaticPaths, err := getLongestPlacementPaths(partition.PlacementRules)
+	if err != nil {
+		return err
+	}
 	for _, staticPath := range placementStaticPaths {
 		queuePath := staticPath.path
+		create := staticPath.create
+		hasDynamicPart := staticPath.hasDynamicPart
+
 		parts := strings.Split(strings.ToLower(queuePath), DOT)
-		result := checkQueueHierarchyForPlacement(parts, staticPath.create, partition.Queues)
-		if result == queueNotParent {
-			return fmt.Errorf("placement rule no. #%d (%s) references a queue (%s) which is a leaf",
+		result, lastQueue := checkQueueHierarchyForPlacement(parts, create, hasDynamicPart, partition.Queues, nil)
+		if result == errQueueNotLeaf {
+			return fmt.Errorf("placement rule no. #%d (%s) references a queue (%s) which is not a leaf",
 				staticPath.ruleNo, staticPath.ruleChain, queuePath)
 		}
-		if result == nonExistingQueue {
+		if result == errNonExistingQueue {
 			return fmt.Errorf("placement rule no. #%d (%s) references non-existing queues (%s) and create is 'false'",
 				staticPath.ruleNo, staticPath.ruleChain, queuePath)
 		}
+		if result == errLastQueueLeaf {
+			return fmt.Errorf("placement rule no. #%d (%s) references non-existing queues (%s) which cannot be created because the last queue (%s) in the hierarchy is a leaf",
+				staticPath.ruleNo, staticPath.ruleChain, queuePath, lastQueue)
+		}
 	}
 
 	return nil
 }
 
-func checkQueueHierarchyForPlacement(path []string, create bool, conf []QueueConfig) placementPathCheckResult {
+func checkQueueHierarchyForPlacement(path []string, create, hasDynamicPart bool, conf []QueueConfig, parentConf *QueueConfig) (result placementPathCheckResult, lastQueueName string) {
 	queueName := path[0]
+	lastQueueName = ""
 
 	// no more queues in the configuration
 	if len(conf) == 0 {
-		if !create {
-			return nonExistingQueue
+		if !parentConf.Parent {
+			// path in the hierarchy is shorter, but the last queue is a leaf
+			result = errLastQueueLeaf
+			lastQueueName = parentConf.Name
+			return
 		}
-		return checkOK
+		if !create {
+			result = errNonExistingQueue
+			return
+		}
+		result = placementOK
+		return
 	}
 
 	var queueConf *QueueConfig
@@ -227,21 +385,34 @@ func checkQueueHierarchyForPlacement(path []string, create bool, conf []QueueCon
 	// queue not found on this level
 	if queueConf == nil {
 		if !create {
-			return nonExistingQueue
+			result = errNonExistingQueue
+			return
 		}
-		return checkOK
-	}
-
-	if !queueConf.Parent {
-		return queueNotParent
+		result = placementOK
+		return
 	}
 
 	if len(path) == 1 {
-		return checkOK
+		if hasDynamicPart {
+			// the "fixed" rule is followed by other rules like tag, user, etc. (root.dev.<user>),
+			// which means that the "fixed" part must point to a parent
+			if queueConf.Parent {
+				result = placementOK
+				return
+			}
+			result = errQueueNotLeaf
+			return
+		}
+		if queueConf.Parent {
+			result = errQueueNotLeaf
+			return
+		}
+		result = placementOK
+		return
 	}
 
 	path = path[1:]
-	return checkQueueHierarchyForPlacement(path, create, queueConf.Queues)
+	return checkQueueHierarchyForPlacement(path, create, hasDynamicPart, queueConf.Queues, queueConf)
 }
 
 // Check the specific rule for syntax.
@@ -254,7 +425,7 @@ func checkPlacementRule(rule PlacementRule) error {
 	// check the parent rule
 	if rule.Parent != nil {
 		if err := checkPlacementRule(*rule.Parent); err != nil {
-			log.Logger().Debug("parent placement rule failed",
+			log.Log(log.Config).Debug("parent placement rule failed",
 				zap.String("rule", rule.Name),
 				zap.String("parentRule", rule.Parent.Name))
 			return err
@@ -262,7 +433,7 @@ func checkPlacementRule(rule PlacementRule) error {
 	}
 	// check filter if given
 	if err := checkPlacementFilter(rule.Filter); err != nil {
-		log.Logger().Debug("placement rule filter failed",
+		log.Log(log.Config).Debug("placement rule filter failed",
 			zap.String("rule", rule.Name),
 			zap.Any("filter", rule.Filter))
 		return err
@@ -364,23 +535,21 @@ func checkLimit(limit Limit, existedUserName map[string]bool, existedGroupName m
 	if len(limit.MaxResources) != 0 {
 		limitResource, err = resources.NewResourceFromConf(limit.MaxResources)
 		if err != nil {
-			log.Logger().Debug("resource parsing failed",
+			log.Log(log.Config).Debug("resource parsing failed",
 				zap.Error(err))
 			return err
 		}
+		if resources.IsZero(limitResource) {
+			return fmt.Errorf("MaxResources is zero in '%s' limit, all resource types are zero", limit.Limit)
+		}
 	}
 	// at least some resource should be not null
-	if limit.MaxApplications == 0 && resources.IsZero(limitResource) {
+	if limit.MaxApplications == 0 && len(limit.MaxResources) == 0 {
 		return fmt.Errorf("invalid resource combination for limit %s all resource limits are null", limit.Limit)
 	}
 
-	if queue.MaxApplications != 0 {
-		if limit.MaxApplications > queue.MaxApplications {
-			return fmt.Errorf("invalid MaxApplications settings for limit %s exeecd current the queue MaxApplications", limit.Limit)
-		}
-		if limit.MaxApplications == 0 {
-			return fmt.Errorf("MaxApplications is 0 in limit name %s, it should be 1 ~ %d", limit.Limit, queue.MaxApplications)
-		}
+	if queue.MaxApplications != 0 && queue.MaxApplications < limit.MaxApplications {
+		return fmt.Errorf("invalid MaxApplications settings for limit %s exceed current the queue MaxApplications", limit.Limit)
 	}
 
 	// If queue is RootQueue, the queue.Resources.Max will be null, we don't need to check for root queue
@@ -388,7 +557,7 @@ func checkLimit(limit Limit, existedUserName map[string]bool, existedGroupName m
 	if queue.Name != RootQueue {
 		queueMaxResource, err := resources.NewResourceFromConf(queue.Resources.Max)
 		if err != nil {
-			log.Logger().Debug("resource parsing failed",
+			log.Log(log.Config).Debug("resource parsing failed",
 				zap.Error(err))
 			return fmt.Errorf("parse queue %s max resource failed: %s", queue.Name, err.Error())
 		}
@@ -407,23 +576,13 @@ func checkLimits(limits []Limit, obj string, queue *QueueConfig) error {
 		return nil
 	}
 	// walk over the list of limits
-	log.Logger().Debug("checking limits configs",
+	log.Log(log.Config).Debug("checking limits configs",
 		zap.String("objName", obj),
 		zap.Int("limitsLength", len(limits)))
 
 	existedUserName := make(map[string]bool)
 	existedGroupName := make(map[string]bool)
 
-	defer func() {
-		for k := range existedUserName {
-			delete(existedUserName, k)
-		}
-
-		for k := range existedGroupName {
-			delete(existedGroupName, k)
-		}
-	}()
-
 	for _, limit := range limits {
 		if err := checkLimit(limit, existedUserName, existedGroupName, queue); err != nil {
 			return err
@@ -506,7 +665,7 @@ func checkQueuesStructure(partition *PartitionConfig) error {
 		return fmt.Errorf("queue config is not set")
 	}
 
-	log.Logger().Debug("checking partition queue config",
+	log.Log(log.Config).Debug("checking partition queue config",
 		zap.String("partitionName", partition.Name))
 
 	// handle no root queue cases
@@ -526,7 +685,7 @@ func checkQueuesStructure(partition *PartitionConfig) error {
 
 	// insert the root queue if not there
 	if insertRoot {
-		log.Logger().Debug("inserting root queue",
+		log.Log(log.Config).Debug("inserting root queue",
 			zap.Int("numOfQueues", len(partition.Queues)))
 		var rootQueue QueueConfig
 		rootQueue.Name = RootQueue
@@ -549,14 +708,7 @@ func checkQueuesStructure(partition *PartitionConfig) error {
 // Check the state dump file path, if configured, is a valid path that can be written to.
 func checkDeprecatedStateDumpFilePath(partition *PartitionConfig) error {
 	if partition.StateDumpFilePath != "" {
-		log.Logger().Warn("Ignoring deprecated partition setting 'statedumpfilepath'. This parameter will be removed in a future release.")
-	}
-	return nil
-}
-
-func ensureDir(fileName string) error {
-	if err := os.MkdirAll(filepath.Dir(fileName), os.ModePerm); err != nil {
-		return err
+		log.Log(log.Deprecation).Warn("Ignoring deprecated partition setting 'statedumpfilepath'. This parameter will be removed in a future release.")
 	}
 	return nil
 }
@@ -615,6 +767,14 @@ func Validate(newConfig *SchedulerConfig) error {
 		if err != nil {
 			return err
 		}
+
+		if err = checkLimitResource(partition.Queues[0], make(map[string]map[string]*resources.Resource), make(map[string]map[string]*resources.Resource), common.Empty); err != nil {
+			return err
+		}
+
+		if err = checkLimitMaxApplications(partition.Queues[0], make(map[string]map[string]uint64), make(map[string]map[string]uint64), common.Empty); err != nil {
+			return err
+		}
 		// write back the partition to keep changes
 		newConfig.Partitions[i] = partition
 	}
@@ -622,51 +782,79 @@ func Validate(newConfig *SchedulerConfig) error {
 }
 
 // returns the longest fixed queue path defined by the placement rule chain
-// e.g. the chain is user->tag->fixed, returns something like "root.users.<tag>.<user>",
+// e.g. the chain is fixed->tag->user, returns something like "root.users.<tag>.<user>",
 // the longest static part is "root.users"
-func getLongestPlacementPaths(rules []PlacementRule) []placementStaticPath {
+func getLongestPlacementPaths(rules []PlacementRule) ([]placementStaticPath, error) {
 	paths := make([]placementStaticPath, 0)
 
 	for i, rule := range rules {
-		path, ruleChain, _ := getLongestStaticPath(rule)
+		path, ruleChain, hasDynamicPart, err := getLongestStaticPath(rule)
+		if err != nil {
+			return nil, err
+		}
+		if strings.Index(path, RootQueue) != 0 {
+			continue
+		}
 		placementPath := placementStaticPath{
-			path:      path,
-			create:    rule.Create,
-			ruleChain: ruleChain,
-			ruleNo:    i,
+			path:           path,
+			create:         rule.Create,
+			ruleChain:      ruleChain,
+			hasDynamicPart: hasDynamicPart,
+			ruleNo:         i,
 		}
 		paths = append(paths, placementPath)
 	}
 
-	return paths
+	return paths, nil
 }
 
-func getLongestStaticPath(rule PlacementRule) (string, string, bool) {
-	var parentPath string
-	var ruleChain string
-	dynamicParent := false
+func getLongestStaticPath(rule PlacementRule) (staticPath, ruleChain string, foundDynamicRule bool, err error) {
+	rules := getRuleChain(rule)
 
-	if rule.Parent != nil {
-		var chain string
-		parentPath, chain, dynamicParent = getLongestStaticPath(*rule.Parent)
-		ruleChain = rule.Name + "->" + chain
-	} else {
-		ruleChain = rule.Name
-		parentPath = RootQueue
-	}
+	for _, r := range rules {
+		if ruleChain == "" {
+			ruleChain = r.Name
+		} else {
+			ruleChain = ruleChain + "->" + r.Name
+		}
+		if foundDynamicRule {
+			continue
+		}
 
-	if rule.Name == types.Fixed {
-		queueName := strings.ToLower(rule.Value)
+		if r.Name != types.Fixed {
+			if staticPath == "" {
+				staticPath = "<dynamic>"
+			}
+			foundDynamicRule = true
+			continue
+		}
+
+		queueName := r.Value
 		qualified := strings.HasPrefix(queueName, RootQueue)
 		if qualified {
-			return queueName, ruleChain, false
+			if staticPath != "" {
+				// error, only the first fixed rule can be fully qualified
+				err = fmt.Errorf("illegal fully qualified 'fixed' rule with value %s", queueName)
+				return staticPath, ruleChain, foundDynamicRule, err
+			}
+			staticPath = queueName
+			continue
 		}
-		// there is a parent rule other than "fixed", we can't do anything about that
-		if dynamicParent {
-			return RootQueue, ruleChain, true
+		if staticPath == "" {
+			staticPath = RootQueue
 		}
-		return parentPath + "." + queueName, ruleChain, false
+		staticPath = staticPath + "." + queueName
 	}
 
-	return parentPath, ruleChain, true
+	return staticPath, ruleChain, foundDynamicRule, nil
+}
+
+func getRuleChain(r PlacementRule) []PlacementRule {
+	rules := make([]PlacementRule, 0)
+	if r.Parent != nil {
+		rules = append(rules, getRuleChain(*r.Parent)...)
+	}
+
+	rules = append(rules, r)
+	return rules
 }
