@@ -47,43 +47,43 @@ func (q Quantity) string() string {
 }
 
 // Util struct to keep track of application resource usage
-type UsedResource struct {
+type TrackedResource struct {
 	// Two level map for aggregated resource usage
 	// With instance type being the top level key, the mapped value is a map:
 	//   resource type (CPU, memory etc) -> the aggregated used time (in seconds) of the resource type
 	//
-	UsedResourceMap map[string]map[string]int64
+	TrackedResourceMap map[string]map[string]int64
 
 	sync.RWMutex
 }
 
-func (ur *UsedResource) Clone() *UsedResource {
+func (ur *TrackedResource) Clone() *TrackedResource {
 	if ur == nil {
 		return nil
 	}
-	ret := NewUsedResource()
+	ret := NewTrackedResource()
 	ur.RLock()
 	defer ur.RUnlock()
-	for k, v := range ur.UsedResourceMap {
+	for k, v := range ur.TrackedResourceMap {
 		dest := make(map[string]int64)
 		for key, element := range v {
 			dest[key] = element
 		}
-		ret.UsedResourceMap[k] = dest
+		ret.TrackedResourceMap[k] = dest
 	}
 	return ret
 }
 
 // Aggregate the resource usage to UsedResourceMap[instType]
 // The time the given resource used is the delta between the resource createTime and currentTime
-func (ur *UsedResource) AggregateUsedResource(instType string,
+func (ur *TrackedResource) AggregateTrackedResource(instType string,
 	resource *Resource, bindTime time.Time) {
 	ur.Lock()
 	defer ur.Unlock()
 
 	releaseTime := time.Now()
 	timeDiff := int64(releaseTime.Sub(bindTime).Seconds())
-	aggregatedResourceTime, ok := ur.UsedResourceMap[instType]
+	aggregatedResourceTime, ok := ur.TrackedResourceMap[instType]
 	if !ok {
 		aggregatedResourceTime = map[string]int64{}
 	}
@@ -95,7 +95,7 @@ func (ur *UsedResource) AggregateUsedResource(instType string,
 		curUsage += int64(element) * timeDiff // resource size times timeDiff
 		aggregatedResourceTime[key] = curUsage
 	}
-	ur.UsedResourceMap[instType] = aggregatedResourceTime
+	ur.TrackedResourceMap[instType] = aggregatedResourceTime
 }
 
 // Never update value of Zero
@@ -157,8 +157,8 @@ func NewResourceFromConf(configMap map[string]string) (*Resource, error) {
 	return res, nil
 }
 
-func NewUsedResource() *UsedResource {
-	return &UsedResource{UsedResourceMap: make(map[string]map[string]int64)}
+func NewTrackedResource() *TrackedResource {
+	return &TrackedResource{TrackedResourceMap: make(map[string]map[string]int64)}
 }
 
 func (r *Resource) String() string {
@@ -295,13 +295,13 @@ func addVal(valA, valB Quantity) Quantity {
 	if (result < valA) != (valB < 0) {
 		if valA < 0 {
 			// return the minimum possible
-			log.Logger().Warn("Resource calculation wrapped: returned minimum value possible",
+			log.Log(log.Resources).Warn("Resource calculation wrapped: returned minimum value possible",
 				zap.Int64("valueA", int64(valA)),
 				zap.Int64("valueB", int64(valB)))
 			return math.MinInt64
 		}
 		// return the maximum possible
-		log.Logger().Warn("Resource calculation wrapped: returned maximum value possible",
+		log.Log(log.Resources).Warn("Resource calculation wrapped: returned maximum value possible",
 			zap.Int64("valueA", int64(valA)),
 			zap.Int64("valueB", int64(valB)))
 		return math.MaxInt64
@@ -326,13 +326,13 @@ func mulVal(valA, valB Quantity) Quantity {
 	if (result/valB != valA) || (valA == math.MinInt64 && valB == -1) {
 		if (valA < 0) != (valB < 0) {
 			// return the minimum possible
-			log.Logger().Warn("Resource calculation wrapped: returned minimum value possible",
+			log.Log(log.Resources).Warn("Resource calculation wrapped: returned minimum value possible",
 				zap.Int64("valueA", int64(valA)),
 				zap.Int64("valueB", int64(valB)))
 			return math.MinInt64
 		}
 		// return the maximum possible
-		log.Logger().Warn("Resource calculation wrapped: returned maximum value possible",
+		log.Log(log.Resources).Warn("Resource calculation wrapped: returned maximum value possible",
 			zap.Int64("valueA", int64(valA)),
 			zap.Int64("valueB", int64(valB)))
 		return math.MaxInt64
@@ -349,14 +349,14 @@ func mulValRatio(value Quantity, ratio float64) Quantity {
 	result := float64(value) * ratio
 	// protect against positive integer overflow
 	if result > math.MaxInt64 {
-		log.Logger().Warn("Multiplication result positive overflow",
+		log.Log(log.Resources).Warn("Multiplication result positive overflow",
 			zap.Float64("value", float64(value)),
 			zap.Float64("ratio", ratio))
 		return math.MaxInt64
 	}
 	// protect against negative integer overflow
 	if result < math.MinInt64 {
-		log.Logger().Warn("Multiplication result negative overflow",
+		log.Log(log.Resources).Warn("Multiplication result negative overflow",
 			zap.Float64("value", float64(value)),
 			zap.Float64("ratio", ratio))
 		return math.MinInt64
@@ -541,7 +541,7 @@ func getShares(res, total *Resource) []float64 {
 		if total == nil || total.Resources[k] == 0 {
 			// negative share is logged
 			if v < 0 {
-				log.Logger().Debug("usage is negative no total, share is also negative",
+				log.Log(log.Resources).Debug("usage is negative no total, share is also negative",
 					zap.String("resource key", k),
 					zap.Int64("resource quantity", int64(v)))
 			}
@@ -552,7 +552,7 @@ func getShares(res, total *Resource) []float64 {
 		shares[idx] = float64(v) / float64(total.Resources[k])
 		// negative share is logged
 		if shares[idx] < 0 {
-			log.Logger().Debug("share set is negative",
+			log.Log(log.Resources).Debug("share set is negative",
 				zap.String("resource key", k),
 				zap.Int64("resource quantity", int64(v)),
 				zap.Int64("total quantity", int64(total.Resources[k])))
@@ -907,6 +907,11 @@ func (r *Resource) HasNegativeValue() bool {
 	return false
 }
 
+// IsEmpty returns true if the resource is nil or has no component resources specified.
+func (r *Resource) IsEmpty() bool {
+	return r == nil || len(r.Resources) == 0
+}
+
 // Returns a new resource with the largest value for each quantity in the resources
 // If either resource passed in is nil a zero resource is returned
 func ComponentWiseMax(left, right *Resource) *Resource {
@@ -936,52 +941,96 @@ func IsZero(zero *Resource) bool {
 	return true
 }
 
+// CalculateAbsUsedCapacity returns absolute used as a percentage, a positive integer value, for each defined resource
+// named in the capacity comparing usage to the capacity.
+// If usage is 0 or below 0, absolute used is always 0
+// if capacity is 0 or below 0, absolute used is always 100
+// if used is larger than capacity a value larger than 100 can be returned. The percentage value returned is capped at
+// math.MaxInt32 (resolved value 2147483647)
 func CalculateAbsUsedCapacity(capacity, used *Resource) *Resource {
 	absResource := NewResource()
 	if capacity == nil || used == nil {
-		log.Logger().Debug("Cannot calculate absolute capacity because of missing capacity or usage")
+		log.Log(log.Resources).Debug("Cannot calculate absolute capacity because of missing capacity or usage")
 		return absResource
 	}
 	missingResources := &strings.Builder{}
-	for resourceName, availableResource := range capacity.Resources {
+	for resourceName, capResource := range capacity.Resources {
 		var absResValue int64
-		if usedResource, ok := used.Resources[resourceName]; ok {
-			if availableResource < usedResource {
-				log.Logger().Warn("Higher usage than max capacity",
-					zap.String("resource", resourceName),
-					zap.Int64("capacity", int64(availableResource)),
-					zap.Int64("usage", int64(usedResource)))
-			}
-			div := float64(usedResource) / float64(availableResource)
-			absResValue = int64(div * 100)
-			// protect against positive integer overflow
-			if absResValue < 0 && div > 0 {
-				log.Logger().Warn("Absolute resource value result positive overflow",
-					zap.String("resource", resourceName),
-					zap.Int64("capacity", int64(availableResource)),
-					zap.Int64("usage", int64(usedResource)))
-				absResValue = math.MaxInt64
-			}
-			// protect against negative integer overflow
-			if absResValue > 0 && div < 0 {
-				log.Logger().Warn("Absolute resource value result negative overflow",
-					zap.String("resource", resourceName),
-					zap.Int64("capacity", int64(availableResource)),
-					zap.Int64("usage", int64(usedResource)))
-				absResValue = math.MinInt64
-			}
-		} else {
+		usedResource, ok := used.Resources[resourceName]
+		// track this for troubleshooting only
+		if !ok {
 			if missingResources.Len() != 0 {
 				missingResources.WriteString(", ")
 			}
 			missingResources.WriteString(resourceName)
 			continue
 		}
+		switch {
+		// used is 0 or below nothing is used -> 0%
+		// below 0 should never happen
+		case usedResource <= 0:
+			absResValue = 0
+		// capacity is 0 or below any usage is full -> 100% (prevents divide by 0)
+		// below 0 should never happen
+		case capResource <= 0:
+			absResValue = 100
+		// calculate percentage: never wraps, could overflow int64 due to percentage conversion ONLY
+		default:
+			div := (float64(usedResource) / float64(capResource)) * 100
+			// we really do not want to show a percentage value that is larger than a 32-bit integer.
+			// even that is already really large and could easily lead to UI render issues.
+			if div > float64(math.MaxInt32) {
+				absResValue = math.MaxInt32
+			} else {
+				absResValue = int64(div)
+			}
+		}
 		absResource.Resources[resourceName] = Quantity(absResValue)
 	}
 	if missingResources.Len() != 0 {
-		log.Logger().Debug("Absolute usage result is missing resource information",
+		log.Log(log.Resources).Debug("Absolute usage result is missing resource information",
 			zap.Stringer("missing resource(s)", missingResources))
 	}
 	return absResource
 }
+
+// DominantResourceType calculates the most used resource type based on the ratio of used compared to
+// the capacity. If a capacity type is set to 0 assume full usage.
+// Dominant type should be calculated with queue usage and capacity. Queue capacities should never
+// contain 0 values when there is a usage also, however in the root queue this could happen. If the
+// last node reporting that resource was removed but not everything has been updated.
+// immediately
+// Ignores resources types that are used but not defined in the capacity.
+func (r *Resource) DominantResourceType(capacity *Resource) string {
+	if r == nil || capacity == nil {
+		return ""
+	}
+	var div, temp float64
+	dominant := ""
+	for name, usedVal := range r.Resources {
+		capVal, ok := capacity.Resources[name]
+		if !ok {
+			log.Log(log.Resources).Debug("missing resource in dominant calculation",
+				zap.String("missing resource", name))
+			continue
+		}
+		// calculate the ratio between usage and capacity
+		// ratio should be somewhere between 0 and 1, but do not restrict
+		// handle 0 values specifically just to be safe should never happen
+		if capVal == 0 {
+			if usedVal == 0 {
+				temp = 0 // no usage, no cap: consider empty
+			} else {
+				temp = 1 // usage, no cap: fully used
+			}
+		} else {
+			temp = float64(usedVal) / float64(capVal) // both not zero calculate ratio
+		}
+		// if we have exactly the same use the latest one
+		if temp >= div {
+			div = temp
+			dominant = name
+		}
+	}
+	return dominant
+}
