@@ -1504,25 +1504,37 @@ func TestCalculateAbsUsedCapacity(t *testing.T) {
 			used:     partialResource,
 			expected: NewResourceFromMap(map[string]Quantity{"memory": 50}),
 		},
-		"positive overflow": {
-			capacity: NewResourceFromMap(map[string]Quantity{"memory": 10}),
+		"positive limit": {
+			capacity: NewResourceFromMap(map[string]Quantity{"memory": 1}),
 			used:     NewResourceFromMap(map[string]Quantity{"memory": math.MaxInt64}),
-			expected: NewResourceFromMap(map[string]Quantity{"memory": math.MaxInt64}),
+			expected: NewResourceFromMap(map[string]Quantity{"memory": math.MaxInt32}),
 		},
 		"negative overflow": {
 			capacity: NewResourceFromMap(map[string]Quantity{"memory": 10}),
 			used:     NewResourceFromMap(map[string]Quantity{"memory": math.MinInt64}),
-			expected: NewResourceFromMap(map[string]Quantity{"memory": math.MinInt64}),
+			expected: NewResourceFromMap(map[string]Quantity{"memory": 0}),
 		},
 		"zero resource, non zero used": {
 			capacity: zeroResource,
 			used:     usageSet,
-			expected: NewResourceFromMap(map[string]Quantity{"memory": math.MaxInt64, "vcores": math.MaxInt64}),
+			expected: NewResourceFromMap(map[string]Quantity{"memory": 100, "vcores": 100}),
+		},
+		"min used": {
+			capacity: NewResourceFromMap(map[string]Quantity{"memory": math.MinInt64}),
+			used:     NewResourceFromMap(map[string]Quantity{"memory": 10}),
+			expected: NewResourceFromMap(map[string]Quantity{"memory": 100}),
+		},
+		"max used": {
+			capacity: NewResourceFromMap(map[string]Quantity{"memory": math.MaxInt64}),
+			used:     NewResourceFromMap(map[string]Quantity{"memory": math.MaxInt64}),
+			expected: NewResourceFromMap(map[string]Quantity{"memory": 100}),
 		},
 	}
-	for _, test := range tests {
-		absCapacity := CalculateAbsUsedCapacity(test.capacity, test.used)
-		assert.DeepEqual(t, test.expected, absCapacity)
+	for name, test := range tests {
+		t.Run(name, func(t *testing.T) {
+			absCapacity := CalculateAbsUsedCapacity(test.capacity, test.used)
+			assert.Assert(t, Equals(test.expected, absCapacity), name)
+		})
 	}
 }
 
@@ -1671,3 +1683,47 @@ func TestHasNegativeValue(t *testing.T) {
 		})
 	}
 }
+
+func TestIsEmpty(t *testing.T) {
+	testCases := []struct {
+		name           string
+		input          *Resource
+		expectedResult bool
+	}{
+		{"Nil resource", nil, true},
+		{"Empty resource", NewResource(), true},
+		{"Positive value", NewResourceFromMap(map[string]Quantity{common.Memory: 100}), false},
+		{"Negative value", NewResourceFromMap(map[string]Quantity{common.Memory: -100}), false},
+	}
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			assert.Equal(t, tc.expectedResult, tc.input.IsEmpty())
+		})
+	}
+}
+
+func TestResource_DominantResource(t *testing.T) {
+	tests := []struct {
+		name     string
+		used     *Resource
+		capacity *Resource
+		wantName string
+	}{
+		{"nil receiver", nil, Zero, ""},
+		{"nil cap", Zero, nil, ""},
+		{"zero cap", NewResourceFromMap(map[string]Quantity{"A": 10}), Zero, ""},
+		{"over cap", NewResourceFromMap(map[string]Quantity{"A": 20}), NewResourceFromMap(map[string]Quantity{"A": 10}), "A"},
+		{"zero usage exist", NewResourceFromMap(map[string]Quantity{"A": 0}), NewResourceFromMap(map[string]Quantity{"A": 10}), "A"},
+		{"usage not in cap", NewResourceFromMap(map[string]Quantity{"B": 10}), NewResourceFromMap(map[string]Quantity{"A": 10}), ""},
+		{"multiple usages", NewResourceFromMap(map[string]Quantity{"B": 10, "A": 10}), NewResourceFromMap(map[string]Quantity{"A": 10, "B": 20}), "A"},
+		{"0 usage with 0 cap", NewResourceFromMap(map[string]Quantity{"A": 0}), NewResourceFromMap(map[string]Quantity{"A": 0}), "A"},
+		{"usage with 0 cap", NewResourceFromMap(map[string]Quantity{"A": 10, "B": 5}), NewResourceFromMap(map[string]Quantity{"A": 0, "B": 10}), "A"},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			if got := tt.used.DominantResourceType(tt.capacity); got != tt.wantName {
+				t.Errorf("DominantResourceType() = %v, want %v", got, tt.wantName)
+			}
+		})
+	}
+}
