@@ -26,6 +26,7 @@ import (
 
 	"go.uber.org/zap"
 
+	"github.com/apache/yunikorn-core/pkg/common"
 	"github.com/apache/yunikorn-core/pkg/log"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
@@ -70,17 +71,17 @@ func GetUserGroupCache(resolver string) *UserGroupCache {
 	once.Do(func() {
 		switch resolver {
 		case "test":
-			log.Logger().Info("creating test user group resolver")
+			log.Log(log.Security).Info("creating test user group resolver")
 			instance = GetUserGroupCacheTest()
 		case "os":
-			log.Logger().Info("creating OS user group resolver")
+			log.Log(log.Security).Info("creating OS user group resolver")
 			instance = GetUserGroupCacheOS()
 		default:
-			log.Logger().Info("creating UserGroupCache without resolver")
+			log.Log(log.Security).Info("creating UserGroupCache without resolver")
 			instance = GetUserGroupNoResolve()
 		}
 		instance.ugs = make(map[string]*UserGroup)
-		log.Logger().Info("starting UserGroupCache cleaner",
+		log.Log(log.Security).Info("starting UserGroupCache cleaner",
 			zap.Stringer("cleanerInterval", instance.interval))
 		go instance.run()
 	})
@@ -93,7 +94,7 @@ func (c *UserGroupCache) run() {
 		time.Sleep(instance.interval)
 		runStart := time.Now()
 		c.cleanUpCache()
-		log.Logger().Debug("time consumed cleaning the UserGroupCache",
+		log.Log(log.Security).Debug("time consumed cleaning the UserGroupCache",
 			zap.Stringer("duration", time.Since(runStart)))
 	}
 }
@@ -116,20 +117,31 @@ func (c *UserGroupCache) cleanUpCache() {
 
 // reset the cached content, test use only
 func (c *UserGroupCache) resetCache() {
-	log.Logger().Debug("UserGroupCache reset")
+	log.Log(log.Security).Debug("UserGroupCache reset")
 	instance.lock.Lock()
 	defer instance.lock.Unlock()
 	c.ugs = make(map[string]*UserGroup)
 }
 
-func (c *UserGroupCache) ConvertUGI(ugi *si.UserGroupInformation) (UserGroup, error) {
+func (c *UserGroupCache) ConvertUGI(ugi *si.UserGroupInformation, force bool) (UserGroup, error) {
 	// check if we have a user to convert
 	if ugi == nil || ugi.User == "" {
-		return UserGroup{}, fmt.Errorf("empty user cannot resolve")
+		if force {
+			// app creation is forced, so we need to synthesize a user / group
+			ugi.User = common.AnonymousUser
+			ugi.Groups = []string{common.AnonymousGroup}
+		} else {
+			return UserGroup{}, fmt.Errorf("empty user cannot resolve")
+		}
 	}
 	// try to resolve the user if group info is empty otherwise we just convert
 	if len(ugi.Groups) == 0 {
-		return c.GetUserGroup(ugi.User)
+		ug, err := c.GetUserGroup(ugi.User)
+		if force && (err != nil || ug.failed) {
+			ugi.Groups = []string{common.AnonymousGroup}
+		} else {
+			return ug, err
+		}
 	}
 	// If groups are already present we should just convert
 	newUG := UserGroup{User: ugi.User}
@@ -171,7 +183,7 @@ func (c *UserGroupCache) GetUserGroup(userName string) (UserGroup, error) {
 	// find the user first, then resolve the groups
 	osUser, err := c.lookup(userName)
 	if err != nil {
-		log.Logger().Error("Error resolving user: does not exist",
+		log.Log(log.Security).Error("Error resolving user: does not exist",
 			zap.String("userName", userName),
 			zap.Error(err))
 		ug.failed = true
@@ -182,7 +194,7 @@ func (c *UserGroupCache) GetUserGroup(userName string) (UserGroup, error) {
 		err = ug.resolveGroups(osUser, c)
 		// log a failure and continue
 		if err != nil {
-			log.Logger().Error("Error resolving groups for user",
+			log.Log(log.Security).Error("Error resolving groups for user",
 				zap.String("userName", userName),
 				zap.Error(err))
 			ug.failed = true
