@@ -31,6 +31,10 @@ import (
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
 
+const (
+	testKey = "testKey"
+)
+
 func TestGetNormalizedPartitionName(t *testing.T) {
 	tests := []struct {
 		partitionName string
@@ -157,6 +161,18 @@ func TestAllowPreemptOther(t *testing.T) {
 	assert.Check(t, !IsAllowPreemptOther(&si.PreemptionPolicy{AllowPreemptOther: false}), "Preempt other should not be allowed if policy does not allow")
 }
 
+func TestIsAppCreationForced(t *testing.T) {
+	assert.Check(t, !IsAppCreationForced(nil), "nil tags should not result in forced app creation")
+	tags := make(map[string]string)
+	assert.Check(t, !IsAppCreationForced(tags), "empty tags should not result in forced app creation")
+	tags[common.AppTagCreateForce] = "false"
+	assert.Check(t, !IsAppCreationForced(tags), "false creation tag should not result in forced app creation")
+	tags[common.AppTagCreateForce] = "invalid"
+	assert.Check(t, !IsAppCreationForced(tags), "invalid creation tag should not result in forced app creation")
+	tags[common.AppTagCreateForce] = "true"
+	assert.Check(t, IsAppCreationForced(tags), "creation tag should result in forced app creation")
+}
+
 func TestConvertSITimeoutWithAdjustment(t *testing.T) {
 	created := time.Now().Unix() - 600
 	defaultTimeout := 15 * time.Minute
@@ -250,3 +266,130 @@ func TestWaitFor(t *testing.T) {
 		})
 	}
 }
+
+func TestMin(t *testing.T) {
+	assert.DeepEqual(t, Min(uint64(1), uint64(2)), uint64(1))
+	assert.DeepEqual(t, Min(uint64(2), uint64(1)), uint64(1))
+	assert.Check(t, Min(uint64(2), uint64(1)) == uint64(1), true)
+	assert.Check(t, Min(uint64(2), uint64(1)) == 1, false)
+}
+
+func TestGetConfigurationBool(t *testing.T) {
+	testCases := []struct {
+		name          string
+		configs       map[string]string
+		defaultValue  bool
+		expectedValue bool
+	}{
+		{
+			name:          "configs is nil",
+			configs:       nil,
+			defaultValue:  true,
+			expectedValue: true,
+		},
+		{
+			name:          "key not exist",
+			configs:       map[string]string{},
+			defaultValue:  true,
+			expectedValue: true,
+		},
+		{
+			name:          "key exist, value is not bool",
+			configs:       map[string]string{testKey: "xyz"},
+			defaultValue:  true,
+			expectedValue: true,
+		},
+		{
+			name:          "key exist, value is different from default value",
+			configs:       map[string]string{testKey: "false"},
+			defaultValue:  true,
+			expectedValue: false,
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			assert.Equal(t, tc.expectedValue, GetConfigurationBool(tc.configs, testKey, tc.defaultValue))
+		})
+	}
+}
+
+func TestGetConfigurationUint(t *testing.T) {
+	testCases := []struct {
+		name          string
+		configs       map[string]string
+		defaultValue  uint64
+		expectedValue uint64
+	}{
+		{
+			name:          "configs is nil",
+			configs:       nil,
+			defaultValue:  100,
+			expectedValue: 100,
+		},
+		{
+			name:          "key not exist",
+			configs:       map[string]string{},
+			defaultValue:  100,
+			expectedValue: 100,
+		},
+		{
+			name:          "key exist, value is not uint64",
+			configs:       map[string]string{testKey: "-1000"},
+			defaultValue:  100,
+			expectedValue: 100,
+		},
+		{
+			name:          "key exist, value is different from default value",
+			configs:       map[string]string{testKey: "1"},
+			defaultValue:  100,
+			expectedValue: 1,
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			assert.Equal(t, tc.expectedValue, GetConfigurationUint(tc.configs, testKey, tc.defaultValue))
+		})
+	}
+}
+
+func TestGetConfigurationInt(t *testing.T) {
+	testCases := []struct {
+		name          string
+		configs       map[string]string
+		defaultValue  int
+		expectedValue int
+	}{
+		{
+			name:          "configs is nil",
+			configs:       nil,
+			defaultValue:  100,
+			expectedValue: 100,
+		},
+		{
+			name:          "key not exist",
+			configs:       map[string]string{},
+			defaultValue:  100,
+			expectedValue: 100,
+		},
+		{
+			name:          "key exist, value is not int",
+			configs:       map[string]string{testKey: "xyz"},
+			defaultValue:  100,
+			expectedValue: 100,
+		},
+		{
+			name:          "key exist, value is different from default value",
+			configs:       map[string]string{testKey: "-1"},
+			defaultValue:  100,
+			expectedValue: -1,
+		},
+	}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			assert.Equal(t, tc.expectedValue, GetConfigurationInt(tc.configs, testKey, tc.defaultValue))
+		})
+	}
+}
