@@ -79,8 +79,8 @@ func TestCreateShimPublisher(t *testing.T) {
 
 // StartService() and Stop() functions should not cause panic
 func TestServiceStartStopInternal(t *testing.T) {
-	store := newEventStoreImpl()
-	publisher := createShimPublisherInternal(store)
+	store := newEventStore()
+	publisher := CreateShimPublisher(store)
 	publisher.StartService()
 	defer publisher.Stop()
 	assert.Equal(t, publisher.getEventStore(), store)
@@ -89,16 +89,16 @@ func TestServiceStartStopInternal(t *testing.T) {
 func TestNoFillWithoutEventPluginRegistered(t *testing.T) {
 	pushEventInterval := 2 * time.Millisecond
 
-	store := newEventStoreImpl()
-	publisher := createShimPublisherWithParameters(store, pushEventInterval)
+	store := newEventStore()
+	publisher := CreateShimPublisher(store)
+	publisher.pushEventInterval = pushEventInterval
 	publisher.StartService()
 	defer publisher.Stop()
 
 	event := &si.EventRecord{
 		Type:          si.EventRecord_REQUEST,
 		ObjectID:      "ask",
-		GroupID:       "app",
-		Reason:        "reason",
+		ReferenceID:   "app",
 		Message:       "message",
 		TimestampNano: 123456,
 	}
@@ -116,16 +116,16 @@ func TestPublisherSendsEvent(t *testing.T) {
 	eventPlugin, err := createEventPluginForTest()
 	assert.NilError(t, err, "could not create event plugin for test")
 
-	store := newEventStoreImpl()
-	publisher := createShimPublisherWithParameters(store, pushEventInterval)
+	store := newEventStore()
+	publisher := CreateShimPublisher(store)
+	publisher.pushEventInterval = pushEventInterval
 	publisher.StartService()
 	defer publisher.Stop()
 
 	event := &si.EventRecord{
 		Type:          si.EventRecord_REQUEST,
 		ObjectID:      "ask",
-		GroupID:       "app",
-		Reason:        "reason",
+		ReferenceID:   "app",
 		Message:       "message",
 		TimestampNano: 123456,
 	}
@@ -137,8 +137,7 @@ func TestPublisherSendsEvent(t *testing.T) {
 		t.Fatal("EventRecord should not be nil!")
 	}
 	assert.Equal(t, eventFromPlugin.ObjectID, "ask")
-	assert.Equal(t, eventFromPlugin.GroupID, "app")
-	assert.Equal(t, eventFromPlugin.Reason, "reason")
+	assert.Equal(t, eventFromPlugin.ReferenceID, "app")
 	assert.Equal(t, eventFromPlugin.Message, "message")
 	assert.Equal(t, eventFromPlugin.TimestampNano, int64(123456))
 }
