@@ -47,13 +47,19 @@ type Metrics struct {
 
 type CoreQueueMetrics interface {
 	IncQueueApplicationsAccepted()
+	GetQueueApplicationsAccepted() (int, error)
 	IncQueueApplicationsRejected()
+	GetQueueApplicationsRejected() (int, error)
 	IncQueueApplicationsRunning()
 	DecQueueApplicationsRunning()
+	GetQueueApplicationsRunning() (int, error)
 	IncQueueApplicationsFailed()
+	GetQueueApplicationsFailed() (int, error)
 	IncQueueApplicationsCompleted()
+	GetQueueApplicationsCompleted() (int, error)
 	IncAllocatedContainer()
 	IncReleasedContainer()
+	AddReleasedContainers(value int)
 	SetQueueGuaranteedResourceMetrics(resourceName string, value float64)
 	SetQueueMaxResourceMetrics(resourceName string, value float64)
 	SetQueueAllocatedResourceMetrics(resourceName string, value float64)
@@ -101,6 +107,7 @@ type CoreSchedulerMetrics interface {
 	// Metrics Ops related to TotalApplicationsRejected
 	IncTotalApplicationsRejected()
 	AddTotalApplicationsRejected(value int)
+	GetTotalApplicationsRejected() (int, error)
 
 	// Metrics Ops related to TotalApplicationsRunning
 	IncTotalApplicationsRunning()
@@ -108,7 +115,7 @@ type CoreSchedulerMetrics interface {
 	DecTotalApplicationsRunning()
 	SubTotalApplicationsRunning(value int)
 	SetTotalApplicationsRunning(value int)
-	getTotalApplicationsRunning() (int, error)
+	GetTotalApplicationsRunning() (int, error)
 
 	// Metrics Ops related to TotalApplicationsFailed
 	IncTotalApplicationsFailed()
@@ -119,6 +126,7 @@ type CoreSchedulerMetrics interface {
 	DecTotalApplicationsCompleted()
 	SubTotalApplicationsCompleted(value int)
 	SetTotalApplicationsCompleted(value int)
+	GetTotalApplicationsCompleted() (int, error)
 
 	// Metrics Ops related to ActiveNodes
 	IncActiveNodes()
@@ -128,6 +136,7 @@ type CoreSchedulerMetrics interface {
 	SetActiveNodes(value int)
 	IncDrainingNodes()
 	DecDrainingNodes()
+	GetDrainingNodes() (int, error)
 	IncUnhealthyNodes()
 	DecUnhealthyNodes()
 	IncTotalDecommissionedNodes()
