@@ -20,6 +20,7 @@ package metrics
 
 import (
 	"github.com/prometheus/client_golang/prometheus"
+	dto "github.com/prometheus/client_model/go"
 	"go.uber.org/zap"
 
 	"github.com/apache/yunikorn-core/pkg/log"
@@ -27,8 +28,9 @@ import (
 
 // QueueMetrics to declare queue metrics
 type QueueMetrics struct {
-	appMetrics      *prometheus.GaugeVec
-	ResourceMetrics *prometheus.GaugeVec
+	appMetrics       *prometheus.GaugeVec
+	containerMetrics *prometheus.CounterVec
+	ResourceMetrics  *prometheus.GaugeVec
 }
 
 // InitQueueMetrics to initialize queue metrics
@@ -42,7 +44,15 @@ func InitQueueMetrics(name string) CoreQueueMetrics {
 			Namespace: Namespace,
 			Subsystem: replaceStr,
 			Name:      "queue_app",
-			Help:      "Queue application metrics. State of the application includes `running`.",
+			Help:      "Queue application metrics. State of the application includes `accepted`, `rejected`, `running`, `failed`, `completed`.",
+		}, []string{"state"})
+
+	q.containerMetrics = prometheus.NewCounterVec(
+		prometheus.CounterOpts{
+			Namespace: Namespace,
+			Subsystem: replaceStr,
+			Name:      "queue_container",
+			Help:      "Queue container metrics. State of the attempt includes `allocated`, `released`.",
 		}, []string{"state"})
 
 	q.ResourceMetrics = prometheus.NewGaugeVec(
@@ -55,6 +65,7 @@ func InitQueueMetrics(name string) CoreQueueMetrics {
 
 	var queueMetricsList = []prometheus.Collector{
 		q.appMetrics,
+		q.containerMetrics,
 		q.ResourceMetrics,
 	}
 
@@ -64,7 +75,7 @@ func InitQueueMetrics(name string) CoreQueueMetrics {
 		// metrics name must be complied with regex: [a-zA-Z_:][a-zA-Z0-9_:]*,
 		// queue name regex: ^[a-zA-Z0-9_-]{1,64}$
 		if err := prometheus.Register(metric); err != nil {
-			log.Logger().Warn("failed to register metrics collector", zap.Error(err))
+			log.Log(log.Metrics).Warn("failed to register metrics collector", zap.Error(err))
 		}
 	}
 
@@ -84,28 +95,77 @@ func (m *QueueMetrics) DecQueueApplicationsRunning() {
 	m.appMetrics.With(prometheus.Labels{"state": "running"}).Dec()
 }
 
+func (m *QueueMetrics) GetQueueApplicationsRunning() (int, error) {
+	metricDto := &dto.Metric{}
+	err := m.appMetrics.With(prometheus.Labels{"state": "running"}).Write(metricDto)
+	if err == nil {
+		return int(*metricDto.Gauge.Value), nil
+	}
+	return -1, err
+}
+
 func (m *QueueMetrics) IncQueueApplicationsAccepted() {
 	m.appMetrics.With(prometheus.Labels{"state": "accepted"}).Inc()
 }
 
+func (m *QueueMetrics) GetQueueApplicationsAccepted() (int, error) {
+	metricDto := &dto.Metric{}
+	err := m.appMetrics.With(prometheus.Labels{"state": "accepted"}).Write(metricDto)
+	if err == nil {
+		return int(*metricDto.Gauge.Value), nil
+	}
+	return -1, err
+}
+
 func (m *QueueMetrics) IncQueueApplicationsRejected() {
 	m.appMetrics.With(prometheus.Labels{"state": "rejected"}).Inc()
 }
 
+func (m *QueueMetrics) GetQueueApplicationsRejected() (int, error) {
+	metricDto := &dto.Metric{}
+	err := m.appMetrics.With(prometheus.Labels{"state": "rejected"}).Write(metricDto)
+	if err == nil {
+		return int(*metricDto.Gauge.Value), nil
+	}
+	return -1, err
+}
+
 func (m *QueueMetrics) IncQueueApplicationsFailed() {
 	m.appMetrics.With(prometheus.Labels{"state": "failed"}).Inc()
 }
 
+func (m *QueueMetrics) GetQueueApplicationsFailed() (int, error) {
+	metricDto := &dto.Metric{}
+	err := m.appMetrics.With(prometheus.Labels{"state": "failed"}).Write(metricDto)
+	if err == nil {
+		return int(*metricDto.Gauge.Value), nil
+	}
+	return -1, err
+}
+
 func (m *QueueMetrics) IncQueueApplicationsCompleted() {
 	m.appMetrics.With(prometheus.Labels{"state": "completed"}).Inc()
 }
 
+func (m *QueueMetrics) GetQueueApplicationsCompleted() (int, error) {
+	metricDto := &dto.Metric{}
+	err := m.appMetrics.With(prometheus.Labels{"state": "completed"}).Write(metricDto)
+	if err == nil {
+		return int(*metricDto.Gauge.Value), nil
+	}
+	return -1, err
+}
+
 func (m *QueueMetrics) IncAllocatedContainer() {
-	m.appMetrics.With(prometheus.Labels{"state": "allocated"}).Inc()
+	m.containerMetrics.With(prometheus.Labels{"state": "allocated"}).Inc()
 }
 
 func (m *QueueMetrics) IncReleasedContainer() {
-	m.appMetrics.With(prometheus.Labels{"state": "released"}).Inc()
+	m.containerMetrics.With(prometheus.Labels{"state": "released"}).Inc()
+}
+
+func (m *QueueMetrics) AddReleasedContainers(value int) {
+	m.containerMetrics.With(prometheus.Labels{"state": "released"}).Add(float64(value))
 }
 
 func (m *QueueMetrics) SetQueueGuaranteedResourceMetrics(resourceName string, value float64) {
