@@ -86,7 +86,7 @@ func InitSchedulerMetrics() *SchedulerMetrics {
 			Namespace: Namespace,
 			Subsystem: SchedulerSubsystem,
 			Name:      "application_total",
-			Help:      "Total number of applications. State of the application includes `running` and `completed`.",
+			Help:      "Total number of applications. State of the application includes `running`, `completed` and `failed`.",
 		}, []string{"state"})
 
 	s.node = prometheus.NewGaugeVec(
@@ -148,7 +148,7 @@ func InitSchedulerMetrics() *SchedulerMetrics {
 	}
 	for _, metric := range metricsList {
 		if err := prometheus.Register(metric); err != nil {
-			log.Logger().Warn("failed to register metrics collector", zap.Error(err))
+			log.Log(log.Metrics).Warn("failed to register metrics collector", zap.Error(err))
 		}
 	}
 	return s
@@ -264,6 +264,15 @@ func (m *SchedulerMetrics) AddTotalApplicationsRejected(value int) {
 	m.applicationSubmission.With(prometheus.Labels{"result": "rejected"}).Add(float64(value))
 }
 
+func (m *SchedulerMetrics) GetTotalApplicationsRejected() (int, error) {
+	metricDto := &dto.Metric{}
+	err := m.applicationSubmission.With(prometheus.Labels{"result": "rejected"}).Write(metricDto)
+	if err == nil {
+		return int(*metricDto.Counter.Value), nil
+	}
+	return -1, err
+}
+
 func (m *SchedulerMetrics) IncTotalApplicationsRunning() {
 	m.application.With(prometheus.Labels{"state": "running"}).Inc()
 }
@@ -284,7 +293,7 @@ func (m *SchedulerMetrics) SetTotalApplicationsRunning(value int) {
 	m.application.With(prometheus.Labels{"state": "running"}).Set(float64(value))
 }
 
-func (m *SchedulerMetrics) getTotalApplicationsRunning() (int, error) {
+func (m *SchedulerMetrics) GetTotalApplicationsRunning() (int, error) {
 	metricDto := &dto.Metric{}
 	err := m.application.With(prometheus.Labels{"state": "running"}).Write(metricDto)
 	if err == nil {
@@ -317,6 +326,15 @@ func (m *SchedulerMetrics) SetTotalApplicationsCompleted(value int) {
 	m.application.With(prometheus.Labels{"state": "completed"}).Set(float64(value))
 }
 
+func (m *SchedulerMetrics) GetTotalApplicationsCompleted() (int, error) {
+	metricDto := &dto.Metric{}
+	err := m.application.With(prometheus.Labels{"state": "completed"}).Write(metricDto)
+	if err == nil {
+		return int(*metricDto.Gauge.Value), nil
+	}
+	return -1, err
+}
+
 func (m *SchedulerMetrics) IncActiveNodes() {
 	m.node.With(prometheus.Labels{"state": "active"}).Inc()
 }
@@ -380,7 +398,7 @@ func (m *SchedulerMetrics) SetNodeResourceUsage(resourceName string, rangeIdx in
 				Help:      "Total resource usage of node, by resource name.",
 			}, []string{"range"})
 		if err := prometheus.Register(resourceMetrics); err != nil {
-			log.Logger().Warn("failed to register metrics collector", zap.Error(err))
+			log.Log(log.Metrics).Warn("failed to register metrics collector", zap.Error(err))
 			return
 		}
 		m.nodeResourceUsage[resourceName] = resourceMetrics
@@ -396,6 +414,15 @@ func (m *SchedulerMetrics) DecDrainingNodes() {
 	m.node.With(prometheus.Labels{"state": "draining"}).Dec()
 }
 
+func (m *SchedulerMetrics) GetDrainingNodes() (int, error) {
+	metricDto := &dto.Metric{}
+	err := m.node.With(prometheus.Labels{"state": "draining"}).Write(metricDto)
+	if err == nil {
+		return int(*metricDto.Gauge.Value), nil
+	}
+	return -1, err
+}
+
 func (m *SchedulerMetrics) IncTotalDecommissionedNodes() {
 	m.node.With(prometheus.Labels{"state": "decommissioned"}).Inc()
 }
