@@ -33,8 +33,6 @@ import (
 	"github.com/apache/yunikorn-core/pkg/webservice/dao"
 )
 
-const defaultPeriod = 30 * time.Second
-
 type HealthChecker struct {
 	context       *ClusterContext
 	confWatcherId string
@@ -76,7 +74,7 @@ func (c *HealthChecker) readPeriod() time.Duration {
 
 	result, err := time.ParseDuration(value)
 	if err != nil {
-		log.Logger().Warn("Failed to parse configuration value",
+		log.Log(log.SchedHealth).Warn("Failed to parse configuration value",
 			zap.String("key", configs.HealthCheckInterval),
 			zap.String("value", value),
 			zap.Error(err))
@@ -112,7 +110,7 @@ func (c *HealthChecker) startInternal(runImmediately bool) {
 		c.period = period
 		c.enabled = true
 
-		log.Logger().Info("Starting periodic health checker", zap.Duration("interval", period))
+		log.Log(log.SchedHealth).Info("Starting periodic health checker", zap.Duration("interval", period))
 
 		go func() {
 			ticker := time.NewTicker(period)
@@ -131,7 +129,7 @@ func (c *HealthChecker) startInternal(runImmediately bool) {
 		c.stopChan = nil
 		c.period = 0
 		c.enabled = false
-		log.Logger().Info("Periodic health checker disabled")
+		log.Log(log.SchedHealth).Info("Periodic health checker disabled")
 	}
 }
 
@@ -142,7 +140,7 @@ func (c *HealthChecker) Stop() {
 	configs.RemoveConfigMapCallback(c.confWatcherId)
 
 	if c.stopChan != nil {
-		log.Logger().Info("Stopping periodic health checker")
+		log.Log(log.SchedHealth).Info("Stopping periodic health checker")
 		*c.stopChan <- struct{}{}
 		close(*c.stopChan)
 		c.stopChan = nil
@@ -173,11 +171,17 @@ func (c *HealthChecker) runOnce() {
 	result := GetSchedulerHealthStatus(schedulerMetrics, c.context)
 	updateSchedulerLastHealthStatus(&result, c.context)
 	if !result.Healthy {
-		log.Logger().Warn("Scheduler is not healthy",
-			zap.Any("health check values", result.HealthChecks))
+		for _, v := range result.HealthChecks {
+			if v.Succeeded {
+				continue
+			}
+			log.Log(log.SchedHealth).Warn("Scheduler is not healthy",
+				zap.String("name", v.Name),
+				zap.String("description", v.Description),
+				zap.String("message", v.DiagnosisMessage))
+		}
 	} else {
-		log.Logger().Debug("Scheduler is healthy",
-			zap.Any("health check values", result.HealthChecks))
+		log.Log(log.SchedHealth).Debug("Scheduler is healthy")
 	}
 }
 
@@ -303,8 +307,8 @@ func checkSchedulingContext(schedulerContext *ClusterContext) []dao.HealthCheckI
 		"Check for negative resources in the nodes",
 		fmt.Sprintf("Nodes with negative resources: %q", nodesWithNegResources))
 	info[2] = CreateCheckInfo(len(allocationMismatch) == 0, "Consistency of data",
-		"Check if a node's allocated resource <= total resource of the node",
-		fmt.Sprintf("Nodes with inconsistent data: %q", allocationMismatch))
+		"Check if a partition's allocated resource <= total resource of the partition",
+		fmt.Sprintf("Partitions with inconsistent data: %q", allocationMismatch))
 	info[3] = CreateCheckInfo(len(totalResourceMismatch) == 0, "Consistency of data",
 		"Check if total partition resource == sum of the node resources from the partition",
 		fmt.Sprintf("Partitions with inconsistent data: %q", totalResourceMismatch))
