@@ -24,6 +24,7 @@ import (
 	"testing"
 	"time"
 
+	"github.com/google/go-cmp/cmp/cmpopts"
 	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common/resources"
@@ -49,7 +50,7 @@ func TestNewAlloc(t *testing.T) {
 	res, err := resources.NewResourceFromConf(map[string]string{"first": "1"})
 	assert.NilError(t, err, "Resource creation failed")
 	ask := newAllocationAsk("ask-1", "app-1", res)
-	alloc := NewAllocation("test-uuid", "node-1", "itype1", ask)
+	alloc := NewAllocation("test-uuid", "node-1", ask)
 	if alloc == nil {
 		t.Fatal("NewAllocation create failed while it should not")
 	}
@@ -57,6 +58,9 @@ func TestNewAlloc(t *testing.T) {
 	assert.Assert(t, resources.Equals(alloc.GetAllocatedResource(), res), "Allocated resource not set correctly")
 	assert.Assert(t, !alloc.IsPlaceholder(), "ask should not have been a placeholder")
 	assert.Equal(t, time.Now().Round(time.Second), alloc.GetCreateTime().Round(time.Second))
+	assert.Equal(t, alloc.GetInstanceType(), "", "Default instance type should be empty")
+	alloc.SetInstanceType(instType1)
+	assert.Equal(t, alloc.GetInstanceType(), instType1, "Instance type not set as expected")
 	allocStr := alloc.String()
 	expected := "applicationID=app-1, uuid=test-uuid, allocationKey=ask-1, Node=node-1, result=Allocated"
 	assert.Equal(t, allocStr, expected, "Strings should have been equal")
@@ -73,7 +77,7 @@ func TestNewAlloc(t *testing.T) {
 	tags[siCommon.CreationTime] = strconv.FormatInt(past, 10)
 	ask.tags = CloneAllocationTags(tags)
 	ask.createTime = time.Unix(past, 0)
-	alloc = NewAllocation("test-uuid", "node-1", "itype1", ask)
+	alloc = NewAllocation("test-uuid", "node-1", ask)
 	assert.Equal(t, alloc.GetCreateTime(), ask.GetCreateTime(), "createTime was not copied from the ask")
 }
 
@@ -81,7 +85,7 @@ func TestNewReservedAlloc(t *testing.T) {
 	res, err := resources.NewResourceFromConf(map[string]string{"first": "1"})
 	assert.NilError(t, err, "Resource creation failed")
 	ask := newAllocationAsk("ask-1", "app-1", res)
-	alloc := newReservedAllocation(Reserved, "node-1", "instType1", ask)
+	alloc := newReservedAllocation("node-1", ask)
 	if alloc == nil {
 		t.Fatal("NewReservedAllocation create failed while it should not")
 	}
@@ -93,6 +97,22 @@ func TestNewReservedAlloc(t *testing.T) {
 	assert.Equal(t, allocStr, expected, "Strings should have been equal")
 }
 
+func TestNewUnreservedAlloc(t *testing.T) {
+	res, err := resources.NewResourceFromConf(map[string]string{"first": "1"})
+	assert.NilError(t, err, "Resource creation failed")
+	ask := newAllocationAsk("ask-1", "app-1", res)
+	alloc := newUnreservedAllocation("node-1", ask)
+	if alloc == nil {
+		t.Fatal("NewReservedAllocation create failed while it should not")
+	}
+	assert.Equal(t, alloc.GetResult(), Unreserved, "NewReservedAlloc should have Reserved result")
+	assert.Equal(t, alloc.GetUUID(), "", "NewReservedAlloc should not have uuid")
+	assert.Assert(t, resources.Equals(alloc.GetAllocatedResource(), res), "Allocated resource not set correctly")
+	allocStr := alloc.String()
+	expected := "applicationID=app-1, uuid=N/A, allocationKey=ask-1, Node=node-1, result=Unreserved"
+	assert.Equal(t, allocStr, expected, "Strings should have been equal")
+}
+
 func TestSIFromNilAlloc(t *testing.T) {
 	defer func() {
 		if r := recover(); r != nil {
@@ -117,12 +137,12 @@ func TestSIFromAlloc(t *testing.T) {
 		ResourcePerAlloc: res.ToProto(),
 	}
 	ask := newAllocationAsk("ask-1", "app-1", res)
-	alloc := NewAllocation("test-uuid", "node-1", "itype1", ask)
+	alloc := NewAllocation("test-uuid", "node-1", ask)
 	if alloc == nil {
 		t.Fatal("NewAllocation create failed while it should not")
 	}
 	allocSI := alloc.NewSIFromAllocation()
-	assert.DeepEqual(t, allocSI, expectedSI)
+	assert.DeepEqual(t, allocSI, expectedSI, cmpopts.IgnoreUnexported(si.Allocation{}, si.Resource{}, si.Quantity{}))
 }
 
 func TestNewAllocFromNilSI(t *testing.T) {
@@ -132,7 +152,7 @@ func TestNewAllocFromNilSI(t *testing.T) {
 		}
 	}()
 	var nilAlloc *Allocation
-	alloc := NewAllocationFromSI(nil, "itype1")
+	alloc := NewAllocationFromSI(nil)
 	assert.Equal(t, alloc, nilAlloc, "Expected nil response from nil SI allocation")
 }
 
@@ -152,16 +172,16 @@ func TestNewAllocFromSI(t *testing.T) {
 		AllocationTags:   tags,
 	}
 	var nilAlloc *Allocation
-	alloc := NewAllocationFromSI(allocSI, "itype1")
+	alloc := NewAllocationFromSI(allocSI)
 	assert.Equal(t, alloc, nilAlloc, "placeholder allocation created without a TaskGroupName")
 	allocSI.TaskGroupName = "testgroup"
-	alloc = NewAllocationFromSI(allocSI, "itype1")
+	alloc = NewAllocationFromSI(allocSI)
 	assert.Assert(t, alloc != nilAlloc, "placeholder ask creation failed unexpectedly")
 	assert.Assert(t, alloc.IsPlaceholder(), "ask should have been a placeholder")
 	assert.Equal(t, alloc.GetTaskGroup(), "testgroup", "TaskGroupName not set as expected")
 	assert.Equal(t, alloc.GetAsk().GetCreateTime(), time.Unix(past, 0)) //nolint:staticcheck
 
 	allocSI.AllocationTags[siCommon.CreationTime] = "xyz"
-	alloc = NewAllocationFromSI(allocSI, "itype1")
+	alloc = NewAllocationFromSI(allocSI)
 	assert.Equal(t, alloc.GetAsk().GetCreateTime().Unix(), int64(-1)) //nolint:staticcheck
 }
