@@ -27,6 +27,7 @@ import (
 	"go.uber.org/zap"
 
 	"github.com/apache/yunikorn-core/pkg/common/resources"
+	"github.com/apache/yunikorn-core/pkg/events"
 	"github.com/apache/yunikorn-core/pkg/log"
 	"github.com/apache/yunikorn-core/pkg/plugins"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/common"
@@ -57,6 +58,7 @@ type Node struct {
 
 	reservations map[string]*reservation // a map of reservations
 	listeners    []NodeListener          // a list of node listeners
+	nodeEvents   *nodeEvents
 
 	sync.RWMutex
 }
@@ -70,7 +72,8 @@ func NewNode(proto *si.NodeInfo) *Node {
 	var ready bool
 	var err error
 	if ready, err = strconv.ParseBool(proto.Attributes[common.NodeReadyAttribute]); err != nil {
-		log.Logger().Error("Could not parse ready flag, assuming true")
+		log.Log(log.SchedNode).Debug("Could not parse ready flag, assuming true",
+			zap.String("nodeID", proto.NodeID))
 		ready = true
 	}
 	sn := &Node{
@@ -84,10 +87,11 @@ func NewNode(proto *si.NodeInfo) *Node {
 		listeners:         make([]NodeListener, 0),
 		ready:             ready,
 	}
+	sn.nodeEvents = newNodeEvents(sn, events.GetEventSystem())
 	// initialise available resources
 	sn.availableResource, err = resources.SubErrorNegative(sn.totalResource, sn.occupiedResource)
 	if err != nil {
-		log.Logger().Error("New node created with no available resources",
+		log.Log(log.SchedNode).Error("New node created with no available resources",
 			zap.Error(err))
 	}
 
@@ -159,12 +163,13 @@ func (sn *Node) SetCapacity(newCapacity *resources.Resource) *resources.Resource
 	sn.Lock()
 	defer sn.Unlock()
 	if resources.Equals(sn.totalResource, newCapacity) {
-		log.Logger().Debug("skip updating capacity, not changed")
+		log.Log(log.SchedNode).Debug("skip updating capacity, not changed")
 		return nil
 	}
 	delta := resources.Sub(newCapacity, sn.totalResource)
 	sn.totalResource = newCapacity
 	sn.refreshAvailableResource()
+	sn.nodeEvents.sendNodeCapacityChangedEvent()
 	return delta
 }
 
@@ -179,10 +184,11 @@ func (sn *Node) SetOccupiedResource(occupiedResource *resources.Resource) {
 	sn.Lock()
 	defer sn.Unlock()
 	if resources.Equals(sn.occupiedResource, occupiedResource) {
-		log.Logger().Debug("skip updating occupiedResource, not changed")
+		log.Log(log.SchedNode).Debug("skip updating occupiedResource, not changed")
 		return
 	}
 	sn.occupiedResource = occupiedResource
+	sn.nodeEvents.sendNodeOccupiedResourceChangedEvent()
 	sn.refreshAvailableResource()
 }
 
@@ -194,7 +200,7 @@ func (sn *Node) refreshAvailableResource() {
 	sn.availableResource.SubFrom(sn.occupiedResource)
 	// check if any quantity is negative: a nil resource is all 0's
 	if !resources.StrictlyGreaterThanOrEquals(sn.availableResource, nil) {
-		log.Logger().Warn("Node update triggered over allocated node",
+		log.Log(log.SchedNode).Warn("Node update triggered over allocated node",
 			zap.Stringer("available", sn.availableResource),
 			zap.Stringer("total", sn.totalResource),
 			zap.Stringer("occupied", sn.occupiedResource),
@@ -232,6 +238,7 @@ func (sn *Node) SetSchedulable(schedulable bool) {
 	sn.Lock()
 	defer sn.Unlock()
 	sn.schedulable = schedulable
+	sn.nodeEvents.sendNodeSchedulableChangedEvent(sn.schedulable)
 }
 
 // Can this node be used in scheduling.
@@ -297,6 +304,7 @@ func (sn *Node) RemoveAllocation(uuid string) *Allocation {
 		delete(sn.allocations, uuid)
 		sn.allocatedResource.SubFrom(alloc.GetAllocatedResource())
 		sn.availableResource.AddTo(alloc.GetAllocatedResource())
+		sn.nodeEvents.sendAllocationRemovedEvent(alloc.allocationKey, alloc.allocatedResource)
 		return alloc
 	}
 
@@ -319,6 +327,7 @@ func (sn *Node) AddAllocation(alloc *Allocation) bool {
 		sn.allocations[alloc.GetUUID()] = alloc
 		sn.allocatedResource.AddTo(res)
 		sn.availableResource.SubFrom(res)
+		sn.nodeEvents.sendAllocationAddedEvent(alloc.allocationKey, res)
 		return true
 	}
 	return false
@@ -341,7 +350,7 @@ func (sn *Node) ReplaceAllocation(uuid string, replace *Allocation, delta *resou
 	sn.allocatedResource.AddTo(delta)
 	sn.availableResource.SubFrom(delta)
 	if !resources.FitIn(before, sn.allocatedResource) {
-		log.Logger().Warn("unexpected increase in node usage after placeholder replacement",
+		log.Log(log.SchedNode).Warn("unexpected increase in node usage after placeholder replacement",
 			zap.String("placeholder uuid", uuid),
 			zap.String("allocation uuid", replace.GetUUID()),
 			zap.Stringer("delta", delta))
@@ -382,7 +391,7 @@ func (sn *Node) preConditions(ask *AllocationAsk, allocate bool) bool {
 			NodeID:        sn.NodeID,
 			Allocate:      allocate,
 		}); err != nil {
-			log.Logger().Debug("running predicates failed",
+			log.Log(log.SchedNode).Debug("running predicates failed",
 				zap.String("allocationKey", allocID),
 				zap.String("nodeID", sn.NodeID),
 				zap.Bool("allocateFlag", allocate),
@@ -401,14 +410,14 @@ func (sn *Node) preConditions(ask *AllocationAsk, allocate bool) bool {
 func (sn *Node) preAllocateCheck(res *resources.Resource, resKey string) bool {
 	// cannot allocate zero or negative resource
 	if !resources.StrictlyGreaterThanZero(res) {
-		log.Logger().Debug("pre alloc check: requested resource is zero",
+		log.Log(log.SchedNode).Debug("pre alloc check: requested resource is zero",
 			zap.String("nodeID", sn.NodeID))
 		return false
 	}
 	// check if the node is reserved for this app/alloc
 	if sn.IsReserved() {
 		if !sn.isReservedForApp(resKey) {
-			log.Logger().Debug("pre alloc check: node reserved for different app or ask",
+			log.Log(log.SchedNode).Debug("pre alloc check: node reserved for different app or ask",
 				zap.String("nodeID", sn.NodeID),
 				zap.String("resKey", resKey))
 			return false
@@ -463,7 +472,7 @@ func (sn *Node) Reserve(app *Application, ask *AllocationAsk) error {
 	// this should really not happen just guard against panic
 	// either app or ask are nil
 	if appReservation == nil {
-		log.Logger().Debug("reservation creation failed unexpectedly",
+		log.Log(log.SchedNode).Debug("reservation creation failed unexpectedly",
 			zap.String("nodeID", sn.NodeID),
 			zap.Any("app", app),
 			zap.Any("ask", ask))
@@ -471,7 +480,7 @@ func (sn *Node) Reserve(app *Application, ask *AllocationAsk) error {
 	}
 	// reservation must fit on the empty node
 	if !resources.FitIn(sn.totalResource, ask.GetAllocatedResource()) {
-		log.Logger().Debug("reservation does not fit on the node",
+		log.Log(log.SchedNode).Debug("reservation does not fit on the node",
 			zap.String("nodeID", sn.NodeID),
 			zap.String("appID", app.ApplicationID),
 			zap.String("ask", ask.GetAllocationKey()),
@@ -479,6 +488,7 @@ func (sn *Node) Reserve(app *Application, ask *AllocationAsk) error {
 		return fmt.Errorf("reservation does not fit on node %s, appID %s, ask %s", sn.NodeID, app.ApplicationID, ask.GetAllocatedResource().String())
 	}
 	sn.reservations[appReservation.getKey()] = appReservation
+	sn.nodeEvents.sendReservedEvent(ask.GetAllocatedResource(), ask.GetAllocationKey())
 	// reservation added successfully
 	return nil
 }
@@ -492,7 +502,7 @@ func (sn *Node) unReserve(app *Application, ask *AllocationAsk) (int, error) {
 	defer sn.Unlock()
 	resKey := reservationKey(nil, app, ask)
 	if resKey == "" {
-		log.Logger().Debug("unreserve reservation key create failed unexpectedly",
+		log.Log(log.SchedNode).Debug("unreserve reservation key create failed unexpectedly",
 			zap.String("nodeID", sn.NodeID),
 			zap.Any("app", app),
 			zap.Any("ask", ask))
@@ -500,10 +510,11 @@ func (sn *Node) unReserve(app *Application, ask *AllocationAsk) (int, error) {
 	}
 	if _, ok := sn.reservations[resKey]; ok {
 		delete(sn.reservations, resKey)
+		sn.nodeEvents.sendUnreservedEvent(ask.GetAllocatedResource(), ask.GetAllocationKey())
 		return 1, nil
 	}
 	// reservation was not found
-	log.Logger().Debug("reservation not found while removing from node",
+	log.Log(log.SchedNode).Debug("reservation not found while removing from node",
 		zap.String("nodeID", sn.NodeID),
 		zap.String("appID", app.ApplicationID),
 		zap.String("ask", ask.GetAllocationKey()))
@@ -583,4 +594,13 @@ func (sn *Node) SetReady(ready bool) {
 	sn.Lock()
 	defer sn.Unlock()
 	sn.ready = ready
+	sn.nodeEvents.sendNodeReadyChangedEvent(sn.ready)
+}
+
+func (sn *Node) SendNodeAddedEvent() {
+	sn.nodeEvents.sendNodeAddedEvent()
+}
+
+func (sn *Node) SendNodeRemovedEvent() {
+	sn.nodeEvents.sendNodeRemovedEvent()
 }
