@@ -26,6 +26,7 @@ import (
 
 	"github.com/apache/yunikorn-core/pkg/common/resources"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/common"
+	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
 
 const testNode = "testnode"
@@ -130,7 +131,7 @@ func TestPreAllocateCheck(t *testing.T) {
 	assert.Assert(t, !node.preAllocateCheck(resLarge, ""), "too large resource should not have fitted on node")
 
 	// set allocated resource
-	node.AddAllocation(newAllocation(appID1, "UUID1", nodeID, "root.default", resSmall))
+	node.AddAllocation(newAllocation(appID1, "UUID1", nodeID, resSmall))
 	assert.Assert(t, node.preAllocateCheck(resSmall, ""), "small resource should have fitted in available allocation")
 	assert.Assert(t, !node.preAllocateCheck(resNode, ""), "node resource should not have fitted in available allocation")
 
@@ -369,7 +370,7 @@ func TestAddAllocation(t *testing.T) {
 	}
 	// allocate half of the resources available and check the calculation
 	half := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 50, "second": 100})
-	node.AddAllocation(newAllocation(appID1, "1", nodeID1, "queue-1", half))
+	node.AddAllocation(newAllocation(appID1, "1", nodeID1, half))
 	if node.GetAllocation("1") == nil {
 		t.Fatal("failed to add allocations: allocation not returned")
 	}
@@ -385,7 +386,7 @@ func TestAddAllocation(t *testing.T) {
 	}
 	// second and check calculation
 	piece := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 25, "second": 50})
-	node.AddAllocation(newAllocation(appID1, "2", nodeID1, "queue-1", piece))
+	node.AddAllocation(newAllocation(appID1, "2", nodeID1, piece))
 	if node.GetAllocation("2") == nil {
 		t.Fatal("failed to add allocations: allocation not returned")
 	}
@@ -411,7 +412,7 @@ func TestRemoveAllocation(t *testing.T) {
 
 	// allocate half of the resources available and check the calculation
 	half := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 50, "second": 100})
-	node.AddAllocation(newAllocation(appID1, "1", nodeID1, "queue-1", half))
+	node.AddAllocation(newAllocation(appID1, "1", nodeID1, half))
 	if node.GetAllocation("1") == nil {
 		t.Fatal("failed to add allocations: allocation not returned")
 	}
@@ -428,7 +429,7 @@ func TestRemoveAllocation(t *testing.T) {
 
 	// add second alloc and remove first check calculation
 	piece := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 25, "second": 50})
-	node.AddAllocation(newAllocation(appID1, "2", nodeID1, "queue-1", piece))
+	node.AddAllocation(newAllocation(appID1, "2", nodeID1, piece))
 	if node.GetAllocation("2") == nil {
 		t.Fatal("failed to add allocations: allocation not returned")
 	}
@@ -456,7 +457,7 @@ func TestNodeReplaceAllocation(t *testing.T) {
 	// allocate half of the resources available and check the calculation
 	phID := "ph-1"
 	half := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 50, "second": 100})
-	ph := newPlaceholderAlloc(appID1, phID, nodeID1, "queue-1", half)
+	ph := newPlaceholderAlloc(appID1, phID, nodeID1, half)
 	node.AddAllocation(ph)
 	assert.Assert(t, node.GetAllocation(phID) != nil, "failed to add placeholder allocation")
 	assert.Assert(t, resources.Equals(node.GetAllocatedResource(), half), "allocated resource not set correctly %v got %v", half, node.GetAllocatedResource())
@@ -464,7 +465,7 @@ func TestNodeReplaceAllocation(t *testing.T) {
 
 	allocID := "real-1"
 	piece := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 25, "second": 50})
-	alloc := newAllocation(appID1, allocID, nodeID1, "queue-1", piece)
+	alloc := newAllocation(appID1, allocID, nodeID1, piece)
 	// calculate the delta: new allocation resource - placeholder (should be negative!)
 	delta := resources.Sub(piece, half)
 	assert.Assert(t, delta.HasNegativeValue(), "expected negative values in delta")
@@ -490,7 +491,7 @@ func TestGetAllocation(t *testing.T) {
 	if alloc != nil {
 		t.Fatalf("allocation should not have been found")
 	}
-	node.AddAllocation(newAllocation(appID1, "1", nodeID1, "queue-1", nil))
+	node.AddAllocation(newAllocation(appID1, "1", nodeID1, nil))
 	alloc = node.GetAllocation("1")
 	if alloc == nil {
 		t.Fatalf("allocation should have been found")
@@ -515,11 +516,11 @@ func TestGetAllocations(t *testing.T) {
 	}
 
 	// allocate
-	node.AddAllocation(newAllocation(appID1, "1", nodeID1, "queue-1", nil))
-	node.AddAllocation(newAllocation(appID1, "2", nodeID1, "queue-1", nil))
+	node.AddAllocation(newAllocation(appID1, "1", nodeID1, nil))
+	node.AddAllocation(newAllocation(appID1, "2", nodeID1, nil))
 	assert.Equal(t, 2, len(node.GetAllAllocations()), "allocation length mismatch")
 	// This should not happen in real code just making sure the code does do what is expected
-	node.AddAllocation(newAllocation(appID1, "2", nodeID1, "queue-1", nil))
+	node.AddAllocation(newAllocation(appID1, "2", nodeID1, nil))
 	assert.Equal(t, 2, len(node.GetAllAllocations()), "allocation length mismatch")
 }
 
@@ -651,3 +652,98 @@ func TestReadyAttribute(t *testing.T) {
 	node = NewNode(proto)
 	assert.Equal(t, false, node.ready, "Node should not be in ready state")
 }
+
+func TestNodeEvents(t *testing.T) {
+	mockEvents := newEventSystemMock()
+	total := resources.NewResourceFromMap(map[string]resources.Quantity{"cpu": 100, "memory": 100})
+	occupied := resources.NewResourceFromMap(map[string]resources.Quantity{"cpu": 10, "memory": 10})
+	proto := newProto(testNode, total, occupied, map[string]string{
+		"ready": "true",
+	})
+	node := NewNode(proto)
+	node.nodeEvents = newNodeEvents(node, mockEvents)
+
+	node.SendNodeAddedEvent()
+	assert.Equal(t, 1, len(mockEvents.events))
+	event := mockEvents.events[0]
+	assert.Equal(t, si.EventRecord_NODE, event.Type)
+	assert.Equal(t, si.EventRecord_ADD, event.EventChangeType)
+
+	mockEvents.Reset()
+	node.SendNodeRemovedEvent()
+	assert.Equal(t, 1, len(mockEvents.events))
+	event = mockEvents.events[0]
+	assert.Equal(t, si.EventRecord_NODE, event.Type)
+	assert.Equal(t, si.EventRecord_REMOVE, event.EventChangeType)
+
+	mockEvents.Reset()
+	node.SetReady(false)
+	assert.Equal(t, 1, len(mockEvents.events))
+	event = mockEvents.events[0]
+	assert.Equal(t, si.EventRecord_NODE, event.Type)
+	assert.Equal(t, si.EventRecord_SET, event.EventChangeType)
+	assert.Equal(t, si.EventRecord_NODE_READY, event.EventChangeDetail)
+
+	mockEvents.Reset()
+	node.AddAllocation(&Allocation{
+		allocatedResource: resources.NewResourceFromMap(map[string]resources.Quantity{"cpu": 1, "memory": 1}),
+		allocationKey:     aKey,
+		uuid:              "uuid-0",
+	})
+	assert.Equal(t, 1, len(mockEvents.events))
+	event = mockEvents.events[0]
+	assert.Equal(t, si.EventRecord_NODE, event.Type)
+	assert.Equal(t, si.EventRecord_ADD, event.EventChangeType)
+	assert.Equal(t, si.EventRecord_NODE_ALLOC, event.EventChangeDetail)
+
+	mockEvents.Reset()
+	node.RemoveAllocation("uuid-0")
+	event = mockEvents.events[0]
+	assert.Equal(t, si.EventRecord_NODE, event.Type)
+	assert.Equal(t, si.EventRecord_REMOVE, event.EventChangeType)
+	assert.Equal(t, si.EventRecord_NODE_ALLOC, event.EventChangeDetail)
+
+	mockEvents.Reset()
+	node.SetOccupiedResource(resources.NewResourceFromMap(map[string]resources.Quantity{"cpu": 20, "memory": 20}))
+	event = mockEvents.events[0]
+	assert.Equal(t, si.EventRecord_NODE, event.Type)
+	assert.Equal(t, si.EventRecord_SET, event.EventChangeType)
+	assert.Equal(t, si.EventRecord_NODE_OCCUPIED, event.EventChangeDetail)
+	assert.Equal(t, int64(20), event.Resource.Resources["cpu"].Value)
+	assert.Equal(t, int64(20), event.Resource.Resources["memory"].Value)
+
+	mockEvents.Reset()
+	node.SetCapacity(resources.NewResourceFromMap(map[string]resources.Quantity{"cpu": 90, "memory": 90}))
+	event = mockEvents.events[0]
+	assert.Equal(t, si.EventRecord_NODE, event.Type)
+	assert.Equal(t, si.EventRecord_SET, event.EventChangeType)
+	assert.Equal(t, si.EventRecord_NODE_CAPACITY, event.EventChangeDetail)
+	assert.Equal(t, int64(90), event.Resource.Resources["cpu"].Value)
+	assert.Equal(t, int64(90), event.Resource.Resources["memory"].Value)
+
+	mockEvents.Reset()
+	node.SetSchedulable(false)
+	event = mockEvents.events[0]
+	assert.Equal(t, si.EventRecord_NODE, event.Type)
+	assert.Equal(t, si.EventRecord_SET, event.EventChangeType)
+	assert.Equal(t, si.EventRecord_NODE_SCHEDULABLE, event.EventChangeDetail)
+
+	mockEvents.Reset()
+	res := resources.NewResourceFromMap(map[string]resources.Quantity{"cpu": 10})
+	ask := newAllocationAsk(aKey, appID1, res)
+	app := newApplication(appID1, "default", "root.unknown")
+	err := node.Reserve(app, ask)
+	assert.NilError(t, err, "could not reserve")
+	event = mockEvents.events[0]
+	assert.Equal(t, si.EventRecord_NODE, event.Type)
+	assert.Equal(t, si.EventRecord_ADD, event.EventChangeType)
+	assert.Equal(t, si.EventRecord_NODE_RESERVATION, event.EventChangeDetail)
+
+	mockEvents.Reset()
+	_, err = node.unReserve(app, ask)
+	assert.NilError(t, err, "could not unreserve")
+	event = mockEvents.events[0]
+	assert.Equal(t, si.EventRecord_NODE, event.Type)
+	assert.Equal(t, si.EventRecord_REMOVE, event.EventChangeType)
+	assert.Equal(t, si.EventRecord_NODE_RESERVATION, event.EventChangeDetail)
+}
