@@ -53,17 +53,17 @@ type Preemptor struct {
 	nodesTried      bool                // flag indicating that scheduling has already been tried on all nodes
 
 	// lazily-populated work structures
-	allocationsByQueue  map[string]*QueuePreemptionSnapshot // map of queue snapshots by queue path
-	queueByAlloc        map[string]*QueuePreemptionSnapshot // map of queue snapshots by allocationID
-	allocationsByNode   map[string][]*Allocation            // map of allocation by nodeID
-	nodeAvailableMap    map[string]*resources.Resource      // map of available resources by nodeID
-	nodeInstanceTypeMap map[string]string
+	allocationsByQueue map[string]*QueuePreemptionSnapshot // map of queue snapshots by queue path
+	queueByAlloc       map[string]*QueuePreemptionSnapshot // map of queue snapshots by allocationID
+	allocationsByNode  map[string][]*Allocation            // map of allocation by nodeID
+	nodeAvailableMap   map[string]*resources.Resource      // map of available resources by nodeID
 }
 
 // QueuePreemptionSnapshot is used to track a snapshot of a queue for preemption
 type QueuePreemptionSnapshot struct {
 	Parent             *QueuePreemptionSnapshot // snapshot of parent queue
 	QueuePath          string                   // fully qualified path to queue
+	Leaf               bool                     // true if queue is a leaf queue
 	AllocatedResource  *resources.Resource      // allocated resources
 	PreemptingResource *resources.Resource      // resources currently flagged for preemption
 	MaxResource        *resources.Resource      // maximum resources for this queue
@@ -143,7 +143,6 @@ func (p *Preemptor) initWorkingState() {
 	allocationsByNode := make(map[string][]*Allocation)
 	queueByAlloc := make(map[string]*QueuePreemptionSnapshot)
 	nodeAvailableMap := make(map[string]*resources.Resource)
-	nodeInstanceTypeMap := make(map[string]string)
 
 	// build a map from NodeID to allocation and from allocationID to queue capacities
 	for _, victims := range p.allocationsByQueue {
@@ -155,16 +154,11 @@ func (p *Preemptor) initWorkingState() {
 			}
 			allocationsByNode[nodeID] = append(allocations, allocation)
 			queueByAlloc[allocation.GetAllocationKey()] = victims
-			instType := allocation.GetInstanceType()
-			if instType != "" && instType != UnknownInstanceType {
-				nodeInstanceTypeMap[nodeID] = instType
-			}
 		}
 	}
 
 	// walk node iterator and track available resources per node
-	for p.iterator.HasNext() {
-		node := p.iterator.Next()
+	p.iterator.ForEachNode(func(node *Node) bool {
 		if !node.IsSchedulable() || (node.IsReserved() && !node.isReservedForApp(reservationKey(nil, p.application, p.ask))) {
 			// node is not available, remove any potential victims from consideration
 			delete(allocationsByNode, node.NodeID)
@@ -172,7 +166,8 @@ func (p *Preemptor) initWorkingState() {
 			// track allocated and available resources
 			nodeAvailableMap[node.NodeID] = node.GetAvailableResource()
 		}
-	}
+		return true
+	})
 
 	// sort the allocations on each node in the order we'd like to try them
 	sortVictimsForPreemption(allocationsByNode)
@@ -180,7 +175,6 @@ func (p *Preemptor) initWorkingState() {
 	p.allocationsByNode = allocationsByNode
 	p.queueByAlloc = queueByAlloc
 	p.nodeAvailableMap = nodeAvailableMap
-	p.nodeInstanceTypeMap = nodeInstanceTypeMap
 }
 
 // checkPreemptionQueueGuarantees verifies that it's possible to free enough resources to fit the given ask
@@ -190,7 +184,7 @@ func (p *Preemptor) checkPreemptionQueueGuarantees() bool {
 	queues := p.duplicateQueueSnapshots()
 	currentQueue, ok := queues[p.queuePath]
 	if !ok {
-		log.Logger().Warn("BUG: Didn't find current queue in snapshot list",
+		log.Log(log.SchedPreemption).Warn("BUG: Didn't find current queue in snapshot list",
 			zap.String("queuePath", p.queuePath))
 		return false
 	}
@@ -227,7 +221,7 @@ func (p *Preemptor) calculateVictimsByNode(nodeAvailable *resources.Resource, po
 	// get the current queue snapshot
 	askQueue, ok := allocationsByQueueSnap[p.queuePath]
 	if !ok {
-		log.Logger().Warn("BUG: Queue not found by name", zap.String("queuePath", p.queuePath))
+		log.Log(log.SchedPreemption).Warn("BUG: Queue not found by name", zap.String("queuePath", p.queuePath))
 		return -1, nil
 	}
 
@@ -281,7 +275,7 @@ func (p *Preemptor) calculateVictimsByNode(nodeAvailable *resources.Resource, po
 	// get the current queue snapshot
 	askQueue, ok2 := allocationsByQueueSnap[p.queuePath]
 	if !ok2 {
-		log.Logger().Warn("BUG: Queue not found by name", zap.String("queuePath", p.queuePath))
+		log.Log(log.SchedPreemption).Warn("BUG: Queue not found by name", zap.String("queuePath", p.queuePath))
 		return -1, nil
 	}
 
@@ -341,6 +335,10 @@ func (p *Preemptor) checkPreemptionPredicates(predicateChecks []*si.PreemptionPr
 
 	// sort predicate checks by number of expected preempted tasks
 	sort.SliceStable(predicateChecks, func(i int, j int) bool {
+		// sort by NodeID if StartIndex are same
+		if predicateChecks[i].StartIndex == predicateChecks[j].StartIndex {
+			return predicateChecks[i].NodeID < predicateChecks[j].NodeID
+		}
 		return predicateChecks[i].StartIndex < predicateChecks[j].StartIndex
 	})
 
@@ -349,7 +347,7 @@ func (p *Preemptor) checkPreemptionPredicates(predicateChecks []*si.PreemptionPr
 	if plugin == nil {
 		// if a plugin isn't registered, assume checks will succeed and synthesize a result
 		check := predicateChecks[0]
-		log.Logger().Debug("No RM callback plugin registered, using first selected node for preemption",
+		log.Log(log.SchedPreemption).Debug("No RM callback plugin registered, using first selected node for preemption",
 			zap.String("NodeID", check.NodeID),
 			zap.String("AllocationKey", check.AllocationKey))
 
@@ -408,7 +406,7 @@ func (p *Preemptor) calculateAdditionalVictims(nodeVictims []*Allocation) ([]*Al
 	// get the current queue snapshot
 	askQueue, ok := allocationsByQueueSnap[p.queuePath]
 	if !ok {
-		log.Logger().Warn("BUG: Queue not found by name", zap.String("queuePath", p.queuePath))
+		log.Log(log.SchedPreemption).Warn("BUG: Queue not found by name", zap.String("queuePath", p.queuePath))
 		return nil, false
 	}
 
@@ -480,7 +478,7 @@ func (p *Preemptor) calculateAdditionalVictims(nodeVictims []*Allocation) ([]*Al
 
 // tryNodes attempts to find potential nodes for scheduling. For each node, potential victims are passed to
 // the shim for evaluation, and the best solution found will be returned.
-func (p *Preemptor) tryNodes() (string, string, []*Allocation, bool) {
+func (p *Preemptor) tryNodes() (string, []*Allocation, bool) {
 	// calculate victim list for each node
 	predicateChecks := make([]*si.PreemptionPredicatesArgs, 0)
 	victimsByNode := make(map[string][]*Allocation)
@@ -511,17 +509,10 @@ func (p *Preemptor) tryNodes() (string, string, []*Allocation, bool) {
 	// call predicates to evaluate each node
 	result := p.checkPreemptionPredicates(predicateChecks, victimsByNode)
 	if result != nil && result.success {
-		instType, ok := p.nodeInstanceTypeMap[result.nodeID]
-		if !ok {
-			// If the key doesn't exist, log error and use UnknownInstanceType
-			log.Logger().Error("BUG: Didn't find instance type in the nodeInstanceTypeMap",
-				zap.String("nodeId", result.nodeID))
-			instType = UnknownInstanceType
-		}
-		return result.nodeID, instType, result.victims, true
+		return result.nodeID, result.victims, true
 	}
 
-	return "", UnknownInstanceType, nil, false
+	return "", nil, false
 }
 
 func (p *Preemptor) TryPreemption() (*Allocation, bool) {
@@ -534,7 +525,7 @@ func (p *Preemptor) TryPreemption() (*Allocation, bool) {
 	p.initWorkingState()
 
 	// try to find a node to schedule on and victims to preempt
-	nodeID, instType, victims, ok := p.tryNodes()
+	nodeID, victims, ok := p.tryNodes()
 	if !ok {
 		// no preemption possible
 		return nil, false
@@ -556,15 +547,21 @@ func (p *Preemptor) TryPreemption() (*Allocation, bool) {
 		if victimQueue := p.queue.FindQueueByAppID(victim.GetApplicationID()); victimQueue != nil {
 			victimQueue.IncPreemptingResource(victim.GetAllocatedResource())
 			victim.MarkPreempted()
-			log.Logger().Info("Preempting task",
-				zap.String("applicationID", victim.GetApplicationID()),
-				zap.String("allocationKey", victim.GetAllocationKey()),
-				zap.String("nodeID", victim.GetNodeID()),
-				zap.Stringer("resources", victim.GetAllocatedResource()))
+			log.Log(log.SchedPreemption).Info("Preempting task",
+				zap.String("askApplicationID", p.ask.applicationID),
+				zap.String("askAllocationKey", p.ask.allocationKey),
+				zap.String("askQueue", p.queue.Name),
+				zap.String("victimApplicationID", victim.GetApplicationID()),
+				zap.String("victimAllocationKey", victim.GetAllocationKey()),
+				zap.Stringer("victimAllocatedResource", victim.GetAllocatedResource()),
+				zap.String("victimNodeID", victim.GetNodeID()),
+				zap.String("victimQueue", victimQueue.Name),
+			)
 		} else {
-			log.Logger().Warn("BUG: Queue not found for preemption victim",
-				zap.String("applicationID", victim.GetApplicationID()),
-				zap.String("allocationKey", victim.GetAllocationKey()))
+			log.Log(log.SchedPreemption).Warn("BUG: Queue not found for preemption victim",
+				zap.String("queue", p.queue.Name),
+				zap.String("victimApplicationID", victim.GetApplicationID()),
+				zap.String("victimAllocationKey", victim.GetAllocationKey()))
 		}
 	}
 
@@ -577,15 +574,15 @@ func (p *Preemptor) TryPreemption() (*Allocation, bool) {
 
 	// reserve the selected node for the new allocation if it will fit
 	if p.headRoom.FitInMaxUndef(p.ask.GetAllocatedResource()) {
-		log.Logger().Info("Reserving node for ask after preemption",
+		log.Log(log.SchedPreemption).Info("Reserving node for ask after preemption",
 			zap.String("allocationKey", p.ask.GetAllocationKey()),
 			zap.String("nodeID", nodeID),
 			zap.Int("victimCount", len(victims)))
-		return newReservedAllocation(Reserved, nodeID, instType, p.ask), true
+		return newReservedAllocation(nodeID, p.ask), true
 	}
 
 	// can't reserve as queue is still too full, but scheduling should succeed after preemption occurs
-	log.Logger().Info("Preempting allocations for ask, but not reserving yet as queue is still above capacity",
+	log.Log(log.SchedPreemption).Info("Preempting allocations for ask, but not reserving yet as queue is still above capacity",
 		zap.String("allocationKey", p.ask.GetAllocationKey()),
 		zap.Int("victimCount", len(victims)))
 
@@ -651,7 +648,7 @@ func (pcr *predicateCheckResult) populateVictims(victimsByNode map[string][]*All
 	// abort if node was not found
 	victimList, ok := victimsByNode[pcr.nodeID]
 	if !ok {
-		log.Logger().Warn("BUG: Unable to find node in victim map", zap.String("nodeID", pcr.nodeID))
+		log.Log(log.SchedPreemption).Warn("BUG: Unable to find node in victim map", zap.String("nodeID", pcr.nodeID))
 		pcr.success = false
 		pcr.index = -1
 		return
@@ -659,7 +656,7 @@ func (pcr *predicateCheckResult) populateVictims(victimsByNode map[string][]*All
 
 	// abort if index is too large
 	if pcr.index >= len(victimList) {
-		log.Logger().Warn("BUG: Got invalid index into allocation list",
+		log.Log(log.SchedPreemption).Warn("BUG: Got invalid index into allocation list",
 			zap.String("nodeID", pcr.nodeID),
 			zap.Int("index", pcr.index),
 			zap.Int("length", len(victimList)))
@@ -693,6 +690,7 @@ func (qps *QueuePreemptionSnapshot) Duplicate(copy map[string]*QueuePreemptionSn
 	snapshot := &QueuePreemptionSnapshot{
 		Parent:             parent,
 		QueuePath:          qps.QueuePath,
+		Leaf:               qps.Leaf,
 		AllocatedResource:  qps.AllocatedResource.Clone(),
 		PreemptingResource: qps.PreemptingResource.Clone(),
 		MaxResource:        qps.MaxResource.Clone(),
@@ -733,6 +731,11 @@ func (qps *QueuePreemptionSnapshot) IsWithinGuaranteedResource() bool {
 		return false
 	}
 	guaranteed := qps.GetGuaranteedResource()
+
+	// if this is a leaf queue and we have not found any guaranteed resources, then we are never within guaranteed usage
+	if qps.Leaf && guaranteed.IsEmpty() {
+		return false
+	}
 	max := qps.GetMaxResource()
 	absGuaranteed := resources.ComponentWiseMinPermissive(guaranteed, max)
 	used := resources.Sub(qps.AllocatedResource, qps.PreemptingResource)
