@@ -20,6 +20,7 @@ package objects
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"strconv"
 	"strings"
@@ -29,15 +30,18 @@ import (
 	"github.com/looplab/fsm"
 	"go.uber.org/zap"
 
+	"github.com/apache/yunikorn-core/pkg/common"
 	"github.com/apache/yunikorn-core/pkg/common/configs"
 	"github.com/apache/yunikorn-core/pkg/common/resources"
 	"github.com/apache/yunikorn-core/pkg/common/security"
+	"github.com/apache/yunikorn-core/pkg/events"
 	"github.com/apache/yunikorn-core/pkg/log"
 	"github.com/apache/yunikorn-core/pkg/metrics"
 	"github.com/apache/yunikorn-core/pkg/scheduler/objects/template"
 	"github.com/apache/yunikorn-core/pkg/scheduler/policies"
+	"github.com/apache/yunikorn-core/pkg/scheduler/ugm"
 	"github.com/apache/yunikorn-core/pkg/webservice/dao"
-	"github.com/apache/yunikorn-scheduler-interface/lib/go/common"
+	siCommon "github.com/apache/yunikorn-scheduler-interface/lib/go/common"
 )
 
 var (
@@ -83,6 +87,7 @@ type Queue struct {
 	runningApps            uint64
 	allocatingAcceptedApps map[string]bool
 	template               *template.Template
+	queueEvents            *queueEvents
 
 	sync.RWMutex
 }
@@ -135,13 +140,33 @@ func NewConfiguredQueue(conf configs.QueueConfig, parent *Queue) (*Queue, error)
 		sq.mergeProperties(parent.getProperties(), conf.Properties)
 	}
 	sq.UpdateQueueProperties()
-
-	log.Logger().Info("configured queue added to scheduler",
+	sq.queueEvents = newQueueEvents(sq, events.GetEventSystem())
+	log.Log(log.SchedQueue).Info("configured queue added to scheduler",
 		zap.String("queueName", sq.QueuePath))
+	sq.queueEvents.sendNewQueueEvent()
 
 	return sq, nil
 }
 
+// NewRecoveryQueue creates a recovery queue if it does not exist. The recovery queue
+// is a dynamic queue, but has an invalid name so that it cannot be directly referenced.
+func NewRecoveryQueue(parent *Queue) (*Queue, error) {
+	if parent == nil {
+		return nil, errors.New("recovery queue cannot be created with nil parent")
+	}
+	if parent.GetQueuePath() != configs.RootQueue {
+		return nil, fmt.Errorf("recovery queue cannot be created with non-root parent: %s", parent.GetQueuePath())
+	}
+	queue, err := newDynamicQueueInternal(common.RecoveryQueue, true, parent)
+	if err == nil {
+		queue.Lock()
+		defer queue.Unlock()
+		queue.submitACL = security.ACL{}
+		queue.sortType = policies.FifoSortPolicy
+	}
+	return queue, err
+}
+
 // NewDynamicQueue creates a new queue to be added to the system based on the placement rules
 // A dynamically added queue can never be the root queue so parent must be set
 // lock free as it cannot be referenced yet
@@ -154,6 +179,10 @@ func NewDynamicQueue(name string, leaf bool, parent *Queue) (*Queue, error) {
 	if !configs.QueueNameRegExp.MatchString(name) {
 		return nil, fmt.Errorf("invalid queue name '%s', a name must only have alphanumeric characters, - or _, and be no longer than 64 characters", name)
 	}
+	return newDynamicQueueInternal(name, leaf, parent)
+}
+
+func newDynamicQueueInternal(name string, leaf bool, parent *Queue) (*Queue, error) {
 	sq := newBlankQueue()
 	sq.Name = strings.ToLower(name)
 	sq.QueuePath = parent.QueuePath + configs.DOT + sq.Name
@@ -169,8 +198,10 @@ func NewDynamicQueue(name string, leaf bool, parent *Queue) (*Queue, error) {
 	}
 
 	sq.UpdateQueueProperties()
-	log.Logger().Info("dynamic queue added to scheduler",
+	sq.queueEvents = newQueueEvents(sq, events.GetEventSystem())
+	log.Log(log.SchedQueue).Info("dynamic queue added to scheduler",
 		zap.String("queueName", sq.QueuePath))
+	sq.queueEvents.sendNewQueueEvent()
 
 	return sq, nil
 }
@@ -279,29 +310,34 @@ func (sq *Queue) applyConf(conf configs.QueueConfig) error {
 	var err error
 	sq.submitACL, err = security.NewACL(conf.SubmitACL)
 	if err != nil {
-		log.Logger().Error("parsing submit ACL failed this should not happen",
+		log.Log(log.SchedQueue).Error("parsing submit ACL failed this should not happen",
 			zap.Error(err))
 		return err
 	}
 	sq.adminACL, err = security.NewACL(conf.AdminACL)
 	if err != nil {
-		log.Logger().Error("parsing admin ACL failed this should not happen",
+		log.Log(log.SchedQueue).Error("parsing admin ACL failed this should not happen",
 			zap.Error(err))
 		return err
 	}
 	// Change from unmanaged to managed
 	if !sq.isManaged {
-		log.Logger().Info("changed dynamic queue to managed",
+		log.Log(log.SchedQueue).Info("changed dynamic queue to managed",
 			zap.String("queue", sq.QueuePath))
 		sq.isManaged = true
 	}
 
+	prevLeaf := sq.isLeaf
 	sq.isLeaf = !conf.Parent
 	// Make sure the parent flag is set correctly: config might expect auto parent type creation
 	if len(conf.Queues) > 0 {
 		sq.isLeaf = false
 	}
 
+	if prevLeaf != sq.isLeaf && sq.queueEvents != nil {
+		sq.queueEvents.sendTypeChangedEvent()
+	}
+
 	if !sq.isLeaf {
 		if err = sq.setTemplate(conf.ChildTemplate); err != nil {
 			return nil
@@ -323,7 +359,7 @@ func (sq *Queue) applyConf(conf configs.QueueConfig) error {
 func (sq *Queue) setResources(resource configs.Resources) error {
 	maxResource, err := resources.NewResourceFromConf(resource.Max)
 	if err != nil {
-		log.Logger().Error("parsing failed on max resources this should not happen",
+		log.Log(log.SchedQueue).Error("parsing failed on max resources this should not happen",
 			zap.Error(err))
 		return err
 	}
@@ -331,23 +367,29 @@ func (sq *Queue) setResources(resource configs.Resources) error {
 	var guaranteedResource *resources.Resource
 	guaranteedResource, err = resources.NewResourceFromConf(resource.Guaranteed)
 	if err != nil {
-		log.Logger().Error("parsing failed on guaranteed resources this should not happen",
+		log.Log(log.SchedQueue).Error("parsing failed on guaranteed resources this should not happen",
 			zap.Error(err))
 		return err
 	}
 
 	if resources.StrictlyGreaterThanZero(maxResource) {
+		if !resources.Equals(sq.maxResource, maxResource) && sq.queueEvents != nil {
+			sq.queueEvents.sendMaxResourceChangedEvent()
+		}
 		sq.maxResource = maxResource
 		sq.updateMaxResourceMetrics()
 	} else {
-		log.Logger().Debug("max resources setting ignored: cannot set zero max resources")
+		log.Log(log.SchedQueue).Debug("max resources setting ignored: cannot set zero max resources")
 	}
 
 	if resources.StrictlyGreaterThanZero(guaranteedResource) {
+		if !resources.Equals(sq.guaranteedResource, guaranteedResource) && sq.queueEvents != nil {
+			sq.queueEvents.sendGuaranteedResourceChangedEvent()
+		}
 		sq.guaranteedResource = guaranteedResource
 		sq.updateGuaranteedResourceMetrics()
 	} else {
-		log.Logger().Debug("guaranteed resources setting ignored: cannot set zero max resources")
+		log.Log(log.SchedQueue).Debug("guaranteed resources setting ignored: cannot set zero max resources")
 	}
 
 	return nil
@@ -368,12 +410,15 @@ func (sq *Queue) setTemplate(conf configs.ChildTemplate) error {
 func (sq *Queue) UpdateQueueProperties() {
 	sq.Lock()
 	defer sq.Unlock()
-
+	if common.IsRecoveryQueue(sq.QueuePath) {
+		// recovery queue properties should never be updated
+		sq.sortType = policies.FifoSortPolicy
+		return
+	}
 	if !sq.isLeaf {
 		// set the sorting type for parent queues
 		sq.sortType = policies.FairSortPolicy
 	}
-
 	// walk over all properties and process
 	var err error
 	for key, value := range sq.properties {
@@ -382,7 +427,7 @@ func (sq *Queue) UpdateQueueProperties() {
 			if sq.isLeaf {
 				sq.sortType, err = policies.SortPolicyFromString(value)
 				if err != nil {
-					log.Logger().Debug("application sort property configuration error",
+					log.Log(log.SchedQueue).Debug("application sort property configuration error",
 						zap.Error(err))
 				}
 				// if it is not defined default to fifo
@@ -393,38 +438,38 @@ func (sq *Queue) UpdateQueueProperties() {
 		case configs.ApplicationSortPriority:
 			sq.prioritySortEnabled, err = applicationSortPriorityEnabled(value)
 			if err != nil {
-				log.Logger().Debug("queue application sort priority configuration error",
+				log.Log(log.SchedQueue).Debug("queue application sort priority configuration error",
 					zap.Error(err))
 			}
 		case configs.PriorityOffset:
 			sq.priorityOffset, err = priorityOffset(value)
 			if err != nil {
-				log.Logger().Debug("queue priority offset configuration error",
+				log.Log(log.SchedQueue).Debug("queue priority offset configuration error",
 					zap.Error(err))
 			}
 		case configs.PriorityPolicy:
 			sq.priorityPolicy, err = policies.PriorityPolicyFromString(value)
 			if err != nil {
-				log.Logger().Debug("queue priority policy configuration error",
+				log.Log(log.SchedQueue).Debug("queue priority policy configuration error",
 					zap.Error(err))
 			}
 		case configs.PreemptionPolicy:
 			sq.preemptionPolicy, err = policies.PreemptionPolicyFromString(value)
 			if err != nil {
-				log.Logger().Debug("queue preemption policy configuration error",
+				log.Log(log.SchedQueue).Debug("queue preemption policy configuration error",
 					zap.Error(err))
 			}
 		case configs.PreemptionDelay:
 			if sq.isLeaf {
 				sq.preemptionDelay, err = preemptionDelay(value)
 				if err != nil {
-					log.Logger().Debug("preemption delay property configuration error",
+					log.Log(log.SchedQueue).Debug("preemption delay property configuration error",
 						zap.Error(err))
 				}
 			}
 		default:
 			// skip unknown properties just log them
-			log.Logger().Debug("queue property skipped",
+			log.Log(log.SchedQueue).Debug("queue property skipped",
 				zap.String("key", key),
 				zap.String("value", value))
 		}
@@ -519,6 +564,10 @@ func (sq *Queue) GetPreemptionDelay() time.Duration {
 // The check is performed recursively: i.e. access to the parent allows access to this queue.
 // This will check both submitACL and adminACL.
 func (sq *Queue) CheckSubmitAccess(user security.UserGroup) bool {
+	if common.IsRecoveryQueue(sq.QueuePath) {
+		// recovery queue can never pass ACL checks
+		return false
+	}
 	sq.RLock()
 	allow := sq.submitACL.CheckAccess(user) || sq.adminACL.CheckAccess(user)
 	sq.RUnlock()
@@ -540,33 +589,6 @@ func (sq *Queue) CheckAdminAccess(user security.UserGroup) bool {
 	return allow
 }
 
-// GetQueueInfo returns the queue hierarchy as an object for a REST call.
-// This object is used by the deprecated REST API and is succeeded by the GetPartitionQueueDAOInfo call.
-func (sq *Queue) GetQueueInfo() dao.QueueDAOInfo {
-	queueInfo := dao.QueueDAOInfo{}
-	for _, child := range sq.GetCopyOfChildren() {
-		queueInfo.ChildQueues = append(queueInfo.ChildQueues, child.GetQueueInfo())
-	}
-
-	// children are done we can now lock just this queue.
-	sq.RLock()
-	defer sq.RUnlock()
-	queueInfo.QueueName = sq.Name
-	queueInfo.Status = sq.stateMachine.Current()
-	queueInfo.Capacities = dao.QueueCapacity{
-		Capacity:     sq.guaranteedResource.DAOMap(),
-		MaxCapacity:  sq.maxResource.DAOMap(),
-		UsedCapacity: sq.allocatedResource.DAOMap(),
-		AbsUsedCapacity: resources.CalculateAbsUsedCapacity(
-			sq.maxResource, sq.allocatedResource).DAOMap(),
-	}
-	queueInfo.Properties = make(map[string]string)
-	for k, v := range sq.properties {
-		queueInfo.Properties[k] = v
-	}
-	return queueInfo
-}
-
 // GetPartitionQueueDAOInfo returns the queue hierarchy as an object for a REST call.
 func (sq *Queue) GetPartitionQueueDAOInfo() dao.PartitionQueueDAOInfo {
 	queueInfo := dao.PartitionQueueDAOInfo{}
@@ -629,6 +651,7 @@ func (sq *Queue) incPendingResource(delta *resources.Resource) {
 	sq.Lock()
 	defer sq.Unlock()
 	sq.pending = resources.Add(sq.pending, delta)
+	sq.updatePendingResourceMetrics()
 }
 
 // decPendingResource decrements pending resource of this queue and its parents.
@@ -646,9 +669,11 @@ func (sq *Queue) decPendingResource(delta *resources.Resource) {
 	var err error
 	sq.pending, err = resources.SubErrorNegative(sq.pending, delta)
 	if err != nil {
-		log.Logger().Warn("Pending resources went negative",
+		log.Log(log.SchedQueue).Warn("Pending resources went negative",
 			zap.String("queueName", sq.QueuePath),
 			zap.Error(err))
+	} else {
+		sq.updatePendingResourceMetrics()
 	}
 }
 
@@ -658,34 +683,67 @@ func (sq *Queue) decPendingResource(delta *resources.Resource) {
 func (sq *Queue) AddApplication(app *Application) {
 	sq.Lock()
 	defer sq.Unlock()
-	sq.applications[app.ApplicationID] = app
-	sq.appPriorities[app.ApplicationID] = app.GetAskMaxPriority()
+	appID := app.ApplicationID
+	sq.applications[appID] = app
+	sq.appPriorities[appID] = app.GetAskMaxPriority()
+	sq.queueEvents.sendNewApplicationEvent(appID)
 	// YUNIKORN-199: update the quota from the namespace
 	// get the tag with the quota
-	quota := app.GetTag(common.AppTagNamespaceResourceQuota)
-	if quota == "" {
+	quota := app.GetTag(siCommon.AppTagNamespaceResourceQuota)
+	// get the tag with the guaranteed resource
+	guaranteed := app.GetTag(siCommon.AppTagNamespaceResourceGuaranteed)
+	if quota == "" && guaranteed == "" {
 		return
 	}
+
+	var quotaRes, guaranteedRes *resources.Resource
+	var quotaErr, guaranteedErr error
+
 	// need to set a quota: convert json string to resource
-	res, err := resources.NewResourceFromString(quota)
-	if err != nil {
-		log.Logger().Warn("application resource quota conversion failure",
-			zap.String("json quota string", quota),
-			zap.Error(err))
-		return
+	if quota != "" {
+		quotaRes, quotaErr = resources.NewResourceFromString(quota)
+		if quotaErr != nil {
+			log.Log(log.SchedQueue).Warn("application resource quota conversion failure",
+				zap.String("json quota string", quota),
+				zap.Error(quotaErr))
+		} else if !resources.StrictlyGreaterThanZero(quotaRes) {
+			log.Log(log.SchedQueue).Warn("Max resource quantities should be greater than zero: cannot set queue max resource",
+				zap.Stringer("maxResource", quotaRes))
+			quotaRes = nil // Skip setting quota if it has a value <= 0
+		}
 	}
-	if !resources.StrictlyGreaterThanZero(res) {
-		log.Logger().Warn("application resource quota has at least one 0 value: cannot set queue limit",
-			zap.Stringer("maxResource", res))
-		return
+
+	// need to set guaranteed resource: convert json string to resource
+	if guaranteed != "" {
+		guaranteedRes, guaranteedErr = resources.NewResourceFromString(guaranteed)
+		if guaranteedErr != nil {
+			log.Log(log.SchedQueue).Warn("application guaranteed resource conversion failure",
+				zap.String("json guaranteed string", guaranteed),
+				zap.Error(guaranteedErr))
+			if quotaErr != nil {
+				return
+			}
+		} else if !resources.StrictlyGreaterThanZero(guaranteedRes) {
+			log.Log(log.SchedQueue).Warn("Guaranteed resource quantities should be greater than zero: cannot set queue guaranteed resource",
+				zap.Stringer("guaranteedResource", guaranteedRes))
+			guaranteedRes = nil // Skip setting guaranteed resource if it has a value <= 0
+		}
 	}
+
 	// set the quota
 	if sq.isManaged {
-		log.Logger().Warn("Trying to set max resources set on a queue that is not an unmanaged leaf",
+		log.Log(log.SchedQueue).Warn("Trying to set max resources on a queue that is not an unmanaged leaf",
 			zap.String("queueName", sq.QueuePath))
 		return
 	}
-	sq.maxResource = res
+
+	if quotaRes != nil {
+		sq.maxResource = quotaRes
+	}
+
+	if guaranteedRes != nil {
+		sq.guaranteedResource = guaranteedRes
+	}
 }
 
 // RemoveApplication removes the app from the list of tracked applications. Make sure that the app
@@ -695,18 +753,19 @@ func (sq *Queue) RemoveApplication(app *Application) {
 	// clean up any outstanding pending resources
 	appID := app.ApplicationID
 	if !sq.appExists(appID) {
-		log.Logger().Debug("Application not found while removing from queue",
+		log.Log(log.SchedQueue).Debug("Application not found while removing from queue",
 			zap.String("queueName", sq.QueuePath),
 			zap.String("applicationID", appID))
 		return
 	}
+	sq.queueEvents.sendRemoveApplicationEvent(appID)
 	if appPending := app.GetPendingResource(); !resources.IsZero(appPending) {
 		sq.decPendingResource(appPending)
 	}
 	// clean up the allocated resource
 	if appAllocated := app.GetAllocatedResource(); !resources.IsZero(appAllocated) {
 		if err := sq.DecAllocatedResource(appAllocated); err != nil {
-			log.Logger().Warn("failed to release allocated resources from queue",
+			log.Log(log.SchedQueue).Warn("failed to release allocated resources from queue",
 				zap.String("appID", appID),
 				zap.Error(err))
 		}
@@ -714,7 +773,7 @@ func (sq *Queue) RemoveApplication(app *Application) {
 	// clean up the allocated placeholder resource
 	if phAllocated := app.GetPlaceholderResource(); !resources.IsZero(phAllocated) {
 		if err := sq.DecAllocatedResource(phAllocated); err != nil {
-			log.Logger().Warn("failed to release placeholder resources from queue",
+			log.Log(log.SchedQueue).Warn("failed to release placeholder resources from queue",
 				zap.String("appID", appID),
 				zap.Error(err))
 		}
@@ -736,10 +795,11 @@ func (sq *Queue) RemoveApplication(app *Application) {
 	delete(sq.allocatingAcceptedApps, appID)
 	priority := sq.recalculatePriority()
 	sq.Unlock()
+	app.appEvents.sendRemoveApplicationEvent()
 
 	sq.parent.UpdateQueuePriority(sq.Name, priority)
 
-	log.Logger().Info("Application completed and removed from queue",
+	log.Log(log.SchedQueue).Info("Application completed and removed from queue",
 		zap.String("queueName", sq.QueuePath),
 		zap.String("applicationID", appID))
 }
@@ -828,7 +888,7 @@ func (sq *Queue) addChildQueue(child *Queue) error {
 		// 1) try to use template if it is not nil
 		// 2) otherwise, child leaf copy the configs.ApplicationSortPolicy from parent (old behavior)
 		if sq.template != nil {
-			log.Logger().Debug("applying child template to new leaf queue",
+			log.Log(log.SchedQueue).Debug("applying child template to new leaf queue",
 				zap.String("child queue", child.QueuePath),
 				zap.String("parent queue", sq.QueuePath),
 				zap.Any("template", sq.template))
@@ -836,7 +896,7 @@ func (sq *Queue) addChildQueue(child *Queue) error {
 		} else {
 			policyValue, ok := sq.properties[configs.ApplicationSortPolicy]
 			if ok {
-				log.Logger().Warn("inheriting application sort policy is deprecated, use child templates",
+				log.Log(log.Deprecation).Warn("inheriting application sort policy is deprecated, use child templates",
 					zap.String("child queue", child.QueuePath),
 					zap.String("parent queue", sq.QueuePath))
 				child.properties[configs.ApplicationSortPolicy] = policyValue
@@ -847,7 +907,7 @@ func (sq *Queue) addChildQueue(child *Queue) error {
 	// don't override the template of non-leaf queue
 	if child.template == nil {
 		child.template = sq.template
-		log.Logger().Debug("new parent queue inheriting template from parent queue",
+		log.Log(log.SchedQueue).Debug("new parent queue inheriting template from parent queue",
 			zap.String("child queue", child.QueuePath),
 			zap.String("parent queue", sq.QueuePath))
 	}
@@ -864,10 +924,10 @@ func (sq *Queue) MarkQueueForRemoval() {
 	// Mark the managed queue for deletion: it is removed from the config let it drain.
 	// Also mark all the managed children for deletion.
 	if sq.isManaged {
-		log.Logger().Info("marking managed queue for deletion",
+		log.Log(log.SchedQueue).Info("marking managed queue for deletion",
 			zap.String("queue", sq.QueuePath))
 		if err := sq.handleQueueEvent(Remove); err != nil {
-			log.Logger().Warn("failed to mark managed queue for deletion",
+			log.Log(log.SchedQueue).Warn("failed to mark managed queue for deletion",
 				zap.String("queue", sq.QueuePath),
 				zap.Error(err))
 		}
@@ -902,9 +962,10 @@ func (sq *Queue) RemoveQueue() bool {
 	if len(sq.children) > 0 || len(sq.applications) > 0 {
 		return false
 	}
-	log.Logger().Info("removing queue", zap.String("queue", sq.QueuePath))
+	log.Log(log.SchedQueue).Info("removing queue", zap.String("queue", sq.QueuePath))
 	// root is always managed and is the only queue with a nil parent: no need to guard
 	sq.parent.removeChildQueue(sq.Name)
+	sq.queueEvents.sendRemoveQueueEvent()
 	return true
 }
 
@@ -947,7 +1008,7 @@ func (sq *Queue) IncAllocatedResource(alloc *resources.Resource, nodeReported bo
 			// only log the warning if we get to the leaf: otherwise we could spam the log with the same message
 			// each time we return from a recursive call. Worst case (hierarchy depth-1) times.
 			if sq.isLeaf {
-				log.Logger().Warn("parent queue exceeds maximum resource",
+				log.Log(log.SchedQueue).Warn("parent queue exceeds maximum resource",
 					zap.String("leafQueue", sq.QueuePath),
 					zap.Stringer("allocationRequest", alloc),
 					zap.Stringer("queueUsage", sq.allocatedResource),
@@ -959,7 +1020,7 @@ func (sq *Queue) IncAllocatedResource(alloc *resources.Resource, nodeReported bo
 	}
 	// all OK update this queue
 	sq.allocatedResource = newAllocated
-	sq.updateAllocatedAndPendingResourceMetrics()
+	sq.updateAllocatedResourceMetrics()
 	return nil
 }
 
@@ -983,7 +1044,7 @@ func (sq *Queue) DecAllocatedResource(alloc *resources.Resource) error {
 			// only log the warning if we get to the leaf: otherwise we spam the log with the same message
 			// each time we return from a recursive call. Worst case (hierarchy depth-1) times.
 			if sq.isLeaf {
-				log.Logger().Warn("released allocation is larger than parent queue allocated resource",
+				log.Log(log.SchedQueue).Warn("released allocation is larger than parent queue allocated resource",
 					zap.String("leafQueue", sq.QueuePath),
 					zap.Stringer("allocationRequest", alloc),
 					zap.Stringer("queueUsage", sq.allocatedResource),
@@ -995,7 +1056,7 @@ func (sq *Queue) DecAllocatedResource(alloc *resources.Resource) error {
 	}
 	// all OK update the queue
 	sq.allocatedResource = resources.Sub(sq.allocatedResource, alloc)
-	sq.updateAllocatedAndPendingResourceMetrics()
+	sq.updateAllocatedResourceMetrics()
 	return nil
 }
 
@@ -1008,7 +1069,7 @@ func (sq *Queue) IncPreemptingResource(alloc *resources.Resource) {
 	defer sq.Unlock()
 	sq.parent.IncPreemptingResource(alloc)
 	sq.preemptingResource = resources.Add(sq.preemptingResource, alloc)
-	sq.updateAllocatedAndPendingResourceMetrics()
+	sq.updatePreemptingResourceMetrics()
 }
 
 // DecPreemptingResource decrements the preempting resources for this queue (recursively).
@@ -1020,7 +1081,7 @@ func (sq *Queue) DecPreemptingResource(alloc *resources.Resource) {
 	defer sq.Unlock()
 	sq.parent.DecPreemptingResource(alloc)
 	sq.preemptingResource = resources.Sub(sq.preemptingResource, alloc)
-	sq.updateAllocatedAndPendingResourceMetrics()
+	sq.updatePreemptingResourceMetrics()
 }
 
 func (sq *Queue) IsPrioritySortEnabled() bool {
@@ -1033,10 +1094,24 @@ func (sq *Queue) IsPrioritySortEnabled() bool {
 // Applications are sorted using the sorting type of the queue.
 // Only applications with a pending resource request are considered.
 // Lock free call all locks are taken when needed in called functions
-func (sq *Queue) sortApplications(filterApps bool) []*Application {
+// If withPlaceholdersOnly is true, then only applications with at least one placeholder allocation are considered.
+func (sq *Queue) sortApplications(filterApps, withPlaceholdersOnly bool) []*Application {
 	if !sq.IsLeafQueue() {
 		return nil
 	}
+
+	apps := sq.GetCopyOfApps()
+	if withPlaceholdersOnly {
+		for key, app := range apps {
+			if !app.HasPlaceholderAllocation() {
+				delete(apps, key)
+			}
+		}
+	}
+	if len(apps) == 0 {
+		return nil
+	}
+
 	// sort applications based on the sorting policy
 	// some apps might be filtered out based on the policy specific conditions.
 	// currently, only the stateAware policy does the filtering (based on app state).
@@ -1047,7 +1122,7 @@ func (sq *Queue) sortApplications(filterApps bool) []*Application {
 		// this is to skip the app filtering in the StateAware policy sorting
 		queueSortType = policies.FifoSortPolicy
 	}
-	return sortApplications(sq.GetCopyOfApps(), queueSortType, sq.IsPrioritySortEnabled(), sq.GetGuaranteedResource())
+	return sortApplications(apps, queueSortType, sq.IsPrioritySortEnabled(), sq.GetGuaranteedResource())
 }
 
 // sortQueues returns a sorted shallow copy of the queues for this parent queue.
@@ -1183,11 +1258,11 @@ func (sq *Queue) SetMaxResource(max *resources.Resource) {
 	defer sq.Unlock()
 
 	if sq.parent != nil {
-		log.Logger().Warn("Max resources set on a queue that is not the root",
+		log.Log(log.SchedQueue).Warn("Max resources set on a queue that is not the root",
 			zap.String("queueName", sq.QueuePath))
 		return
 	}
-	log.Logger().Info("updating root queue max resources",
+	log.Log(log.SchedQueue).Info("updating root queue max resources",
 		zap.Stringer("current max", sq.maxResource),
 		zap.Stringer("new max", max))
 	sq.maxResource = max.Clone()
@@ -1222,7 +1297,7 @@ func (sq *Queue) canRunApp(appID string) bool {
 // resources are skipped.
 // Applications are sorted based on the application sortPolicy. Applications without pending resources are skipped.
 // Lock free call this all locks are taken when needed in called functions
-func (sq *Queue) TryAllocate(iterator func() NodeIterator, fullIterator func() NodeIterator, getnode func(string) *Node) *Allocation {
+func (sq *Queue) TryAllocate(iterator func() NodeIterator, fullIterator func() NodeIterator, getnode func(string) *Node, allowPreemption bool) *Allocation {
 	if sq.IsLeafQueue() {
 		// get the headroom
 		headRoom := sq.getHeadRoom()
@@ -1230,13 +1305,13 @@ func (sq *Queue) TryAllocate(iterator func() NodeIterator, fullIterator func() N
 		preemptAttemptsRemaining := maxPreemptionsPerQueue
 
 		// process the apps (filters out app without pending requests)
-		for _, app := range sq.sortApplications(true) {
-			if app.IsAccepted() && !sq.canRunApp(app.ApplicationID) {
+		for _, app := range sq.sortApplications(true, false) {
+			if app.IsAccepted() && (!sq.canRunApp(app.ApplicationID) || !ugm.GetUserManager().CanRunApp(sq.QueuePath, app.ApplicationID, app.user)) {
 				continue
 			}
-			alloc := app.tryAllocate(headRoom, preemptionDelay, &preemptAttemptsRemaining, iterator, fullIterator, getnode)
+			alloc := app.tryAllocate(headRoom, allowPreemption, preemptionDelay, &preemptAttemptsRemaining, iterator, fullIterator, getnode)
 			if alloc != nil {
-				log.Logger().Debug("allocation found on queue",
+				log.Log(log.SchedQueue).Debug("allocation found on queue",
 					zap.String("queueName", sq.QueuePath),
 					zap.String("appID", app.ApplicationID),
 					zap.Stringer("allocation", alloc))
@@ -1251,7 +1326,7 @@ func (sq *Queue) TryAllocate(iterator func() NodeIterator, fullIterator func() N
 	} else {
 		// process the child queues (filters out queues without pending requests)
 		for _, child := range sq.sortQueues() {
-			alloc := child.TryAllocate(iterator, fullIterator, getnode)
+			alloc := child.TryAllocate(iterator, fullIterator, getnode, allowPreemption)
 			if alloc != nil {
 				return alloc
 			}
@@ -1269,10 +1344,10 @@ func (sq *Queue) TryAllocate(iterator func() NodeIterator, fullIterator func() N
 func (sq *Queue) TryPlaceholderAllocate(iterator func() NodeIterator, getnode func(string) *Node) *Allocation {
 	if sq.IsLeafQueue() {
 		// process the apps (filters out app without pending requests)
-		for _, app := range sq.sortApplications(true) {
+		for _, app := range sq.sortApplications(true, true) {
 			alloc := app.tryPlaceholderAllocate(iterator, getnode)
 			if alloc != nil {
-				log.Logger().Debug("allocation found on queue",
+				log.Log(log.SchedQueue).Debug("allocation found on queue",
 					zap.String("queueName", sq.QueuePath),
 					zap.String("appID", app.ApplicationID),
 					zap.Stringer("allocation", alloc))
@@ -1299,8 +1374,10 @@ func (sq *Queue) GetQueueOutstandingRequests(total *[]*AllocationAsk) {
 		// e.g. StateAware filters apps by state in order to schedule app one by one.
 		// we calculate all the requests that can fit into the queue's headroom,
 		// all these requests are qualified to trigger the up scaling.
-		for _, app := range sq.sortApplications(false) {
-			app.getOutstandingRequests(headRoom, total)
+		for _, app := range sq.sortApplications(false, false) {
+			// calculate the users' headroom
+			userHeadroom := ugm.GetUserManager().Headroom(app.queuePath, app.ApplicationID, app.user)
+			app.getOutstandingRequests(headRoom, userHeadroom, total)
 		}
 	} else {
 		for _, child := range sq.sortQueues() {
@@ -1325,23 +1402,23 @@ func (sq *Queue) TryReservedAllocate(iterator func() NodeIterator) *Allocation {
 			// process the apps
 			for appID, numRes := range reservedCopy {
 				if numRes > 1 {
-					log.Logger().Debug("multiple reservations found for application trying to allocate one",
+					log.Log(log.SchedQueue).Debug("multiple reservations found for application trying to allocate one",
 						zap.String("appID", appID),
 						zap.Int("reservations", numRes))
 				}
 				app := sq.GetApplication(appID)
 				if app == nil {
-					log.Logger().Debug("reservation(s) found but application did not exist in queue",
+					log.Log(log.SchedQueue).Debug("reservation(s) found but application did not exist in queue",
 						zap.String("queueName", sq.QueuePath),
 						zap.String("appID", appID))
 					return nil
 				}
-				if app.IsAccepted() && !sq.canRunApp(appID) {
+				if app.IsAccepted() && (!sq.canRunApp(appID) || !ugm.GetUserManager().CanRunApp(sq.QueuePath, appID, app.user)) {
 					continue
 				}
 				alloc := app.tryReservedAllocate(headRoom, iterator)
 				if alloc != nil {
-					log.Logger().Debug("reservation found for allocation found on queue",
+					log.Log(log.SchedQueue).Debug("reservation found for allocation found on queue",
 						zap.String("queueName", sq.QueuePath),
 						zap.String("appID", appID),
 						zap.Stringer("allocation", alloc),
@@ -1480,14 +1557,22 @@ func (sq *Queue) updateMaxResourceMetrics() {
 	}
 }
 
-// updateAllocatedAndPendingResourceMetrics updates allocated and pending resource metrics for all queue types.
-func (sq *Queue) updateAllocatedAndPendingResourceMetrics() {
+// updateAllocatedResourceMetrics updates allocated resource metrics for all queue types.
+func (sq *Queue) updateAllocatedResourceMetrics() {
 	for k, v := range sq.allocatedResource.Resources {
 		metrics.GetQueueMetrics(sq.QueuePath).SetQueueAllocatedResourceMetrics(k, float64(v))
 	}
+}
+
+// updatePendingResourceMetrics updates pending resource metrics for all queue types.
+func (sq *Queue) updatePendingResourceMetrics() {
 	for k, v := range sq.pending.Resources {
 		metrics.GetQueueMetrics(sq.QueuePath).SetQueuePendingResourceMetrics(k, float64(v))
 	}
+}
+
+// updatePreemptingResourceMetrics updates preempting resource metrics for all queue types.
+func (sq *Queue) updatePreemptingResourceMetrics() {
 	for k, v := range sq.preemptingResource.Resources {
 		metrics.GetQueueMetrics(sq.QueuePath).SetQueuePreemptingResourceMetrics(k, float64(v))
 	}
@@ -1516,7 +1601,7 @@ func (sq *Queue) incRunningApps(appID string) {
 	sq.runningApps++
 	// sanity check
 	if sq.maxRunningApps > 0 && sq.runningApps > sq.maxRunningApps {
-		log.Logger().Debug("queue running apps went over maximum",
+		log.Log(log.SchedQueue).Debug("queue running apps went over maximum",
 			zap.String("queueName", sq.QueuePath))
 		sq.runningApps = sq.maxRunningApps
 	}
@@ -1538,7 +1623,7 @@ func (sq *Queue) decRunningApps() {
 	if sq.runningApps > 0 {
 		sq.runningApps--
 	} else {
-		log.Logger().Debug("queue running apps went negative",
+		log.Log(log.SchedQueue).Debug("queue running apps went negative",
 			zap.String("queueName", sq.QueuePath))
 	}
 }
@@ -1596,7 +1681,7 @@ func (sq *Queue) FindEligiblePreemptionVictims(queuePath string, ask *Allocation
 	queuePriority, ok := priorityMap[fence.QueuePath]
 	if !ok {
 		// this shouldn't happen
-		log.Logger().Error("BUG: No computed priority found for queue", zap.String("queue", fence.QueuePath))
+		log.Log(log.SchedQueue).Error("BUG: No computed priority found for queue", zap.String("queue", fence.QueuePath))
 		return nil
 	}
 
@@ -1623,6 +1708,7 @@ func (sq *Queue) createPreemptionSnapshot(cache map[string]*QueuePreemptionSnaps
 	snapshot = &QueuePreemptionSnapshot{
 		Parent:             parentSnapshot,
 		QueuePath:          sq.QueuePath,
+		Leaf:               sq.isLeaf,
 		AllocatedResource:  sq.allocatedResource.Clone(),
 		PreemptingResource: sq.preemptingResource.Clone(),
 		MaxResource:        sq.maxResource.Clone(),
@@ -1666,6 +1752,11 @@ func (sq *Queue) findEligiblePreemptionVictims(results map[string]*QueuePreempti
 					continue
 				}
 
+				// skip placeholder tasks which are marked released
+				if alloc.IsReleased() {
+					continue
+				}
+
 				// if we have encountered a fence then all tasks are eligible for preemption
 				// otherwise the task is a candidate if its priority is less than or equal to the ask priority
 				if fenced || int64(alloc.GetPriority()) <= askPriority {
@@ -1777,7 +1868,7 @@ func (sq *Queue) updateApplicationPriorityInternal(applicationID string, priorit
 	defer sq.Unlock()
 
 	if _, ok := sq.applications[applicationID]; !ok {
-		log.Logger().Debug("Unknown application", zap.String("applicationID", applicationID))
+		log.Log(log.SchedQueue).Debug("Unknown application", zap.String("applicationID", applicationID))
 		return sq.currentPriority
 	}
 
@@ -1798,7 +1889,7 @@ func (sq *Queue) updateQueuePriorityInternal(queueName string, priority int32) i
 	defer sq.Unlock()
 
 	if _, ok := sq.children[queueName]; !ok {
-		log.Logger().Debug("Unknown queue", zap.String("queueName", queueName))
+		log.Log(log.SchedQueue).Debug("Unknown queue", zap.String("queueName", queueName))
 		return sq.currentPriority
 	}
 
@@ -1823,3 +1914,7 @@ func (sq *Queue) recalculatePriority() int32 {
 	sq.currentPriority = curr
 	return priorityValueByPolicy(sq.priorityPolicy, sq.priorityOffset, curr)
 }
+
+func (sq *Queue) SendRemoveQueueEvent() {
+	sq.queueEvents.sendRemoveQueueEvent()
+}
