@@ -28,17 +28,19 @@ import (
 	"github.com/apache/yunikorn-core/pkg/common/configs"
 	"github.com/apache/yunikorn-core/pkg/common/resources"
 	"github.com/apache/yunikorn-core/pkg/common/security"
+	"github.com/apache/yunikorn-core/pkg/events"
 	"github.com/apache/yunikorn-core/pkg/rmproxy"
 	"github.com/apache/yunikorn-core/pkg/scheduler/ugm"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
 
 const (
-	appID0  = "app-0"
-	appID1  = "app-1"
-	appID2  = "app-2"
-	aKey    = "alloc-1"
-	nodeID1 = "node-1"
+	appID0    = "app-0"
+	appID1    = "app-1"
+	appID2    = "app-2"
+	aKey      = "alloc-1"
+	aUUID     = "alloc-uuid-1"
+	nodeID1   = "node-1"
 	instType1 = "itype-1"
 )
 
@@ -159,7 +161,7 @@ func newNodeRes(nodeID string, total *resources.Resource) *Node {
 }
 
 func newNodeInternal(nodeID string, total, occupied *resources.Resource) *Node {
-	return &Node{
+	sn := &Node{
 		NodeID:            nodeID,
 		Hostname:          "",
 		Rackname:          "",
@@ -173,6 +175,8 @@ func newNodeInternal(nodeID string, total, occupied *resources.Resource) *Node {
 		schedulable:       true,
 		reservations:      make(map[string]*reservation),
 	}
+	sn.nodeEvents = newNodeEvents(sn, events.GetEventSystem())
+	return sn
 }
 
 func newProto(nodeID string, totalResource, occupiedResource *resources.Resource, attributes map[string]string) *si.NodeInfo {
@@ -204,18 +208,18 @@ func newProto(nodeID string, totalResource, occupiedResource *resources.Resource
 }
 
 // Create a new Allocation with a random ask key
-func newAllocation(appID, uuid, nodeID, queueName string, res *resources.Resource) *Allocation {
+func newAllocation(appID, uuid, nodeID string, res *resources.Resource) *Allocation {
 	askKey := strconv.FormatInt((time.Now()).UnixNano(), 10)
 	ask := newAllocationAsk(askKey, appID, res)
-	return NewAllocation(uuid, nodeID, instType1, ask)
+	return NewAllocation(uuid, nodeID, ask)
 }
 
 // Create a new Allocation with a random ask key
-func newPlaceholderAlloc(appID, uuid, nodeID, queueName string, res *resources.Resource) *Allocation {
+func newPlaceholderAlloc(appID, uuid, nodeID string, res *resources.Resource) *Allocation {
 	askKey := strconv.FormatInt((time.Now()).UnixNano(), 10)
 	ask := newAllocationAsk(askKey, appID, res)
 	ask.placeholder = true
-	return NewAllocation(uuid, nodeID, instType1, ask)
+	return NewAllocation(uuid, nodeID, ask)
 }
 
 func newAllocationAsk(allocKey, appID string, res *resources.Resource) *AllocationAsk {
@@ -261,7 +265,7 @@ func assertUserGroupResource(t *testing.T, userGroup security.UserGroup, expecte
 	userResource := ugm.GetUserResources(userGroup)
 	groupResource := ugm.GetGroupResources(userGroup.Groups[0])
 	assert.Equal(t, resources.Equals(userResource, expected), true)
-	assert.Equal(t, resources.Equals(groupResource, expected), true)
+	assert.Equal(t, resources.Equals(groupResource, nil), true)
 }
 
 func assertUserResourcesAndGroupResources(t *testing.T, userGroup security.UserGroup, expectedUserResources *resources.Resource, expectedGroupResources *resources.Resource, i int) {
