@@ -39,6 +39,7 @@ import (
 	"github.com/apache/yunikorn-core/pkg/scheduler/objects"
 	"github.com/apache/yunikorn-core/pkg/scheduler/placement"
 	"github.com/apache/yunikorn-core/pkg/scheduler/policies"
+	"github.com/apache/yunikorn-core/pkg/scheduler/ugm"
 	"github.com/apache/yunikorn-core/pkg/webservice/dao"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
@@ -61,6 +62,9 @@ type PartitionContext struct {
 	userGroupCache         *security.UserGroupCache        // user cache per partition
 	totalPartitionResource *resources.Resource             // Total node resources
 	allocations            int                             // Number of allocations on the partition
+	reservations           int                             // number of reservations
+	placeholderAllocations int                             // number of placeholder allocations
+	preemptionEnabled      bool                            // whether preemption is enabled or not
 
 	// The partition write lock must not be held while manipulating an application.
 	// Scheduling is running continuously as a lock free background task. Scheduling an application
@@ -76,7 +80,7 @@ type PartitionContext struct {
 
 func newPartitionContext(conf configs.PartitionConfig, rmID string, cc *ClusterContext) (*PartitionContext, error) {
 	if conf.Name == "" || rmID == "" {
-		log.Logger().Info("partition cannot be created",
+		log.Log(log.SchedPartition).Info("partition cannot be created",
 			zap.String("partition name", conf.Name),
 			zap.String("rmID", rmID),
 			zap.Any("cluster context", cc))
@@ -116,7 +120,7 @@ func (pc *PartitionContext) initialPartitionFromConfig(conf configs.PartitionCon
 	if err = pc.addQueue(queueConf.Queues, pc.root); err != nil {
 		return err
 	}
-	log.Logger().Info("root queue added",
+	log.Log(log.SchedPartition).Info("root queue added",
 		zap.String("partitionName", pc.Name),
 		zap.String("rmID", pc.RmID))
 
@@ -128,7 +132,10 @@ func (pc *PartitionContext) initialPartitionFromConfig(conf configs.PartitionCon
 	// TODO get the resolver from the config
 	pc.userGroupCache = security.GetUserGroupCache("")
 	pc.updateNodeSortingPolicy(conf)
-	return nil
+	pc.updatePreemption(conf)
+
+	// update limit settings: start at the root
+	return ugm.GetUserManager().UpdateConfig(queueConf, conf.Queues[0].Name)
 }
 
 // NOTE: this is a lock free call. It should only be called holding the PartitionContext lock.
@@ -136,39 +143,36 @@ func (pc *PartitionContext) updateNodeSortingPolicy(conf configs.PartitionConfig
 	var configuredPolicy policies.SortingPolicy
 	configuredPolicy, err := policies.SortingPolicyFromString(conf.NodeSortPolicy.Type)
 	if err != nil {
-		log.Logger().Debug("NodeSorting policy incorrectly set or unknown",
+		log.Log(log.SchedPartition).Debug("NodeSorting policy incorrectly set or unknown",
 			zap.Error(err))
-		log.Logger().Info(fmt.Sprintf("NodeSorting policy not set using '%s' as default", configuredPolicy))
+		log.Log(log.SchedPartition).Info(fmt.Sprintf("NodeSorting policy not set using '%s' as default", configuredPolicy))
 	} else {
-		log.Logger().Info("NodeSorting policy set from config",
+		log.Log(log.SchedPartition).Info("NodeSorting policy set from config",
 			zap.Stringer("policyName", configuredPolicy))
 	}
 	pc.nodes.SetNodeSortingPolicy(objects.NewNodeSortingPolicy(conf.NodeSortPolicy.Type, conf.NodeSortPolicy.ResourceWeights))
 }
 
+// NOTE: this is a lock free call. It should only be called holding the PartitionContext lock.
+func (pc *PartitionContext) updatePreemption(conf configs.PartitionConfig) {
+	pc.preemptionEnabled = conf.Preemption.Enabled == nil || *conf.Preemption.Enabled
+}
+
 func (pc *PartitionContext) updatePartitionDetails(conf configs.PartitionConfig) error {
 	pc.Lock()
 	defer pc.Unlock()
 	if len(conf.Queues) == 0 || conf.Queues[0].Name != configs.RootQueue {
 		return fmt.Errorf("partition cannot be created without root queue")
 	}
-
-	if pc.placementManager.IsInitialised() {
-		log.Logger().Info("Updating placement manager rules on config reload")
-		err := pc.placementManager.UpdateRules(conf.PlacementRules)
-		if err != nil {
-			log.Logger().Info("New placement rules not activated, config reload failed", zap.Error(err))
-			return err
-		}
-		pc.rules = &conf.PlacementRules
-	} else {
-		log.Logger().Info("Creating new placement manager on config reload")
-		pc.rules = &conf.PlacementRules
-		// We need to pass in the locked version of the GetQueue function.
-		// Placing an application will not have a lock on the partition context.
-		pc.placementManager = placement.NewPlacementManager(*pc.rules, pc.GetQueue)
+	log.Log(log.SchedPartition).Info("Updating placement manager rules on config reload")
+	err := pc.placementManager.UpdateRules(conf.PlacementRules)
+	if err != nil {
+		log.Log(log.SchedPartition).Info("New placement rules not activated, config reload failed", zap.Error(err))
+		return err
 	}
+	pc.rules = &conf.PlacementRules
 	pc.updateNodeSortingPolicy(conf)
+	pc.updatePreemption(conf)
 	// start at the root: there is only one queue
 	queueConf := conf.Queues[0]
 	root := pc.root
@@ -178,7 +182,11 @@ func (pc *PartitionContext) updatePartitionDetails(conf configs.PartitionConfig)
 	}
 	root.UpdateQueueProperties()
 	// update the rest of the queues recursively
-	return pc.updateQueues(queueConf.Queues, root)
+	if err := pc.updateQueues(queueConf.Queues, root); err != nil {
+		return err
+	}
+	// update limit settings: start at the root
+	return ugm.GetUserManager().UpdateConfig(queueConf, conf.Queues[0].Name)
 }
 
 // Process the config structure and create a queue info tree for this partition
@@ -242,7 +250,7 @@ func (pc *PartitionContext) updateQueues(config []configs.QueueConfig, parent *o
 // The current cleanup sequence is "immediate". This is implemented to allow a graceful cleanup.
 func (pc *PartitionContext) markPartitionForRemoval() {
 	if err := pc.handlePartitionEvent(objects.Remove); err != nil {
-		log.Logger().Error("failed to mark partition for deletion",
+		log.Log(log.SchedPartition).Error("failed to mark partition for deletion",
 			zap.String("partitionName", pc.Name),
 			zap.Error(err))
 	}
@@ -254,10 +262,6 @@ func (pc *PartitionContext) isDraining() bool {
 	return pc.stateMachine.Current() == objects.Draining.String()
 }
 
-func (pc *PartitionContext) isRunning() bool {
-	return pc.stateMachine.Current() == objects.Active.String()
-}
-
 func (pc *PartitionContext) isStopped() bool {
 	return pc.stateMachine.Current() == objects.Stopped.String()
 }
@@ -299,38 +303,41 @@ func (pc *PartitionContext) AddApplication(app *objects.Application) error {
 	}
 
 	// Put app under the queue
-	queueName := app.GetQueuePath()
 	pm := pc.getPlacementManager()
-	if pm.IsInitialised() {
-		err := pm.PlaceApplication(app)
-		if err != nil {
-			return fmt.Errorf("failed to place application %s: %v", appID, err)
-		}
-		queueName = app.GetQueuePath()
-		if queueName == "" {
-			return fmt.Errorf("application rejected by placement rules: %s", appID)
-		}
+	err := pm.PlaceApplication(app)
+	if err != nil {
+		return fmt.Errorf("failed to place application %s: %v", appID, err)
 	}
+	queueName := app.GetQueuePath()
+	if queueName == "" {
+		return fmt.Errorf("application rejected by placement rules: %s", appID)
+	}
+
 	// lock the partition and make the last change: we need to do this before creating the queues.
 	// queue cleanup might otherwise remove the queue again before we can add the application
 	pc.Lock()
 	defer pc.Unlock()
 	// we have a queue name either from placement or direct, get the queue
 	queue := pc.getQueueInternal(queueName)
+
+	// create the queue if necessary
 	if queue == nil {
-		// queue must exist if not using placement rules
-		if !pm.IsInitialised() {
-			return fmt.Errorf("application '%s' rejected, cannot create queue '%s' without placement rules", appID, queueName)
-		}
-		// with placement rules the hierarchy might not exist so try and create it
 		var err error
-		queue, err = pc.createQueue(queueName, app.GetUser())
-		if err != nil {
-			return fmt.Errorf("failed to create rule based queue %s for application %s", queueName, appID)
+		if common.IsRecoveryQueue(queueName) {
+			queue, err = pc.createRecoveryQueue()
+			if err != nil {
+				return fmt.Errorf("failed to create recovery queue %s for application %s", common.RecoveryQueueFull, appID)
+			}
+		} else {
+			queue, err = pc.createQueue(queueName, app.GetUser())
+			if err != nil {
+				return fmt.Errorf("failed to create rule based queue %s for application %s", queueName, appID)
+			}
 		}
 	}
-	// check the queue: is a leaf queue with submit access
-	if !queue.IsLeafQueue() || !queue.CheckSubmitAccess(app.GetUser()) {
+
+	// check the queue: is a leaf queue
+	if !queue.IsLeafQueue() {
 		return fmt.Errorf("failed to find queue %s for application %s", queueName, appID)
 	}
 
@@ -387,13 +394,13 @@ func (pc *PartitionContext) removeApplication(appID string) []*objects.Allocatio
 			currentUUID := alloc.GetUUID()
 			node := pc.GetNode(alloc.GetNodeID())
 			if node == nil {
-				log.Logger().Warn("unknown node: not found in active node list",
+				log.Log(log.SchedPartition).Warn("unknown node: not found in active node list",
 					zap.String("appID", appID),
 					zap.String("nodeID", alloc.GetNodeID()))
 				continue
 			}
 			if nodeAlloc := node.RemoveAllocation(currentUUID); nodeAlloc == nil {
-				log.Logger().Warn("unknown allocation: not found on the node",
+				log.Log(log.SchedPartition).Warn("unknown allocation: not found on the node",
 					zap.String("appID", appID),
 					zap.String("allocationId", currentUUID),
 					zap.String("nodeID", alloc.GetNodeID()))
@@ -465,16 +472,15 @@ func (pc *PartitionContext) getQueueInternal(name string) *objects.Queue {
 }
 
 // Get the queue info for the whole queue structure to pass to the webservice
-func (pc *PartitionContext) GetQueueInfo() dao.QueueDAOInfo {
-	return pc.root.GetQueueInfo()
+func (pc *PartitionContext) GetPartitionQueues() dao.PartitionQueueDAOInfo {
+	partitionQueueDAOInfo := pc.root.GetPartitionQueueDAOInfo()
+	partitionQueueDAOInfo.Partition = common.GetPartitionNameWithoutClusterID(pc.Name)
+	return partitionQueueDAOInfo
 }
 
-// Get the queue info for the whole queue structure to pass to the webservice
-func (pc *PartitionContext) GetPartitionQueues() dao.PartitionQueueDAOInfo {
-	var PartitionQueueDAOInfo = dao.PartitionQueueDAOInfo{}
-	PartitionQueueDAOInfo = pc.root.GetPartitionQueueDAOInfo()
-	PartitionQueueDAOInfo.Partition = common.GetPartitionNameWithoutClusterID(pc.Name)
-	return PartitionQueueDAOInfo
+// Create the recovery queue.
+func (pc *PartitionContext) createRecoveryQueue() (*objects.Queue, error) {
+	return objects.NewRecoveryQueue(pc.root)
 }
 
 // Create a queue with full hierarchy. This is called when a new queue is created from a placement rule.
@@ -488,7 +494,7 @@ func (pc *PartitionContext) createQueue(name string, user security.UserGroup) (*
 	}
 	current := name
 	queue := pc.getQueueInternal(current)
-	log.Logger().Debug("Checking queue creation")
+	log.Log(log.SchedPartition).Debug("Checking queue creation")
 	for queue == nil {
 		toCreate = append(toCreate, current[strings.LastIndex(current, configs.DOT)+1:])
 		current = current[0:strings.LastIndex(current, configs.DOT)]
@@ -502,7 +508,7 @@ func (pc *PartitionContext) createQueue(name string, user security.UserGroup) (*
 	if queue.IsLeafQueue() {
 		return nil, fmt.Errorf("creation of queue %s failed parent is already a leaf: %s", name, current)
 	}
-	log.Logger().Debug("Creating queue(s)",
+	log.Log(log.SchedPartition).Debug("Creating queue(s)",
 		zap.String("parent", current),
 		zap.String("fullPath", name))
 	for i := len(toCreate) - 1; i >= 0; i-- {
@@ -510,7 +516,7 @@ func (pc *PartitionContext) createQueue(name string, user security.UserGroup) (*
 		var err error
 		queue, err = objects.NewDynamicQueue(toCreate[i], i == 0, queue)
 		if err != nil {
-			log.Logger().Warn("Queue auto create failed unexpected",
+			log.Log(log.SchedPartition).Warn("Queue auto create failed unexpected",
 				zap.String("queueName", toCreate[i]),
 				zap.Error(err))
 			return nil, err
@@ -533,7 +539,7 @@ func (pc *PartitionContext) AddNode(node *objects.Node, existingAllocations []*o
 	if node == nil {
 		return fmt.Errorf("cannot add 'nil' node to partition %s", pc.Name)
 	}
-	log.Logger().Info("adding node to partition",
+	log.Log(log.SchedPartition).Info("adding node to partition",
 		zap.String("partition", pc.Name),
 		zap.String("nodeID", node.NodeID))
 	if pc.isDraining() || pc.isStopped() {
@@ -549,7 +555,7 @@ func (pc *PartitionContext) AddNode(node *objects.Node, existingAllocations []*o
 			if err := pc.addAllocation(alloc); err != nil {
 				// not expecting any inflight replacements on node recovery
 				released, _ := pc.removeNode(node.NodeID)
-				log.Logger().Info("Failed to add existing allocations, changes reversed",
+				log.Log(log.SchedPartition).Info("Failed to add existing allocations, changes reversed",
 					zap.String("nodeID", node.NodeID),
 					zap.Int("existingAllocations", len(existingAllocations)),
 					zap.Int("releasedAllocations", len(released)),
@@ -598,7 +604,7 @@ func (pc *PartitionContext) addNodeToList(node *objects.Node) error {
 		pc.totalPartitionResource.AddTo(node.GetCapacity())
 	}
 	pc.root.SetMaxResource(pc.totalPartitionResource)
-	log.Logger().Info("Updated available resources from added node",
+	log.Log(log.SchedPartition).Info("Updated available resources from added node",
 		zap.String("partitionName", pc.Name),
 		zap.String("nodeID", node.NodeID),
 		zap.Stringer("partitionResource", pc.totalPartitionResource))
@@ -613,7 +619,7 @@ func (pc *PartitionContext) removeNodeFromList(nodeID string) *objects.Node {
 	defer pc.Unlock()
 	node := pc.nodes.RemoveNode(nodeID)
 	if node == nil {
-		log.Logger().Debug("node was not found, node already removed",
+		log.Log(log.SchedPartition).Debug("node was not found, node already removed",
 			zap.String("nodeID", nodeID),
 			zap.String("partitionName", pc.Name))
 		return nil
@@ -625,7 +631,7 @@ func (pc *PartitionContext) removeNodeFromList(nodeID string) *objects.Node {
 	// found the node cleanup the available resources, partition resources cannot be nil at this point
 	pc.totalPartitionResource.SubFrom(node.GetCapacity())
 	pc.root.SetMaxResource(pc.totalPartitionResource)
-	log.Logger().Info("Updated available resources from removed node",
+	log.Log(log.SchedPartition).Info("Updated available resources from removed node",
 		zap.String("partitionName", pc.Name),
 		zap.String("nodeID", node.NodeID),
 		zap.Stringer("partitionResource", pc.totalPartitionResource))
@@ -638,7 +644,7 @@ func (pc *PartitionContext) removeNodeFromList(nodeID string) *objects.Node {
 // other nodes.
 // NOTE: this is a lock free call. It must NOT be called holding the PartitionContext lock.
 func (pc *PartitionContext) removeNode(nodeID string) ([]*objects.Allocation, []*objects.Allocation) {
-	log.Logger().Info("removing node from partition",
+	log.Log(log.SchedPartition).Info("removing node from partition",
 		zap.String("partition", pc.Name),
 		zap.String("nodeID", nodeID))
 
@@ -672,11 +678,14 @@ func (pc *PartitionContext) removeNodeAllocations(node *objects.Node) ([]*object
 		// was already removed
 		app := pc.getApplication(alloc.GetApplicationID())
 		if app == nil {
-			log.Logger().Info("app is not found, skipping while removing the node",
+			log.Log(log.SchedPartition).Info("app is not found, skipping while removing the node",
 				zap.String("appID", alloc.GetApplicationID()),
 				zap.String("nodeID", node.NodeID))
 			continue
 		}
+		// Processing a removal while in the Completing state could race with the state change.
+		// Retrieve the queue early before a possible race.
+		queue := app.GetQueue()
 		// check for an inflight replacement.
 		if alloc.GetReleaseCount() != 0 {
 			release := alloc.GetFirstRelease()
@@ -698,16 +707,16 @@ func (pc *PartitionContext) removeNodeAllocations(node *objects.Node) ([]*object
 					// The reverse case is handled during allocation.
 					if delta.HasNegativeValue() {
 						// this looks incorrect but the delta is negative and the result will be a real decrease
-						err := app.GetQueue().IncAllocatedResource(delta, false)
+						err := queue.IncAllocatedResource(delta, false)
 						// this should not happen as we really decrease the value
 						if err != nil {
-							log.Logger().Warn("unexpected failure during queue update: replacing placeholder",
+							log.Log(log.SchedPartition).Warn("unexpected failure during queue update: replacing placeholder",
 								zap.String("appID", alloc.GetApplicationID()),
 								zap.String("placeholderID", alloc.GetUUID()),
 								zap.String("allocationID", release.GetUUID()),
 								zap.Error(err))
 						}
-						log.Logger().Warn("replacing placeholder: placeholder is larger than real allocation",
+						log.Log(log.SchedPartition).Warn("replacing placeholder: placeholder is larger than real allocation",
 							zap.String("allocationID", release.GetUUID()),
 							zap.Stringer("requested resource", release.GetAllocatedResource()),
 							zap.String("placeholderID", alloc.GetUUID()),
@@ -717,7 +726,7 @@ func (pc *PartitionContext) removeNodeAllocations(node *objects.Node) ([]*object
 					confirmed = append(confirmed, release)
 					// the allocation is removed so add it to the list that we return
 					released = append(released, alloc)
-					log.Logger().Info("allocation removed from node and replacement confirmed",
+					log.Log(log.SchedPartition).Info("allocation removed from node and replacement confirmed",
 						zap.String("nodeID", node.NodeID),
 						zap.String("allocationId", allocID),
 						zap.String("replacement nodeID", release.GetNodeID()),
@@ -732,13 +741,13 @@ func (pc *PartitionContext) removeNodeAllocations(node *objects.Node) ([]*object
 			// update the repeat on the real alloc to get it re-scheduled
 			_, err := app.UpdateAskRepeat(askAlloc.GetAsk().GetAllocationKey(), 1)
 			if err == nil {
-				log.Logger().Info("inflight placeholder replacement reversed due to node removal",
+				log.Log(log.SchedPartition).Info("inflight placeholder replacement reversed due to node removal",
 					zap.String("appID", askAlloc.GetApplicationID()),
 					zap.String("allocationKey", askAlloc.GetAsk().GetAllocationKey()),
 					zap.String("nodeID", node.NodeID),
 					zap.String("replacement allocationId", askAlloc.GetUUID()))
 			} else {
-				log.Logger().Error("node removal: repeat update failure for inflight replacement",
+				log.Log(log.SchedPartition).Error("node removal: repeat update failure for inflight replacement",
 					zap.String("appID", askAlloc.GetApplicationID()),
 					zap.String("allocationKey", askAlloc.GetAsk().GetAllocationKey()),
 					zap.String("nodeID", node.NodeID),
@@ -747,27 +756,29 @@ func (pc *PartitionContext) removeNodeAllocations(node *objects.Node) ([]*object
 		}
 		// check allocations on the app
 		if app.RemoveAllocation(allocID, si.TerminationType_UNKNOWN_TERMINATION_TYPE) == nil {
-			log.Logger().Info("allocation is not found, skipping while removing the node",
+			log.Log(log.SchedPartition).Info("allocation is not found, skipping while removing the node",
 				zap.String("allocationId", allocID),
 				zap.String("appID", app.ApplicationID),
 				zap.String("nodeID", node.NodeID))
 			continue
 		}
-		if err := app.GetQueue().DecAllocatedResource(alloc.GetAllocatedResource()); err != nil {
-			log.Logger().Warn("failed to release resources from queue",
+		if err := queue.DecAllocatedResource(alloc.GetAllocatedResource()); err != nil {
+			log.Log(log.SchedPartition).Warn("failed to release resources from queue",
 				zap.String("appID", alloc.GetApplicationID()),
 				zap.Error(err))
-		} else {
-			metrics.GetQueueMetrics(app.GetQueuePath()).IncReleasedContainer()
 		}
 		// remove preempted resources
 		if alloc.IsPreempted() {
-			app.GetQueue().DecPreemptingResource(alloc.GetAllocatedResource())
+			queue.DecPreemptingResource(alloc.GetAllocatedResource())
+		}
+		if alloc.IsPlaceholder() {
+			pc.decPhAllocationCount(1)
 		}
 
 		// the allocation is removed so add it to the list that we return
 		released = append(released, alloc)
-		log.Logger().Info("allocation removed from node",
+		metrics.GetQueueMetrics(queue.GetQueuePath()).IncReleasedContainer()
+		log.Log(log.SchedPartition).Info("allocation removed from node",
 			zap.String("nodeID", node.NodeID),
 			zap.String("allocationId", allocID))
 	}
@@ -793,7 +804,7 @@ func (pc *PartitionContext) tryAllocate() *objects.Allocation {
 		return nil
 	}
 	// try allocating from the root down
-	alloc := pc.root.TryAllocate(pc.GetNodeIterator, pc.GetFullNodeIterator, pc.GetNode)
+	alloc := pc.root.TryAllocate(pc.GetNodeIterator, pc.GetFullNodeIterator, pc.GetNode, pc.isPreemptionEnabled())
 	if alloc != nil {
 		return pc.allocate(alloc)
 	}
@@ -803,6 +814,9 @@ func (pc *PartitionContext) tryAllocate() *objects.Allocation {
 // Try process reservations for the partition
 // Lock free call this all locks are taken when needed in called functions
 func (pc *PartitionContext) tryReservedAllocate() *objects.Allocation {
+	if pc.getReservationCount() == 0 {
+		return nil
+	}
 	if !resources.StrictlyGreaterThanZero(pc.root.GetPendingResource()) {
 		// nothing to do just return
 		return nil
@@ -818,6 +832,9 @@ func (pc *PartitionContext) tryReservedAllocate() *objects.Allocation {
 // Try process placeholder for the partition
 // Lock free call this all locks are taken when needed in called functions
 func (pc *PartitionContext) tryPlaceholderAllocate() *objects.Allocation {
+	if pc.getPhAllocationCount() == 0 {
+		return nil
+	}
 	if !resources.StrictlyGreaterThanZero(pc.root.GetPendingResource()) {
 		// nothing to do just return
 		return nil
@@ -825,7 +842,7 @@ func (pc *PartitionContext) tryPlaceholderAllocate() *objects.Allocation {
 	// try allocating from the root down
 	alloc := pc.root.TryPlaceholderAllocate(pc.GetNodeIterator, pc.GetNode)
 	if alloc != nil {
-		log.Logger().Info("scheduler replace placeholder processed",
+		log.Log(log.SchedPartition).Info("scheduler replace placeholder processed",
 			zap.String("appID", alloc.GetApplicationID()),
 			zap.String("allocationKey", alloc.GetAllocationKey()),
 			zap.String("uuid", alloc.GetUUID()),
@@ -843,7 +860,7 @@ func (pc *PartitionContext) allocate(alloc *objects.Allocation) *objects.Allocat
 	appID := alloc.GetApplicationID()
 	app := pc.getApplication(appID)
 	if app == nil {
-		log.Logger().Info("Application was removed while allocating",
+		log.Log(log.SchedPartition).Info("Application was removed while allocating",
 			zap.String("appID", appID))
 		return nil
 	}
@@ -855,18 +872,19 @@ func (pc *PartitionContext) allocate(alloc *objects.Allocation) *objects.Allocat
 		nodeID = alloc.GetNodeID()
 	} else {
 		nodeID = alloc.GetReservedNodeID()
-		log.Logger().Debug("Reservation allocated on different node",
+		log.Log(log.SchedPartition).Debug("Reservation allocated on different node",
 			zap.String("current node", alloc.GetNodeID()),
 			zap.String("reserved node", nodeID),
 			zap.String("appID", appID))
 	}
 	node := pc.GetNode(nodeID)
 	if node == nil {
-		log.Logger().Info("Node was removed while allocating",
+		log.Log(log.SchedPartition).Info("Node was removed while allocating",
 			zap.String("nodeID", nodeID),
 			zap.String("appID", appID))
 		return nil
 	}
+	alloc.SetInstanceType(node.GetInstanceType())
 	// reservation
 	if alloc.GetResult() == objects.Reserved {
 		pc.reserve(app, node, alloc.GetAsk())
@@ -884,8 +902,11 @@ func (pc *PartitionContext) allocate(alloc *objects.Allocation) *objects.Allocat
 
 	// track the number of allocations
 	pc.updateAllocationCount(1)
+	if alloc.IsPlaceholder() {
+		pc.incPhAllocationCount()
+	}
 
-	log.Logger().Info("scheduler allocation processed",
+	log.Log(log.SchedPartition).Info("scheduler allocation processed",
 		zap.String("appID", alloc.GetApplicationID()),
 		zap.String("allocationKey", alloc.GetAllocationKey()),
 		zap.String("uuid", alloc.GetUUID()),
@@ -902,22 +923,23 @@ func (pc *PartitionContext) reserve(app *objects.Application, node *objects.Node
 	appID := app.ApplicationID
 	// app has node already reserved cannot reserve again
 	if app.IsReservedOnNode(node.NodeID) {
-		log.Logger().Info("Application is already reserved on node",
+		log.Log(log.SchedPartition).Info("Application is already reserved on node",
 			zap.String("appID", appID),
 			zap.String("nodeID", node.NodeID))
 		return
 	}
 	// all ok, add the reservation to the app, this will also reserve the node
 	if err := app.Reserve(node, ask); err != nil {
-		log.Logger().Debug("Failed to handle reservation, error during update of app",
+		log.Log(log.SchedPartition).Debug("Failed to handle reservation, error during update of app",
 			zap.Error(err))
 		return
 	}
 
 	// add the reservation to the queue list
 	app.GetQueue().Reserve(appID)
+	pc.incReservationCount()
 
-	log.Logger().Info("allocation ask is reserved",
+	log.Log(log.SchedPartition).Info("allocation ask is reserved",
 		zap.String("appID", appID),
 		zap.String("queue", app.GetQueuePath()),
 		zap.String("allocationKey", ask.GetAllocationKey()),
@@ -931,15 +953,16 @@ func (pc *PartitionContext) unReserve(app *objects.Application, node *objects.No
 	var err error
 	var num int
 	if num, err = app.UnReserve(node, ask); err != nil {
-		log.Logger().Info("Failed to unreserve, error during allocate on the app",
+		log.Log(log.SchedPartition).Info("Failed to unreserve, error during allocate on the app",
 			zap.Error(err))
 		return
 	}
 	// remove the reservation of the queue
 	appID := app.ApplicationID
 	app.GetQueue().UnReserve(appID, num)
+	pc.decReservationCount(num)
 
-	log.Logger().Info("allocation ask is unreserved",
+	log.Log(log.SchedPartition).Info("allocation ask is unreserved",
 		zap.String("appID", appID),
 		zap.String("queue", app.GetQueuePath()),
 		zap.String("allocationKey", ask.GetAllocationKey()),
@@ -1112,7 +1135,7 @@ func (pc *PartitionContext) addAllocation(alloc *objects.Allocation) error {
 			alloc.GetAllocationKey(), alloc.GetApplicationID())
 	}
 
-	log.Logger().Info("adding recovered allocation",
+	log.Log(log.SchedPartition).Info("adding recovered allocation",
 		zap.String("partitionName", pc.Name),
 		zap.String("appID", alloc.GetApplicationID()),
 		zap.String("allocKey", alloc.GetAllocationKey()),
@@ -1143,13 +1166,17 @@ func (pc *PartitionContext) addAllocation(alloc *objects.Allocation) error {
 
 	metrics.GetQueueMetrics(queue.GetQueuePath()).IncAllocatedContainer()
 	node.AddAllocation(alloc)
+	alloc.SetInstanceType(node.GetInstanceType())
 	app.RecoverAllocationAsk(alloc.GetAsk())
 	app.AddAllocation(alloc)
 
 	// track the number of allocations
 	pc.updateAllocationCount(1)
+	if alloc.IsPlaceholder() {
+		pc.incPhAllocationCount()
+	}
 
-	log.Logger().Info("recovered allocation",
+	log.Log(log.SchedPartition).Info("recovered allocation",
 		zap.String("partitionName", pc.Name),
 		zap.String("appID", alloc.GetApplicationID()),
 		zap.String("allocKey", alloc.GetAllocationKey()),
@@ -1158,10 +1185,10 @@ func (pc *PartitionContext) addAllocation(alloc *objects.Allocation) error {
 	return nil
 }
 
-func (pc *PartitionContext) convertUGI(ugi *si.UserGroupInformation) (security.UserGroup, error) {
+func (pc *PartitionContext) convertUGI(ugi *si.UserGroupInformation, forced bool) (security.UserGroup, error) {
 	pc.RLock()
 	defer pc.RUnlock()
-	return pc.userGroupCache.ConvertUGI(ugi)
+	return pc.userGroupCache.ConvertUGI(ugi, forced)
 }
 
 // calculate overall nodes resource usage and returns a map as the result,
@@ -1218,31 +1245,47 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 	app := pc.getApplication(appID)
 	// no app nothing to do everything should already be clean
 	if app == nil {
-		log.Logger().Info("Application not found while releasing allocation",
+		log.Log(log.SchedPartition).Info("Application not found while releasing allocation",
 			zap.String("appID", appID),
 			zap.String("allocationId", uuid),
 			zap.Stringer("terminationType", release.TerminationType))
 		return nil, nil
 	}
+	// Processing a removal while in the Completing state could race with the state change.
+	// The race occurs between removing the allocation and updating the queue after node processing.
+	// If the state change removes the queue link before we get to updating the queue after the node we
+	// leave the resources as allocated on the queue. The queue cannot be removed yet at this point as
+	// there are still allocations left. So retrieve the queue early to sidestep the race.
+	queue := app.GetQueue()
 	// temp store for allocations manipulated
 	released := make([]*objects.Allocation, 0)
 	var confirmed *objects.Allocation
 	// when uuid is not specified, remove all allocations from the app
 	if uuid == "" {
-		log.Logger().Info("remove all allocations",
+		log.Log(log.SchedPartition).Info("remove all allocations",
 			zap.String("appID", appID))
 		released = append(released, app.RemoveAllAllocations()...)
+		total := 0
+		for _, r := range released {
+			if r.IsPlaceholder() {
+				total++
+			}
+		}
+		pc.decPhAllocationCount(total)
 	} else {
 		// if we have an uuid the termination type is important
 		if release.TerminationType == si.TerminationType_PLACEHOLDER_REPLACED {
-			log.Logger().Info("replacing placeholder allocation",
+			log.Log(log.SchedPartition).Info("replacing placeholder allocation",
 				zap.String("appID", appID),
 				zap.String("allocationId", uuid))
 			if alloc := app.ReplaceAllocation(uuid); alloc != nil {
 				released = append(released, alloc)
+				if alloc.IsPlaceholder() {
+					pc.decPhAllocationCount(1)
+				}
 			}
 		} else {
-			log.Logger().Info("removing allocation from application",
+			log.Log(log.SchedPartition).Info("removing allocation from application",
 				zap.String("appID", appID),
 				zap.String("allocationId", uuid),
 				zap.Stringer("terminationType", release.TerminationType))
@@ -1258,7 +1301,7 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 	for _, alloc := range released {
 		node := pc.GetNode(alloc.GetNodeID())
 		if node == nil {
-			log.Logger().Info("node not found while releasing allocation",
+			log.Log(log.SchedPartition).Warn("node not found while releasing allocation",
 				zap.String("appID", appID),
 				zap.String("allocationId", alloc.GetUUID()),
 				zap.String("nodeID", alloc.GetNodeID()))
@@ -1275,7 +1318,7 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 				// This looks incorrect but the delta is negative and the result will be an increase of the
 				// total tracked. The total will later be deducted from the queue usage.
 				total.SubFrom(delta)
-				log.Logger().Warn("replacing placeholder: placeholder is larger than real allocation",
+				log.Log(log.SchedPartition).Warn("replacing placeholder: placeholder is larger than real allocation",
 					zap.String("allocationID", confirmed.GetUUID()),
 					zap.Stringer("requested resource", confirmed.GetAllocatedResource()),
 					zap.String("placeholderID", alloc.GetUUID()),
@@ -1289,14 +1332,14 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 				// we have already added the real allocation to the new node, just remove the placeholder
 				node.RemoveAllocation(alloc.GetUUID())
 			}
-			log.Logger().Info("replacing placeholder allocation on node",
+			log.Log(log.SchedPartition).Info("replacing placeholder allocation on node",
 				zap.String("nodeID", alloc.GetNodeID()),
 				zap.String("allocationId", alloc.GetUUID()),
 				zap.String("allocation nodeID", confirmed.GetNodeID()))
 		} else if node.RemoveAllocation(alloc.GetUUID()) != nil {
 			// all non replacement are real removes: must update the queue usage
 			total.AddTo(alloc.GetAllocatedResource())
-			log.Logger().Info("removing allocation from node",
+			log.Log(log.SchedPartition).Info("removing allocation from node",
 				zap.String("nodeID", alloc.GetNodeID()),
 				zap.String("allocationId", alloc.GetUUID()))
 		}
@@ -1305,18 +1348,15 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 		}
 	}
 	if resources.StrictlyGreaterThanZero(total) {
-		queue := app.GetQueue()
 		if err := queue.DecAllocatedResource(total); err != nil {
-			log.Logger().Warn("failed to release resources from queue",
+			log.Log(log.SchedPartition).Warn("failed to release resources from queue",
 				zap.String("appID", appID),
 				zap.String("allocationId", uuid),
 				zap.Error(err))
-		} else {
-			metrics.GetQueueMetrics(queue.GetQueuePath()).IncReleasedContainer()
 		}
 	}
 	if resources.StrictlyGreaterThanZero(totalPreempting) {
-		app.GetQueue().DecPreemptingResource(totalPreempting)
+		queue.DecPreemptingResource(totalPreempting)
 	}
 
 	// if confirmed is set we can assume there will just be one alloc in the released
@@ -1326,10 +1366,11 @@ func (pc *PartitionContext) removeAllocation(release *si.AllocationRelease) ([]*
 	}
 	// track the number of allocations, when we replace the result is no change
 	pc.updateAllocationCount(-len(released))
+	metrics.GetQueueMetrics(queue.GetQueuePath()).AddReleasedContainers(len(released))
 
-	// if the termination type is timeout, we don't notify the shim, because it's
-	// originated from that side
-	if release.TerminationType == si.TerminationType_TIMEOUT {
+	// if the termination type is TIMEOUT/PREEMPTED_BY_SCHEDULER, we don't notify the shim,
+	// because it's originated from that side
+	if release.TerminationType == si.TerminationType_TIMEOUT || release.TerminationType == si.TerminationType_PREEMPTED_BY_SCHEDULER {
 		released = nil
 	}
 	return released, confirmed
@@ -1345,14 +1386,14 @@ func (pc *PartitionContext) removeAllocationAsk(release *si.AllocationAskRelease
 	allocKey := release.AllocationKey
 	// A timeout termination is send by the core to the shim, ignore on return.
 	if release.TerminationType == si.TerminationType_TIMEOUT {
-		log.Logger().Debug("Ignoring ask release with termination type Timeout",
+		log.Log(log.SchedPartition).Debug("Ignoring ask release with termination type Timeout",
 			zap.String("appID", appID),
 			zap.String("ask", allocKey))
 		return
 	}
 	app := pc.getApplication(appID)
 	if app == nil {
-		log.Logger().Info("Invalid ask release requested by shim",
+		log.Log(log.SchedPartition).Info("Invalid ask release requested by shim",
 			zap.String("appID", appID),
 			zap.String("ask", allocKey),
 			zap.Stringer("terminationType", release.TerminationType))
@@ -1400,22 +1441,28 @@ func (pc *PartitionContext) GetNodeSortingResourceWeights() map[string]float64 {
 	return policy.ResourceWeights()
 }
 
+func (pc *PartitionContext) isPreemptionEnabled() bool {
+	pc.RLock()
+	defer pc.RUnlock()
+	return pc.preemptionEnabled
+}
+
 func (pc *PartitionContext) moveTerminatedApp(appID string) {
 	app := pc.getApplication(appID)
 	// nothing to do if the app is not found on the partition
 	if app == nil {
-		log.Logger().Debug("Application already removed from app list",
+		log.Log(log.SchedPartition).Debug("Application already removed from app list",
 			zap.String("appID", appID))
 		return
 	}
 	app.UnSetQueue()
 	// new ID as completedApplications map key, use negative value to get a divider
 	newID := appID + strconv.FormatInt(-(time.Now()).Unix(), 10)
-	log.Logger().Info("Removing terminated application from the application list",
+	log.Log(log.SchedPartition).Info("Removing terminated application from the application list",
 		zap.String("appID", appID),
 		zap.String("app status", app.CurrentState()))
 	app.LogAppSummary(pc.RmID)
-	app.CleanupUsedResource()
+	app.CleanupTrackedResource()
 	pc.Lock()
 	defer pc.Unlock()
 	delete(pc.applications, appID)
@@ -1424,7 +1471,7 @@ func (pc *PartitionContext) moveTerminatedApp(appID string) {
 
 func (pc *PartitionContext) AddRejectedApplication(rejectedApplication *objects.Application, rejectedMessage string) {
 	if err := rejectedApplication.RejectApplication(rejectedMessage); err != nil {
-		log.Logger().Warn("BUG: Unexpected failure: Application state not changed to Rejected",
+		log.Log(log.SchedPartition).Warn("BUG: Unexpected failure: Application state not changed to Rejected",
 			zap.String("currentState", rejectedApplication.CurrentState()),
 			zap.Error(err))
 	}
@@ -1433,3 +1480,39 @@ func (pc *PartitionContext) AddRejectedApplication(rejectedApplication *objects.
 	}
 	pc.rejectedApplications[rejectedApplication.ApplicationID] = rejectedApplication
 }
+
+func (pc *PartitionContext) incPhAllocationCount() {
+	pc.Lock()
+	defer pc.Unlock()
+	pc.placeholderAllocations++
+}
+
+func (pc *PartitionContext) decPhAllocationCount(num int) {
+	pc.Lock()
+	defer pc.Unlock()
+	pc.placeholderAllocations -= num
+}
+
+func (pc *PartitionContext) getPhAllocationCount() int {
+	pc.RLock()
+	defer pc.RUnlock()
+	return pc.placeholderAllocations
+}
+
+func (pc *PartitionContext) incReservationCount() {
+	pc.Lock()
+	defer pc.Unlock()
+	pc.reservations++
+}
+
+func (pc *PartitionContext) decReservationCount(num int) {
+	pc.Lock()
+	defer pc.Unlock()
+	pc.reservations -= num
+}
+
+func (pc *PartitionContext) getReservationCount() int {
+	pc.RLock()
+	defer pc.RUnlock()
+	return pc.reservations
+}
