@@ -38,8 +38,9 @@ type rule interface {
 
 	// Execute the rule and return the queue getName the application is placed in.
 	// Returns the fully qualified queue getName if the rule finds a queue or an empty string if the rule did not match.
+	// Additionally, returns true if ACLs should be checked or false otherwise.
 	// The error must only be set if there is a failure while executing the rule not if the rule did not match.
-	placeApplication(app *objects.Application, queueFn func(string) *objects.Queue) (string, error)
+	placeApplication(app *objects.Application, queueFn func(string) *objects.Queue) (string, bool, error)
 
 	// Return the getName of the rule which is defined in the rule.
 	// The basicRule provides a "unnamed rule" implementation.
@@ -105,10 +106,10 @@ func newRule(conf configs.PlacementRule) (rule, error) {
 	// initialise the rule: do not expect the rule to log errors
 	err = newRule.initialise(conf)
 	if err != nil {
-		log.Logger().Error("Rule init failed", zap.Error(err))
+		log.Log(log.Config).Error("Rule init failed", zap.Error(err))
 		return nil, err
 	}
-	log.Logger().Debug("New rule created", zap.Any("ruleConf", conf))
+	log.Log(log.Config).Debug("New rule created", zap.Any("ruleConf", conf))
 	return newRule, nil
 }
 
