@@ -21,8 +21,11 @@ package ugm
 import (
 	"sync"
 
+	"go.uber.org/zap"
+
+	"github.com/apache/yunikorn-core/pkg/common"
 	"github.com/apache/yunikorn-core/pkg/common/resources"
-	"github.com/apache/yunikorn-core/pkg/common/security"
+	"github.com/apache/yunikorn-core/pkg/log"
 	"github.com/apache/yunikorn-core/pkg/webservice/dao"
 )
 
@@ -39,43 +42,71 @@ type UserTracker struct {
 	sync.RWMutex
 }
 
-func newUserTracker(user security.UserGroup) *UserTracker {
+func newUserTracker(user string) *UserTracker {
 	queueTracker := newRootQueueTracker()
 	userTracker := &UserTracker{
-		userName:         user.User,
+		userName:         user,
 		appGroupTrackers: make(map[string]*GroupTracker),
 		queueTracker:     queueTracker,
 	}
 	return userTracker
 }
 
-func (ut *UserTracker) increaseTrackedResource(queuePath, applicationID string, usage *resources.Resource) error {
+func (ut *UserTracker) increaseTrackedResource(hierarchy []string, applicationID string, usage *resources.Resource) bool {
 	ut.Lock()
 	defer ut.Unlock()
-	return ut.queueTracker.increaseTrackedResource(queuePath, applicationID, usage)
+	increased := ut.queueTracker.increaseTrackedResource(hierarchy, applicationID, user, usage)
+	if increased {
+		gt := ut.appGroupTrackers[applicationID]
+		log.Log(log.SchedUGM).Debug("Increasing resource usage for group",
+			zap.String("group", gt.getName()),
+			zap.Strings("queue path", hierarchy),
+			zap.String("application", applicationID),
+			zap.Stringer("resource", usage))
+		increasedGroupUsage := gt.increaseTrackedResource(hierarchy, applicationID, usage, ut.userName)
+		if !increasedGroupUsage {
+			_, decreased := ut.queueTracker.decreaseTrackedResource(hierarchy, applicationID, usage, false)
+			if !decreased {
+				log.Log(log.SchedUGM).Error("User resource usage rollback has failed",
+					zap.Strings("queue path", hierarchy),
+					zap.String("application", applicationID),
+					zap.String("user", ut.userName))
+			}
+		}
+		return increasedGroupUsage
+	}
+	return increased
 }
 
-func (ut *UserTracker) decreaseTrackedResource(queuePath, applicationID string, usage *resources.Resource, removeApp bool) (bool, error) {
+func (ut *UserTracker) decreaseTrackedResource(hierarchy []string, applicationID string, usage *resources.Resource, removeApp bool) (bool, bool) {
 	ut.Lock()
 	defer ut.Unlock()
 	if removeApp {
 		delete(ut.appGroupTrackers, applicationID)
 	}
-	return ut.queueTracker.decreaseTrackedResource(queuePath, applicationID, usage, removeApp)
+	return ut.queueTracker.decreaseTrackedResource(hierarchy, applicationID, usage, removeApp)
 }
 
 func (ut *UserTracker) hasGroupForApp(applicationID string) bool {
 	ut.RLock()
 	defer ut.RUnlock()
-	return ut.appGroupTrackers[applicationID] != nil
+	_, ok := ut.appGroupTrackers[applicationID]
+	return ok
 }
 
 func (ut *UserTracker) setGroupForApp(applicationID string, groupTrack *GroupTracker) {
 	ut.Lock()
 	defer ut.Unlock()
-	if ut.appGroupTrackers[applicationID] == nil {
-		ut.appGroupTrackers[applicationID] = groupTrack
+	ut.appGroupTrackers[applicationID] = groupTrack
+}
+
+func (ut *UserTracker) getGroupForApp(applicationID string) string {
+	ut.Lock()
+	defer ut.Unlock()
+	if ut.appGroupTrackers[applicationID] != nil {
+		return ut.appGroupTrackers[applicationID].groupName
 	}
+	return common.Empty
 }
 
 func (ut *UserTracker) getTrackedApplications() map[string]*GroupTracker {
@@ -84,6 +115,18 @@ func (ut *UserTracker) getTrackedApplications() map[string]*GroupTracker {
 	return ut.appGroupTrackers
 }
 
+func (ut *UserTracker) setLimits(hierarchy []string, resource *resources.Resource, maxApps uint64) {
+	ut.Lock()
+	defer ut.Unlock()
+	ut.queueTracker.setLimit(hierarchy, resource, maxApps)
+}
+
+func (ut *UserTracker) headroom(hierarchy []string) *resources.Resource {
+	ut.Lock()
+	defer ut.Unlock()
+	return ut.queueTracker.headroom(hierarchy, user)
+}
+
 func (ut *UserTracker) GetUserResourceUsageDAOInfo() *dao.UserResourceUsageDAOInfo {
 	ut.RLock()
 	defer ut.RUnlock()
@@ -92,8 +135,40 @@ func (ut *UserTracker) GetUserResourceUsageDAOInfo() *dao.UserResourceUsageDAOIn
 	}
 	userResourceUsage.UserName = ut.userName
 	for app, gt := range ut.appGroupTrackers {
-		userResourceUsage.Groups[app] = gt.groupName
+		if gt != nil {
+			userResourceUsage.Groups[app] = gt.groupName
+		}
 	}
-	userResourceUsage.Queues = ut.queueTracker.getResourceUsageDAOInfo("")
+	userResourceUsage.Queues = ut.queueTracker.getResourceUsageDAOInfo(common.Empty)
 	return userResourceUsage
 }
+
+func (ut *UserTracker) IsQueuePathTrackedCompletely(hierarchy []string) bool {
+	ut.RLock()
+	defer ut.RUnlock()
+	return ut.queueTracker.IsQueuePathTrackedCompletely(hierarchy)
+}
+
+func (ut *UserTracker) IsUnlinkRequired(hierarchy []string) bool {
+	ut.RLock()
+	defer ut.RUnlock()
+	return ut.queueTracker.IsUnlinkRequired(hierarchy)
+}
+
+func (ut *UserTracker) UnlinkQT(hierarchy []string) bool {
+	ut.RLock()
+	defer ut.RUnlock()
+	return ut.queueTracker.UnlinkQT(hierarchy)
+}
+
+func (ut *UserTracker) canBeRemoved() bool {
+	ut.RLock()
+	defer ut.RUnlock()
+	return ut.queueTracker.canBeRemoved()
+}
+
+func (ut *UserTracker) canRunApp(hierarchy []string, applicationID string) bool {
+	ut.Lock()
+	defer ut.Unlock()
+	return ut.queueTracker.canRunApp(hierarchy, applicationID, user)
+}
