@@ -30,14 +30,14 @@ import (
 	"strings"
 
 	"github.com/julienschmidt/httprouter"
-
 	"github.com/prometheus/client_golang/prometheus/promhttp"
 	"go.uber.org/zap"
-	"gopkg.in/yaml.v2"
+	"gopkg.in/yaml.v3"
 
 	"github.com/apache/yunikorn-core/pkg/common"
 	"github.com/apache/yunikorn-core/pkg/common/configs"
 	"github.com/apache/yunikorn-core/pkg/common/resources"
+	"github.com/apache/yunikorn-core/pkg/events"
 	"github.com/apache/yunikorn-core/pkg/log"
 	metrics2 "github.com/apache/yunikorn-core/pkg/metrics"
 	"github.com/apache/yunikorn-core/pkg/metrics/history"
@@ -48,15 +48,17 @@ import (
 	"github.com/apache/yunikorn-core/pkg/webservice/dao"
 )
 
-const PartitionDoesNotExists = "Partition not found"
-const MissingParamsName = "Missing parameters"
-const QueueDoesNotExists = "Queue not found"
-const UserDoesNotExists = "User not found"
-const GroupDoesNotExists = "Group not found"
-const UserNameMissing = "User name is missing"
-const GroupNameMissing = "Group name is missing"
-const ApplicationDoesNotExists = "Application not found"
-const NodeDoesNotExists = "Node not found"
+const (
+	PartitionDoesNotExists   = "Partition not found"
+	MissingParamsName        = "Missing parameters"
+	QueueDoesNotExists       = "Queue not found"
+	UserDoesNotExists        = "User not found"
+	GroupDoesNotExists       = "Group not found"
+	UserNameMissing          = "User name is missing"
+	GroupNameMissing         = "Group name is missing"
+	ApplicationDoesNotExists = "Application not found"
+	NodeDoesNotExists        = "Node not found"
+)
 
 func getStackInfo(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
@@ -71,7 +73,7 @@ func getStackInfo(w http.ResponseWriter, r *http.Request) {
 		}
 	}
 	if _, err := w.Write(stack()); err != nil {
-		log.Logger().Error("GetStackInfo error", zap.Error(err))
+		log.Log(log.REST).Error("GetStackInfo error", zap.Error(err))
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
 }
@@ -130,7 +132,7 @@ func buildJSONErrorResponse(w http.ResponseWriter, detail string, code int) {
 	w.WriteHeader(code)
 	errorInfo := dao.NewYAPIError(nil, code, detail)
 	if jsonErr := json.NewEncoder(w).Encode(errorInfo); jsonErr != nil {
-		log.Logger().Error(fmt.Sprintf("Problem in sending error response in JSON format. Error response: %s", detail))
+		log.Log(log.REST).Error(fmt.Sprintf("Problem in sending error response in JSON format. Error response: %s", detail))
 	}
 }
 
@@ -146,7 +148,7 @@ func getClusterJSON(partition *scheduler.PartitionContext) *dao.ClusterDAOInfo {
 
 func getClusterUtilJSON(partition *scheduler.PartitionContext) []*dao.ClusterUtilDAOInfo {
 	var utils []*dao.ClusterUtilDAOInfo
-	var getResource bool = true
+	var getResource = true
 	total := partition.GetTotalPartitionResource()
 	if resources.IsZero(total) {
 		getResource = false
@@ -162,7 +164,7 @@ func getClusterUtilJSON(partition *scheduler.PartitionContext) []*dao.ClusterUti
 				ResourceType: name,
 				Total:        int64(total.Resources[name]),
 				Used:         int64(used.Resources[name]),
-				Usage:        fmt.Sprintf("%d", int64(value)) + "%",
+				Usage:        fmt.Sprintf("%d%%", int64(value)),
 			}
 			utils = append(utils, utilization)
 		}
@@ -348,29 +350,57 @@ func getNodesDAO(entries []*objects.Node) []*dao.NodeDAOInfo {
 	return nodesDAO
 }
 
+// getNodeUtilisation loads the node utilisation based on the dominant resource used
+// for the default partition. Dominant resource is defined as the highest utilised resource
+// type on the root queue based on the registered resources.
+// Use the default partition until we get YUNIKORN-2088 fixed
+func getNodeUtilisation(w http.ResponseWriter, r *http.Request) {
+	writeHeaders(w)
+	partitionContext := schedulerContext.GetPartitionWithoutClusterID(configs.DefaultPartition)
+	if partitionContext == nil {
+		buildJSONErrorResponse(w, PartitionDoesNotExists, http.StatusBadRequest)
+		return
+	}
+	// calculate the dominant resource based on root queue usage and size
+	rootQ := partitionContext.GetQueue(configs.RootQueue)
+	rootMax := rootQ.GetMaxResource()
+	// if no nodes have been registered return an empty object
+	nodesDao := &dao.NodesUtilDAOInfo{}
+	if !resources.IsZero(rootMax) {
+		// if nothing is used we get an empty dominant resource and return an empty object
+		rootUsed := rootQ.GetAllocatedResource()
+		dominant := rootUsed.DominantResourceType(rootMax)
+		nodesDao = getNodesUtilJSON(partitionContext, dominant)
+	}
+	if err := json.NewEncoder(w).Encode(nodesDao); err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
+	}
+}
+
+// getNodesUtilJSON loads the nodes utilisation for a partition for a specific resource type.
 func getNodesUtilJSON(partition *scheduler.PartitionContext, name string) *dao.NodesUtilDAOInfo {
 	mapResult := make([]int, 10)
 	mapName := make([][]string, 10)
 	var v float64
 	var nodeUtil []*dao.NodeUtilDAOInfo
+	var idx int
 	for _, node := range partition.GetNodes() {
-		resourceExist := true
-		// check resource exist or not
+		// check resource exist or not: only count if node advertises the resource
 		total := node.GetCapacity()
-		if total.Resources[name] <= 0 {
-			resourceExist = false
+		if _, ok := total.Resources[name]; !ok {
+			continue
 		}
 		resourceAllocated := node.GetAllocatedResource()
-		if _, ok := resourceAllocated.Resources[name]; !ok {
-			resourceExist = false
-		}
-		// if resource exist in node, record the bucket it should go
-		if resourceExist {
+		// if resource exist in node, record the bucket it should go into,
+		// otherwise none is used, and it should end up in the 0 bucket
+		if _, ok := resourceAllocated.Resources[name]; ok {
 			v = float64(resources.CalculateAbsUsedCapacity(total, resourceAllocated).Resources[name])
-			idx := int(math.Dim(math.Ceil(v/10), 1))
-			mapResult[idx]++
-			mapName[idx] = append(mapName[idx], node.NodeID)
+			idx = int(math.Dim(math.Ceil(v/10), 1))
+		} else {
+			idx = 0
 		}
+		mapResult[idx]++
+		mapName[idx] = append(mapName[idx], node.NodeID)
 	}
 	// put number of nodes and node name to different buckets
 	for k := 0; k < 10; k++ {
@@ -424,9 +454,11 @@ func getContainerHistory(w http.ResponseWriter, r *http.Request) {
 func getClusterConfig(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
 
-	conf := configs.ConfigContext.Get(schedulerContext.GetPolicyGroup())
 	var marshalledConf []byte
 	var err error
+
+	conf := getClusterConfigDAO()
+
 	// check if we have a request for json output
 	if r.Header.Get("Accept") == "application/json" {
 		marshalledConf, err = json.Marshal(&conf)
@@ -442,6 +474,16 @@ func getClusterConfig(w http.ResponseWriter, r *http.Request) {
 	}
 }
 
+func getClusterConfigDAO() *dao.ConfigDAOInfo {
+	// merge core config with extra config
+	conf := dao.ConfigDAOInfo{
+		SchedulerConfig: configs.ConfigContext.Get(schedulerContext.GetPolicyGroup()),
+		Extra:           configs.GetConfigMap(),
+	}
+
+	return &conf
+}
+
 func checkHealthStatus(w http.ResponseWriter, r *http.Request) {
 	writeHeaders(w)
 
@@ -449,16 +491,16 @@ func checkHealthStatus(w http.ResponseWriter, r *http.Request) {
 	result := schedulerContext.GetLastHealthCheckResult()
 	if result != nil {
 		if !result.Healthy {
-			log.Logger().Error("Scheduler is not healthy", zap.Any("health check info", *result))
+			log.Log(log.SchedHealth).Error("Scheduler is not healthy", zap.Any("health check info", *result))
 			buildJSONErrorResponse(w, "Scheduler is not healthy", http.StatusServiceUnavailable)
 		} else {
-			log.Logger().Info("Scheduler is healthy", zap.Any("health check info", *result))
+			log.Log(log.SchedHealth).Info("Scheduler is healthy", zap.Any("health check info", *result))
 			if err := json.NewEncoder(w).Encode(result); err != nil {
 				buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 			}
 		}
 	} else {
-		log.Logger().Info("The healthy status of scheduler is not found", zap.Any("health check info", ""))
+		log.Log(log.SchedHealth).Info("The healthy status of scheduler is not found", zap.Any("health check info", ""))
 		buildJSONErrorResponse(w, "The healthy status of scheduler is not found", http.StatusNotFound)
 	}
 }
@@ -470,7 +512,7 @@ func buildUpdateResponse(err error, w http.ResponseWriter) {
 			buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 		}
 	} else {
-		log.Logger().Info("Configuration update failed with errors",
+		log.Log(log.REST).Info("Configuration update failed with errors",
 			zap.Error(err))
 		buildJSONErrorResponse(w, err.Error(), http.StatusConflict)
 	}
@@ -622,28 +664,6 @@ func getApplication(w http.ResponseWriter, r *http.Request) {
 	}
 }
 
-func setLogLevel(w http.ResponseWriter, r *http.Request) {
-	writeHeaders(w)
-	vars := httprouter.ParamsFromContext(r.Context())
-	if vars == nil {
-		buildJSONErrorResponse(w, MissingParamsName, http.StatusBadRequest)
-		return
-	}
-	level := vars.ByName("level")
-	if err := log.SetLogLevel(level); err != nil {
-		buildJSONErrorResponse(w, err.Error(), http.StatusBadRequest)
-	}
-}
-
-func getLogLevel(w http.ResponseWriter, r *http.Request) {
-	writeHeaders(w)
-	zapConfig := log.GetConfig()
-	if _, err := w.Write([]byte(zapConfig.Level.Level().String())); err != nil {
-		log.Logger().Error("Could not get log level", zap.Error(err))
-		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
-	}
-}
-
 func getPartitionInfoDAO(lists map[string]*scheduler.PartitionContext) []*dao.PartitionInfo {
 	var result []*dao.PartitionInfo
 
@@ -789,14 +809,14 @@ func getResourceManagerDiagnostics() map[string]interface{} {
 	dumpStr, err := plugin.GetStateDump()
 	if err != nil {
 		// might be not implemented
-		log.Logger().Debug("Unable to get RM state dump", zap.Error(err))
+		log.Log(log.REST).Debug("Unable to get RM state dump", zap.Error(err))
 		result["Error"] = err.Error()
 		return result
 	}
 
 	// convert to JSON map
 	if err = json.Unmarshal([]byte(dumpStr), &result); err != nil {
-		log.Logger().Warn("Unable to parse RM state dump", zap.Error(err))
+		log.Log(log.REST).Warn("Unable to parse RM state dump", zap.Error(err))
 		result["Error"] = err.Error()
 	}
 
@@ -879,3 +899,52 @@ func getGroupResourceUsage(w http.ResponseWriter, r *http.Request) {
 		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
 	}
 }
+
+func getEvents(w http.ResponseWriter, r *http.Request) {
+	writeHeaders(w)
+	eventSystem := events.GetEventSystem()
+	if !eventSystem.IsEventTrackingEnabled() {
+		buildJSONErrorResponse(w, "Event tracking is disabled", http.StatusBadRequest)
+		return
+	}
+
+	count := uint64(10000)
+	var start uint64
+
+	if countStr := r.URL.Query().Get("count"); countStr != "" {
+		c, err := strconv.ParseInt(countStr, 10, 64)
+		if err != nil {
+			buildJSONErrorResponse(w, err.Error(), http.StatusBadRequest)
+			return
+		}
+		if c <= 0 {
+			buildJSONErrorResponse(w, fmt.Sprintf("Illegal number of events: %d", c), http.StatusBadRequest)
+			return
+		}
+		count = uint64(c)
+	}
+
+	if startStr := r.URL.Query().Get("start"); startStr != "" {
+		i, err := strconv.ParseInt(startStr, 10, 64)
+		if err != nil {
+			buildJSONErrorResponse(w, err.Error(), http.StatusBadRequest)
+			return
+		}
+		if i < 0 {
+			buildJSONErrorResponse(w, fmt.Sprintf("Illegal id: %d", i), http.StatusBadRequest)
+			return
+		}
+		start = uint64(i)
+	}
+
+	records, lowestID, highestID := eventSystem.GetEventsFromID(start, count)
+	eventDao := dao.EventRecordDAO{
+		InstanceUUID: schedulerContext.GetUUID(),
+		LowestID:     lowestID,
+		HighestID:    highestID,
+		EventRecords: records,
+	}
+	if err := json.NewEncoder(w).Encode(eventDao); err != nil {
+		buildJSONErrorResponse(w, err.Error(), http.StatusInternalServerError)
+	}
+}
