@@ -27,17 +27,18 @@ import (
 	"reflect"
 	"strings"
 	"testing"
+	"time"
 
 	"github.com/julienschmidt/httprouter"
-
 	"github.com/prometheus/client_golang/prometheus/promhttp"
-	"gopkg.in/yaml.v2"
+	"gopkg.in/yaml.v3"
 	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common"
 	"github.com/apache/yunikorn-core/pkg/common/configs"
 	"github.com/apache/yunikorn-core/pkg/common/resources"
 	"github.com/apache/yunikorn-core/pkg/common/security"
+	"github.com/apache/yunikorn-core/pkg/events"
 	"github.com/apache/yunikorn-core/pkg/metrics/history"
 	"github.com/apache/yunikorn-core/pkg/scheduler"
 	"github.com/apache/yunikorn-core/pkg/scheduler/objects"
@@ -196,11 +197,72 @@ partitions:
                 vcore: 10000
 `
 
+const userGroupLimitsConfig = `
+partitions:
+    - name: default
+      queues:
+        - name: root
+          parent: true
+          submitacl: '*'
+          queues:
+            - name: parent1
+              parent: true
+              limits:
+                - limit: ""
+                  users:
+                    - test_user
+                  maxapplications: 0
+                  maxresources:
+                    cpu: "200"
+`
+
+const userGroupLimitsInvalidConfig = `
+partitions:
+    - name: default
+      queues:
+        - name: root
+          parent: true
+          submitacl: '*'
+          queues:
+            - name: parent1
+              parent: true
+              limits:
+                - limit: ""
+                  users:
+                    - test_user
+                  maxapplications: 1
+                  maxresources:
+                    cpu: "0"
+`
+
+const userGroupLimitsInvalidConfig1 = `
+partitions:
+    - name: default
+      queues:
+        - name: root
+          parent: true
+          submitacl: '*'
+          queues:
+            - name: parent1
+              parent: true
+              limits:
+                - limit: ""
+                  users:
+                    - test_user
+`
+
 const rmID = "rm-123"
 const policyGroup = "default-policy-group"
 const queueName = "root.default"
 const nodeID = "node-1"
-const instType = "itype-1"
+
+var (
+	updatedExtraConf = map[string]string{
+		"log.level":                  "info",
+		"service.schedulingInterval": "1s",
+		"admissionController.accessControl.bypassAuth": "false",
+	}
+)
 
 // setup To take care of setting up config, cluster, partitions etc
 func setup(t *testing.T, config string, partitionCount int) *scheduler.PartitionContext {
@@ -265,6 +327,47 @@ func TestValidateConf(t *testing.T) {
 	}
 }
 
+func TestUserGroupLimits(t *testing.T) {
+	confTests := []struct {
+		content          string
+		expectedResponse dao.ValidateConfResponse
+	}{
+		{
+			content: userGroupLimitsConfig,
+			expectedResponse: dao.ValidateConfResponse{
+				Allowed: true,
+				Reason:  common.Empty,
+			},
+		},
+		{
+			content: userGroupLimitsInvalidConfig,
+			expectedResponse: dao.ValidateConfResponse{
+				Allowed: false,
+				Reason:  "MaxResources is zero in '' limit, all resource types are zero",
+			},
+		},
+		{
+			content: userGroupLimitsInvalidConfig1,
+			expectedResponse: dao.ValidateConfResponse{
+				Allowed: false,
+				Reason:  "invalid resource combination for limit  all resource limits are null",
+			},
+		},
+	}
+	for _, test := range confTests {
+		// No err check: new request always returns correctly
+		//nolint: errcheck
+		req, _ := http.NewRequest("POST", "", strings.NewReader(test.content))
+		resp := &MockResponseWriter{}
+		validateConf(resp, req)
+		var vcr dao.ValidateConfResponse
+		err := json.Unmarshal(resp.outputBytes, &vcr)
+		assert.NilError(t, err, "failed to unmarshal ValidateConfResponse from response body")
+		assert.Equal(t, vcr.Allowed, test.expectedResponse.Allowed, "allowed flag incorrect")
+		assert.Equal(t, vcr.Reason, test.expectedResponse.Reason, "response text not as expected")
+	}
+}
+
 func TestApplicationHistory(t *testing.T) {
 	// make sure the history is nil when we finish this test
 	defer ResetIMHistory()
@@ -385,7 +488,7 @@ func TestGetConfigYAML(t *testing.T) {
 	resp := &MockResponseWriter{}
 	getClusterConfig(resp, req)
 	// yaml unmarshal handles the checksum add the end automatically in this implementation
-	conf := &configs.SchedulerConfig{}
+	conf := &dao.ConfigDAOInfo{}
 	err = yaml.Unmarshal(resp.outputBytes, conf)
 	assert.NilError(t, err, "failed to unmarshal config from response body")
 	assert.Equal(t, conf.Partitions[0].NodeSortPolicy.Type, "fair", "node sort policy set incorrectly, not fair")
@@ -396,6 +499,8 @@ func TestGetConfigYAML(t *testing.T) {
 	// change the config
 	err = schedulerContext.UpdateRMSchedulerConfig(rmID, []byte(updatedConf))
 	assert.NilError(t, err, "Error when updating clusterInfo from config")
+	configs.SetConfigMap(updatedExtraConf)
+
 	// check that we return yaml by default, unmarshal will error when we don't
 	req.Header.Set("Accept", "unknown")
 	getClusterConfig(resp, req)
@@ -403,6 +508,10 @@ func TestGetConfigYAML(t *testing.T) {
 	assert.NilError(t, err, "failed to unmarshal config from response body (updated config)")
 	assert.Equal(t, conf.Partitions[0].NodeSortPolicy.Type, "binpacking", "node sort policy not updated")
 	assert.Assert(t, startConfSum != conf.Checksum, "checksums did not change in output")
+	assert.DeepEqual(t, conf.Extra, updatedExtraConf)
+
+	// reset extra config map
+	configs.SetConfigMap(map[string]string{})
 }
 
 func TestGetConfigJSON(t *testing.T) {
@@ -414,7 +523,7 @@ func TestGetConfigJSON(t *testing.T) {
 	resp := &MockResponseWriter{}
 	getClusterConfig(resp, req)
 
-	conf := &configs.SchedulerConfig{}
+	conf := &dao.ConfigDAOInfo{}
 	err := json.Unmarshal(resp.outputBytes, conf)
 	assert.NilError(t, err, "failed to unmarshal config from response body (json)")
 	startConfSum := conf.Checksum
@@ -423,12 +532,17 @@ func TestGetConfigJSON(t *testing.T) {
 	// change the config
 	err = schedulerContext.UpdateRMSchedulerConfig(rmID, []byte(updatedConf))
 	assert.NilError(t, err, "Error when updating clusterInfo from config")
+	configs.SetConfigMap(updatedExtraConf)
 
 	getClusterConfig(resp, req)
 	err = json.Unmarshal(resp.outputBytes, conf)
 	assert.NilError(t, err, "failed to unmarshal config from response body (json, updated config)")
 	assert.Assert(t, startConfSum != conf.Checksum, "checksums did not change in json output: %s, %s", startConfSum, conf.Checksum)
 	assert.Equal(t, conf.Partitions[0].NodeSortPolicy.Type, "binpacking", "node sort policy not updated (json)")
+	assert.DeepEqual(t, conf.Extra, updatedExtraConf)
+
+	// reset extra config map
+	configs.SetConfigMap(map[string]string{})
 }
 
 func TestBuildUpdateResponseSuccess(t *testing.T) {
@@ -495,8 +609,8 @@ func TestGetClusterUtilJSON(t *testing.T) {
 	resAlloc2 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 300, siCommon.CPU: 200})
 	ask1 := objects.NewAllocationAsk("alloc-1", appID, resAlloc1)
 	ask2 := objects.NewAllocationAsk("alloc-2", appID, resAlloc2)
-	alloc1 := objects.NewAllocation("alloc-1-uuid", nodeID, instType, ask1)
-	alloc2 := objects.NewAllocation("alloc-2-uuid", nodeID, instType, ask2)
+	alloc1 := objects.NewAllocation("alloc-1-uuid", nodeID, ask1)
+	alloc2 := objects.NewAllocation("alloc-2-uuid", nodeID, ask2)
 	allocs := []*objects.Allocation{alloc1, alloc2}
 	err = partition.AddNode(node1, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
@@ -547,53 +661,146 @@ func TestGetNodesUtilJSON(t *testing.T) {
 
 	// create test nodes
 	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 1000, siCommon.CPU: 1000}).ToProto()
-	nodeRes2 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 1000, siCommon.CPU: 1000, "GPU": 10}).ToProto()
 	node1ID := "node-1"
 	node1 := objects.NewNode(&si.NodeInfo{NodeID: node1ID, SchedulableResource: nodeRes})
 	node2ID := "node-2"
+	nodeRes2 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 1000, siCommon.CPU: 1000, "GPU": 10}).ToProto()
 	node2 := objects.NewNode(&si.NodeInfo{NodeID: node2ID, SchedulableResource: nodeRes2})
+	node3ID := "node-3"
+	nodeCPU := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.CPU: 1000}).ToProto()
+	node3 := objects.NewNode(&si.NodeInfo{NodeID: node3ID, SchedulableResource: nodeCPU})
 
 	// create test allocations
 	resAlloc1 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 500, siCommon.CPU: 300})
 	resAlloc2 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 300, siCommon.CPU: 500, "GPU": 5})
 	ask1 := objects.NewAllocationAsk("alloc-1", appID, resAlloc1)
 	ask2 := objects.NewAllocationAsk("alloc-2", appID, resAlloc2)
-	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, instType, ask1)}
+	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, ask1)}
 	err = partition.AddNode(node1, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
-	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, instType, ask2)}
+	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, ask2)}
 	err = partition.AddNode(node2, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
+	err = partition.AddNode(node3, nil)
+	assert.NilError(t, err, "add node to partition should not have failed")
+
+	// two nodes advertise memory: must show up in the list
+	result := getNodesUtilJSON(partition, siCommon.Memory)
+	subResult := result.NodesUtil
+	assert.Equal(t, result.ResourceType, siCommon.Memory)
+	assert.Equal(t, subResult[2].NumOfNodes, int64(1))
+	assert.Equal(t, subResult[4].NumOfNodes, int64(1))
+	assert.Equal(t, subResult[2].NodeNames[0], node2ID)
+	assert.Equal(t, subResult[4].NodeNames[0], node1ID)
+
+	// three nodes advertise cpu: must show up in the list
+	result = getNodesUtilJSON(partition, siCommon.CPU)
+	subResult = result.NodesUtil
+	assert.Equal(t, result.ResourceType, siCommon.CPU)
+	assert.Equal(t, subResult[0].NumOfNodes, int64(1))
+	assert.Equal(t, subResult[0].NodeNames[0], node3ID)
+	assert.Equal(t, subResult[2].NumOfNodes, int64(1))
+	assert.Equal(t, subResult[2].NodeNames[0], node1ID)
+	assert.Equal(t, subResult[4].NumOfNodes, int64(1))
+	assert.Equal(t, subResult[4].NodeNames[0], node2ID)
+
+	// one node advertise GPU: must show up in the list
+	result = getNodesUtilJSON(partition, "GPU")
+	subResult = result.NodesUtil
+	assert.Equal(t, result.ResourceType, "GPU")
+	assert.Equal(t, subResult[4].NumOfNodes, int64(1))
+	assert.Equal(t, subResult[4].NodeNames[0], node2ID)
+
+	result = getNodesUtilJSON(partition, "non-exist")
+	subResult = result.NodesUtil
+	assert.Equal(t, result.ResourceType, "non-exist")
+	assert.Equal(t, subResult[0].NumOfNodes, int64(0))
+	assert.Equal(t, len(subResult[0].NodeNames), 0)
+}
+
+func TestGetNodeUtilisation(t *testing.T) {
+	NewWebApp(&scheduler.ClusterContext{}, nil)
+
+	// var req *http.Request
+	req, err := http.NewRequest("GET", "/ws/v1/scheduler/node-utilization", strings.NewReader(""))
+	assert.NilError(t, err, "Get node utilisation Handler request failed")
+	req = req.WithContext(context.TODO())
+	resp := &MockResponseWriter{}
+
+	getNodeUtilisation(resp, req)
+	var errInfo dao.YAPIError
+	err = json.Unmarshal(resp.outputBytes, &errInfo)
+	assert.NilError(t, err, "getNodeUtilisation should have returned and error")
+
+	partition := setup(t, configDefault, 1)
+	utilisation := &dao.NodesUtilDAOInfo{}
+	err = json.Unmarshal(resp.outputBytes, utilisation)
+	assert.NilError(t, err, "getNodeUtilisation should have returned an empty object")
+	assert.Equal(t, utilisation.ResourceType, "", "unexpected type returned")
+	assert.Equal(t, len(utilisation.NodesUtil), 0, "no nodes should be returned")
+	assert.Assert(t, confirmNodeCount(utilisation.NodesUtil, 0), "unexpected number of nodes returned should be 0")
+
+	// create test nodes
+	nodeRes := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 10}).ToProto()
+	nodeRes2 := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 10, "second": 5}).ToProto()
+	node1ID := "node-1"
+	node1 := objects.NewNode(&si.NodeInfo{NodeID: node1ID, SchedulableResource: nodeRes})
+	node2ID := "node-2"
+	node2 := objects.NewNode(&si.NodeInfo{NodeID: node2ID, SchedulableResource: nodeRes2})
+
+	err = partition.AddNode(node1, nil)
+	assert.NilError(t, err, "add node to partition should not have failed")
+	err = partition.AddNode(node2, nil)
+	assert.NilError(t, err, "add node to partition should not have failed")
 
 	// get nodes utilization
-	res1 := getNodesUtilJSON(partition, siCommon.Memory)
-	res2 := getNodesUtilJSON(partition, siCommon.CPU)
-	res3 := getNodesUtilJSON(partition, "GPU")
-	resNon := getNodesUtilJSON(partition, "non-exist")
-	subres1 := res1.NodesUtil
-	subres2 := res2.NodesUtil
-	subres3 := res3.NodesUtil
-	subresNon := resNon.NodesUtil
+	getNodeUtilisation(resp, req)
+	utilisation = &dao.NodesUtilDAOInfo{}
+	err = json.Unmarshal(resp.outputBytes, utilisation)
+	assert.NilError(t, err, "getNodeUtilisation should have returned an object")
+	assert.Equal(t, utilisation.ResourceType, "", "unexpected type returned")
+	assert.Equal(t, len(utilisation.NodesUtil), 10, "empty usage: unexpected bucket count returned")
+	assert.Assert(t, confirmNodeCount(utilisation.NodesUtil, 0), "unexpected number of nodes returned should be 0")
 
-	assert.Equal(t, res1.ResourceType, siCommon.Memory)
-	assert.Equal(t, subres1[2].NumOfNodes, int64(1))
-	assert.Equal(t, subres1[4].NumOfNodes, int64(1))
-	assert.Equal(t, subres1[2].NodeNames[0], node2ID)
-	assert.Equal(t, subres1[4].NodeNames[0], node1ID)
+	resAlloc := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 10})
+	ask := objects.NewAllocationAsk("alloc-1", "app", resAlloc)
+	alloc := objects.NewAllocation("alloc-1-uuid", node1ID, ask)
+	assert.Assert(t, node1.AddAllocation(alloc), "unexpected failure adding allocation to node")
+	rootQ := partition.GetQueue("root")
+	err = rootQ.IncAllocatedResource(resAlloc, false)
+	assert.NilError(t, err, "unexpected error returned setting allocated resource on queue")
+	// get nodes utilization
+	getNodeUtilisation(resp, req)
+	utilisation = &dao.NodesUtilDAOInfo{}
+	err = json.Unmarshal(resp.outputBytes, utilisation)
+	assert.NilError(t, err, "getNodeUtilisation should have returned an object")
+	assert.Equal(t, utilisation.ResourceType, "first", "expected first as type returned")
+	assert.Equal(t, len(utilisation.NodesUtil), 10, "empty usage: unexpected bucket count returned")
+	assert.Assert(t, confirmNodeCount(utilisation.NodesUtil, 2), "unexpected number of nodes returned should be 2")
 
-	assert.Equal(t, res2.ResourceType, siCommon.CPU)
-	assert.Equal(t, subres2[2].NumOfNodes, int64(1))
-	assert.Equal(t, subres2[4].NumOfNodes, int64(1))
-	assert.Equal(t, subres2[2].NodeNames[0], node1ID)
-	assert.Equal(t, subres2[4].NodeNames[0], node2ID)
+	// make second type dominant by using all
+	resAlloc = resources.NewResourceFromMap(map[string]resources.Quantity{"second": 5})
+	ask = objects.NewAllocationAsk("alloc-2", "app", resAlloc)
+	alloc = objects.NewAllocation("alloc-2-uuid", node2ID, ask)
+	assert.Assert(t, node2.AddAllocation(alloc), "unexpected failure adding allocation to node")
+	err = rootQ.IncAllocatedResource(resAlloc, false)
+	assert.NilError(t, err, "unexpected error returned setting allocated resource on queue")
+	// get nodes utilization
+	getNodeUtilisation(resp, req)
+	utilisation = &dao.NodesUtilDAOInfo{}
+	err = json.Unmarshal(resp.outputBytes, utilisation)
+	assert.NilError(t, err, "getNodeUtilisation should have returned an object")
+	assert.Equal(t, utilisation.ResourceType, "second", "expected second as type returned")
+	assert.Equal(t, len(utilisation.NodesUtil), 10, "empty usage: unexpected bucket count returned")
+	assert.Assert(t, confirmNodeCount(utilisation.NodesUtil, 1), "unexpected number of nodes returned should be 1")
+}
 
-	assert.Equal(t, res3.ResourceType, "GPU")
-	assert.Equal(t, subres3[4].NumOfNodes, int64(1))
-	assert.Equal(t, subres3[4].NodeNames[0], node2ID)
-
-	assert.Equal(t, resNon.ResourceType, "non-exist")
-	assert.Equal(t, subresNon[0].NumOfNodes, int64(0))
-	assert.Equal(t, len(subresNon[0].NodeNames), 0)
+func confirmNodeCount(info []*dao.NodeUtilDAOInfo, count int64) bool {
+	var total int64
+	for _, node := range info {
+		total += node.NumOfNodes
+	}
+	return total == count
 }
 
 func addAndConfirmApplicationExists(t *testing.T, partitionName string, partition *scheduler.PartitionContext, appName string) *objects.Application {
@@ -654,10 +861,10 @@ func TestPartitions(t *testing.T) {
 	resAlloc2 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 200, siCommon.CPU: 300})
 	ask1 := objects.NewAllocationAsk("alloc-1", app6.ApplicationID, resAlloc1)
 	ask2 := objects.NewAllocationAsk("alloc-2", app3.ApplicationID, resAlloc2)
-	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, instType, ask1)}
+	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, ask1)}
 	err := defaultPartition.AddNode(node1, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
-	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, instType, ask2)}
+	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, ask2)}
 	err = defaultPartition.AddNode(node2, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
 
@@ -805,10 +1012,10 @@ func TestGetPartitionNodes(t *testing.T) {
 	resAlloc2 := resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.Memory: 300, siCommon.CPU: 500})
 	ask1 := objects.NewAllocationAsk("alloc-1", appID, resAlloc1)
 	ask2 := objects.NewAllocationAsk("alloc-2", appID, resAlloc2)
-	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, instType, ask1)}
+	allocs := []*objects.Allocation{objects.NewAllocation("alloc-1-uuid", node1ID, ask1)}
 	err = partition.AddNode(node1, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
-	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, instType, ask2)}
+	allocs = []*objects.Allocation{objects.NewAllocation("alloc-2-uuid", node2ID, ask2)}
 	err = partition.AddNode(node2, allocs)
 	assert.NilError(t, err, "add node to partition should not have failed")
 
@@ -1114,7 +1321,7 @@ func assertApplicationExists(t *testing.T, resp *MockResponseWriter) {
 	err := json.Unmarshal(resp.outputBytes, &errInfo)
 	assert.NilError(t, err, "failed to unmarshal applications dao response from response body")
 	assert.Equal(t, http.StatusBadRequest, resp.statusCode, "Incorrect Status code")
-	assert.Equal(t, errInfo.Message, "Application not found", "JSON error message is incorrect")
+	assert.Equal(t, errInfo.Message, ApplicationDoesNotExists, "JSON error message is incorrect")
 	assert.Equal(t, errInfo.StatusCode, http.StatusBadRequest)
 }
 
@@ -1123,7 +1330,7 @@ func assertUserExists(t *testing.T, resp *MockResponseWriter) {
 	err := json.Unmarshal(resp.outputBytes, &errInfo)
 	assert.NilError(t, err, "failed to unmarshal applications dao response from response body")
 	assert.Equal(t, http.StatusBadRequest, resp.statusCode, "Incorrect Status code")
-	assert.Equal(t, errInfo.Message, "User not found", "JSON error message is incorrect")
+	assert.Equal(t, errInfo.Message, UserDoesNotExists, "JSON error message is incorrect")
 	assert.Equal(t, errInfo.StatusCode, http.StatusBadRequest)
 }
 
@@ -1132,7 +1339,7 @@ func assertUserNameExists(t *testing.T, resp *MockResponseWriter) {
 	err := json.Unmarshal(resp.outputBytes, &errInfo)
 	assert.NilError(t, err, "failed to unmarshal applications dao response from response body")
 	assert.Equal(t, http.StatusBadRequest, resp.statusCode, "Incorrect Status code")
-	assert.Equal(t, errInfo.Message, "User name is missing", "JSON error message is incorrect")
+	assert.Equal(t, errInfo.Message, UserNameMissing, "JSON error message is incorrect")
 	assert.Equal(t, errInfo.StatusCode, http.StatusBadRequest)
 }
 
@@ -1141,7 +1348,7 @@ func assertGroupExists(t *testing.T, resp *MockResponseWriter) {
 	err := json.Unmarshal(resp.outputBytes, &errInfo)
 	assert.NilError(t, err, "failed to unmarshal applications dao response from response body")
 	assert.Equal(t, http.StatusBadRequest, resp.statusCode, "Incorrect Status code")
-	assert.Equal(t, errInfo.Message, "Group not found", "JSON error message is incorrect")
+	assert.Equal(t, errInfo.Message, GroupDoesNotExists, "JSON error message is incorrect")
 	assert.Equal(t, errInfo.StatusCode, http.StatusBadRequest)
 }
 
@@ -1150,7 +1357,7 @@ func assertGroupNameExists(t *testing.T, resp *MockResponseWriter) {
 	err := json.Unmarshal(resp.outputBytes, &errInfo)
 	assert.NilError(t, err, "failed to unmarshal applications dao response from response body")
 	assert.Equal(t, http.StatusBadRequest, resp.statusCode, "Incorrect Status code")
-	assert.Equal(t, errInfo.Message, "Group name is missing", "JSON error message is incorrect")
+	assert.Equal(t, errInfo.Message, GroupNameMissing, "JSON error message is incorrect")
 	assert.Equal(t, errInfo.StatusCode, http.StatusBadRequest)
 }
 
@@ -1159,7 +1366,7 @@ func assertNodeIDExists(t *testing.T, resp *MockResponseWriter) {
 	err := json.Unmarshal(resp.outputBytes, &errInfo)
 	assert.NilError(t, err, "failed to unmarshal node dao response from response body")
 	assert.Equal(t, http.StatusBadRequest, resp.statusCode, "Incorrect Status code")
-	assert.Equal(t, errInfo.Message, "Node not found", "JSON error message is incorrect")
+	assert.Equal(t, errInfo.Message, NodeDoesNotExists, "JSON error message is incorrect")
 	assert.Equal(t, errInfo.StatusCode, http.StatusBadRequest)
 }
 
@@ -1177,6 +1384,15 @@ func TestValidateQueue(t *testing.T) {
 }
 
 func TestFullStateDumpPath(t *testing.T) {
+	original := configs.GetConfigMap()
+	defer func() {
+		configs.SetConfigMap(original)
+	}()
+	configMap := map[string]string{
+		"log.level": "WARN",
+	}
+	configs.SetConfigMap(configMap)
+
 	schedulerContext = prepareSchedulerContext(t, false)
 
 	partitionContext := schedulerContext.GetPartitionMapClone()
@@ -1199,45 +1415,6 @@ func TestFullStateDumpPath(t *testing.T) {
 	verifyStateDumpJSON(t, &aggregated)
 }
 
-func TestGetLoggerLevel(t *testing.T) {
-	req, err := http.NewRequest("GET", "/ws/v1/loglevel", strings.NewReader(""))
-	assert.NilError(t, err)
-
-	rr := httptest.NewRecorder()
-	handler := http.HandlerFunc(getLogLevel)
-	handler.ServeHTTP(rr, req)
-
-	expected := "debug"
-	assert.Equal(t, rr.Body.String(), expected,
-		fmt.Sprintf("handler returned unexpected body: got %v want %v", rr.Body.String(), expected))
-	assert.Equal(t, rr.Code, http.StatusOK)
-}
-
-func TestSetLoggerLevel(t *testing.T) {
-	// invalid
-	req, err := http.NewRequest("PUT", "/ws/v1/loglevel", strings.NewReader(""))
-	assert.NilError(t, err)
-	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
-		httprouter.Param{Key: "level", Value: "invalid"},
-	}))
-	rr := httptest.NewRecorder()
-
-	handler := http.HandlerFunc(setLogLevel)
-	handler.ServeHTTP(rr, req)
-	assert.Equal(t, rr.Code, http.StatusBadRequest)
-
-	// valid
-	req, err = http.NewRequest("PUT", "/ws/v1/loglevel", strings.NewReader(""))
-	assert.NilError(t, err)
-
-	req = req.WithContext(context.WithValue(req.Context(), httprouter.ParamsKey, httprouter.Params{
-		httprouter.Param{Key: "level", Value: "error"},
-	}))
-	rr = httptest.NewRecorder()
-	handler.ServeHTTP(rr, req)
-	assert.Equal(t, rr.Code, http.StatusOK)
-}
-
 func TestSpecificUserAndGroupResourceUsage(t *testing.T) {
 	prepareUserAndGroupContext(t)
 	// Test user name is missing
@@ -1293,8 +1470,7 @@ func TestSpecificUserAndGroupResourceUsage(t *testing.T) {
 	getGroupResourceUsage(resp, req)
 	err = json.Unmarshal(resp.outputBytes, &groupResourceUsageDao)
 	assert.NilError(t, err, "failed to unmarshal group resource usage dao response from response body: %s", string(resp.outputBytes))
-	assert.Equal(t, groupResourceUsageDao.Queues.ResourceUsage.String(),
-		resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.CPU: 1}).String())
+	assert.DeepEqual(t, groupResourceUsageDao, &dao.GroupResourceUsageDAOInfo{GroupName: "", Applications: nil, Queues: nil})
 
 	// Test non-existing group query
 	req, err = http.NewRequest("GET", "/ws/v1/partition/default/usage/group/", strings.NewReader(""))
@@ -1335,15 +1511,163 @@ func TestUsersAndGroupsResourceUsage(t *testing.T) {
 	assert.NilError(t, err, "Get Groups Resource Usage Handler request failed")
 
 	var groupsResourceUsageDao []*dao.GroupResourceUsageDAOInfo
+	var expGroupsResourceUsageDao []*dao.GroupResourceUsageDAOInfo
 	getGroupsResourceUsage(resp, req)
 	err = json.Unmarshal(resp.outputBytes, &groupsResourceUsageDao)
 	assert.NilError(t, err, "failed to unmarshal groups resource usage dao response from response body: %s", string(resp.outputBytes))
-	assert.Equal(t, groupsResourceUsageDao[0].Queues.ResourceUsage.String(),
-		resources.NewResourceFromMap(map[string]resources.Quantity{siCommon.CPU: 1}).String())
+	assert.DeepEqual(t, groupsResourceUsageDao, expGroupsResourceUsageDao)
 
 	// Assert existing groups
-	assert.Equal(t, len(groupsResourceUsageDao), 1)
-	assert.Equal(t, groupsResourceUsageDao[0].GroupName, "testgroup")
+	assert.Equal(t, len(groupsResourceUsageDao), 0)
+}
+
+func TestGetEvents(t *testing.T) {
+	prepareSchedulerContext(t, false)
+	appEvent, nodeEvent, queueEvent := addEvents(t)
+
+	checkAllEvents(t, []*si.EventRecord{appEvent, nodeEvent, queueEvent})
+
+	checkSingleEvent(t, appEvent, "count=1")
+	checkSingleEvent(t, queueEvent, "start=2")
+
+	// illegal requests
+	checkIllegalBatchRequest(t, "count=xyz", "strconv.ParseInt: parsing \"xyz\": invalid syntax")
+	checkIllegalBatchRequest(t, "count=-100", "Illegal number of events: -100")
+	checkIllegalBatchRequest(t, "start=xyz", "strconv.ParseInt: parsing \"xyz\": invalid syntax")
+	checkIllegalBatchRequest(t, "start=-100", "Illegal id: -100")
+}
+
+func TestGetEventsWhenTrackingDisabled(t *testing.T) {
+	original := configs.GetConfigMap()
+	defer func() {
+		ev := events.GetEventSystem().(*events.EventSystemImpl) //nolint:errcheck
+		ev.Stop()
+		configs.SetConfigMap(original)
+	}()
+	configMap := map[string]string{
+		configs.CMEventTrackingEnabled: "false",
+	}
+	configs.SetConfigMap(configMap)
+	events.Init()
+	ev := events.GetEventSystem().(*events.EventSystemImpl) //nolint:errcheck
+	ev.StartServiceWithPublisher(false)
+
+	req, err := http.NewRequest("GET", "/ws/v1/events/batch", strings.NewReader(""))
+	assert.NilError(t, err)
+	readIllegalRequest(t, req, "Event tracking is disabled")
+}
+
+func addEvents(t *testing.T) (appEvent, nodeEvent, queueEvent *si.EventRecord) {
+	events.Init()
+	ev := events.GetEventSystem().(*events.EventSystemImpl) //nolint:errcheck
+	ev.StartServiceWithPublisher(false)
+	protoRes := resources.NewResourceFromMap(map[string]resources.Quantity{
+		"cpu": 1,
+	}).ToProto()
+
+	appEvent = &si.EventRecord{
+		Type:              si.EventRecord_APP,
+		TimestampNano:     100,
+		Message:           "app event",
+		EventChangeType:   si.EventRecord_ADD,
+		EventChangeDetail: si.EventRecord_APP_ALLOC,
+		ObjectID:          "app",
+		ReferenceID:       "alloc",
+		Resource:          protoRes,
+	}
+	ev.AddEvent(appEvent)
+	nodeEvent = &si.EventRecord{
+		Type:              si.EventRecord_NODE,
+		TimestampNano:     101,
+		Message:           "node event",
+		EventChangeType:   si.EventRecord_ADD,
+		EventChangeDetail: si.EventRecord_APP_ALLOC,
+		ObjectID:          "node",
+		ReferenceID:       "alloc",
+		Resource:          protoRes,
+	}
+	ev.AddEvent(nodeEvent)
+	queueEvent = &si.EventRecord{
+		Type:              si.EventRecord_QUEUE,
+		TimestampNano:     102,
+		Message:           "queue event",
+		EventChangeType:   si.EventRecord_REMOVE,
+		EventChangeDetail: si.EventRecord_QUEUE_APP,
+		ObjectID:          "root.default",
+		ReferenceID:       "app",
+	}
+	ev.AddEvent(queueEvent)
+	noEvents := 0
+	err := common.WaitFor(10*time.Millisecond, time.Second, func() bool {
+		noEvents = ev.Store.CountStoredEvents()
+		return noEvents == 3
+	})
+	assert.NilError(t, err, "wanted 3 events, got %d", noEvents)
+	return appEvent, nodeEvent, queueEvent
+}
+
+func checkSingleEvent(t *testing.T, event *si.EventRecord, query string) {
+	req, err := http.NewRequest("GET", "/ws/v1/events/batch?"+query, strings.NewReader(""))
+	assert.NilError(t, err)
+	eventDao := getEventRecordDao(t, req)
+	assert.Assert(t, eventDao.InstanceUUID != "")
+	assert.Equal(t, 1, len(eventDao.EventRecords))
+	compareEvents(t, event, eventDao.EventRecords[0])
+}
+
+func checkIllegalBatchRequest(t *testing.T, query, msg string) {
+	req, err := http.NewRequest("GET", "/ws/v1/events/batch?"+query, strings.NewReader(""))
+	assert.NilError(t, err)
+	readIllegalRequest(t, req, msg)
+}
+
+func readIllegalRequest(t *testing.T, req *http.Request, errMsg string) {
+	rr := httptest.NewRecorder()
+	handler := http.HandlerFunc(getEvents)
+	handler.ServeHTTP(rr, req)
+	assert.Equal(t, http.StatusBadRequest, rr.Code)
+	jsonBytes := make([]byte, 256)
+	n, err := rr.Body.Read(jsonBytes)
+	assert.NilError(t, err, "cannot read response body")
+	var errObject dao.YAPIError
+	err = json.Unmarshal(jsonBytes[:n], &errObject)
+	assert.NilError(t, err, "cannot unmarshal events dao")
+	assert.Assert(t, strings.Contains(errObject.Message, errMsg))
+}
+
+func checkAllEvents(t *testing.T, events []*si.EventRecord) {
+	req, err := http.NewRequest("GET", "/ws/v1/events/batch/", strings.NewReader(""))
+	assert.NilError(t, err)
+	eventDao := getEventRecordDao(t, req)
+
+	for i := 0; i < len(events); i++ {
+		compareEvents(t, events[i], eventDao.EventRecords[i])
+	}
+}
+
+func compareEvents(t *testing.T, event, eventFromDao *si.EventRecord) {
+	assert.Equal(t, event.TimestampNano, eventFromDao.TimestampNano)
+	assert.Equal(t, event.EventChangeType, eventFromDao.EventChangeType)
+	assert.Equal(t, event.EventChangeDetail, eventFromDao.EventChangeDetail)
+	assert.Equal(t, event.ObjectID, eventFromDao.ObjectID)
+	assert.Equal(t, event.ReferenceID, eventFromDao.ReferenceID)
+	assert.Equal(t, event.Message, eventFromDao.Message)
+	res0 := resources.NewResourceFromProto(event.Resource)
+	res1 := resources.NewResourceFromProto(eventFromDao.Resource)
+	assert.Assert(t, resources.Equals(res0, res1))
+}
+
+func getEventRecordDao(t *testing.T, req *http.Request) dao.EventRecordDAO {
+	rr := httptest.NewRecorder()
+	handler := http.HandlerFunc(getEvents)
+	handler.ServeHTTP(rr, req)
+	jsonBytes := make([]byte, 2048)
+	n, err := rr.Body.Read(jsonBytes)
+	assert.NilError(t, err, "cannot read response body")
+	var eventDao dao.EventRecordDAO
+	err = json.Unmarshal(jsonBytes[:n], &eventDao)
+	assert.NilError(t, err, "cannot unmarshal events dao")
+	return eventDao
 }
 
 func prepareSchedulerContext(t *testing.T, stateDumpConf bool) *scheduler.ClusterContext {
@@ -1385,7 +1709,7 @@ func prepareUserAndGroupContext(t *testing.T) {
 
 	// add an alloc
 	uuid := "uuid-1"
-	allocInfo := objects.NewAllocation(uuid, "node-1", instType, ask)
+	allocInfo := objects.NewAllocation(uuid, "node-1", ask)
 	app.AddAllocation(allocInfo)
 	assert.Assert(t, app.IsStarting(), "Application did not return starting state after alloc: %s", app.CurrentState())
 
@@ -1399,4 +1723,6 @@ func verifyStateDumpJSON(t *testing.T, aggregated *AggregatedStateInfo) {
 	assert.Check(t, len(aggregated.ClusterInfo) > 0)
 	assert.Check(t, len(aggregated.Queues) > 0)
 	assert.Check(t, len(aggregated.LogLevel) > 0)
+	assert.Check(t, len(aggregated.Config.SchedulerConfig.Partitions) > 0)
+	assert.Check(t, len(aggregated.Config.Extra) > 0)
 }
