@@ -26,7 +26,6 @@ import (
 	"sync"
 
 	"go.uber.org/zap"
-	"go.uber.org/zap/zapcore"
 	v1 "k8s.io/api/core/v1"
 	informersv1 "k8s.io/client-go/informers/core/v1"
 	"k8s.io/client-go/tools/cache"
@@ -228,7 +227,7 @@ type configMapUpdateHandler struct {
 	conf *AdmissionControllerConf
 }
 
-func (h *configMapUpdateHandler) OnAdd(obj interface{}) {
+func (h *configMapUpdateHandler) OnAdd(obj interface{}, _ bool) {
 	cm := utils.Convert2ConfigMap(obj)
 	if idx, ok := h.configMapIndex(cm); ok {
 		h.conf.configUpdated(idx, cm)
@@ -250,7 +249,7 @@ func (h *configMapUpdateHandler) OnDelete(obj interface{}) {
 	case cache.DeletedFinalStateUnknown:
 		cm = utils.Convert2ConfigMap(obj)
 	default:
-		log.Logger().Warn("unable to convert to configmap")
+		log.Log(log.AdmissionConf).Warn("unable to convert to configmap")
 		return
 	}
 	if idx, ok := h.configMapIndex(cm); ok {
@@ -296,7 +295,7 @@ func (acc *AdmissionControllerConf) updateConfigMaps(configMaps []*v1.ConfigMap,
 
 	// check for enable config hot refresh
 	if !initial && !acc.enableConfigHotRefresh {
-		log.Logger().Warn("Config hot-refresh is disabled, ignoring configuration update")
+		log.Log(log.AdmissionConf).Warn("Config hot-refresh is disabled, ignoring configuration update")
 		return
 	}
 
@@ -306,10 +305,6 @@ func (acc *AdmissionControllerConf) updateConfigMaps(configMaps []*v1.ConfigMap,
 	// hot refresh
 	acc.enableConfigHotRefresh = parseConfigBool(configs, schedulerconf.CMSvcEnableConfigHotRefresh, schedulerconf.DefaultEnableConfigHotRefresh)
 
-	// logging
-	logLevel := parseConfigInt(configs, schedulerconf.CMLogLevel, schedulerconf.DefaultLoggingLevel)
-	log.GetZapConfigs().Level.SetLevel(zapcore.Level(logLevel))
-
 	// scheduler
 	acc.policyGroup = parseConfigString(configs, schedulerconf.CMSvcPolicyGroup, schedulerconf.DefaultPolicyGroup)
 
@@ -333,6 +328,14 @@ func (acc *AdmissionControllerConf) updateConfigMaps(configMaps []*v1.ConfigMap,
 
 	// labeling
 	acc.defaultQueueName = parseConfigString(configs, AMFilteringDefaultQueueName, DefaultFilteringQueueName)
+	if acc.defaultQueueName != "" && !strings.HasPrefix(acc.defaultQueueName, constants.RootQueue) {
+		log.Log(log.AdmissionConf).Warn("invalid default queue. defaultQueue must be fully qualified. Resetting to "+DefaultFilteringQueueName,
+			zap.String(AMFilteringDefaultQueueName, acc.defaultQueueName))
+		acc.defaultQueueName = DefaultFilteringQueueName
+	}
+
+	// logging
+	log.UpdateLoggingConfig(configs)
 
 	acc.dumpConfigurationInternal()
 }
@@ -344,7 +347,7 @@ func (acc *AdmissionControllerConf) DumpConfiguration() {
 }
 
 func (acc *AdmissionControllerConf) dumpConfigurationInternal() {
-	log.Logger().Info("Loaded admission controller configuration",
+	log.Log(log.AdmissionConf).Info("Loaded admission controller configuration",
 		zap.String("namespace", acc.namespace),
 		zap.String("kubeConfig", acc.kubeConfig),
 		zap.String("policyGroup", acc.policyGroup),
@@ -373,11 +376,11 @@ func parseConfigRegexps(config map[string]string, key string, defaultValue strin
 	value := parseConfigString(config, key, defaultValue)
 	result, err := parseRegexes(value)
 	if err != nil {
-		log.Logger().Error("Unable to parse regex values, using default",
+		log.Log(log.AdmissionConf).Error("Unable to parse regex values, using default",
 			zap.String("key", key), zap.String("value", value), zap.String("default", defaultValue), zap.Error(err))
 		result, err = parseRegexes(defaultValue)
 		if err != nil {
-			log.Logger().Fatal("BUG: can't parse default regex pattern", zap.Error(err))
+			log.Log(log.AdmissionConf).Fatal("BUG: can't parse default regex pattern", zap.Error(err))
 		}
 	}
 	return result
@@ -387,24 +390,13 @@ func parseConfigBool(config map[string]string, key string, defaultValue bool) bo
 	value := parseConfigString(config, key, fmt.Sprintf("%t", defaultValue))
 	result, err := strconv.ParseBool(value)
 	if err != nil {
-		log.Logger().Error("Unable to parse bool value, using default",
+		log.Log(log.AdmissionConf).Error("Unable to parse bool value, using default",
 			zap.String("key", key), zap.String("value", value), zap.Bool("default", defaultValue), zap.Error(err))
 		result = defaultValue
 	}
 	return result
 }
 
-func parseConfigInt(config map[string]string, key string, defaultValue int) int {
-	value := parseConfigString(config, key, fmt.Sprintf("%d", defaultValue))
-	result, err := strconv.ParseInt(value, 10, 31)
-	if err != nil {
-		log.Logger().Error("Unable to parse int value, using default",
-			zap.String("key", key), zap.String("value", value), zap.Int("default", defaultValue), zap.Error(err))
-		return defaultValue
-	}
-	return int(result)
-}
-
 func parseConfigString(config map[string]string, key string, defaultValue string) string {
 	if value, ok := config[key]; ok {
 		return value
@@ -421,7 +413,7 @@ func parseRegexes(patterns string) ([]*regexp.Regexp, error) {
 		}
 		re, err := regexp.Compile(pattern)
 		if err != nil {
-			log.Logger().Error("Unable to compile regular expression", zap.String("pattern", pattern), zap.Error(err))
+			log.Log(log.AdmissionConf).Error("Unable to compile regular expression", zap.String("pattern", pattern), zap.Error(err))
 			return nil, err
 		}
 		result = append(result, re)
