@@ -76,7 +76,7 @@ type priorityClassUpdateHandler struct {
 // OnAdd adds or replaces the priority class entry in the cache.
 // The cached value is only the resulting value of the annotation, not the whole PriorityClass object.
 // An empty string for the Name is technically possible but should not occur.
-func (h *priorityClassUpdateHandler) OnAdd(obj interface{}) {
+func (h *priorityClassUpdateHandler) OnAdd(obj interface{}, _ bool) {
 	pc := utils.Convert2PriorityClass(obj)
 	if pc == nil {
 		return
@@ -90,7 +90,7 @@ func (h *priorityClassUpdateHandler) OnAdd(obj interface{}) {
 
 // OnUpdate calls OnAdd for processing the PriorityClass cache update.
 func (h *priorityClassUpdateHandler) OnUpdate(_, newObj interface{}) {
-	h.OnAdd(newObj)
+	h.OnAdd(newObj, false)
 }
 
 // OnDelete removes the PriorityClass from the cache.
@@ -102,7 +102,7 @@ func (h *priorityClassUpdateHandler) OnDelete(obj interface{}) {
 	case cache.DeletedFinalStateUnknown:
 		pc = utils.Convert2PriorityClass(obj)
 	default:
-		log.Logger().Warn("unable to convert to PriorityClass")
+		log.Log(log.Admission).Warn("unable to convert to PriorityClass")
 		return
 	}
 	if pc == nil {
