@@ -25,11 +25,8 @@ import (
 
 	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/general"
 	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
-	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/sparkoperator"
 	"github.com/apache/yunikorn-k8shim/pkg/client"
-	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
 	"github.com/apache/yunikorn-k8shim/pkg/conf"
-	"github.com/apache/yunikorn-k8shim/pkg/controller/application"
 	"github.com/apache/yunikorn-k8shim/pkg/log"
 )
 
@@ -55,22 +52,12 @@ func NewAMService(amProtocol interfaces.ApplicationManagementProtocol,
 		podEventHandler: podEventHandler,
 	}
 
-	log.Logger().Info("Initializing new AppMgmt service")
-
-	if !apiProvider.IsTestingMode() {
-		log.Logger().Info("Registering Spark operator with the AppMgmt service")
-		appManager.register(
-			// registered app plugins
-			// for general apps
-			general.NewManager(apiProvider, podEventHandler),
-			// for spark operator - SparkApplication
-			sparkoperator.NewManager(amProtocol, apiProvider),
-			// for application crds
-			application.NewAppManager(amProtocol, apiProvider))
-	} else {
-		podEventHandler = general.NewPodEventHandler(amProtocol, false)
-		appManager.register(general.NewManager(apiProvider, podEventHandler))
-	}
+	log.Log(log.ShimAppMgmt).Info("Initializing new AppMgmt service")
+	appManager.register(
+		// registered app plugins
+		// for general apps
+		general.NewManager(apiProvider, podEventHandler),
+	)
 
 	return appManager
 }
@@ -91,11 +78,11 @@ func (svc *AppManagementService) GetManagerByName(name string) interfaces.AppMan
 func (svc *AppManagementService) register(managers ...interfaces.AppManager) {
 	for _, mgr := range managers {
 		if conf.GetSchedulerConf().IsOperatorPluginEnabled(mgr.Name()) {
-			log.Logger().Info("registering app management service",
+			log.Log(log.ShimAppMgmt).Info("registering app management service",
 				zap.String("serviceName", mgr.Name()))
 			svc.managers = append(svc.managers, mgr)
 		} else {
-			log.Logger().Info("skip registering app management service",
+			log.Log(log.ShimAppMgmt).Info("skip registering app management service",
 				zap.String("serviceName", mgr.Name()))
 		}
 	}
@@ -105,22 +92,22 @@ func (svc *AppManagementService) Start() error {
 	for _, optService := range svc.managers {
 		// init service before starting
 		if err := optService.ServiceInit(); err != nil {
-			log.Logger().Error("service init fails",
+			log.Log(log.ShimAppMgmt).Error("service init fails",
 				zap.String("serviceName", optService.Name()),
 				zap.Error(err))
 			return err
 		}
 
-		log.Logger().Info("starting app management service",
+		log.Log(log.ShimAppMgmt).Info("starting app management service",
 			zap.String("serviceName", optService.Name()))
 		if err := optService.Start(); err != nil {
-			log.Logger().Error("failed to start management service",
+			log.Log(log.ShimAppMgmt).Error("failed to start management service",
 				zap.String("serviceName", optService.Name()),
 				zap.Error(err))
 			return err
 		}
 
-		log.Logger().Info("app management service started",
+		log.Log(log.ShimAppMgmt).Info("app management service started",
 			zap.String("serviceName", optService.Name()))
 	}
 
@@ -128,22 +115,8 @@ func (svc *AppManagementService) Start() error {
 }
 
 func (svc *AppManagementService) Stop() {
-	log.Logger().Info("shutting down app management services")
+	log.Log(log.ShimAppMgmt).Info("shutting down app management services")
 	for _, optService := range svc.managers {
 		optService.Stop()
 	}
 }
-
-func (svc *AppManagementService) ApplicationStateUpdateEventHandler() func(obj interface{}) {
-	// when there is a app state update event received
-	// call the corresponding appManager to handle it, right now, only need to call the appCRD manager to handle this
-	mgr := svc.GetManagerByName(constants.AppManagerHandlerName)
-	if appMgr, ok := mgr.(*application.AppManager); ok {
-		return appMgr.HandleApplicationStateUpdate()
-	}
-	log.Logger().Warn("App manager is not registered",
-		zap.String("app manager name", constants.AppManagerHandlerName))
-	return func(obj interface{}) {
-		// noop
-	}
-}
