@@ -67,7 +67,7 @@ func (p *PodEventHandler) handleEventFromInformers(eventType EventType, source E
 	defer p.Unlock()
 
 	if p.recoveryRunning && source == Informers {
-		log.Logger().Debug("Storing async event", zap.Int("eventType", int(eventType)),
+		log.Log(log.ShimAppMgmtGeneral).Debug("Storing async event", zap.Int("eventType", int(eventType)),
 			zap.String("pod", pod.GetName()))
 		p.asyncEvents = append(p.asyncEvents, &podAsyncEvent{eventType, pod})
 		return true
@@ -84,7 +84,7 @@ func (p *PodEventHandler) internalHandle(eventType EventType, source EventSource
 	case DeletePod:
 		return p.deletePod(pod)
 	default:
-		log.Logger().Error("Unknown pod eventType", zap.Int("eventType", int(eventType)))
+		log.Log(log.ShimAppMgmtGeneral).Error("Unknown pod eventType", zap.Int("eventType", int(eventType)))
 		return nil
 	}
 }
@@ -95,7 +95,7 @@ func (p *PodEventHandler) RecoveryDone(terminatedPods map[string]bool) {
 
 	noOfEvents := len(p.asyncEvents)
 	if noOfEvents > 0 {
-		log.Logger().Info("Processing async events that arrived during recovery",
+		log.Log(log.ShimAppMgmtGeneral).Info("Processing async events that arrived during recovery",
 			zap.Int("no. of events", noOfEvents))
 		for _, event := range p.asyncEvents {
 			// ignore all events for pods that have already been determined to
@@ -107,7 +107,7 @@ func (p *PodEventHandler) RecoveryDone(terminatedPods map[string]bool) {
 			p.internalHandle(event.eventType, Informers, event.pod)
 		}
 	} else {
-		log.Logger().Info("No async pod events to process")
+		log.Log(log.ShimAppMgmtGeneral).Info("No async pod events to process")
 	}
 
 	p.recoveryRunning = false
@@ -148,7 +148,7 @@ func (p *PodEventHandler) addPod(pod *v1.Pod, eventSource EventSource) interface
 	if recovery && !appExists {
 		err := managedApp.TriggerAppRecovery()
 		if err != nil {
-			log.Logger().Error("failed to recover app", zap.Error(err))
+			log.Log(log.ShimAppMgmtGeneral).Error("failed to recover app", zap.Error(err))
 		}
 	}
 
