@@ -20,13 +20,13 @@ package cache
 
 import (
 	"fmt"
-	"reflect"
 	"sort"
 	"strings"
 	"sync"
 	"testing"
 	"time"
 
+	"github.com/google/go-cmp/cmp/cmpopts"
 	"gotest.tools/v3/assert"
 	is "gotest.tools/v3/assert/cmp"
 	v1 "k8s.io/api/core/v1"
@@ -34,7 +34,6 @@ import (
 	apis "k8s.io/apimachinery/pkg/apis/meta/v1"
 	k8sEvents "k8s.io/client-go/tools/events"
 
-	"github.com/apache/yunikorn-k8shim/pkg/apis/yunikorn.apache.org/v1alpha1"
 	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/general"
 	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
 	"github.com/apache/yunikorn-k8shim/pkg/client"
@@ -62,7 +61,7 @@ func TestNewApplication(t *testing.T) {
 	assert.Equal(t, len(app.taskMap), 0)
 	assert.Equal(t, app.GetApplicationState(), ApplicationStates().New)
 	assert.Equal(t, app.queue, "root.queue")
-	assert.Assert(t, reflect.DeepEqual(app.groups, []string{"dev", "yunikorn"}))
+	assert.DeepEqual(t, app.groups, []string{"dev", "yunikorn"})
 }
 
 func TestSubmitApplication(t *testing.T) {
@@ -595,24 +594,10 @@ func TestGetNonTerminatedTaskAlias(t *testing.T) {
 
 func TestSetTaskGroupsAndSchedulingPolicy(t *testing.T) {
 	app := NewApplication("app01", "root.a", "test-user", testGroups, map[string]string{}, newMockSchedulerAPI())
-	assert.Assert(t, app.getSchedulingPolicy().Type == "")
 	assert.Equal(t, len(app.getTaskGroups()), 0)
 
-	app.setSchedulingPolicy(v1alpha1.SchedulingPolicy{
-		Type: v1alpha1.TryReserve,
-		Parameters: map[string]string{
-			"option-1": "value-1",
-			"option-2": "value-2",
-		},
-	})
-
-	assert.Equal(t, app.getSchedulingPolicy().Type, v1alpha1.TryReserve)
-	assert.Equal(t, len(app.getSchedulingPolicy().Parameters), 2)
-	assert.Equal(t, app.getSchedulingPolicy().Parameters["option-1"], "value-1", "incorrect parameter value")
-	assert.Equal(t, app.getSchedulingPolicy().Parameters["option-2"], "value-2", "incorrect parameter value")
-
 	duration := int64(3000)
-	app.setTaskGroups([]v1alpha1.TaskGroup{
+	app.setTaskGroups([]interfaces.TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 10,
@@ -674,7 +659,7 @@ func TestSetTaskGroupsAndSchedulingPolicy(t *testing.T) {
 	// Pods should always be equal to the sum of the MinMember values
 	expectedPlaceholderAsk := common.NewResourceBuilder().AddResource("pods", 30).AddResource(siCommon.Memory, 25000*1024*1024).AddResource(siCommon.CPU, 25000).Build()
 	actualPlaceholderAsk := app.getPlaceholderAsk()
-	assert.DeepEqual(t, actualPlaceholderAsk, expectedPlaceholderAsk)
+	assert.DeepEqual(t, actualPlaceholderAsk, expectedPlaceholderAsk, cmpopts.IgnoreUnexported(si.Resource{}, si.Quantity{}))
 }
 
 type threadSafePodsMap struct {
@@ -722,17 +707,8 @@ func TestTryReserve(t *testing.T) {
 		testGroups, map[string]string{}, mockedAPIProvider.GetAPIs().SchedulerAPI)
 	context.addApplication(app)
 
-	// set app scheduling policy
-	app.setSchedulingPolicy(v1alpha1.SchedulingPolicy{
-		Type: v1alpha1.TryReserve,
-		Parameters: map[string]string{
-			"option-1": "value-1",
-			"option-2": "value-2",
-		},
-	})
-
 	// set taskGroups
-	app.setTaskGroups([]v1alpha1.TaskGroup{
+	app.setTaskGroups([]interfaces.TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 10,
@@ -798,7 +774,7 @@ func TestTryReservePostRestart(t *testing.T) {
 	context.addApplication(app)
 
 	// set taskGroups
-	app.setTaskGroups([]v1alpha1.TaskGroup{
+	app.setTaskGroups([]interfaces.TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 10,
@@ -967,7 +943,7 @@ func TestSkipReservationStage(t *testing.T) {
 	task2.sm.SetState(TaskStates().Allocated)
 	app.addTask(task1)
 	app.addTask(task2)
-	app.setTaskGroups([]v1alpha1.TaskGroup{
+	app.setTaskGroups([]interfaces.TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 10,
@@ -989,7 +965,7 @@ func TestSkipReservationStage(t *testing.T) {
 	task2.sm.SetState(TaskStates().New)
 	app.addTask(task1)
 	app.addTask(task2)
-	app.setTaskGroups([]v1alpha1.TaskGroup{
+	app.setTaskGroups([]interfaces.TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 10,
@@ -1278,7 +1254,7 @@ func TestApplication_onReservationStateChange(t *testing.T) {
 	assertAppState(t, app, ApplicationStates().Running, 1*time.Second)
 
 	// set taskGroups
-	app.setTaskGroups([]v1alpha1.TaskGroup{
+	app.setTaskGroups([]interfaces.TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 1,
