@@ -19,6 +19,7 @@
 package cache
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"strconv"
@@ -29,7 +30,7 @@ import (
 	v1 "k8s.io/api/core/v1"
 	schedulingv1 "k8s.io/api/scheduling/v1"
 	"k8s.io/client-go/tools/cache"
-	podutil "k8s.io/kubernetes/pkg/api/v1/pod"
+	"k8s.io/kubernetes/pkg/scheduler/framework"
 	"k8s.io/kubernetes/pkg/scheduler/framework/plugins/volumebinding"
 
 	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
@@ -107,18 +108,9 @@ func (ctx *Context) AddSchedulingEventHandlers() {
 
 	ctx.apiProvider.AddEventHandler(&client.ResourceEventHandlers{
 		Type:     client.PodInformerHandlers,
-		FilterFn: ctx.filterPods,
-		AddFn:    ctx.addPodToCache,
-		UpdateFn: ctx.updatePodInCache,
-		DeleteFn: ctx.removePodFromCache,
-	})
-
-	nodeCoordinator := newNodeResourceCoordinator(ctx.nodes)
-	ctx.apiProvider.AddEventHandler(&client.ResourceEventHandlers{
-		Type:     client.PodInformerHandlers,
-		FilterFn: nodeCoordinator.filterPods,
-		UpdateFn: nodeCoordinator.updatePod,
-		DeleteFn: nodeCoordinator.deletePod,
+		AddFn:    ctx.addPod,
+		UpdateFn: ctx.updatePod,
+		DeleteFn: ctx.deletePod,
 	})
 
 	ctx.apiProvider.AddEventHandler(&client.ResourceEventHandlers{
@@ -141,19 +133,18 @@ func (ctx *Context) IsPluginMode() bool {
 	return ctx.pluginMode
 }
 
-func (ctx *Context) SetPluginMode(pluginMode bool) {
-	ctx.pluginMode = pluginMode
-}
-
 func (ctx *Context) addNode(obj interface{}) {
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+
 	node, err := convertToNode(obj)
 	if err != nil {
-		log.Logger().Error("node conversion failed", zap.Error(err))
+		log.Log(log.ShimContext).Error("node conversion failed", zap.Error(err))
 		return
 	}
 
 	// add node to secondary scheduler cache
-	log.Logger().Warn("adding node to cache", zap.String("NodeName", node.Name))
+	log.Log(log.ShimContext).Warn("adding node to cache", zap.String("NodeName", node.Name))
 	ctx.schedulerCache.AddNode(node)
 
 	// add node to internal cache
@@ -165,17 +156,20 @@ func (ctx *Context) addNode(obj interface{}) {
 }
 
 func (ctx *Context) updateNode(oldObj, newObj interface{}) {
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+
 	// we only trigger update when resource changes
 	oldNode, err := convertToNode(oldObj)
 	if err != nil {
-		log.Logger().Error("old node conversion failed",
+		log.Log(log.ShimContext).Error("old node conversion failed",
 			zap.Error(err))
 		return
 	}
 
 	newNode, err := convertToNode(newObj)
 	if err != nil {
-		log.Logger().Error("new node conversion failed",
+		log.Log(log.ShimContext).Error("new node conversion failed",
 			zap.Error(err))
 		return
 	}
@@ -188,6 +182,9 @@ func (ctx *Context) updateNode(oldObj, newObj interface{}) {
 }
 
 func (ctx *Context) deleteNode(obj interface{}) {
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+
 	var node *v1.Node
 	switch t := obj.(type) {
 	case *v1.Node:
@@ -196,16 +193,16 @@ func (ctx *Context) deleteNode(obj interface{}) {
 		var ok bool
 		node, ok = t.Obj.(*v1.Node)
 		if !ok {
-			log.Logger().Error("cannot convert to *v1.Node", zap.Any("object", t.Obj))
+			log.Log(log.ShimContext).Error("cannot convert to *v1.Node", zap.Any("object", t.Obj))
 			return
 		}
 	default:
-		log.Logger().Error("cannot convert to *v1.Node", zap.Any("object", t))
+		log.Log(log.ShimContext).Error("cannot convert to *v1.Node", zap.Any("object", t))
 		return
 	}
 
 	// delete node from secondary cache
-	log.Logger().Debug("delete node from cache", zap.String("nodeName", node.Name))
+	log.Log(log.ShimContext).Debug("delete node from cache", zap.String("nodeName", node.Name))
 	ctx.schedulerCache.RemoveNode(node)
 
 	// delete node from primary cache
@@ -216,25 +213,90 @@ func (ctx *Context) deleteNode(obj interface{}) {
 		fmt.Sprintf("node %s is deleted from the scheduler", node.Name))
 }
 
-func (ctx *Context) addPodToCache(obj interface{}) {
+func (ctx *Context) addPod(obj interface{}) {
 	pod, err := utils.Convert2Pod(obj)
 	if err != nil {
-		log.Logger().Error("failed to add pod to cache", zap.Error(err))
+		log.Log(log.ShimContext).Error("failed to add pod", zap.Error(err))
 		return
 	}
+	if utils.GetApplicationIDFromPod(pod) == "" {
+		ctx.updateForeignPod(pod)
+	} else {
+		ctx.updateYuniKornPod(pod)
+	}
+}
 
-	// treat a terminated pod like a removal
+func (ctx *Context) updatePod(_, newObj interface{}) {
+	pod, err := utils.Convert2Pod(newObj)
+	if err != nil {
+		log.Log(log.ShimContext).Error("failed to update pod", zap.Error(err))
+		return
+	}
+	if utils.GetApplicationIDFromPod(pod) == "" {
+		ctx.updateForeignPod(pod)
+	} else {
+		ctx.updateYuniKornPod(pod)
+	}
+}
+
+func (ctx *Context) updateYuniKornPod(pod *v1.Pod) {
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+
+	// treat terminated pods like a remove
 	if utils.IsPodTerminated(pod) {
-		log.Logger().Debug("Request to add terminated pod, removing from cache", zap.String("podName", pod.Name))
+		log.Log(log.ShimContext).Debug("Request to update terminated pod, removing from cache", zap.String("podName", pod.Name))
 		ctx.schedulerCache.RemovePod(pod)
 		return
 	}
-
-	log.Logger().Debug("adding pod to cache", zap.String("podName", pod.Name))
-	ctx.schedulerCache.AddPod(pod)
+	ctx.schedulerCache.UpdatePod(pod)
 }
 
-func (ctx *Context) removePodFromCache(obj interface{}) {
+func (ctx *Context) updateForeignPod(pod *v1.Pod) {
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+
+	podStatusBefore := ""
+	oldPod, ok := ctx.schedulerCache.GetPod(string(pod.UID))
+	if ok {
+		podStatusBefore = string(oldPod.Status.Phase)
+	}
+
+	// conditions for allocate:
+	//   1. pod was previously assigned
+	//   2. pod is now assigned
+	//   3. pod is not in terminated state
+	if oldPod == nil && utils.IsAssignedPod(pod) && !utils.IsPodTerminated(pod) {
+		log.Log(log.ShimContext).Debug("pod is assigned to a node, trigger occupied resource update",
+			zap.String("namespace", pod.Namespace),
+			zap.String("podName", pod.Name),
+			zap.String("podStatusBefore", podStatusBefore),
+			zap.String("podStatusCurrent", string(pod.Status.Phase)))
+		podResource := common.GetPodResource(pod)
+		ctx.nodes.updateNodeOccupiedResources(pod.Spec.NodeName, podResource, AddOccupiedResource)
+		ctx.schedulerCache.AddPod(pod)
+		return
+	}
+
+	// conditions for release:
+	//   1. pod was previously assigned
+	//   2. pod is now in a terminated state
+	if oldPod != nil && utils.IsPodTerminated(pod) {
+		log.Log(log.ShimContext).Debug("pod terminated, trigger occupied resource update",
+			zap.String("namespace", pod.Namespace),
+			zap.String("podName", pod.Name),
+			zap.String("podStatusBefore", podStatusBefore),
+			zap.String("podStatusCurrent", string(pod.Status.Phase)))
+		// this means pod is terminated
+		// we need sub the occupied resource and re-sync with the scheduler-core
+		podResource := common.GetPodResource(pod)
+		ctx.nodes.updateNodeOccupiedResources(pod.Spec.NodeName, podResource, SubOccupiedResource)
+		ctx.schedulerCache.RemovePod(pod)
+		return
+	}
+}
+
+func (ctx *Context) deletePod(obj interface{}) {
 	var pod *v1.Pod
 	switch t := obj.(type) {
 	case *v1.Pod:
@@ -243,47 +305,55 @@ func (ctx *Context) removePodFromCache(obj interface{}) {
 		var ok bool
 		pod, ok = t.Obj.(*v1.Pod)
 		if !ok {
-			log.Logger().Error("Cannot convert to *v1.Pod", zap.Any("pod", obj))
+			log.Log(log.ShimContext).Error("Cannot convert to *v1.Pod", zap.Any("pod", obj))
 			return
 		}
 	default:
-		log.Logger().Error("Cannot convert to *v1.Pod", zap.Any("pod", obj))
+		log.Log(log.ShimContext).Error("Cannot convert to *v1.Pod", zap.Any("pod", obj))
 		return
 	}
 
-	log.Logger().Debug("removing pod from cache", zap.String("podName", pod.Name))
+	if utils.GetApplicationIDFromPod(pod) == "" {
+		ctx.deleteForeignPod(pod)
+	} else {
+		ctx.deleteYuniKornPod(pod)
+	}
+}
+
+func (ctx *Context) deleteYuniKornPod(pod *v1.Pod) {
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+
+	log.Log(log.ShimContext).Debug("removing pod from cache", zap.String("podName", pod.Name))
 	ctx.schedulerCache.RemovePod(pod)
 }
 
-func (ctx *Context) updatePodInCache(oldObj, newObj interface{}) {
-	_, err := utils.Convert2Pod(oldObj)
-	if err != nil {
-		log.Logger().Error("failed to update pod in cache", zap.Error(err))
-		return
-	}
-	newPod, err := utils.Convert2Pod(newObj)
-	if err != nil {
-		log.Logger().Error("failed to update pod in cache", zap.Error(err))
+func (ctx *Context) deleteForeignPod(pod *v1.Pod) {
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+
+	oldPod, ok := ctx.schedulerCache.GetPod(string(pod.UID))
+	if !ok {
+		// if pod is not in scheduler cache, no node updates are needed
+		log.Log(log.ShimContext).Debug("unknown foreign pod deleted, no resource updated needed",
+			zap.String("namespace", pod.Namespace),
+			zap.String("podName", pod.Name))
 		return
 	}
 
-	// treat terminated pods like a remove
-	if utils.IsPodTerminated(newPod) {
-		log.Logger().Debug("Request to update terminated pod, removing from cache", zap.String("podName", newPod.Name))
-		ctx.schedulerCache.RemovePod(newPod)
-		return
-	}
-
-	ctx.schedulerCache.UpdatePod(newPod)
-}
-
-// filter pods by scheduler name and state
-func (ctx *Context) filterPods(obj interface{}) bool {
-	switch obj := obj.(type) {
-	case *v1.Pod:
-		return utils.GetApplicationIDFromPod(obj) != ""
-	default:
-		return false
+	// conditions for release:
+	//   1. pod is already assigned to a node
+	if oldPod != nil {
+		log.Log(log.ShimContext).Debug("foreign pod deleted, triggering occupied resource update",
+			zap.String("namespace", pod.Namespace),
+			zap.String("podName", pod.Name),
+			zap.String("podStatusBefore", string(oldPod.Status.Phase)),
+			zap.String("podStatusCurrent", string(pod.Status.Phase)))
+		// this means pod is terminated
+		// we need sub the occupied resource and re-sync with the scheduler-core
+		podResource := common.GetPodResource(pod)
+		ctx.nodes.updateNodeOccupiedResources(pod.Spec.NodeName, podResource, SubOccupiedResource)
+		ctx.schedulerCache.RemovePod(pod)
 	}
 }
 
@@ -301,7 +371,7 @@ func (ctx *Context) filterConfigMaps(obj interface{}) bool {
 
 // when the configMap for the scheduler is added, trigger hot-refresh
 func (ctx *Context) addConfigMaps(obj interface{}) {
-	log.Logger().Debug("configMap added")
+	log.Log(log.ShimContext).Debug("configMap added")
 	configmap := utils.Convert2ConfigMap(obj)
 	switch configmap.Name {
 	case constants.DefaultConfigMapName:
@@ -316,7 +386,7 @@ func (ctx *Context) addConfigMaps(obj interface{}) {
 
 // when the configMap for the scheduler is updated, trigger hot-refresh
 func (ctx *Context) updateConfigMaps(_, newObj interface{}) {
-	log.Logger().Debug("configMap updated")
+	log.Log(log.ShimContext).Debug("configMap updated")
 	configmap := utils.Convert2ConfigMap(newObj)
 	switch configmap.Name {
 	case constants.DefaultConfigMapName:
@@ -331,7 +401,7 @@ func (ctx *Context) updateConfigMaps(_, newObj interface{}) {
 
 // when the configMap for the scheduler is deleted, trigger refresh using default config
 func (ctx *Context) deleteConfigMaps(obj interface{}) {
-	log.Logger().Debug("configMap deleted")
+	log.Log(log.ShimContext).Debug("configMap deleted")
 	var configmap *v1.ConfigMap = nil
 	switch t := obj.(type) {
 	case *v1.ConfigMap:
@@ -339,7 +409,7 @@ func (ctx *Context) deleteConfigMaps(obj interface{}) {
 	case cache.DeletedFinalStateUnknown:
 		configmap = utils.Convert2ConfigMap(obj)
 	default:
-		log.Logger().Warn("unable to convert to configmap")
+		log.Log(log.ShimContext).Warn("unable to convert to configmap")
 		return
 	}
 
@@ -366,7 +436,10 @@ func (ctx *Context) filterPriorityClasses(obj interface{}) bool {
 }
 
 func (ctx *Context) addPriorityClass(obj interface{}) {
-	log.Logger().Debug("priority class added")
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+
+	log.Log(log.ShimContext).Debug("priority class added")
 	priorityClass := utils.Convert2PriorityClass(obj)
 	if priorityClass != nil {
 		ctx.schedulerCache.AddPriorityClass(priorityClass)
@@ -374,7 +447,10 @@ func (ctx *Context) addPriorityClass(obj interface{}) {
 }
 
 func (ctx *Context) updatePriorityClass(_, newObj interface{}) {
-	log.Logger().Debug("priority class updated")
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+
+	log.Log(log.ShimContext).Debug("priority class updated")
 	priorityClass := utils.Convert2PriorityClass(newObj)
 	if priorityClass != nil {
 		ctx.schedulerCache.UpdatePriorityClass(priorityClass)
@@ -382,7 +458,10 @@ func (ctx *Context) updatePriorityClass(_, newObj interface{}) {
 }
 
 func (ctx *Context) deletePriorityClass(obj interface{}) {
-	log.Logger().Debug("priorityClass deleted")
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+
+	log.Log(log.ShimContext).Debug("priorityClass deleted")
 	var priorityClass *schedulingv1.PriorityClass
 	switch t := obj.(type) {
 	case *schedulingv1.PriorityClass:
@@ -390,7 +469,7 @@ func (ctx *Context) deletePriorityClass(obj interface{}) {
 	case cache.DeletedFinalStateUnknown:
 		priorityClass = utils.Convert2PriorityClass(obj)
 	default:
-		log.Logger().Warn("unable to convert to priorityClass")
+		log.Log(log.ShimContext).Warn("unable to convert to priorityClass")
 		return
 	}
 	if priorityClass != nil {
@@ -404,21 +483,21 @@ func (ctx *Context) triggerReloadConfig(index int, configMap *v1.ConfigMap) {
 
 	conf := ctx.apiProvider.GetAPIs().GetConf()
 	if !conf.EnableConfigHotRefresh {
-		log.Logger().Info("hot-refresh disabled, skipping scheduler configuration update")
+		log.Log(log.ShimContext).Info("hot-refresh disabled, skipping scheduler configuration update")
 		return
 	}
 
 	ctx.configMaps[index] = configMap
 	err := schedulerconf.UpdateConfigMaps(ctx.configMaps, false)
 	if err != nil {
-		log.Logger().Error("Unable to update configmap, ignoring changes", zap.Error(err))
+		log.Log(log.ShimContext).Error("Unable to update configmap, ignoring changes", zap.Error(err))
 		return
 	}
 
 	confMap := schedulerconf.FlattenConfigMaps(ctx.configMaps)
 
 	conf = ctx.apiProvider.GetAPIs().GetConf()
-	log.Logger().Info("reloading scheduler configuration")
+	log.Log(log.ShimContext).Info("reloading scheduler configuration")
 	config := utils.GetCoreSchedulerConfigFromConfigMap(confMap)
 	extraConfig := utils.GetExtraConfigFromConfigMap(confMap)
 
@@ -429,10 +508,15 @@ func (ctx *Context) triggerReloadConfig(index int, configMap *v1.ConfigMap) {
 		ExtraConfig: extraConfig,
 	}
 	if err := ctx.apiProvider.GetAPIs().SchedulerAPI.UpdateConfiguration(request); err != nil {
-		log.Logger().Error("reload configuration failed", zap.Error(err))
+		log.Log(log.ShimContext).Error("reload configuration failed", zap.Error(err))
 	}
 }
 
+// EventsToRegister returns the Kubernetes events that should be watched for updates which may effect predicate processing
+func (ctx *Context) EventsToRegister() []framework.ClusterEvent {
+	return ctx.predManager.EventsToRegister()
+}
+
 // evaluate given predicates based on current context
 func (ctx *Context) IsPodFitNode(name, node string, allocate bool) error {
 	ctx.lock.RLock()
@@ -468,7 +552,7 @@ func (ctx *Context) IsPodFitNodeViaPreemption(name, node string, allocations []s
 			// look up each victim in the scheduler cache
 			victims := make([]*v1.Pod, 0)
 			for _, uid := range allocations {
-				if victim, ok := ctx.schedulerCache.GetPod(uid); ok {
+				if victim, ok := ctx.schedulerCache.GetPodNoLock(uid); ok {
 					victims = append(victims, victim)
 				} else {
 					// if pod isn't found, add a placeholder so that the list is still the same size
@@ -496,38 +580,36 @@ func (ctx *Context) bindPodVolumes(pod *v1.Pod) error {
 	// then here we just need to retrieve that value from cache, to skip bindings if volumes are already bound.
 	if assumedPod, exist := ctx.schedulerCache.GetPod(podKey); exist {
 		if ctx.schedulerCache.ArePodVolumesAllBound(podKey) {
-			log.Logger().Info("Binding Pod Volumes skipped: all volumes already bound",
+			log.Log(log.ShimContext).Info("Binding Pod Volumes skipped: all volumes already bound",
 				zap.String("podName", pod.Name))
 		} else {
-			log.Logger().Info("Binding Pod Volumes", zap.String("podName", pod.Name))
-			boundClaims, claimsToBind, unboundClaimsImmediate, err := ctx.apiProvider.GetAPIs().VolumeBinder.GetPodVolumes(assumedPod)
+			log.Log(log.ShimContext).Info("Binding Pod Volumes", zap.String("podName", pod.Name))
+
+			// retrieve the volume claims
+			podVolumeClaims, err := ctx.apiProvider.GetAPIs().VolumeBinder.GetPodVolumeClaims(pod)
 			if err != nil {
-				log.Logger().Error("Failed to get pod volumes",
-					zap.String("podName", assumedPod.Name),
-					zap.Error(err))
-				return err
-			}
-			if len(unboundClaimsImmediate) > 0 {
-				err = fmt.Errorf("pod %s has unbound immediate claims", pod.Name)
-				log.Logger().Error("Pod has unbound immediate claims",
+				log.Log(log.ShimContext).Error("Failed to get pod volume claims",
 					zap.String("podName", assumedPod.Name),
 					zap.Error(err))
 				return err
 			}
+
+			// get node information
 			node, err := ctx.schedulerCache.GetNodeInfo(assumedPod.Spec.NodeName)
 			if err != nil {
-				log.Logger().Error("Failed to get node info",
+				log.Log(log.ShimContext).Error("Failed to get node info",
 					zap.String("podName", assumedPod.Name),
 					zap.String("nodeName", assumedPod.Spec.NodeName),
 					zap.Error(err))
 				return err
 			}
-			volumes, reasons, err := ctx.apiProvider.GetAPIs().VolumeBinder.FindPodVolumes(assumedPod, boundClaims, claimsToBind, node)
+
+			// retrieve volumes
+			volumes, reasons, err := ctx.apiProvider.GetAPIs().VolumeBinder.FindPodVolumes(pod, podVolumeClaims, node)
 			if err != nil {
-				log.Logger().Error("Failed to find pod volumes",
+				log.Log(log.ShimContext).Error("Failed to find pod volumes",
 					zap.String("podName", assumedPod.Name),
 					zap.String("nodeName", assumedPod.Spec.NodeName),
-					zap.Int("claimsToBind", len(claimsToBind)),
 					zap.Error(err))
 				return err
 			}
@@ -538,10 +620,9 @@ func (ctx *Context) bindPodVolumes(pod *v1.Pod) error {
 				}
 				sReason := strings.Join(sReasons, ", ")
 				err = fmt.Errorf("pod %s has conflicting volume claims: %s", pod.Name, sReason)
-				log.Logger().Error("Pod has conflicting volume claims",
+				log.Log(log.ShimContext).Error("Pod has conflicting volume claims",
 					zap.String("podName", assumedPod.Name),
 					zap.String("nodeName", assumedPod.Spec.NodeName),
-					zap.Int("claimsToBind", len(claimsToBind)),
 					zap.Error(err))
 				return err
 			}
@@ -553,9 +634,9 @@ func (ctx *Context) bindPodVolumes(pod *v1.Pod) error {
 				// convert nil to empty array
 				volumes.DynamicProvisions = make([]*v1.PersistentVolumeClaim, 0)
 			}
-			err = ctx.apiProvider.GetAPIs().VolumeBinder.BindPodVolumes(assumedPod, volumes)
+			err = ctx.apiProvider.GetAPIs().VolumeBinder.BindPodVolumes(context.Background(), assumedPod, volumes)
 			if err != nil {
-				log.Logger().Error("Failed to bind pod volumes",
+				log.Log(log.ShimContext).Error("Failed to bind pod volumes",
 					zap.String("podName", assumedPod.Name),
 					zap.String("nodeName", assumedPod.Spec.NodeName),
 					zap.Int("dynamicProvisions", len(volumes.DynamicProvisions)),
@@ -588,12 +669,35 @@ func (ctx *Context) AssumePod(name string, node string) error {
 			// volume builder might be null in UTs
 			if ctx.apiProvider.GetAPIs().VolumeBinder != nil {
 				var err error
-				boundClaims, claimsToBind, _, err := ctx.apiProvider.GetAPIs().VolumeBinder.GetPodVolumes(assumedPod)
+				// retrieve the volume claims
+				podVolumeClaims, err := ctx.apiProvider.GetAPIs().VolumeBinder.GetPodVolumeClaims(pod)
 				if err != nil {
+					log.Log(log.ShimContext).Error("Failed to get pod volume claims",
+						zap.String("podName", assumedPod.Name),
+						zap.Error(err))
 					return err
 				}
-				volumes, _, err := ctx.apiProvider.GetAPIs().VolumeBinder.FindPodVolumes(pod, boundClaims, claimsToBind, targetNode.Node())
+
+				// retrieve volumes
+				volumes, reasons, err := ctx.apiProvider.GetAPIs().VolumeBinder.FindPodVolumes(pod, podVolumeClaims, targetNode.Node())
 				if err != nil {
+					log.Log(log.ShimContext).Error("Failed to find pod volumes",
+						zap.String("podName", assumedPod.Name),
+						zap.String("nodeName", assumedPod.Spec.NodeName),
+						zap.Error(err))
+					return err
+				}
+				if len(reasons) > 0 {
+					sReasons := make([]string, 0)
+					for _, reason := range reasons {
+						sReasons = append(sReasons, string(reason))
+					}
+					sReason := strings.Join(sReasons, ", ")
+					err = fmt.Errorf("pod %s has conflicting volume claims: %s", pod.Name, sReason)
+					log.Log(log.ShimContext).Error("Pod has conflicting volume claims",
+						zap.String("podName", assumedPod.Name),
+						zap.String("nodeName", assumedPod.Spec.NodeName),
+						zap.Error(err))
 					return err
 				}
 				allBound, err = ctx.apiProvider.GetAPIs().VolumeBinder.AssumePodVolumes(pod, node, volumes)
@@ -617,11 +721,11 @@ func (ctx *Context) ForgetPod(name string) {
 	defer ctx.lock.Unlock()
 
 	if pod, ok := ctx.schedulerCache.GetPod(name); ok {
-		log.Logger().Debug("forget pod", zap.String("pod", pod.Name))
+		log.Log(log.ShimContext).Debug("forget pod", zap.String("pod", pod.Name))
 		ctx.schedulerCache.ForgetPod(pod)
 		return
 	}
-	log.Logger().Debug("unable to forget pod: not found in cache", zap.String("pod", name))
+	log.Log(log.ShimContext).Debug("unable to forget pod: not found in cache", zap.String("pod", name))
 }
 
 func (ctx *Context) UpdateApplication(app *Application) {
@@ -657,7 +761,7 @@ func (ctx *Context) StartPodAllocation(podKey string, nodeID string) bool {
 // either way we need to release all allocations (if exists) for this application
 func (ctx *Context) NotifyApplicationComplete(appID string) {
 	if app := ctx.GetApplication(appID); app != nil {
-		log.Logger().Debug("NotifyApplicationComplete",
+		log.Log(log.ShimContext).Debug("NotifyApplicationComplete",
 			zap.String("appID", appID),
 			zap.String("currentAppState", app.GetApplicationState()))
 		ev := NewSimpleApplicationEvent(appID, CompleteApplication)
@@ -667,7 +771,7 @@ func (ctx *Context) NotifyApplicationComplete(appID string) {
 
 func (ctx *Context) NotifyApplicationFail(appID string) {
 	if app := ctx.GetApplication(appID); app != nil {
-		log.Logger().Debug("NotifyApplicationFail",
+		log.Log(log.ShimContext).Debug("NotifyApplicationFail",
 			zap.String("appID", appID),
 			zap.String("currentAppState", app.GetApplicationState()))
 		ev := NewSimpleApplicationEvent(appID, FailApplication)
@@ -676,11 +780,11 @@ func (ctx *Context) NotifyApplicationFail(appID string) {
 }
 
 func (ctx *Context) NotifyTaskComplete(appID, taskID string) {
-	log.Logger().Debug("NotifyTaskComplete",
+	log.Log(log.ShimContext).Debug("NotifyTaskComplete",
 		zap.String("appID", appID),
 		zap.String("taskID", taskID))
 	if app := ctx.GetApplication(appID); app != nil {
-		log.Logger().Debug("release allocation",
+		log.Log(log.ShimContext).Debug("release allocation",
 			zap.String("appID", appID),
 			zap.String("taskID", taskID))
 		ev := NewSimpleTaskEvent(appID, taskID, CompleteTask)
@@ -702,10 +806,19 @@ func (ctx *Context) updateApplicationTags(request *interfaces.AddApplicationRequ
 	// add resource quota info as an app tag
 	resourceQuota := utils.GetNamespaceQuotaFromAnnotation(namespaceObj)
 	if resourceQuota != nil && !common.IsZero(resourceQuota) {
-		if quotaStr, err := json.Marshal(resourceQuota); err == nil {
-			request.Metadata.Tags[siCommon.AppTagNamespaceResourceQuota] = string(quotaStr)
+		if quota, err := json.Marshal(resourceQuota); err == nil {
+			request.Metadata.Tags[siCommon.AppTagNamespaceResourceQuota] = string(quota)
 		}
 	}
+
+	// add guaranteed resource info as an app tag
+	guaranteedResource := utils.GetNamespaceGuaranteedFromAnnotation(namespaceObj)
+	if guaranteedResource != nil && !common.IsZero(guaranteedResource) {
+		if guaranteed, err := json.Marshal(guaranteedResource); err == nil {
+			request.Metadata.Tags[siCommon.AppTagNamespaceResourceGuaranteed] = string(guaranteed)
+		}
+	}
+
 	// add parent queue info as an app tag
 	parentQueue := utils.GetNameSpaceAnnotationValue(namespaceObj, constants.AnnotationParentQueue)
 	if parentQueue != "" {
@@ -717,7 +830,7 @@ func (ctx *Context) updateApplicationTags(request *interfaces.AddApplicationRequ
 // if the namespace is unable to be listed from api-server, a nil is returned
 func (ctx *Context) getNamespaceObject(namespace string) *v1.Namespace {
 	if namespace == "" {
-		log.Logger().Debug("could not get namespace from empty string")
+		log.Log(log.ShimContext).Debug("could not get namespace from empty string")
 		return nil
 	}
 
@@ -727,14 +840,14 @@ func (ctx *Context) getNamespaceObject(namespace string) *v1.Namespace {
 		// every app should belong to a namespace,
 		// if we cannot list the namespace here, probably something is wrong
 		// log an error here and skip retrieving the resource quota
-		log.Logger().Error("failed to get app namespace", zap.Error(err))
+		log.Log(log.ShimContext).Error("failed to get app namespace", zap.Error(err))
 		return nil
 	}
 	return namespaceObj
 }
 
 func (ctx *Context) AddApplication(request *interfaces.AddApplicationRequest) interfaces.ManagedApp {
-	log.Logger().Debug("AddApplication", zap.Any("Request", request))
+	log.Log(log.ShimContext).Debug("AddApplication", zap.Any("Request", request))
 	if app := ctx.GetApplication(request.Metadata.ApplicationID); app != nil {
 		return app
 	}
@@ -743,7 +856,7 @@ func (ctx *Context) AddApplication(request *interfaces.AddApplicationRequest) in
 	defer ctx.lock.Unlock()
 
 	if ns, ok := request.Metadata.Tags[constants.AppTagNamespace]; ok {
-		log.Logger().Debug("app namespace info",
+		log.Log(log.ShimContext).Debug("app namespace info",
 			zap.String("appID", request.Metadata.ApplicationID),
 			zap.String("namespace", ns))
 		ctx.updateApplicationTags(request, ns)
@@ -770,7 +883,7 @@ func (ctx *Context) AddApplication(request *interfaces.AddApplicationRequest) in
 
 	// add into cache
 	ctx.applications[app.applicationID] = app
-	log.Logger().Info("app added",
+	log.Log(log.ShimContext).Info("app added",
 		zap.String("appID", app.applicationID))
 
 	return app
@@ -814,11 +927,11 @@ func (ctx *Context) RemoveApplication(appID string) error {
 		}
 		// send the update request to scheduler core
 		rr := common.CreateUpdateRequestForRemoveApplication(app.applicationID, app.partition)
-		if err := ctx.apiProvider.GetAPIs().SchedulerAPI.UpdateApplication(&rr); err != nil {
-			log.Logger().Error("failed to send remove application request to core", zap.Error(err))
+		if err := ctx.apiProvider.GetAPIs().SchedulerAPI.UpdateApplication(rr); err != nil {
+			log.Log(log.ShimContext).Error("failed to send remove application request to core", zap.Error(err))
 		}
 		delete(ctx.applications, appID)
-		log.Logger().Info("app removed",
+		log.Log(log.ShimContext).Info("app removed",
 			zap.String("appID", appID))
 
 		return nil
@@ -830,7 +943,7 @@ func (ctx *Context) RemoveApplicationInternal(appID string) {
 	ctx.lock.Lock()
 	defer ctx.lock.Unlock()
 	if _, exist := ctx.applications[appID]; !exist {
-		log.Logger().Debug("Attempted to remove non-existent application", zap.String("appID", appID))
+		log.Log(log.ShimContext).Debug("Attempted to remove non-existent application", zap.String("appID", appID))
 		return
 	}
 	delete(ctx.applications, appID)
@@ -838,7 +951,7 @@ func (ctx *Context) RemoveApplicationInternal(appID string) {
 
 // this implements ApplicationManagementProtocol
 func (ctx *Context) AddTask(request *interfaces.AddTaskRequest) interfaces.ManagedTask {
-	log.Logger().Debug("AddTask",
+	log.Log(log.ShimContext).Debug("AddTask",
 		zap.String("appID", request.Metadata.ApplicationID),
 		zap.String("taskID", request.Metadata.TaskID))
 	if managedApp := ctx.GetApplication(request.Metadata.ApplicationID); managedApp != nil {
@@ -860,17 +973,17 @@ func (ctx *Context) AddTask(request *interfaces.AddTaskRequest) interfaces.Manag
 				}
 				task := NewFromTaskMeta(request.Metadata.TaskID, app, ctx, request.Metadata, originator)
 				app.addTask(task)
-				log.Logger().Info("task added",
+				log.Log(log.ShimContext).Info("task added",
 					zap.String("appID", app.applicationID),
 					zap.String("taskID", task.taskID),
 					zap.String("taskState", task.GetTaskState()))
 				if originator {
 					if app.GetOriginatingTask() != nil {
-						log.Logger().Error("Inconsistent state - found another originator task for an application",
+						log.Log(log.ShimContext).Error("Inconsistent state - found another originator task for an application",
 							zap.String("taskId", task.GetTaskID()))
 					}
 					app.setOriginatingTask(task)
-					log.Logger().Info("app request originating pod added",
+					log.Log(log.ShimContext).Info("app request originating pod added",
 						zap.String("appID", app.applicationID),
 						zap.String("original task", task.GetTaskID()))
 				}
@@ -887,7 +1000,7 @@ func (ctx *Context) RemoveTask(appID, taskID string) {
 	defer ctx.lock.RUnlock()
 	app, ok := ctx.applications[appID]
 	if !ok {
-		log.Logger().Debug("Attempted to remove task from non-existent application", zap.String("appID", appID))
+		log.Log(log.ShimContext).Debug("Attempted to remove task from non-existent application", zap.String("appID", appID))
 		return
 	}
 	app.removeTask(taskID)
@@ -898,20 +1011,20 @@ func (ctx *Context) getTask(appID string, taskID string) *Task {
 	defer ctx.lock.RUnlock()
 	app := ctx.getApplication(appID)
 	if app == nil {
-		log.Logger().Debug("application is not found in the context",
+		log.Log(log.ShimContext).Debug("application is not found in the context",
 			zap.String("appID", appID))
 		return nil
 	}
 	managedTask, err := app.GetTask(taskID)
 	if err != nil {
-		log.Logger().Debug("task is not found in applications",
+		log.Log(log.ShimContext).Debug("task is not found in applications",
 			zap.String("taskID", taskID),
 			zap.String("appID", appID))
 		return nil
 	}
 	task, valid := managedTask.(*Task)
 	if !valid {
-		log.Logger().Debug("managedTask conversion failed",
+		log.Log(log.ShimContext).Debug("managedTask conversion failed",
 			zap.String("taskID", taskID))
 		return nil
 	}
@@ -935,38 +1048,38 @@ func (ctx *Context) PublishEvents(eventRecords []*si.EventRecord) {
 		for _, record := range eventRecords {
 			switch record.Type {
 			case si.EventRecord_REQUEST:
-				taskID := record.ObjectID
-				appID := record.GroupID
+				appID := record.ObjectID
+				taskID := record.ReferenceID
 				if task := ctx.getTask(appID, taskID); task != nil {
 					events.GetRecorder().Eventf(task.GetTaskPod().DeepCopy(), nil,
-						v1.EventTypeNormal, record.Reason, record.Reason, record.Message)
+						v1.EventTypeNormal, "", "", record.Message)
 				} else {
-					log.Logger().Warn("task event is not published because task is not found",
+					log.Log(log.ShimContext).Warn("task event is not published because task is not found",
 						zap.String("appID", appID),
 						zap.String("taskID", taskID),
 						zap.Stringer("event", record))
 				}
 			case si.EventRecord_NODE:
+				if !isPublishableNodeEvent(record) {
+					continue
+				}
 				nodeID := record.ObjectID
 				nodeInfo := ctx.schedulerCache.GetNode(nodeID)
 				if nodeInfo == nil {
-					log.Logger().Warn("node event is not published because nodeInfo is not found",
+					log.Log(log.ShimContext).Warn("node event is not published because nodeInfo is not found",
 						zap.String("nodeID", nodeID),
 						zap.Stringer("event", record))
 					continue
 				}
 				node := nodeInfo.Node()
 				if node == nil {
-					log.Logger().Warn("node event is not published because node is not found",
+					log.Log(log.ShimContext).Warn("node event is not published because node is not found",
 						zap.String("nodeID", nodeID),
 						zap.Stringer("event", record))
 					continue
 				}
 				events.GetRecorder().Eventf(node.DeepCopy(), nil,
-					v1.EventTypeNormal, record.Reason, record.Reason, record.Message)
-			default:
-				log.Logger().Warn("Unsupported event type, currently only supports to publish request event records",
-					zap.Stringer("type", record.Type))
+					v1.EventTypeNormal, "", "", record.Message)
 			}
 		}
 	}
@@ -978,26 +1091,17 @@ func (ctx *Context) updatePodCondition(task *Task, podCondition *v1.PodCondition
 	if task.GetTaskState() == TaskStates().Scheduling {
 		// only update the pod when pod condition changes
 		// minimize the overhead added to the api-server/etcd
-		if !utils.PodUnderCondition(task.pod, podCondition) {
-			log.Logger().Debug("updating pod condition",
-				zap.String("namespace", task.pod.Namespace),
-				zap.String("name", task.pod.Name),
-				zap.Any("podCondition", podCondition))
-			// call api-server to do the pod condition update
-			if podutil.UpdatePodCondition(&task.pod.Status, podCondition) {
-				if !ctx.apiProvider.IsTestingMode() {
-					podCopy := task.pod.DeepCopy()
-					_, err := ctx.apiProvider.GetAPIs().KubeClient.UpdateStatus(podCopy)
-					if err == nil {
-						return true
-					}
-					// only log the error here, no need to handle it if the update failed
-					log.Logger().Error("update pod condition failed",
-						zap.Error(err))
-				}
+		if ok, podCopy := task.UpdatePodCondition(podCondition); ok {
+			_, err := ctx.apiProvider.GetAPIs().KubeClient.UpdateStatus(podCopy)
+			if err == nil {
+				return true
 			}
+			// only log the error here, no need to handle it if the update failed
+			log.Log(log.ShimContext).Error("update pod condition failed",
+				zap.Error(err))
 		}
 	}
+
 	return false
 }
 
@@ -1006,11 +1110,12 @@ func (ctx *Context) updatePodCondition(task *Task, podCondition *v1.PodCondition
 // pod condition in order to trigger auto-scaling.
 func (ctx *Context) HandleContainerStateUpdate(request *si.UpdateContainerSchedulingStateRequest) {
 	// the allocationKey equals to the taskID
-	if task := ctx.getTask(request.ApplicartionID, request.AllocationKey); task != nil {
+	if task := ctx.getTask(request.ApplicationID, request.AllocationKey); task != nil {
 		switch request.State {
 		case si.UpdateContainerSchedulingStateRequest_SKIPPED:
 			// auto-scaler scans pods whose pod condition is PodScheduled=false && reason=Unschedulable
 			// if the pod is skipped because the queue quota has been exceed, we do not trigger the auto-scaling
+			task.SetTaskSchedulingState(interfaces.TaskSchedSkipped)
 			if ctx.updatePodCondition(task,
 				&v1.PodCondition{
 					Type:    v1.PodScheduled,
@@ -1023,6 +1128,7 @@ func (ctx *Context) HandleContainerStateUpdate(request *si.UpdateContainerSchedu
 					"Task %s is skipped from scheduling because the queue quota has been exceed", task.alias)
 			}
 		case si.UpdateContainerSchedulingStateRequest_FAILED:
+			task.SetTaskSchedulingState(interfaces.TaskSchedFailed)
 			// set pod condition to Unschedulable in order to trigger auto-scaling
 			if ctx.updatePodCondition(task,
 				&v1.PodCondition{
@@ -1036,7 +1142,7 @@ func (ctx *Context) HandleContainerStateUpdate(request *si.UpdateContainerSchedu
 					"Task %s is pending for the requested resources become available", task.alias)
 			}
 		default:
-			log.Logger().Warn("no handler for container scheduling state",
+			log.Log(log.ShimContext).Warn("no handler for container scheduling state",
 				zap.Stringer("state", request.State))
 		}
 	}
@@ -1047,7 +1153,7 @@ func (ctx *Context) ApplicationEventHandler() func(obj interface{}) {
 		if event, ok := obj.(events.ApplicationEvent); ok {
 			managedApp := ctx.GetApplication(event.GetApplicationID())
 			if managedApp == nil {
-				log.Logger().Error("failed to handle application event",
+				log.Log(log.ShimContext).Error("failed to handle application event",
 					zap.String("reason", "application not exist"))
 				return
 			}
@@ -1055,7 +1161,7 @@ func (ctx *Context) ApplicationEventHandler() func(obj interface{}) {
 			if app, ok := managedApp.(*Application); ok {
 				if app.canHandle(event) {
 					if err := app.handle(event); err != nil {
-						log.Logger().Error("failed to handle application event",
+						log.Log(log.ShimContext).Error("failed to handle application event",
 							zap.String("event", event.GetEvent()),
 							zap.Error(err))
 					}
@@ -1070,12 +1176,12 @@ func (ctx *Context) TaskEventHandler() func(obj interface{}) {
 		if event, ok := obj.(events.TaskEvent); ok {
 			task := ctx.getTask(event.GetApplicationID(), event.GetTaskID())
 			if task == nil {
-				log.Logger().Error("failed to handle application event")
+				log.Log(log.ShimContext).Error("failed to handle application event")
 				return
 			}
 			if task.canHandle(event) {
 				if err := task.handle(event); err != nil {
-					log.Logger().Error("failed to handle task event",
+					log.Log(log.ShimContext).Error("failed to handle task event",
 						zap.String("applicationID", task.applicationID),
 						zap.String("taskID", task.taskID),
 						zap.String("event", event.GetEvent()),
@@ -1111,7 +1217,7 @@ func (ctx *Context) LoadConfigMaps() ([]*v1.ConfigMap, error) {
 }
 
 func (ctx *Context) GetStateDump() (string, error) {
-	log.Logger().Info("State dump requested")
+	log.Log(log.ShimContext).Info("State dump requested")
 
 	dump := map[string]interface{}{
 		"cache": ctx.schedulerCache.GetSchedulerCacheDao(),
@@ -1124,6 +1230,17 @@ func (ctx *Context) GetStateDump() (string, error) {
 	return string(bytes), nil
 }
 
+func isPublishableNodeEvent(event *si.EventRecord) bool {
+	// we only send node added & removed event
+	if event.Type == si.EventRecord_NODE &&
+		((event.EventChangeDetail == si.EventRecord_DETAILS_NONE && event.EventChangeType == si.EventRecord_ADD) ||
+			(event.EventChangeDetail == si.EventRecord_NODE_DECOMISSION && event.EventChangeType == si.EventRecord_REMOVE)) {
+		return true
+	}
+
+	return false
+}
+
 // VisibleForTesting
 func (ctx *Context) GetSchedulerCache() *schedulercache.SchedulerCache {
 	return ctx.schedulerCache
