@@ -19,13 +19,11 @@
 package cache
 
 import (
-	"context"
 	"fmt"
 	"time"
 
 	"go.uber.org/zap"
 	corev1 "k8s.io/api/core/v1"
-	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/labels"
 
 	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
@@ -38,14 +36,9 @@ import (
 )
 
 func (ctx *Context) WaitForRecovery(recoverableAppManagers []interfaces.Recoverable, maxTimeout time.Duration) error {
-	// Currently, disable recovery when testing in a mocked cluster,
-	// because mock pod/node lister is not easy. We do have unit tests for
-	// waitForAppRecovery/recover separately.
-	if !ctx.apiProvider.IsTestingMode() {
-		if err := ctx.recover(recoverableAppManagers, maxTimeout); err != nil {
-			log.Logger().Error("nodes recovery failed", zap.Error(err))
-			return err
-		}
+	if err := ctx.recover(recoverableAppManagers, maxTimeout); err != nil {
+		log.Log(log.ShimContext).Error("nodes recovery failed", zap.Error(err))
+		return err
 	}
 
 	return nil
@@ -80,92 +73,83 @@ func (ctx *Context) recover(mgr []interfaces.Recoverable, due time.Duration) err
 		ctx.nodes.addAndReportNode(node, false)
 	}
 
-	// current, disable getting pods for a node during test,
-	// because in the tests, we don't really send existing allocations
-	// we simply simulate to accept or reject nodes on conditions.
-	if !ctx.apiProvider.IsTestingMode() {
-		var podList *corev1.PodList
-		podList, err = ctx.apiProvider.GetAPIs().KubeClient.GetClientSet().
-			CoreV1().Pods("").
-			List(context.Background(), metav1.ListOptions{})
-		if err != nil {
-			return err
-		}
+	pods, err := ctx.apiProvider.GetAPIs().PodInformer.Lister().List(labels.Everything())
+	if err != nil {
+		return err
+	}
 
-		nodeOccupiedResources := make(map[string]*si.Resource)
-		for i := 0; i < len(podList.Items); i++ {
-			pod := podList.Items[i]
-			// only handle assigned pods
-			if !utils.IsAssignedPod(&pod) {
-				log.Logger().Info("Skipping unassigned pod",
+	nodeOccupiedResources := make(map[string]*si.Resource)
+	for _, pod := range pods {
+		// only handle assigned pods
+		if !utils.IsAssignedPod(pod) {
+			log.Log(log.ShimContext).Info("Skipping unassigned pod",
+				zap.String("podUID", string(pod.UID)),
+				zap.String("podName", fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)))
+			continue
+		}
+		// yunikorn scheduled pods add to existing allocations
+		ykPod := utils.GetApplicationIDFromPod(pod) != ""
+		switch {
+		case ykPod:
+			if existingAlloc := getExistingAllocation(mgr, pod); existingAlloc != nil {
+				log.Log(log.ShimContext).Debug("Adding resources for existing pod",
+					zap.String("appID", existingAlloc.ApplicationID),
 					zap.String("podUID", string(pod.UID)),
-					zap.String("podName", fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)))
-				continue
-			}
-			// yunikorn scheduled pods add to existing allocations
-			ykPod := utils.GetApplicationIDFromPod(&pod) != ""
-			switch {
-			case ykPod:
-				if existingAlloc := getExistingAllocation(mgr, &pod); existingAlloc != nil {
-					log.Logger().Debug("Adding resources for existing pod",
-						zap.String("appID", existingAlloc.ApplicationID),
-						zap.String("podUID", string(pod.UID)),
-						zap.String("podName", fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)),
-						zap.String("nodeName", existingAlloc.NodeID),
-						zap.Stringer("resources", common.GetPodResource(&pod)))
-					existingAlloc.AllocationTags = common.CreateTagsForTask(&pod)
-					if err = ctx.nodes.addExistingAllocation(existingAlloc); err != nil {
-						log.Logger().Warn("Failed to add existing allocation", zap.Error(err))
-					}
-				} else {
-					log.Logger().Warn("No allocation found for existing pod",
-						zap.String("podUID", string(pod.UID)),
-						zap.String("podName", fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)),
-						zap.String("nodeName", pod.Spec.NodeName),
-						zap.Stringer("resources", common.GetPodResource(&pod)))
+					zap.String("podName", fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)),
+					zap.String("nodeName", existingAlloc.NodeID),
+					zap.Stringer("resources", common.GetPodResource(pod)))
+				existingAlloc.AllocationTags = common.CreateTagsForTask(pod)
+				if err = ctx.nodes.addExistingAllocation(existingAlloc); err != nil {
+					log.Log(log.ShimContext).Warn("Failed to add existing allocation", zap.Error(err))
 				}
-			case !utils.IsPodTerminated(&pod):
-				// pod is not terminated (succeed or failed) state,
-				// and it has a node assigned, that means the scheduler
-				// has already allocated the pod onto a node
-				// we should report this occupied resource to scheduler-core
-				occupiedResource := nodeOccupiedResources[pod.Spec.NodeName]
-				if occupiedResource == nil {
-					occupiedResource = common.NewResourceBuilder().Build()
-				}
-				podResource := common.GetPodResource(&pod)
-				log.Logger().Debug("Adding resources for occupied pod",
+			} else {
+				log.Log(log.ShimContext).Warn("No allocation found for existing pod",
 					zap.String("podUID", string(pod.UID)),
 					zap.String("podName", fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)),
 					zap.String("nodeName", pod.Spec.NodeName),
-					zap.Stringer("resources", podResource))
-				occupiedResource = common.Add(occupiedResource, podResource)
-				nodeOccupiedResources[pod.Spec.NodeName] = occupiedResource
-				ctx.nodes.cache.AddPod(&pod)
-			default:
-				log.Logger().Debug("Skipping terminated pod",
-					zap.String("podUID", string(pod.UID)),
-					zap.String("podName", fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)))
+					zap.Stringer("resources", common.GetPodResource(pod)))
 			}
+		case !utils.IsPodTerminated(pod):
+			// pod is not terminated (succeed or failed) state,
+			// and it has a node assigned, that means the scheduler
+			// has already allocated the pod onto a node
+			// we should report this occupied resource to scheduler-core
+			occupiedResource := nodeOccupiedResources[pod.Spec.NodeName]
+			if occupiedResource == nil {
+				occupiedResource = common.NewResourceBuilder().Build()
+			}
+			podResource := common.GetPodResource(pod)
+			log.Log(log.ShimContext).Debug("Adding resources for occupied pod",
+				zap.String("podUID", string(pod.UID)),
+				zap.String("podName", fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)),
+				zap.String("nodeName", pod.Spec.NodeName),
+				zap.Stringer("resources", podResource))
+			occupiedResource = common.Add(occupiedResource, podResource)
+			nodeOccupiedResources[pod.Spec.NodeName] = occupiedResource
+			ctx.nodes.cache.AddPod(pod)
+		default:
+			log.Log(log.ShimContext).Debug("Skipping terminated pod",
+				zap.String("podUID", string(pod.UID)),
+				zap.String("podName", fmt.Sprintf("%s/%s", pod.Namespace, pod.Name)))
 		}
+	}
 
-		// why we need to calculate the occupied resources here? why not add an event-handler
-		// in node_coordinator#addPod?
-		// this is because the occupied resources must be calculated and counted before the
-		// scheduling started. If we do both updating existing occupied resources along with
-		// new pods scheduling, due to the fact that we cannot predicate the ordering of K8s
-		// events, it could be dangerous because we might schedule pods onto some node that
-		// doesn't have enough capacity (occupied resources not yet reported).
-		for nodeName, occupiedResource := range nodeOccupiedResources {
-			if cachedNode := ctx.nodes.getNode(nodeName); cachedNode != nil {
-				cachedNode.updateOccupiedResource(occupiedResource, AddOccupiedResource)
-			}
+	// why we need to calculate the occupied resources here? why not add an event-handler
+	// in node_coordinator#addPod?
+	// this is because the occupied resources must be calculated and counted before the
+	// scheduling started. If we do both updating existing occupied resources along with
+	// new pods scheduling, due to the fact that we cannot predicate the ordering of K8s
+	// events, it could be dangerous because we might schedule pods onto some node that
+	// doesn't have enough capacity (occupied resources not yet reported).
+	for nodeName, occupiedResource := range nodeOccupiedResources {
+		if cachedNode := ctx.nodes.getNode(nodeName); cachedNode != nil {
+			cachedNode.updateOccupiedResource(occupiedResource, AddOccupiedResource)
 		}
 	}
 
 	// start new nodes
 	for _, node := range ctx.nodes.nodesMap {
-		log.Logger().Info("node state",
+		log.Log(log.ShimContext).Info("node state",
 			zap.String("nodeName", node.name),
 			zap.String("nodeState", node.getNodeState()))
 		if node.getNodeState() == SchedulerNodeStates().New {
@@ -180,7 +164,7 @@ func (ctx *Context) recover(mgr []interfaces.Recoverable, due time.Duration) err
 	if err = utils.WaitForCondition(func() bool {
 		nodesRecovered := 0
 		for _, node := range ctx.nodes.nodesMap {
-			log.Logger().Info("node state",
+			log.Log(log.ShimContext).Info("node state",
 				zap.String("nodeName", node.name),
 				zap.String("nodeState", node.getNodeState()))
 			switch node.getNodeState() {
@@ -194,11 +178,11 @@ func (ctx *Context) recover(mgr []interfaces.Recoverable, due time.Duration) err
 		}
 
 		if nodesRecovered == len(allNodes) {
-			log.Logger().Info("nodes recovery is successful",
+			log.Log(log.ShimContext).Info("nodes recovery is successful",
 				zap.Int("recoveredNodes", nodesRecovered))
 			return true
 		}
-		log.Logger().Info("still waiting for recovering nodes",
+		log.Log(log.ShimContext).Info("still waiting for recovering nodes",
 			zap.Int("totalNodes", len(allNodes)),
 			zap.Int("recoveredNodes", nodesRecovered))
 		return false
