@@ -55,6 +55,7 @@ type SchedulerCache struct {
 	assumedPods           map[string]bool   // map of assumed pods, value indicates if pod volumes are all bound
 	pendingAllocations    map[string]string // map of pod to node ID, presence indicates a pending allocation for scheduler
 	inProgressAllocations map[string]string // map of pod to node ID, presence indicates an in-process allocation for scheduler
+	pvcRefCounts          map[string]map[string]int
 	lock                  sync.RWMutex
 	clients               *client.Clients // client APIs
 
@@ -73,6 +74,7 @@ func NewSchedulerCache(clients *client.Clients) *SchedulerCache {
 		assumedPods:           make(map[string]bool),
 		pendingAllocations:    make(map[string]string),
 		inProgressAllocations: make(map[string]string),
+		pvcRefCounts:          make(map[string]map[string]int),
 		clients:               clients,
 	}
 	return cache
@@ -171,16 +173,17 @@ func (cache *SchedulerCache) UpdateNode(newNode *v1.Node) {
 func (cache *SchedulerCache) updateNode(node *v1.Node) {
 	nodeInfo, ok := cache.nodesMap[node.Name]
 	if !ok {
-		log.Logger().Debug("Adding node to cache", zap.String("nodeName", node.Name))
+		log.Log(log.ShimCacheExternal).Debug("Adding node to cache", zap.String("nodeName", node.Name))
 		nodeInfo = framework.NewNodeInfo()
 		cache.nodesMap[node.Name] = nodeInfo
 		cache.nodesInfo = nil
 	} else {
-		log.Logger().Debug("Updating node in cache", zap.String("nodeName", node.Name))
+		log.Log(log.ShimCacheExternal).Debug("Updating node in cache", zap.String("nodeName", node.Name))
 	}
 	nodeInfo.SetNode(node)
 	cache.nodesInfoPodsWithAffinity = nil
 	cache.nodesInfoPodsWithReqAntiAffinity = nil
+	cache.updatePVCRefCounts(nodeInfo, false)
 }
 
 func (cache *SchedulerCache) RemoveNode(node *v1.Node) {
@@ -195,7 +198,7 @@ func (cache *SchedulerCache) RemoveNode(node *v1.Node) {
 func (cache *SchedulerCache) removeNode(node *v1.Node) {
 	nodeInfo, ok := cache.nodesMap[node.Name]
 	if !ok {
-		log.Logger().Debug("Attempted to remove non-existent node", zap.String("nodeName", node.Name))
+		log.Log(log.ShimCacheExternal).Debug("Attempted to remove non-existent node", zap.String("nodeName", node.Name))
 		return
 	}
 
@@ -207,11 +210,12 @@ func (cache *SchedulerCache) removeNode(node *v1.Node) {
 		delete(cache.inProgressAllocations, key)
 	}
 
-	log.Logger().Debug("Removing node from cache", zap.String("nodeName", node.Name))
+	log.Log(log.ShimCacheExternal).Debug("Removing node from cache", zap.String("nodeName", node.Name))
 	delete(cache.nodesMap, node.Name)
 	cache.nodesInfo = nil
 	cache.nodesInfoPodsWithAffinity = nil
 	cache.nodesInfoPodsWithReqAntiAffinity = nil
+	cache.updatePVCRefCounts(nodeInfo, true)
 }
 
 func (cache *SchedulerCache) GetPriorityClass(name string) *schedulingv1.PriorityClass {
@@ -245,9 +249,9 @@ func (cache *SchedulerCache) UpdatePriorityClass(priorityClass *schedulingv1.Pri
 func (cache *SchedulerCache) updatePriorityClass(priorityClass *schedulingv1.PriorityClass) {
 	_, ok := cache.pcMap[priorityClass.Name]
 	if !ok {
-		log.Logger().Debug("Adding priorityClass to cache", zap.String("name", priorityClass.Name))
+		log.Log(log.ShimCacheExternal).Debug("Adding priorityClass to cache", zap.String("name", priorityClass.Name))
 	} else {
-		log.Logger().Debug("Updating priorityClass in cache", zap.String("name", priorityClass.Name))
+		log.Log(log.ShimCacheExternal).Debug("Updating priorityClass in cache", zap.String("name", priorityClass.Name))
 	}
 	cache.pcMap[priorityClass.Name] = priorityClass
 }
@@ -262,7 +266,7 @@ func (cache *SchedulerCache) RemovePriorityClass(priorityClass *schedulingv1.Pri
 }
 
 func (cache *SchedulerCache) removePriorityClass(priorityClass *schedulingv1.PriorityClass) {
-	log.Logger().Debug("Removing priorityClass from cache", zap.String("name", priorityClass.Name))
+	log.Log(log.ShimCacheExternal).Debug("Removing priorityClass from cache", zap.String("name", priorityClass.Name))
 	delete(cache.pcMap, priorityClass.Name)
 }
 
@@ -367,13 +371,18 @@ func (cache *SchedulerCache) updatePod(pod *v1.Pod) {
 			nodeInfo, ok := cache.nodesMap[nodeName]
 			if ok {
 				if err := nodeInfo.RemovePod(currState); err != nil {
-					log.Logger().Warn("BUG: Failed to remove pod from node",
+					log.Log(log.ShimCacheExternal).Warn("BUG: Failed to remove pod from node",
 						zap.String("podName", currState.Name),
 						zap.String("nodeName", nodeName),
 						zap.Error(err))
 				}
-				cache.nodesInfoPodsWithAffinity = nil
-				cache.nodesInfoPodsWithReqAntiAffinity = nil
+				cache.updatePVCRefCounts(nodeInfo, false)
+				if podWithAffinity(pod) {
+					cache.nodesInfoPodsWithAffinity = nil
+				}
+				if podWithRequiredAntiAffinity(pod) {
+					cache.nodesInfoPodsWithReqAntiAffinity = nil
+				}
 			}
 			if pod.Spec.NodeName == "" {
 				// new pod wasn't assigned to a node, so use existing assignment
@@ -402,16 +411,21 @@ func (cache *SchedulerCache) updatePod(pod *v1.Pod) {
 		}
 		nodeInfo.AddPod(pod)
 		cache.assignedPods[key] = pod.Spec.NodeName
-		cache.nodesInfoPodsWithAffinity = nil
-		cache.nodesInfoPodsWithReqAntiAffinity = nil
+		if podWithAffinity(pod) {
+			cache.nodesInfoPodsWithAffinity = nil
+		}
+		if podWithRequiredAntiAffinity(pod) {
+			cache.nodesInfoPodsWithReqAntiAffinity = nil
+		}
+		cache.updatePVCRefCounts(nodeInfo, false)
 	}
 
 	// if pod is not in a terminal state, add it back into cache
 	if !utils.IsPodTerminated(pod) {
-		log.Logger().Debug("Putting pod in cache", zap.String("podName", pod.Name), zap.String("podKey", key))
+		log.Log(log.ShimCacheExternal).Debug("Putting pod in cache", zap.String("podName", pod.Name), zap.String("podKey", key))
 		cache.podsMap[key] = pod
 	} else {
-		log.Logger().Debug("Removing terminated pod from cache", zap.String("podName", pod.Name), zap.String("podKey", key))
+		log.Log(log.ShimCacheExternal).Debug("Removing terminated pod from cache", zap.String("podName", pod.Name), zap.String("podKey", key))
 		delete(cache.podsMap, key)
 		delete(cache.assignedPods, key)
 		delete(cache.assumedPods, key)
@@ -431,18 +445,19 @@ func (cache *SchedulerCache) RemovePod(pod *v1.Pod) {
 
 func (cache *SchedulerCache) removePod(pod *v1.Pod) {
 	key := string(pod.UID)
-	log.Logger().Debug("Removing deleted pod from cache", zap.String("podName", pod.Name), zap.String("podKey", key))
+	log.Log(log.ShimCacheExternal).Debug("Removing deleted pod from cache", zap.String("podName", pod.Name), zap.String("podKey", key))
 	nodeName, ok := cache.assignedPods[key]
 	if ok {
 		nodeInfo, ok := cache.nodesMap[nodeName]
 		if ok {
 			if err := nodeInfo.RemovePod(pod); err != nil {
-				log.Logger().Warn("BUG: Failed to remove pod from node",
+				log.Log(log.ShimCacheExternal).Warn("BUG: Failed to remove pod from node",
 					zap.String("podName", pod.Name),
 					zap.String("nodeName", nodeName),
 					zap.Error(err))
 			}
 		}
+		cache.updatePVCRefCounts(nodeInfo, false)
 	}
 	delete(cache.podsMap, key)
 	delete(cache.assignedPods, key)
@@ -456,6 +471,10 @@ func (cache *SchedulerCache) removePod(pod *v1.Pod) {
 func (cache *SchedulerCache) GetPod(uid string) (*v1.Pod, bool) {
 	cache.lock.RLock()
 	defer cache.lock.RUnlock()
+	return cache.GetPodNoLock(uid)
+}
+
+func (cache *SchedulerCache) GetPodNoLock(uid string) (*v1.Pod, bool) {
 	if pod, ok := cache.podsMap[uid]; ok {
 		return pod, true
 	}
@@ -473,7 +492,7 @@ func (cache *SchedulerCache) AssumePod(pod *v1.Pod, allBound bool) {
 func (cache *SchedulerCache) assumePod(pod *v1.Pod, allBound bool) {
 	key := string(pod.UID)
 
-	log.Logger().Debug("Adding assumed pod to cache",
+	log.Log(log.ShimCacheExternal).Debug("Adding assumed pod to cache",
 		zap.String("podName", pod.Name),
 		zap.String("podKey", key),
 		zap.String("node", pod.Spec.NodeName),
@@ -498,7 +517,7 @@ func (cache *SchedulerCache) forgetPod(pod *v1.Pod) {
 	cache.updatePod(pod)
 
 	// remove assigned allocation
-	log.Logger().Debug("Removing assumed pod from cache",
+	log.Log(log.ShimCacheExternal).Debug("Removing assumed pod from cache",
 		zap.String("podName", pod.Name),
 		zap.String("podKey", key))
 
@@ -554,8 +573,8 @@ func (cache *SchedulerCache) GetPersistentVolumeInfo(name string) (*v1.Persisten
 
 // dumpState dumps summary statistics for the cache. Must be called with lock already acquired
 func (cache *SchedulerCache) dumpState(context string) {
-	if log.Logger().Core().Enabled(zapcore.DebugLevel) {
-		log.Logger().Debug("Scheduler cache state ("+context+")",
+	if log.Log(log.ShimCacheExternal).Core().Enabled(zapcore.DebugLevel) {
+		log.Log(log.ShimCacheExternal).Debug("Scheduler cache state ("+context+")",
 			zap.Int("nodes", len(cache.nodesMap)),
 			zap.Int("pods", len(cache.podsMap)),
 			zap.Int("assumed", len(cache.assumedPods)),
@@ -589,6 +608,34 @@ func (cache *SchedulerCache) nodePodCount() int {
 	return result
 }
 
+// IsPVCUsedByPods determines if a given volume claim is in use by any current pods. This is explicitly for the use
+// of the predicate shared lister and requires that the scheduler cache lock be held while accessing.
+func (cache *SchedulerCache) IsPVCUsedByPods(key string) bool {
+	_, ok := cache.pvcRefCounts[key]
+	return ok
+}
+
+func (cache *SchedulerCache) updatePVCRefCounts(node *framework.NodeInfo, removeNode bool) {
+	nodeName := node.Node().Name
+	for k, v := range cache.pvcRefCounts {
+		delete(v, nodeName)
+		if len(v) == 0 {
+			delete(cache.pvcRefCounts, k)
+		}
+	}
+
+	if !removeNode {
+		for k, count := range node.PVCRefCounts {
+			entry, ok := cache.pvcRefCounts[k]
+			if !ok {
+				entry = make(map[string]int)
+				cache.pvcRefCounts[k] = entry
+			}
+			entry[nodeName] = count
+		}
+	}
+}
+
 func (cache *SchedulerCache) GetSchedulerCacheDao() SchedulerCacheDao {
 	cache.lock.RLock()
 	defer cache.lock.RUnlock()
@@ -705,3 +752,14 @@ func (cache *SchedulerCache) GetSchedulerCacheDao() SchedulerCacheDao {
 		SchedulingPods:  podSchedulingInfoByName,
 	}
 }
+
+func podWithAffinity(p *v1.Pod) bool {
+	affinity := p.Spec.Affinity
+	return affinity != nil && (affinity.PodAffinity != nil || affinity.PodAntiAffinity != nil)
+}
+
+func podWithRequiredAntiAffinity(p *v1.Pod) bool {
+	affinity := p.Spec.Affinity
+	return affinity != nil && affinity.PodAntiAffinity != nil &&
+		len(affinity.PodAntiAffinity.RequiredDuringSchedulingIgnoredDuringExecution) != 0
+}
