@@ -25,7 +25,7 @@ import (
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
-	"github.com/apache/yunikorn-k8shim/pkg/apis/yunikorn.apache.org/v1alpha1"
+	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
 	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
 	"github.com/apache/yunikorn-k8shim/pkg/common/utils"
 	"github.com/apache/yunikorn-k8shim/pkg/conf"
@@ -47,7 +47,7 @@ type Placeholder struct {
 	pod           *v1.Pod
 }
 
-func newPlaceholder(placeholderName string, app *Application, taskGroup v1alpha1.TaskGroup) *Placeholder {
+func newPlaceholder(placeholderName string, app *Application, taskGroup interfaces.TaskGroup) *Placeholder {
 	// Here the owner reference is always the originator pod
 	ownerRefs := app.getPlaceholderOwnerReferences()
 	annotations := utils.MergeMaps(taskGroup.Annotations, map[string]string{
@@ -79,6 +79,13 @@ func newPlaceholder(placeholderName string, app *Application, taskGroup v1alpha1
 		}
 	}
 
+	var priorityClassName string
+	if task := app.GetOriginatingTask(); task != nil {
+		priorityClassName = task.GetTaskPod().Spec.PriorityClassName
+	}
+
+	// prepare the resource lists
+	requests := utils.GetPlaceholderResourceRequests(taskGroup.MinResource)
 	placeholderPod := &v1.Pod{
 		ObjectMeta: metav1.ObjectMeta{
 			Name:      placeholderName,
@@ -103,15 +110,17 @@ func newPlaceholder(placeholderName string, app *Application, taskGroup v1alpha1
 					Image:           conf.GetSchedulerConf().PlaceHolderImage,
 					ImagePullPolicy: v1.PullIfNotPresent,
 					Resources: v1.ResourceRequirements{
-						Requests: utils.GetPlaceholderResourceRequest(taskGroup.MinResource),
+						Requests: requests,
+						Limits:   requests,
 					},
 				},
 			},
-			RestartPolicy: constants.PlaceholderPodRestartPolicy,
-			SchedulerName: constants.SchedulerName,
-			NodeSelector:  taskGroup.NodeSelector,
-			Tolerations:   taskGroup.Tolerations,
-			Affinity:      taskGroup.Affinity,
+			RestartPolicy:     constants.PlaceholderPodRestartPolicy,
+			SchedulerName:     constants.SchedulerName,
+			NodeSelector:      taskGroup.NodeSelector,
+			Tolerations:       taskGroup.Tolerations,
+			Affinity:          taskGroup.Affinity,
+			PriorityClassName: priorityClassName,
 		},
 	}
 
