@@ -25,7 +25,10 @@ import (
 	"sync"
 	"time"
 
+	"github.com/looplab/fsm"
 	"go.uber.org/zap"
+	v1 "k8s.io/api/core/v1"
+	podutil "k8s.io/kubernetes/pkg/api/v1/pod"
 
 	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
 	"github.com/apache/yunikorn-k8shim/pkg/common"
@@ -35,9 +38,6 @@ import (
 	"github.com/apache/yunikorn-k8shim/pkg/dispatcher"
 	"github.com/apache/yunikorn-k8shim/pkg/log"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
-
-	"github.com/looplab/fsm"
-	v1 "k8s.io/api/core/v1"
 )
 
 type Task struct {
@@ -48,14 +48,15 @@ type Task struct {
 	allocationUUID  string
 	resource        *si.Resource
 	pod             *v1.Pod
+	podStatus       v1.PodStatus // pod status, maintained separately for efficiency reasons
 	context         *Context
 	nodeName        string
 	createTime      time.Time
 	taskGroupName   string
 	placeholder     bool
 	terminationType string
-	pluginMode      bool
 	originator      bool
+	schedulingState interfaces.TaskSchedulingState
 	sm              *fsm.FSM
 	lock            *sync.RWMutex
 }
@@ -86,25 +87,22 @@ func NewFromTaskMeta(tid string, app *Application, ctx *Context, metadata interf
 
 func createTaskInternal(tid string, app *Application, resource *si.Resource,
 	pod *v1.Pod, placeholder bool, taskGroupName string, ctx *Context, originator bool) *Task {
-	var pluginMode bool
-	if ctx != nil {
-		pluginMode = ctx.IsPluginMode()
-	}
 	task := &Task{
-		taskID:        tid,
-		alias:         fmt.Sprintf("%s/%s", pod.Namespace, pod.Name),
-		applicationID: app.GetApplicationID(),
-		application:   app,
-		pod:           pod,
-		resource:      resource,
-		createTime:    pod.GetCreationTimestamp().Time,
-		placeholder:   placeholder,
-		taskGroupName: taskGroupName,
-		pluginMode:    pluginMode,
-		originator:    originator,
-		context:       ctx,
-		sm:            newTaskState(),
-		lock:          &sync.RWMutex{},
+		taskID:          tid,
+		alias:           fmt.Sprintf("%s/%s", pod.Namespace, pod.Name),
+		applicationID:   app.GetApplicationID(),
+		application:     app,
+		pod:             pod,
+		podStatus:       *pod.Status.DeepCopy(),
+		resource:        resource,
+		createTime:      pod.GetCreationTimestamp().Time,
+		placeholder:     placeholder,
+		taskGroupName:   taskGroupName,
+		originator:      originator,
+		context:         ctx,
+		sm:              newTaskState(),
+		schedulingState: interfaces.TaskSchedPending,
+		lock:            &sync.RWMutex{},
 	}
 	if tgName := utils.GetTaskGroupFromPodSpec(pod); tgName != "" {
 		task.taskGroupName = tgName
@@ -215,7 +213,7 @@ func (task *Task) initialize() {
 		task.allocationUUID = string(task.pod.UID)
 		task.nodeName = task.pod.Spec.NodeName
 		task.sm.SetState(TaskStates().Bound)
-		log.Logger().Info("set task as Bound",
+		log.Log(log.ShimCacheTask).Info("set task as Bound",
 			zap.String("appID", task.applicationID),
 			zap.String("taskID", task.taskID),
 			zap.String("allocationUUID", task.allocationUUID),
@@ -230,7 +228,7 @@ func (task *Task) initialize() {
 		task.allocationUUID = string(task.pod.UID)
 		task.nodeName = task.pod.Spec.NodeName
 		task.sm.SetState(TaskStates().Completed)
-		log.Logger().Info("set task as Completed",
+		log.Log(log.ShimCacheTask).Info("set task as Completed",
 			zap.String("appID", task.applicationID),
 			zap.String("taskID", task.taskID),
 			zap.String("allocationUUID", task.allocationUUID),
@@ -271,8 +269,20 @@ func (task *Task) isPreemptOtherAllowed() bool {
 	}
 }
 
+func (task *Task) SetTaskSchedulingState(state interfaces.TaskSchedulingState) {
+	task.lock.Lock()
+	defer task.lock.Unlock()
+	task.schedulingState = state
+}
+
+func (task *Task) GetTaskSchedulingState() interfaces.TaskSchedulingState {
+	task.lock.RLock()
+	defer task.lock.RUnlock()
+	return task.schedulingState
+}
+
 func (task *Task) handleSubmitTaskEvent() {
-	log.Logger().Debug("scheduling pod",
+	log.Log(log.ShimCacheTask).Debug("scheduling pod",
 		zap.String("podName", task.pod.Name))
 
 	// build preemption policy
@@ -291,9 +301,9 @@ func (task *Task) handleSubmitTaskEvent() {
 		task.pod,
 		task.originator,
 		preemptionPolicy)
-	log.Logger().Debug("send update request", zap.Stringer("request", &rr))
-	if err := task.context.apiProvider.GetAPIs().SchedulerAPI.UpdateAllocation(&rr); err != nil {
-		log.Logger().Debug("failed to send scheduling request to scheduler", zap.Error(err))
+	log.Log(log.ShimCacheTask).Debug("send update request", zap.Stringer("request", rr))
+	if err := task.context.apiProvider.GetAPIs().SchedulerAPI.UpdateAllocation(rr); err != nil {
+		log.Log(log.ShimCacheTask).Debug("failed to send scheduling request to scheduler", zap.Error(err))
 		return
 	}
 
@@ -328,8 +338,8 @@ func (task *Task) postTaskAllocated() {
 		defer task.lock.Unlock()
 
 		// plugin mode means we delegate this work to the default scheduler
-		if task.pluginMode {
-			log.Logger().Debug("allocating pod",
+		if utils.IsPluginMode() {
+			log.Log(log.ShimCacheTask).Debug("allocating pod",
 				zap.String("podName", task.pod.Name),
 				zap.String("podUID", string(task.pod.UID)))
 
@@ -346,7 +356,7 @@ func (task *Task) postTaskAllocated() {
 				"Successfully assigned %s to node %s", task.alias, task.nodeName)
 
 			// before binding pod to node, first bind volumes to pod
-			log.Logger().Debug("bind pod volumes",
+			log.Log(log.ShimCacheTask).Debug("bind pod volumes",
 				zap.String("podName", task.pod.Name),
 				zap.String("podUID", string(task.pod.UID)))
 			if task.context.apiProvider.GetAPIs().VolumeBinder != nil {
@@ -359,25 +369,27 @@ func (task *Task) postTaskAllocated() {
 				}
 			}
 
-			log.Logger().Debug("bind pod",
+			log.Log(log.ShimCacheTask).Debug("bind pod",
 				zap.String("podName", task.pod.Name),
 				zap.String("podUID", string(task.pod.UID)))
 
 			if err := task.context.apiProvider.GetAPIs().KubeClient.Bind(task.pod, task.nodeName); err != nil {
 				errorMessage := fmt.Sprintf("bind pod to node failed, name: %s, %s", task.alias, err.Error())
-				log.Logger().Error(errorMessage)
+				log.Log(log.ShimCacheTask).Error(errorMessage)
 				dispatcher.Dispatch(NewFailTaskEvent(task.applicationID, task.taskID, errorMessage))
 				events.GetRecorder().Eventf(task.pod.DeepCopy(), nil,
 					v1.EventTypeWarning, "PodBindFailure", "PodBindFailure", errorMessage)
 				return
 			}
 
-			log.Logger().Info("successfully bound pod", zap.String("podName", task.pod.Name))
+			log.Log(log.ShimCacheTask).Info("successfully bound pod", zap.String("podName", task.pod.Name))
 			dispatcher.Dispatch(NewBindTaskEvent(task.applicationID, task.taskID))
 			events.GetRecorder().Eventf(task.pod.DeepCopy(), nil,
 				v1.EventTypeNormal, "PodBindSuccessful", "PodBindSuccessful",
 				"Pod %s is successfully bound to node %s", task.alias, task.nodeName)
 		}
+
+		task.schedulingState = interfaces.TaskSchedAllocated
 	}()
 }
 
@@ -396,7 +408,7 @@ func (task *Task) beforeTaskAllocated(eventSrc string, allocUUID string, nodeID
 	// Notify the core to release this allocation to avoid resource leak.
 	// The ask is not relevant at this point.
 	if eventSrc == TaskStates().Completed {
-		log.Logger().Info("task is already completed, invalidate the allocation",
+		log.Log(log.ShimCacheTask).Info("task is already completed, invalidate the allocation",
 			zap.String("currentTaskState", eventSrc),
 			zap.String("allocUUID", allocUUID),
 			zap.String("allocatedNode", nodeID))
@@ -405,7 +417,7 @@ func (task *Task) beforeTaskAllocated(eventSrc string, allocUUID string, nodeID
 }
 
 func (task *Task) postTaskBound() {
-	if task.pluginMode {
+	if utils.IsPluginMode() {
 		// When the pod is actively scheduled by YuniKorn, it can be  moved to the default-scheduler's
 		// UnschedulablePods structure. If the pod does not change, the pod will stay in the UnschedulablePods
 		// structure for podMaxInUnschedulablePodsDuration (default 5 minutes). Here we update the pod
@@ -417,14 +429,14 @@ func (task *Task) postTaskBound() {
 			if pod.Annotations == nil {
 				pod.Annotations = make(map[string]string)
 			}
-			pod.Annotations["yunikorn.apache.org/scheduled-at"] = strconv.FormatInt(time.Now().UnixNano(), 10)
+			pod.Annotations[constants.DomainYuniKorn+"scheduled-at"] = strconv.FormatInt(time.Now().UnixNano(), 10)
 		}); err != nil {
-			log.Logger().Warn("failed to update pod status", zap.Error(err))
+			log.Log(log.ShimCacheTask).Warn("failed to update pod status", zap.Error(err))
 		}
 	}
 
 	if task.placeholder {
-		log.Logger().Info("placeholder is bound",
+		log.Log(log.ShimCacheTask).Info("placeholder is bound",
 			zap.String("appID", task.applicationID),
 			zap.String("taskName", task.alias),
 			zap.String("taskGroupName", task.taskGroupName))
@@ -452,7 +464,7 @@ func (task *Task) beforeTaskFail() {
 }
 
 func (task *Task) postTaskFailed(reason string) {
-	log.Logger().Error("task failed",
+	log.Log(log.ShimCacheTask).Error("task failed",
 		zap.String("appID", task.applicationID),
 		zap.String("taskID", task.taskID),
 		zap.String("reason", reason))
@@ -476,7 +488,7 @@ func (task *Task) beforeTaskCompleted() {
 func (task *Task) releaseAllocation() {
 	// scheduler api might be nil in some tests
 	if task.context.apiProvider.GetAPIs().SchedulerAPI != nil {
-		log.Logger().Debug("prepare to send release request",
+		log.Log(log.ShimCacheTask).Debug("prepare to send release request",
 			zap.String("applicationID", task.applicationID),
 			zap.String("taskID", task.taskID),
 			zap.String("taskAlias", task.alias),
@@ -487,7 +499,7 @@ func (task *Task) releaseAllocation() {
 		// The message depends on current task state, generate requests accordingly.
 		// If allocated send an AllocationReleaseRequest,
 		// If not allocated yet send an AllocationAskReleaseRequest
-		var releaseRequest si.AllocationRequest
+		var releaseRequest *si.AllocationRequest
 		s := TaskStates()
 		switch task.GetTaskState() {
 		case s.New, s.Pending, s.Scheduling, s.Rejected:
@@ -495,7 +507,7 @@ func (task *Task) releaseAllocation() {
 				task.applicationID, task.taskID, task.application.partition)
 		default:
 			if task.allocationUUID == "" {
-				log.Logger().Warn("BUG: task allocation UUID is empty on release",
+				log.Log(log.ShimCacheTask).Warn("BUG: task allocation UUID is empty on release",
 					zap.String("applicationID", task.applicationID),
 					zap.String("taskID", task.taskID),
 					zap.String("taskAlias", task.alias),
@@ -507,12 +519,12 @@ func (task *Task) releaseAllocation() {
 		}
 
 		if releaseRequest.Releases != nil {
-			log.Logger().Info("releasing allocations",
+			log.Log(log.ShimCacheTask).Info("releasing allocations",
 				zap.Int("numOfAsksToRelease", len(releaseRequest.Releases.AllocationAsksToRelease)),
 				zap.Int("numOfAllocationsToRelease", len(releaseRequest.Releases.AllocationsToRelease)))
 		}
-		if err := task.context.apiProvider.GetAPIs().SchedulerAPI.UpdateAllocation(&releaseRequest); err != nil {
-			log.Logger().Debug("failed to send scheduling request to scheduler", zap.Error(err))
+		if err := task.context.apiProvider.GetAPIs().SchedulerAPI.UpdateAllocation(releaseRequest); err != nil {
+			log.Log(log.ShimCacheTask).Debug("failed to send scheduling request to scheduler", zap.Error(err))
 		}
 	}
 }
@@ -531,7 +543,7 @@ func (task *Task) sanityCheckBeforeScheduling() error {
 			continue
 		}
 		pvcName := volume.PersistentVolumeClaim.ClaimName
-		log.Logger().Debug("checking PVC", zap.String("name", pvcName))
+		log.Log(log.ShimCacheTask).Debug("checking PVC", zap.String("name", pvcName))
 		pvc, err := task.context.apiProvider.GetAPIs().PVCInformer.Lister().PersistentVolumeClaims(namespace).Get(pvcName)
 		if err != nil {
 			return err
@@ -542,3 +554,25 @@ func (task *Task) sanityCheckBeforeScheduling() error {
 	}
 	return nil
 }
+
+func (task *Task) UpdatePodCondition(podCondition *v1.PodCondition) (bool, *v1.Pod) {
+	task.lock.Lock()
+	defer task.lock.Unlock()
+
+	status := task.podStatus.DeepCopy()
+	pod := task.pod.DeepCopy()
+	pod.Status = *status
+	if !utils.PodUnderCondition(pod, podCondition) {
+		log.Log(log.ShimContext).Debug("updating pod condition",
+			zap.String("namespace", task.pod.Namespace),
+			zap.String("name", task.pod.Name),
+			zap.Any("podCondition", podCondition))
+		if podutil.UpdatePodCondition(&task.podStatus, podCondition) {
+			status = task.podStatus.DeepCopy()
+			pod.Status = *status
+			return true, pod
+		}
+	}
+
+	return false, pod
+}
