@@ -21,7 +21,7 @@ import (
 	"strconv"
 	"testing"
 
-	"gotest.tools/assert"
+	"gotest.tools/v3/assert"
 	v1 "k8s.io/api/core/v1"
 	apis "k8s.io/apimachinery/pkg/apis/meta/v1"
 
@@ -31,7 +31,6 @@ import (
 )
 
 const nodeID = "node-01"
-const nodeLabels = "{\"label1\":\"key1\",\"label2\":\"key2\"}"
 
 func TestCreateReleaseAllocationRequest(t *testing.T) {
 	request := CreateReleaseAllocationRequestForTask("app01", "alloc01", "default", "STOPPED_BY_RM")
@@ -218,16 +217,28 @@ func TestCreateUpdateRequestForNewNode(t *testing.T) {
 	occupied := NewResourceBuilder().AddResource(common.Memory, 50).AddResource(common.CPU, 1).Build()
 	var existingAllocations []*si.Allocation
 	ready := true
+	nodeLabels := map[string]string{
+		"label1":                           "key1",
+		"label2":                           "key2",
+		"node.kubernetes.io/instance-type": "HighMem",
+	}
 	request := CreateUpdateRequestForNewNode(nodeID, nodeLabels, capacity, occupied, existingAllocations, ready)
 	assert.Equal(t, len(request.Nodes), 1)
 	assert.Equal(t, request.Nodes[0].NodeID, nodeID)
 	assert.Equal(t, request.Nodes[0].SchedulableResource, capacity)
 	assert.Equal(t, request.Nodes[0].OccupiedResource, occupied)
-	assert.Equal(t, len(request.Nodes[0].Attributes), 4)
+	assert.Equal(t, len(request.Nodes[0].Attributes), 7)
 	assert.Equal(t, request.Nodes[0].Attributes[constants.DefaultNodeAttributeHostNameKey], nodeID)
 	assert.Equal(t, request.Nodes[0].Attributes[constants.DefaultNodeAttributeRackNameKey], constants.DefaultRackName)
-	assert.Equal(t, request.Nodes[0].Attributes[constants.DefaultNodeAttributeNodeLabelsKey], nodeLabels)
 	assert.Equal(t, request.Nodes[0].Attributes[common.NodeReadyAttribute], strconv.FormatBool(ready))
+
+	// Make sure include nodeLabel
+	assert.Equal(t, request.Nodes[0].Attributes["label1"], "key1")
+	assert.Equal(t, request.Nodes[0].Attributes["label2"], "key2")
+	assert.Equal(t, request.Nodes[0].Attributes["node.kubernetes.io/instance-type"], "HighMem")
+
+	// Make sure include the instanceType
+	assert.Equal(t, request.Nodes[0].Attributes[common.InstanceType], "HighMem")
 }
 
 func TestCreateUpdateRequestForUpdatedNode(t *testing.T) {
