@@ -20,6 +20,7 @@ package utils
 
 import (
 	"fmt"
+	"math/rand"
 	"strconv"
 	"strings"
 
@@ -27,13 +28,12 @@ import (
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 
-	"github.com/apache/yunikorn-k8shim/pkg/apis/yunikorn.apache.org/v1alpha1"
 	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
 	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
 	"github.com/apache/yunikorn-k8shim/pkg/log"
 )
 
-func FindAppTaskGroup(appTaskGroups []*v1alpha1.TaskGroup, groupName string) (*v1alpha1.TaskGroup, error) {
+func FindAppTaskGroup(appTaskGroups []*interfaces.TaskGroup, groupName string) (*interfaces.TaskGroup, error) {
 	if groupName == "" {
 		// task has no group defined
 		return nil, nil
@@ -55,21 +55,37 @@ func FindAppTaskGroup(appTaskGroups []*v1alpha1.TaskGroup, groupName string) (*v
 	return nil, fmt.Errorf("taskGroup %s is not defined in the application", groupName)
 }
 
-// the placeholder name is the pod name, pod name can not be longer than 63 chars,
-// taskGroup name and appID will be truncated if they go over 20/28 chars respectively,
-// each taskGroup is assigned with an incremental index starting from 0.
-func GeneratePlaceholderName(taskGroupName, appID string, index int32) string {
-	// taskGroup name no longer than 20 chars
+// GeneratePlaceholderName creates the placeholder name for a pod, pod name can not be longer than 63 chars,
+// appID and taskGroupName will be truncated if they go over 28/20 chars respectively,
+// and each name will be suffixed with a randomly generated 10-character nonce.
+func GeneratePlaceholderName(taskGroupName, appID string) string {
 	// appID no longer than 28 chars
-	// total length no longer than 20 + 28 + 5 + 10 = 63
-	shortTaskGroupName := fmt.Sprintf("%.20s", taskGroupName)
-	shortAppID := fmt.Sprintf("%.28s", appID)
-	return "tg-" + shortTaskGroupName + "-" + shortAppID + fmt.Sprintf("-%d", index)
+	// taskGroup name no longer than 20 chars
+	// nonce equal to 10 chars
+	// total length no longer than 28 + 20 + 5 + 10 = 63
+	return fmt.Sprintf("tg-%.28s-%.20s-%s", appID, taskGroupName, generateNonce(10))
 }
 
-func GetPlaceholderResourceRequest(resources map[string]resource.Quantity) v1.ResourceList {
+const letterBytes = "abcdefghijklmnopqrstuvwxyz0123456789"
+
+// generateNonce generates an alphanumeric string (i.e. base-36) of the given length
+func generateNonce(length int) string {
+	buf := make([]byte, length)
+	for i := range buf {
+		// this uses the built-in rand package as it does not need to be hardened
+		buf[i] = letterBytes[rand.Intn(len(letterBytes))] //nolint:gosec
+	}
+	return string(buf)
+}
+
+// GetPlaceholderResourceRequests converts the map of resources requested into a list of resources for the request
+// specification that can be added to a pod. No checks on what is requested.
+func GetPlaceholderResourceRequests(resources map[string]resource.Quantity) v1.ResourceList {
 	resourceReq := v1.ResourceList{}
 	for k, v := range resources {
+		if k == "" {
+			continue
+		}
 		resourceReq[v1.ResourceName(k)] = v
 	}
 	return resourceReq
@@ -88,19 +104,19 @@ func GetSchedulingPolicyParam(pod *v1.Pod) *interfaces.SchedulingPolicyParameter
 	for _, p := range params {
 		param := strings.Split(p, "=")
 		if len(param) != 2 {
-			log.Logger().Warn("Skipping malformed scheduling policy parameter: ", zap.String("namespace", pod.Namespace), zap.String("name", pod.Name), zap.String("Scheduling Policy parameters passed in annotation: ", p))
+			log.Log(log.ShimUtils).Warn("Skipping malformed scheduling policy parameter: ", zap.String("namespace", pod.Namespace), zap.String("name", pod.Name), zap.String("Scheduling Policy parameters passed in annotation: ", p))
 			continue
 		}
 		if param[0] == constants.SchedulingPolicyTimeoutParam {
 			timeout, err = strconv.ParseInt(param[1], 10, 64)
 			if err != nil {
-				log.Logger().Warn("Failed to parse timeout value from annotation", zap.String("namespace", pod.Namespace), zap.String("name", pod.Name), zap.Int64("Using Placeholder timeout: ", timeout), zap.String("Placeholder timeout passed in annotation: ", p))
+				log.Log(log.ShimUtils).Warn("Failed to parse timeout value from annotation", zap.String("namespace", pod.Namespace), zap.String("name", pod.Name), zap.Int64("Using Placeholder timeout: ", timeout), zap.String("Placeholder timeout passed in annotation: ", p))
 			}
 		} else if param[0] == constants.SchedulingPolicyStyleParam {
 			style = constants.SchedulingPolicyStyleParamValues[param[1]]
 			if style == "" {
 				style = constants.SchedulingPolicyStyleParamDefault
-				log.Logger().Warn("Unknown gang scheduling style, using "+constants.SchedulingPolicyStyleParamDefault+" style as default",
+				log.Log(log.ShimUtils).Warn("Unknown gang scheduling style, using "+constants.SchedulingPolicyStyleParamDefault+" style as default",
 					zap.String("namespace", pod.Namespace), zap.String("name", pod.Name), zap.String("Gang scheduling style passed in annotation: ", p))
 			}
 		}
