@@ -30,11 +30,9 @@ import (
 
 	v1 "k8s.io/api/core/v1"
 	schedulingv1 "k8s.io/api/scheduling/v1"
-	"k8s.io/apimachinery/pkg/api/resource"
-	apis "k8s.io/apimachinery/pkg/apis/meta/v1"
 	podv1 "k8s.io/kubernetes/pkg/api/v1/pod"
 
-	"github.com/apache/yunikorn-k8shim/pkg/apis/yunikorn.apache.org/v1alpha1"
+	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
 	"github.com/apache/yunikorn-k8shim/pkg/common"
 	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
 	"github.com/apache/yunikorn-k8shim/pkg/conf"
@@ -45,6 +43,16 @@ import (
 
 const userInfoKey = siCommon.DomainYuniKorn + "user.info"
 
+var pluginMode bool
+
+func SetPluginMode(value bool) {
+	pluginMode = value
+}
+
+func IsPluginMode() bool {
+	return pluginMode
+}
+
 func Convert2Pod(obj interface{}) (*v1.Pod, error) {
 	pod, ok := obj.(*v1.Pod)
 	if !ok {
@@ -57,7 +65,7 @@ func Convert2ConfigMap(obj interface{}) *v1.ConfigMap {
 	if configmap, ok := obj.(*v1.ConfigMap); ok {
 		return configmap
 	}
-	log.Logger().Warn("cannot convert to *v1.ConfigMap", zap.Stringer("type", reflect.TypeOf(obj)))
+	log.Log(log.ShimUtils).Warn("cannot convert to *v1.ConfigMap", zap.Stringer("type", reflect.TypeOf(obj)))
 	return nil
 }
 
@@ -65,7 +73,7 @@ func Convert2PriorityClass(obj interface{}) *schedulingv1.PriorityClass {
 	if priorityClass, ok := obj.(*schedulingv1.PriorityClass); ok {
 		return priorityClass
 	}
-	log.Logger().Warn("cannot convert to *schedulingv1.PriorityClass", zap.Stringer("type", reflect.TypeOf(obj)))
+	log.Log(log.ShimUtils).Warn("cannot convert to *schedulingv1.PriorityClass", zap.Stringer("type", reflect.TypeOf(obj)))
 	return nil
 }
 
@@ -104,35 +112,62 @@ func GetQueueNameFromPod(pod *v1.Pod) string {
 	return queueName
 }
 
-// GetApplicationIDFromPod returns the applicationID (if present) from a Pod or an empty string if not present.
-// If an applicationID is present, the Pod is managed by YuniKorn. Otherwise, it is managed by an external scheduler.
+// GenerateApplicationID generates an appID based on the namespace value
+// if configured to generate unique appID, generate appID as <namespace>-<pod-uid> namespace capped at 26chars
+// if not set or configured as false, appID generated as <autogen-prefix>-<namespace>-<autogen-suffix>
+func GenerateApplicationID(namespace string, generateUniqueAppIds bool, podUID string) string {
+	var generatedID string
+	if generateUniqueAppIds {
+		generatedID = fmt.Sprintf("%.26s-%s", namespace, podUID)
+	} else {
+		generatedID = fmt.Sprintf("%s-%s-%s", constants.AutoGenAppPrefix, namespace, constants.AutoGenAppSuffix)
+	}
+
+	return fmt.Sprintf("%.63s", generatedID)
+}
+
+// GetApplicationIDFromPod returns the Application for a Pod. If a Pod is marked as schedulable by YuniKorn but is
+// missing an ApplicationID, one will be generated here (if YuniKorn is running in standard mode) or an empty string
+// will be returned (if YuniKorn is running in plugin mode).
+// If an Application ID is returned, the Pod is managed by YuniKorn. Otherwise, it is managed by an external scheduler.
 func GetApplicationIDFromPod(pod *v1.Pod) string {
 	// SchedulerName needs to match
 	if strings.Compare(pod.Spec.SchedulerName, constants.SchedulerName) != 0 {
 		return ""
 	}
-	// if pod was tagged with ignore-application, return
-	if value := GetPodAnnotationValue(pod, constants.AnnotationIgnoreApplication); value != "" {
-		ignore, err := strconv.ParseBool(value)
-		if err != nil {
-			log.Logger().Warn("Failed to parse annotation "+constants.AnnotationIgnoreApplication, zap.Error(err))
-		} else if ignore {
-			return ""
+
+	// If pod was tagged with ignore-application and plugin mode is active, return
+	if pluginMode {
+		if value := GetPodAnnotationValue(pod, constants.AnnotationIgnoreApplication); value != "" {
+			ignore, err := strconv.ParseBool(value)
+			if err != nil {
+				log.Log(log.ShimUtils).Warn("Failed to parse annotation "+constants.AnnotationIgnoreApplication, zap.Error(err))
+			} else if ignore {
+				return ""
+			}
 		}
 	}
-	// application ID can be defined in annotations
+
+	// Application ID can be defined in annotation
 	if value := GetPodAnnotationValue(pod, constants.AnnotationApplicationID); value != "" {
 		return value
 	}
+	// Application ID can be defined in label
 	if value := GetPodLabelValue(pod, constants.LabelApplicationID); value != "" {
 		return value
 	}
-	// application ID can be defined in labels
+	// Spark can also define application ID
 	if value := GetPodLabelValue(pod, constants.SparkLabelAppID); value != "" {
 		return value
 	}
-	// no application ID found, this is not a YuniKorn-managed Pod
-	return ""
+
+	// If plugin mode, interpret missing Application ID as a non-YuniKorn pod
+	if pluginMode {
+		return ""
+	}
+
+	// Standard deployment mode, so we need a valid Application ID to proceed. Generate one now.
+	return GenerateApplicationID(pod.Namespace, conf.GetSchedulerConf().GenerateUniqueAppIds, string(pod.UID))
 }
 
 // compare the existing pod condition with the given one, return true if the pod condition remains not changed.
@@ -142,6 +177,21 @@ func PodUnderCondition(pod *v1.Pod, condition *v1.PodCondition) bool {
 	return current != nil && current.Status == condition.Status && current.Reason == condition.Reason
 }
 
+// get namespace guaranteed resource from namespace annotation
+func GetNamespaceGuaranteedFromAnnotation(namespaceObj *v1.Namespace) *si.Resource {
+	// retrieve guaranteed resource info from annotations
+	namespaceGuaranteed := GetNameSpaceAnnotationValue(namespaceObj, constants.NamespaceGuaranteed)
+	var namespaceGuaranteedMap map[string]string
+	err := json.Unmarshal([]byte(namespaceGuaranteed), &namespaceGuaranteedMap)
+	if err != nil {
+		log.Log(log.ShimUtils).Warn("Unable to process namespace.guaranteed annotation",
+			zap.String("namespace", namespaceObj.Name),
+			zap.String("namespace.guaranteed is", namespaceGuaranteed))
+		return nil
+	}
+	return common.GetResource(namespaceGuaranteedMap)
+}
+
 func GetNamespaceQuotaFromAnnotation(namespaceObj *v1.Namespace) *si.Resource {
 	// retrieve resource quota info from annotations
 	cpuQuota := GetNameSpaceAnnotationValue(namespaceObj, constants.CPUQuota)
@@ -154,20 +204,20 @@ func GetNamespaceQuotaFromAnnotation(namespaceObj *v1.Namespace) *si.Resource {
 	switch {
 	case namespaceQuota != "":
 		if cpuQuota != "" || memQuota != "" {
-			log.Logger().Warn("Using namespace.quota instead of namespace.max.* (deprecated) annotation to set cpu and/or memory for namespace though both are available.",
+			log.Log(log.ShimUtils).Warn("Using namespace.quota instead of namespace.max.* (deprecated) annotation to set cpu and/or memory for namespace though both are available.",
 				zap.String("namespace", namespaceObj.Name))
 		}
 		var namespaceQuotaMap map[string]string
 		err := json.Unmarshal([]byte(namespaceQuota), &namespaceQuotaMap)
 		if err != nil {
-			log.Logger().Warn("Unable to process namespace.quota annotation",
+			log.Log(log.ShimUtils).Warn("Unable to process namespace.quota annotation",
 				zap.String("namespace", namespaceObj.Name),
-				zap.String("namespace.quota val is", namespaceQuota))
+				zap.String("namespace.quota is", namespaceQuota))
 			return nil
 		}
 		return common.GetResource(namespaceQuotaMap)
 	case cpuQuota != "" || memQuota != "":
-		log.Logger().Warn("Please use namespace.quota instead of namespace.max.* (deprecated) annotation. Using deprecated annotation to set cpu and/or memory for namespace. ",
+		log.Log(log.ShimUtils).Warn("Please use namespace.quota instead of namespace.max.* (deprecated) annotation. Using deprecated annotation to set cpu and/or memory for namespace. ",
 			zap.String("namespace", namespaceObj.Name))
 		return common.ParseResource(cpuQuota, memQuota)
 	default:
@@ -175,19 +225,6 @@ func GetNamespaceQuotaFromAnnotation(namespaceObj *v1.Namespace) *si.Resource {
 	}
 }
 
-type K8sResource struct {
-	ResourceName v1.ResourceName
-	Value        int64
-}
-
-func NewK8sResourceList(resources ...K8sResource) map[v1.ResourceName]resource.Quantity {
-	resourceList := make(map[v1.ResourceName]resource.Quantity)
-	for _, r := range resources {
-		resourceList[r.ResourceName] = *resource.NewQuantity(r.Value, resource.DecimalSI)
-	}
-	return resourceList
-}
-
 func WaitForCondition(eval func() bool, interval time.Duration, timeout time.Duration) error {
 	deadline := time.Now().Add(timeout)
 	for {
@@ -203,53 +240,6 @@ func WaitForCondition(eval func() bool, interval time.Duration, timeout time.Dur
 	}
 }
 
-func PodForTest(podName, memory, cpu string) *v1.Pod {
-	containers := make([]v1.Container, 0)
-	c1Resources := make(map[v1.ResourceName]resource.Quantity)
-	c1Resources[v1.ResourceMemory] = resource.MustParse(memory)
-	c1Resources[v1.ResourceCPU] = resource.MustParse(cpu)
-	containers = append(containers, v1.Container{
-		Name: "container-01",
-		Resources: v1.ResourceRequirements{
-			Requests: c1Resources,
-		},
-	})
-
-	return &v1.Pod{
-		TypeMeta: apis.TypeMeta{
-			Kind:       "Pod",
-			APIVersion: "v1",
-		},
-		ObjectMeta: apis.ObjectMeta{
-			Name: podName,
-		},
-		Spec: v1.PodSpec{
-			Containers: containers,
-		},
-	}
-}
-
-func NodeForTest(nodeID, memory, cpu string) *v1.Node {
-	resourceList := make(map[v1.ResourceName]resource.Quantity)
-	resourceList[v1.ResourceName("memory")] = resource.MustParse(memory)
-	resourceList[v1.ResourceName("cpu")] = resource.MustParse(cpu)
-	return &v1.Node{
-		TypeMeta: apis.TypeMeta{
-			Kind:       "Node",
-			APIVersion: "v1",
-		},
-		ObjectMeta: apis.ObjectMeta{
-			Name:      nodeID,
-			Namespace: "default",
-			UID:       "uid_0001",
-		},
-		Spec: v1.NodeSpec{},
-		Status: v1.NodeStatus{
-			Allocatable: resourceList,
-		},
-	}
-}
-
 // merge two string maps
 // if the same key defined in the first and second maps
 // the value will be set by the second map
@@ -274,16 +264,16 @@ func GetUserFromPod(pod *v1.Pod) (string, []string) {
 		var userGroup si.UserGroupInformation
 		err := json.Unmarshal([]byte(userInfoJSON), &userGroup)
 		if err != nil {
-			log.Logger().Error("unable to process user info annotation", zap.Error(err))
+			log.Log(log.ShimUtils).Error("unable to process user info annotation", zap.Error(err))
 			return constants.DefaultUser, nil
 		}
 		user := userGroup.User
 		groups := userGroup.Groups
 		if user == "" {
-			log.Logger().Warn("got empty username, using default")
+			log.Log(log.ShimUtils).Warn("got empty username, using default")
 			user = constants.DefaultUser
 		}
-		log.Logger().Info("found user info from pod annotations",
+		log.Log(log.ShimUtils).Info("found user info from pod annotations",
 			zap.String("username", user), zap.Strings("groups", groups))
 		return user, groups
 	}
@@ -296,13 +286,13 @@ func GetUserFromPod(pod *v1.Pod) (string, []string) {
 	}
 	// User name to be defined in labels
 	if username := GetPodLabelValue(pod, userLabelKey); username != "" && len(username) > 0 {
-		log.Logger().Info("Found user name from pod labels.",
+		log.Log(log.ShimUtils).Info("Found user name from pod labels.",
 			zap.String("userLabel", userLabelKey), zap.String("user", username))
 		return username, nil
 	}
 	value := constants.DefaultUser
 
-	log.Logger().Debug("Unable to retrieve user name from pod labels. Empty user label",
+	log.Log(log.ShimUtils).Debug("Unable to retrieve user name from pod labels. Empty user label",
 		zap.String("userLabel", userLabelKey))
 
 	return value, nil
@@ -374,13 +364,13 @@ func GetPlaceholderFlagFromPodSpec(pod *v1.Pod) bool {
 	return false
 }
 
-func GetTaskGroupsFromAnnotation(pod *v1.Pod) ([]v1alpha1.TaskGroup, error) {
+func GetTaskGroupsFromAnnotation(pod *v1.Pod) ([]interfaces.TaskGroup, error) {
 	taskGroupInfo := GetPodAnnotationValue(pod, constants.AnnotationTaskGroups)
 	if taskGroupInfo == "" {
 		return nil, nil
 	}
 
-	taskGroups := []v1alpha1.TaskGroup{}
+	taskGroups := []interfaces.TaskGroup{}
 	err := json.Unmarshal([]byte(taskGroupInfo), &taskGroups)
 	if err != nil {
 		return nil, err
