@@ -19,10 +19,14 @@
 package conf
 
 import (
+	"bytes"
+	"compress/gzip"
+	"encoding/base64"
 	"encoding/json"
 	"errors"
 	"flag"
 	"fmt"
+	"io"
 	"os"
 	"strconv"
 	"strings"
@@ -33,7 +37,7 @@ import (
 	"go.uber.org/zap"
 	"go.uber.org/zap/zapcore"
 	v1 "k8s.io/api/core/v1"
-	"k8s.io/klog"
+	"k8s.io/klog/v2"
 
 	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
 	"github.com/apache/yunikorn-k8shim/pkg/log"
@@ -48,9 +52,10 @@ const (
 	EnvNamespace  = "NAMESPACE"
 
 	// prefixes
-	PrefixService    = "service."
-	PrefixLog        = "log."
-	PrefixKubernetes = "kubernetes."
+	PrefixService             = "service."
+	PrefixLog                 = "log."
+	PrefixKubernetes          = "kubernetes."
+	PrefixAdmissionController = "admissionController."
 
 	// service
 	CMSvcClusterID                    = PrefixService + "clusterId"
@@ -65,39 +70,39 @@ const (
 	CMSvcPlaceholderImage             = PrefixService + "placeholderImage"
 	CMSvcNodeInstanceTypeNodeLabelKey = PrefixService + "nodeInstanceTypeNodeLabelKey"
 
-	// log
-	CMLogLevel = PrefixLog + "level"
-
 	// kubernetes
 	CMKubeQPS   = PrefixKubernetes + "qps"
 	CMKubeBurst = PrefixKubernetes + "burst"
 
+	// admissioncontroller
+	PrefixAMFiltering               = PrefixAdmissionController + "filtering."
+	AMFilteringGenerateUniqueAppIds = PrefixAMFiltering + "generateUniqueAppId"
+
 	// defaults
-	DefaultNamespace              = "default"
-	DefaultClusterID              = "mycluster"
-	DefaultPolicyGroup            = "queues"
-	DefaultSchedulingInterval     = time.Second
-	DefaultVolumeBindTimeout      = 10 * time.Second
-	DefaultEventChannelCapacity   = 1024 * 1024
-	DefaultDispatchTimeout        = 300 * time.Second
-	DefaultOperatorPlugins        = "general"
-	DefaultDisableGangScheduling  = false
-	DefaultEnableConfigHotRefresh = true
-	DefaultLoggingLevel           = 0
-	DefaultLogEncoding            = "console"
-	DefaultKubeQPS                = 1000
-	DefaultKubeBurst              = 1000
+	DefaultNamespace                       = "default"
+	DefaultClusterID                       = "mycluster"
+	DefaultPolicyGroup                     = "queues"
+	DefaultSchedulingInterval              = time.Second
+	DefaultVolumeBindTimeout               = 10 * time.Second
+	DefaultEventChannelCapacity            = 1024 * 1024
+	DefaultDispatchTimeout                 = 300 * time.Second
+	DefaultOperatorPlugins                 = "general"
+	DefaultDisableGangScheduling           = false
+	DefaultEnableConfigHotRefresh          = true
+	DefaultKubeQPS                         = 1000
+	DefaultKubeBurst                       = 1000
+	DefaultAMFilteringGenerateUniqueAppIds = false
 )
 
 var (
-	BuildVersion    string
-	BuildDate       string
-	IsPluginVersion bool
-	GoVersion       string
-	Arch            string
-	CoreSHA         string
-	SiSHA           string
-	ShimSHA         string
+	buildVersion    string
+	buildDate       string
+	isPluginVersion string
+	goVersion       string
+	arch            string
+	coreSHA         string
+	siSHA           string
+	shimSHA         string
 )
 
 var once sync.Once
@@ -112,7 +117,6 @@ type SchedulerConf struct {
 	PolicyGroup              string        `json:"policyGroup"`
 	Interval                 time.Duration `json:"schedulingIntervalSecond"`
 	KubeConfig               string        `json:"absoluteKubeConfigFilePath"`
-	LoggingLevel             int           `json:"loggingLevel"`
 	VolumeBindTimeout        time.Duration `json:"volumeBindTimeout"`
 	TestMode                 bool          `json:"testMode"`
 	EventChannelCapacity     int           `json:"eventChannelCapacity"`
@@ -126,6 +130,7 @@ type SchedulerConf struct {
 	PlaceHolderImage         string        `json:"placeHolderImage"`
 	InstanceTypeNodeLabelKey string        `json:"instanceTypeNodeLabelKey"`
 	Namespace                string        `json:"namespace"`
+	GenerateUniqueAppIds     bool          `json:"generateUniqueAppIds"`
 	sync.RWMutex
 }
 
@@ -134,30 +139,31 @@ func (conf *SchedulerConf) Clone() *SchedulerConf {
 	defer conf.RUnlock()
 
 	return &SchedulerConf{
-		SchedulerName:          conf.SchedulerName,
-		ClusterID:              conf.ClusterID,
-		ClusterVersion:         conf.ClusterVersion,
-		PolicyGroup:            conf.PolicyGroup,
-		Interval:               conf.Interval,
-		KubeConfig:             conf.KubeConfig,
-		LoggingLevel:           conf.LoggingLevel,
-		VolumeBindTimeout:      conf.VolumeBindTimeout,
-		TestMode:               conf.TestMode,
-		EventChannelCapacity:   conf.EventChannelCapacity,
-		DispatchTimeout:        conf.DispatchTimeout,
-		KubeQPS:                conf.KubeQPS,
-		KubeBurst:              conf.KubeBurst,
-		OperatorPlugins:        conf.OperatorPlugins,
-		EnableConfigHotRefresh: conf.EnableConfigHotRefresh,
-		DisableGangScheduling:  conf.DisableGangScheduling,
-		UserLabelKey:           conf.UserLabelKey,
-		PlaceHolderImage:       conf.PlaceHolderImage,
-		Namespace:              conf.Namespace,
+		SchedulerName:            conf.SchedulerName,
+		ClusterID:                conf.ClusterID,
+		ClusterVersion:           conf.ClusterVersion,
+		PolicyGroup:              conf.PolicyGroup,
+		Interval:                 conf.Interval,
+		KubeConfig:               conf.KubeConfig,
+		VolumeBindTimeout:        conf.VolumeBindTimeout,
+		TestMode:                 conf.TestMode,
+		EventChannelCapacity:     conf.EventChannelCapacity,
+		DispatchTimeout:          conf.DispatchTimeout,
+		KubeQPS:                  conf.KubeQPS,
+		KubeBurst:                conf.KubeBurst,
+		OperatorPlugins:          conf.OperatorPlugins,
+		EnableConfigHotRefresh:   conf.EnableConfigHotRefresh,
+		DisableGangScheduling:    conf.DisableGangScheduling,
+		UserLabelKey:             conf.UserLabelKey,
+		PlaceHolderImage:         conf.PlaceHolderImage,
+		InstanceTypeNodeLabelKey: conf.InstanceTypeNodeLabelKey,
+		Namespace:                conf.Namespace,
+		GenerateUniqueAppIds:     conf.GenerateUniqueAppIds,
 	}
 }
 
 func UpdateConfigMaps(configMaps []*v1.ConfigMap, initial bool) error {
-	log.Logger().Info("reloading configuration")
+	log.Log(log.ShimConfig).Info("reloading configuration")
 
 	// start with defaults
 	prev := CreateDefaultConfig()
@@ -169,7 +175,7 @@ func UpdateConfigMaps(configMaps []*v1.ConfigMap, initial bool) error {
 	newConf, cmErrors := parseConfig(config, prev)
 	if cmErrors != nil {
 		for _, err := range cmErrors {
-			log.Logger().Error("failed to parse configmap entry", zap.Error(err))
+			log.Log(log.ShimConfig).Error("failed to parse configmap entry", zap.Error(err))
 		}
 		return errors.New("failed to load configmap")
 	}
@@ -182,13 +188,13 @@ func UpdateConfigMaps(configMaps []*v1.ConfigMap, initial bool) error {
 
 	// update scheduler config with merged version
 	SetSchedulerConf(newConf)
-	conf := GetSchedulerConf()
+	_ = GetSchedulerConf()
 
 	// update logger configuration
-	log.GetZapConfigs().Level.SetLevel(zapcore.Level(conf.LoggingLevel))
+	log.UpdateLoggingConfig(config)
 
 	// update Kubernetes logger configuration
-	updateKubeLogger(conf)
+	updateKubeLogger()
 
 	// dump new scheduler configuration
 	DumpConfiguration()
@@ -210,34 +216,35 @@ func handleNonReloadableConfig(old *SchedulerConf, new *SchedulerConf) {
 	checkNonReloadableBool(CMSvcDisableGangScheduling, &old.DisableGangScheduling, &new.DisableGangScheduling)
 	checkNonReloadableString(CMSvcPlaceholderImage, &old.PlaceHolderImage, &new.PlaceHolderImage)
 	checkNonReloadableString(CMSvcNodeInstanceTypeNodeLabelKey, &old.InstanceTypeNodeLabelKey, &new.InstanceTypeNodeLabelKey)
+	checkNonReloadableBool(AMFilteringGenerateUniqueAppIds, &old.GenerateUniqueAppIds, &new.GenerateUniqueAppIds)
 }
 
 const warningNonReloadable = "ignoring non-reloadable configuration change (restart required to update)"
 
 func checkNonReloadableString(name string, old *string, new *string) {
 	if *old != *new {
-		log.Logger().Warn(warningNonReloadable, zap.String("config", name), zap.String("existing", *old), zap.String("new", *new))
+		log.Log(log.ShimConfig).Warn(warningNonReloadable, zap.String("config", name), zap.String("existing", *old), zap.String("new", *new))
 		*new = *old
 	}
 }
 
 func checkNonReloadableDuration(name string, old *time.Duration, new *time.Duration) {
 	if *old != *new {
-		log.Logger().Warn(warningNonReloadable, zap.String("config", name), zap.Duration("existing", *old), zap.Duration("new", *new))
+		log.Log(log.ShimConfig).Warn(warningNonReloadable, zap.String("config", name), zap.Duration("existing", *old), zap.Duration("new", *new))
 		*new = *old
 	}
 }
 
 func checkNonReloadableInt(name string, old *int, new *int) {
 	if *old != *new {
-		log.Logger().Warn(warningNonReloadable, zap.String("config", name), zap.Int("existing", *old), zap.Int("new", *new))
+		log.Log(log.ShimConfig).Warn(warningNonReloadable, zap.String("config", name), zap.Int("existing", *old), zap.Int("new", *new))
 		*new = *old
 	}
 }
 
 func checkNonReloadableBool(name string, old *bool, new *bool) {
 	if *old != *new {
-		log.Logger().Warn(warningNonReloadable, zap.String("config", name), zap.Bool("existing", *old), zap.Bool("new", *new))
+		log.Log(log.ShimConfig).Warn(warningNonReloadable, zap.String("config", name), zap.Bool("existing", *old), zap.Bool("new", *new))
 		*new = *old
 	}
 }
@@ -323,11 +330,10 @@ func CreateDefaultConfig() *SchedulerConf {
 		SchedulerName:            constants.SchedulerName,
 		Namespace:                GetSchedulerNamespace(),
 		ClusterID:                DefaultClusterID,
-		ClusterVersion:           BuildVersion,
+		ClusterVersion:           buildVersion,
 		PolicyGroup:              DefaultPolicyGroup,
 		Interval:                 DefaultSchedulingInterval,
 		KubeConfig:               GetDefaultKubeConfigPath(),
-		LoggingLevel:             DefaultLoggingLevel,
 		VolumeBindTimeout:        DefaultVolumeBindTimeout,
 		TestMode:                 false,
 		EventChannelCapacity:     DefaultEventChannelCapacity,
@@ -340,6 +346,7 @@ func CreateDefaultConfig() *SchedulerConf {
 		UserLabelKey:             constants.DefaultUserLabel,
 		PlaceHolderImage:         constants.PlaceholderContainerImage,
 		InstanceTypeNodeLabelKey: constants.DefaultNodeInstanceTypeNodeLabelKey,
+		GenerateUniqueAppIds:     DefaultAMFilteringGenerateUniqueAppIds,
 	}
 }
 
@@ -366,13 +373,13 @@ func parseConfig(config map[string]string, prev *SchedulerConf) (*SchedulerConf,
 	parser.stringVar(&conf.PlaceHolderImage, CMSvcPlaceholderImage)
 	parser.stringVar(&conf.InstanceTypeNodeLabelKey, CMSvcNodeInstanceTypeNodeLabelKey)
 
-	// log
-	parser.intVar(&conf.LoggingLevel, CMLogLevel)
-
 	// kubernetes
 	parser.intVar(&conf.KubeQPS, CMKubeQPS)
 	parser.intVar(&conf.KubeBurst, CMKubeBurst)
 
+	// admission controller
+	parser.boolVar(&conf.GenerateUniqueAppIds, AMFilteringGenerateUniqueAppIds)
+
 	if len(parser.errors) > 0 {
 		return nil, parser.errors
 	}
@@ -402,7 +409,7 @@ func (cp *configParser) intVar(p *int, name string) {
 		int64Value, err := strconv.ParseInt(newValue, 10, 32)
 		intValue := int(int64Value)
 		if err != nil {
-			log.Logger().Error("Unable to parse configmap entry", zap.String("key", name), zap.String("value", newValue), zap.Error(err))
+			log.Log(log.ShimConfig).Error("Unable to parse configmap entry", zap.String("key", name), zap.String("value", newValue), zap.Error(err))
 			cp.errors = append(cp.errors, err)
 			return
 		}
@@ -414,7 +421,7 @@ func (cp *configParser) boolVar(p *bool, name string) {
 	if newValue, ok := cp.config[name]; ok {
 		boolValue, err := strconv.ParseBool(newValue)
 		if err != nil {
-			log.Logger().Error("Unable to parse configmap entry", zap.String("key", name), zap.String("value", newValue), zap.Error(err))
+			log.Log(log.ShimConfig).Error("Unable to parse configmap entry", zap.String("key", name), zap.String("value", newValue), zap.Error(err))
 			cp.errors = append(cp.errors, err)
 			return
 		}
@@ -426,7 +433,7 @@ func (cp *configParser) durationVar(p *time.Duration, name string) {
 	if newValue, ok := cp.config[name]; ok {
 		durationValue, err := time.ParseDuration(newValue)
 		if err != nil {
-			log.Logger().Error("Unable to parse configmap entry", zap.String("key", name), zap.String("value", newValue), zap.Error(err))
+			log.Log(log.ShimConfig).Error("Unable to parse configmap entry", zap.String("key", name), zap.String("value", newValue), zap.Error(err))
 			cp.errors = append(cp.errors, err)
 			return
 		}
@@ -434,14 +441,14 @@ func (cp *configParser) durationVar(p *time.Duration, name string) {
 	}
 }
 
-func updateKubeLogger(conf *SchedulerConf) {
+func updateKubeLogger() {
 	// if log level is debug, enable klog and set its log level verbosity to 4 (represents debug level),
 	// For details refer to the Logging Conventions of klog at
 	// https://github.com/kubernetes/community/blob/master/contributors/devel/sig-instrumentation/logging.md
 
 	// danger, this can only be called once!
 	kubeLoggerOnce.Do(func() {
-		if zapcore.Level(conf.LoggingLevel).Enabled(zapcore.DebugLevel) {
+		if log.Log(log.Kubernetes).Core().Enabled(zapcore.DebugLevel) {
 			klog.InitFlags(nil)
 			// cannot really handle the error here ignore it
 			//nolint:errcheck
@@ -454,7 +461,7 @@ func DumpConfiguration() {
 	configs := GetSchedulerConf()
 	c, err := json.MarshalIndent(configs, "", " ")
 
-	logger := log.Logger()
+	logger := log.Log(log.ShimConfig)
 
 	//nolint:errcheck
 	defer logger.Sync()
@@ -466,6 +473,40 @@ func DumpConfiguration() {
 	}
 }
 
+func Decompress(key string, value []byte) (string, string) {
+	var uncompressedData string
+	decodedValue := make([]byte, base64.StdEncoding.DecodedLen(len(value)))
+	n, err := base64.StdEncoding.Decode(decodedValue, value)
+	if err != nil {
+		log.Log(log.ShimConfig).Error("failed to decode schedulerConfig entry", zap.Error(err))
+		return "", ""
+	}
+	decodedValue = decodedValue[:n]
+	splitKey := strings.Split(key, ".")
+	compressionAlgo := splitKey[len(splitKey)-1]
+	if strings.EqualFold(compressionAlgo, constants.GzipSuffix) {
+		reader := bytes.NewReader(decodedValue)
+		gzReader, err := gzip.NewReader(reader)
+		if err != nil {
+			log.Log(log.ShimConfig).Error("failed to decompress decoded schedulerConfig entry", zap.Error(err))
+			return "", ""
+		}
+		defer func() {
+			if err = gzReader.Close(); err != nil {
+				log.Log(log.ShimConfig).Debug("gzip Reader could not be closed ", zap.Error(err))
+			}
+		}()
+		decompressedBytes, err := io.ReadAll(gzReader)
+		if err != nil {
+			log.Log(log.ShimConfig).Error("failed to decompress decoded schedulerConfig entry", zap.Error(err))
+			return "", ""
+		}
+		uncompressedData = string(decompressedBytes)
+	}
+	strippedKey, _ := strings.CutSuffix(key, "."+compressionAlgo)
+	return strippedKey, uncompressedData
+}
+
 func FlattenConfigMaps(configMaps []*v1.ConfigMap) map[string]string {
 	result := make(map[string]string)
 	for _, configMap := range configMaps {
@@ -473,7 +514,31 @@ func FlattenConfigMaps(configMaps []*v1.ConfigMap) map[string]string {
 			for k, v := range configMap.Data {
 				result[k] = v
 			}
+			for k, v := range configMap.BinaryData {
+				strippedKey, uncompressedData := Decompress(k, v)
+				result[strippedKey] = uncompressedData
+			}
 		}
 	}
 	return result
 }
+
+func GetBuildInfoMap() map[string]string {
+	return map[string]string{
+		"buildVersion":    buildVersion,
+		"buildDate":       buildDate,
+		"isPluginVersion": isPluginVersion,
+		"goVersion":       goVersion,
+		"arch":            arch,
+		"coreSHA":         coreSHA,
+		"siSHA":           siSHA,
+		"shimSHA":         shimSHA,
+	}
+}
+
+func GetBuildInfoString() string {
+	return fmt.Sprintf(
+		"Build info: version=%s date=%s isPluginVersion=%s goVersion=%s arch=%s coreSHA=%s siSHA=%s shimSHA=%s",
+		buildVersion, buildDate, isPluginVersion, goVersion, arch, coreSHA, siSHA, shimSHA,
+	)
+}
