@@ -43,9 +43,9 @@ import (
 	"k8s.io/kubernetes/pkg/scheduler/framework/plugins/podtopologyspread"
 	"k8s.io/kubernetes/pkg/util/taints"
 
-	"github.com/apache/yunikorn-core/pkg/log"
 	"github.com/apache/yunikorn-k8shim/pkg/client"
 	"github.com/apache/yunikorn-k8shim/pkg/conf"
+	"github.com/apache/yunikorn-k8shim/pkg/log"
 	"github.com/apache/yunikorn-k8shim/pkg/plugin/support"
 )
 
@@ -61,7 +61,7 @@ func TestPreemptionPredicatesEmpty(t *testing.T) {
 	lister := lister()
 	handle := support.NewFrameworkHandle(lister, informerFactory, clientSet)
 
-	ep := enabledPlugins(noderesources.FitName)
+	ep := enabledPlugins(noderesources.Name)
 	predicateManager := newPredicateManagerInternal(handle, ep, ep, ep, ep)
 
 	pod := &v1.Pod{}
@@ -80,7 +80,7 @@ func TestPreemptionPredicates(t *testing.T) {
 	lister := lister()
 	handle := support.NewFrameworkHandle(lister, informerFactory, clientSet)
 
-	ep := enabledPlugins(noderesources.FitName)
+	ep := enabledPlugins(noderesources.Name)
 	predicateManager := newPredicateManagerInternal(handle, ep, ep, ep, ep)
 
 	pod := newResourcePod(framework.Resource{MilliCPU: 500, Memory: 5000000})
@@ -128,6 +128,24 @@ func TestPreemptionPredicates(t *testing.T) {
 	assert.Equal(t, index, -1, "wrong index")
 }
 
+func TestEventsToRegister(t *testing.T) {
+	conf.GetSchedulerConf().SetTestMode(true)
+	clientSet := clientSet()
+	informerFactory := informerFactory(clientSet)
+	lister := lister()
+	handle := support.NewFrameworkHandle(lister, informerFactory, clientSet)
+
+	ep := enabledPlugins(nodename.Name, interpodaffinity.Name, podtopologyspread.Name)
+	predicateManager := newPredicateManagerInternal(handle, ep, ep, ep, ep)
+
+	events := predicateManager.EventsToRegister()
+	assert.Equal(t, 2, len(events), "wrong event count")
+	assert.Equal(t, events[0].Resource, framework.Node, "wrong resource (0)")
+	assert.Equal(t, events[0].ActionType, framework.All, "wrong action type (0)")
+	assert.Equal(t, events[1].Resource, framework.Pod, "wrong resource (1)")
+	assert.Equal(t, events[1].ActionType, framework.All, "wrong action type (1)")
+}
+
 func TestPodFitsHost(t *testing.T) {
 	conf.GetSchedulerConf().SetTestMode(true)
 	clientSet := clientSet()
@@ -1104,7 +1122,7 @@ func TestRunGeneralPredicates(t *testing.T) {
 	informerFactory := informerFactory(clientSet)
 	lister := lister()
 	handle := support.NewFrameworkHandle(lister, informerFactory, clientSet)
-	ep := enabledPlugins(noderesources.FitName, nodename.Name, nodeports.Name, nodevolumelimits.CSIName)
+	ep := enabledPlugins(noderesources.Name, nodename.Name, nodeports.Name, nodevolumelimits.CSIName)
 	predicateManager := newPredicateManagerInternal(handle, ep, ep, ep, ep)
 
 	resourceTests := []struct {
@@ -2206,7 +2224,7 @@ func TestReserveNodeSelector(t *testing.T) {
 			pod.Spec.NodeSelector = tc.nodeSelectors
 			node.Labels = tc.nodeLabels
 			plugin, err := predicateManager.Predicates(pod, nodeInfo, false)
-			log.Logger().Info("reservation predicates called", zap.Error(err), zap.String("plugin", plugin))
+			log.Log(log.Test).Info("reservation predicates called", zap.Error(err), zap.String("plugin", plugin))
 			if tc.errorExpected {
 				assert.Assert(t, err != nil, "An error is expected")
 			} else {
@@ -2241,13 +2259,18 @@ func enabledPlugins(name ...string) map[string]bool {
 }
 
 type sharedListerMock struct {
-	nodeLister *nodeListerMock
+	nodeLister    *nodeListerMock
+	storageLister *storageListerMock
 }
 
 func (s *sharedListerMock) NodeInfos() framework.NodeInfoLister {
 	return s.nodeLister
 }
 
+func (s *sharedListerMock) StorageInfos() framework.StorageInfoLister {
+	return s.storageLister
+}
+
 type nodeListerMock struct {
 	nodeInfos []*framework.NodeInfo
 }
@@ -2285,5 +2308,13 @@ func (n *nodeListerMock) Get(nodeName string) (*framework.NodeInfo, error) {
 	return nil, errors.New("node not found")
 }
 
+type storageListerMock struct {
+}
+
+func (s *storageListerMock) IsPVCUsedByPods(key string) bool {
+	return false
+}
+
 var _ framework.SharedLister = &sharedListerMock{}
 var _ framework.NodeInfoLister = &nodeListerMock{}
+var _ framework.StorageInfoLister = &storageListerMock{}
