@@ -36,36 +36,11 @@ import (
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/yunikorn"
 )
 
-const userInfoAnnotation = "yunikorn.apache.org/user.info"
+const userInfoAnnotation = constants.DomainYuniKorn + "user.info"
 const nonExistentNode = "non-existent-node"
 const defaultPodTimeout = 10 * time.Second
 const cronJobPodTimeout = 65 * time.Second
 
-type EventHandler struct {
-	updateCh chan struct{}
-}
-
-func (e *EventHandler) OnAdd(_ interface{}) {}
-
-func (e *EventHandler) OnUpdate(_, _ interface{}) {
-	e.updateCh <- struct{}{}
-}
-
-func (e *EventHandler) OnDelete(_ interface{}) {}
-
-func (e *EventHandler) WaitForUpdate(timeout time.Duration) bool {
-	t := time.After(timeout)
-
-	for {
-		select {
-		case <-t:
-			return false
-		case <-e.updateCh:
-			return true
-		}
-	}
-}
-
 var _ = ginkgo.Describe("AdmissionController", func() {
 	ginkgo.BeforeEach(func() {
 		kubeClient = k8s.KubeCtl{}
@@ -222,11 +197,11 @@ var _ = ginkgo.Describe("AdmissionController", func() {
 		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
 		pod, err = kubeClient.GetPod(pod.Name, ns)
 		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
-		userinfo := pod.Annotations["yunikorn.apache.org/user.info"]
+		userinfo := pod.Annotations[constants.DomainYuniKorn+"user.info"]
 		gomega.Ω(userinfo).Should(gomega.Not(gomega.BeNil()))
 
 		ginkgo.By("Attempt to update userinfo annotation")
-		_, err = kubeClient.UpdatePodWithAnnotation(pod, ns, "yunikorn.apache.org/user.info", "shouldnotsucceed")
+		_, err = kubeClient.UpdatePodWithAnnotation(pod, ns, constants.DomainYuniKorn+"user.info", "shouldnotsucceed")
 		gomega.Ω(err).Should(gomega.HaveOccurred())
 	})
 
@@ -317,16 +292,11 @@ var _ = ginkgo.Describe("AdmissionController", func() {
 		}
 		configMap.Data[amConf.AMAccessControlTrustControllers] = "false"
 		ginkgo.By("Update configmap")
-		stopChan := make(chan struct{})
-		eventHandler := &EventHandler{updateCh: make(chan struct{})}
-		err = kubeClient.StartConfigMapInformer(configmanager.YuniKornTestConfig.YkNamespace, stopChan, eventHandler)
-		defer close(stopChan)
-		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
-		_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
-		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
-		updateOk := eventHandler.WaitForUpdate(30 * time.Second)
-		gomega.Ω(updateOk).To(gomega.Equal(true))
-		time.Sleep(time.Second)
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
+			gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
+		})
 
 		ginkgo.By("Create a deployment")
 		deployment, err2 := kubeClient.CreateDeployment(&testDeployment, ns)
@@ -353,11 +323,11 @@ var _ = ginkgo.Describe("AdmissionController", func() {
 		configMap, err = kubeClient.GetConfigMap(constants.ConfigMapName, configmanager.YuniKornTestConfig.YkNamespace)
 		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
 		configMap.Data[amConf.AMAccessControlTrustControllers] = "true"
-		_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
-		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
-		updateOk = eventHandler.WaitForUpdate(30 * time.Second)
-		gomega.Ω(updateOk).To(gomega.Equal(true))
-		time.Sleep(time.Second)
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
+			gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
+		})
 
 		// pod is expected to appear
 		ginkgo.By("Check for sleep pod")
@@ -375,16 +345,11 @@ var _ = ginkgo.Describe("AdmissionController", func() {
 		}
 		configMap.Data[amConf.AMAccessControlExternalUsers] = ""
 		ginkgo.By("Update configmap")
-		stopChan := make(chan struct{})
-		eventHandler := &EventHandler{updateCh: make(chan struct{})}
-		err = kubeClient.StartConfigMapInformer(configmanager.YuniKornTestConfig.YkNamespace, stopChan, eventHandler)
-		defer close(stopChan)
-		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
-		_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
-		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
-		updateOk := eventHandler.WaitForUpdate(30 * time.Second)
-		gomega.Ω(updateOk).To(gomega.Equal(true))
-		time.Sleep(time.Second)
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
+			gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
+		})
 
 		ginkgo.By("Create a deployment")
 		deployment := testDeployment.DeepCopy()
@@ -400,11 +365,12 @@ var _ = ginkgo.Describe("AdmissionController", func() {
 		configMap, err = kubeClient.GetConfigMap(constants.ConfigMapName, configmanager.YuniKornTestConfig.YkNamespace)
 		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
 		configMap.Data[amConf.AMAccessControlExternalUsers] = "(^minikube-user$|^kubernetes-admin$)" // works with Minikube & KIND
-		_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
-		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
-		updateOk = eventHandler.WaitForUpdate(30 * time.Second)
-		gomega.Ω(updateOk).To(gomega.Equal(true))
-		time.Sleep(time.Second)
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
+			gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
+
+		})
 
 		// submit deployment again
 		ginkgo.By("Submit deployment again")
@@ -428,16 +394,11 @@ var _ = ginkgo.Describe("AdmissionController", func() {
 		}
 		configMap.Data[amConf.AMAccessControlBypassAuth] = "true"
 		ginkgo.By("Update configmap (bypassAuth -> true)")
-		stopChan := make(chan struct{})
-		eventHandler := &EventHandler{updateCh: make(chan struct{})}
-		err = kubeClient.StartConfigMapInformer(configmanager.YuniKornTestConfig.YkNamespace, stopChan, eventHandler)
-		defer close(stopChan)
-		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
-		_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
-		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
-		updateOk := eventHandler.WaitForUpdate(30 * time.Second)
-		gomega.Ω(updateOk).To(gomega.Equal(true))
-		time.Sleep(time.Second)
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
+			gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
+		})
 
 		ginkgo.By("Submit a deployment")
 		deployment := testDeployment.DeepCopy()
@@ -453,11 +414,11 @@ var _ = ginkgo.Describe("AdmissionController", func() {
 		configMap, err = kubeClient.GetConfigMap(constants.ConfigMapName, configmanager.YuniKornTestConfig.YkNamespace)
 		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
 		configMap.Data[amConf.AMAccessControlBypassAuth] = "false"
-		_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
-		gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
-		updateOk = eventHandler.WaitForUpdate(30 * time.Second)
-		gomega.Ω(updateOk).To(gomega.Equal(true))
-		time.Sleep(time.Second)
+		// The wait wrapper still can't fully guarantee that the config in AdmissionController has been updated.
+		yunikorn.WaitForAdmissionControllerRefreshConfAfterAction(func() {
+			_, err = kubeClient.UpdateConfigMap(configMap, configmanager.YuniKornTestConfig.YkNamespace)
+			gomega.Ω(err).ShouldNot(gomega.HaveOccurred())
+		})
 
 		ginkgo.By("Update container image in deployment")
 		deployment, err = kubeClient.GetDeployment(deployment.Name, ns)
@@ -514,7 +475,7 @@ func runWorkloadTest(workloadType k8s.WorkloadType, create func() (string, error
 	fmt.Fprintf(ginkgo.GinkgoWriter, "Running pod is %s\n", pods.Items[0].Name)
 	pod, err2 := kubeClient.GetPod(pods.Items[0].Name, ns)
 	gomega.Ω(err2).ShouldNot(gomega.HaveOccurred())
-	userinfo := pod.Annotations["yunikorn.apache.org/user.info"]
+	userinfo := pod.Annotations[constants.DomainYuniKorn+"user.info"]
 	gomega.Ω(userinfo).Should(gomega.Not(gomega.BeNil()))
 }
 
