@@ -21,61 +21,71 @@ package basicscheduling_test
 import (
 	"time"
 
-	"github.com/onsi/ginkgo"
+	"github.com/onsi/ginkgo/v2"
 	"github.com/onsi/gomega"
 	v1 "k8s.io/api/core/v1"
 
 	"github.com/apache/yunikorn-core/pkg/webservice/dao"
+	tests "github.com/apache/yunikorn-k8shim/test/e2e"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/common"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/k8s"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/yunikorn"
 )
 
+var kClient k8s.KubeCtl
+var restClient yunikorn.RClient
+var sleepRespPod *v1.Pod
+var dev = "dev" + common.RandSeq(5)
+var appsInfo *dao.ApplicationDAOInfo
+var annotation = "ann-" + common.RandSeq(10)
+var oldConfigMap = new(v1.ConfigMap)
+
+// Define sleepPod
+var sleepPodConfigs = k8s.SleepPodConfig{Name: "sleepjob", NS: dev}
+
+var _ = ginkgo.BeforeSuite(func() {
+	// Initializing kubectl client
+	kClient = k8s.KubeCtl{}
+	gomega.Ω(kClient.SetClient()).To(gomega.BeNil())
+	// Initializing rest client
+	restClient = yunikorn.RClient{}
+
+	yunikorn.EnsureYuniKornConfigsPresent()
+
+	By("Port-forward the scheduler pod")
+	err := kClient.PortForwardYkSchedulerPod()
+	Ω(err).NotTo(HaveOccurred())
+
+	yunikorn.UpdateConfigMapWrapper(oldConfigMap, "fifo", annotation)
+
+	ginkgo.By("create development namespace")
+	ns1, err := kClient.CreateNamespace(dev, nil)
+	gomega.Ω(err).NotTo(gomega.HaveOccurred())
+	gomega.Ω(ns1.Status.Phase).To(gomega.Equal(v1.NamespaceActive))
+
+	ginkgo.By("Deploy the sleep pod to the development namespace")
+	initPod, podErr := k8s.InitSleepPod(sleepPodConfigs)
+	gomega.Ω(podErr).NotTo(gomega.HaveOccurred())
+	sleepRespPod, err = kClient.CreatePod(initPod, dev)
+	gomega.Ω(err).NotTo(gomega.HaveOccurred())
+	// Wait for pod to move to running state
+	err = kClient.WaitForPodRunning(dev, sleepPodConfigs.Name, 30*time.Second)
+	gomega.Ω(err).NotTo(gomega.HaveOccurred())
+
+	appsInfo, err = restClient.GetAppInfo("default", "root."+dev, sleepRespPod.ObjectMeta.Labels["applicationId"])
+	gomega.Ω(err).NotTo(gomega.HaveOccurred())
+	gomega.Ω(appsInfo).NotTo(gomega.BeNil())
+})
+
+var _ = ginkgo.AfterSuite(func() {
+	ginkgo.By("Tear down namespace: " + dev)
+	err := kClient.TearDownNamespace(dev)
+	Ω(err).NotTo(HaveOccurred())
+
+	yunikorn.RestoreConfigMapWrapper(oldConfigMap, annotation)
+})
+
 var _ = ginkgo.Describe("", func() {
-	var kClient k8s.KubeCtl
-	var restClient yunikorn.RClient
-	var sleepRespPod *v1.Pod
-	var dev = "dev" + common.RandSeq(5)
-	var appsInfo *dao.ApplicationDAOInfo
-	var annotation = "ann-" + common.RandSeq(10)
-	var oldConfigMap = new(v1.ConfigMap)
-
-	// Define sleepPod
-	sleepPodConfigs := k8s.SleepPodConfig{Name: "sleepjob", NS: dev}
-
-	ginkgo.BeforeSuite(func() {
-		// Initializing kubectl client
-		kClient = k8s.KubeCtl{}
-		gomega.Ω(kClient.SetClient()).To(gomega.BeNil())
-		// Initializing rest client
-		restClient = yunikorn.RClient{}
-
-		yunikorn.EnsureYuniKornConfigsPresent()
-
-		By("Port-forward the scheduler pod")
-		err := kClient.PortForwardYkSchedulerPod()
-		Ω(err).NotTo(HaveOccurred())
-
-		yunikorn.UpdateConfigMapWrapper(oldConfigMap, "fifo", annotation)
-
-		ginkgo.By("create development namespace")
-		ns1, err := kClient.CreateNamespace(dev, nil)
-		gomega.Ω(err).NotTo(gomega.HaveOccurred())
-		gomega.Ω(ns1.Status.Phase).To(gomega.Equal(v1.NamespaceActive))
-
-		ginkgo.By("Deploy the sleep pod to the development namespace")
-		initPod, podErr := k8s.InitSleepPod(sleepPodConfigs)
-		gomega.Ω(podErr).NotTo(gomega.HaveOccurred())
-		sleepRespPod, err = kClient.CreatePod(initPod, dev)
-		gomega.Ω(err).NotTo(gomega.HaveOccurred())
-		// Wait for pod to move to running state
-		err = kClient.WaitForPodRunning(dev, sleepPodConfigs.Name, 30*time.Second)
-		gomega.Ω(err).NotTo(gomega.HaveOccurred())
-
-		appsInfo, err = restClient.GetAppInfo("default", "root."+dev, sleepRespPod.ObjectMeta.Labels["applicationId"])
-		gomega.Ω(err).NotTo(gomega.HaveOccurred())
-		gomega.Ω(appsInfo).NotTo(gomega.BeNil())
-	})
 
 	ginkgo.It("Verify_App_Queue_Info", func() {
 		ginkgo.By("Verify that the sleep pod is mapped to development queue")
@@ -109,18 +119,15 @@ var _ = ginkgo.Describe("", func() {
 	})
 
 	ginkgo.AfterEach(func() {
+		testDescription := ginkgo.CurrentSpecReport()
+		if testDescription.Failed() {
+			tests.LogTestClusterInfoWrapper(testDescription.FailureMessage(), []string{dev})
+			tests.LogYunikornContainer(testDescription.FailureMessage())
+		}
 		// call the healthCheck api to check scheduler health
 		ginkgo.By("Check Yunikorn's health")
 		checks, err := yunikorn.GetFailedHealthChecks()
 		Ω(err).NotTo(HaveOccurred())
 		Ω(checks).To(gomega.Equal(""), checks)
 	})
-
-	ginkgo.AfterSuite(func() {
-		ginkgo.By("Tear down namespace: " + dev)
-		err := kClient.TearDownNamespace(dev)
-		Ω(err).NotTo(HaveOccurred())
-
-		yunikorn.RestoreConfigMapWrapper(oldConfigMap, annotation)
-	})
 })
