@@ -39,10 +39,8 @@ const (
 	normalSleepJobPrefix = "normal-sleep-job"
 	taskGroupA           = "groupa"
 	taskGroupB           = "groupb"
-	taskGroupAprefix     = "tg-" + taskGroupA + "-" + gangSleepJobPrefix
-	taskGroupBprefix     = "tg-" + taskGroupB + "-" + gangSleepJobPrefix
 	taskGroupE2E         = "e2e-task-group"
-	taskGroupE2EPrefix   = "tg-" + taskGroupE2E
+	taskGroupE2EPrefix   = "tg-" + gangSleepJobPrefix + "-" + taskGroupE2E
 	parallelism          = 3
 	taintKey             = "e2e_test"
 )
@@ -216,11 +214,15 @@ var _ = ginkgo.Describe("", func() {
 		Ω(createErr).NotTo(gomega.HaveOccurred())
 
 		ginkgo.By("Waiting for placeholders in task group A (expected state: Running)")
-		err = kClient.WaitForPlaceholders(dev, taskGroupAprefix, parallelism, 30*time.Second, v1.PodRunning)
+		groupAPrefix := "tg-" + appID + "-" + taskGroupA + "-"
+		stateRunning := v1.PodRunning
+		err = kClient.WaitForPlaceholders(dev, groupAPrefix, parallelism, 30*time.Second, &stateRunning)
 		Ω(err).NotTo(gomega.HaveOccurred())
 
 		ginkgo.By("Waiting for placeholders in task group B (expected state: Pending)")
-		err = kClient.WaitForPlaceholders(dev, taskGroupBprefix, parallelism+1, 30*time.Second, v1.PodPending)
+		groupBPrefix := "tg-" + appID + "-" + taskGroupB + "-"
+		statePending := v1.PodPending
+		err = kClient.WaitForPlaceholders(dev, groupBPrefix, parallelism+1, 30*time.Second, &statePending)
 		Ω(err).NotTo(gomega.HaveOccurred())
 
 		ginkgo.By("Restart the scheduler pod")
@@ -252,12 +254,12 @@ var _ = ginkgo.Describe("", func() {
 			podPhase := pod.Status.Phase
 			podName := pod.GetName()
 			fmt.Fprintf(ginkgo.GinkgoWriter, "Pod name: %-40s\tStatus: %s\n", podName, podPhase)
-			if strings.HasPrefix(podName, taskGroupAprefix) {
+			if strings.HasPrefix(podName, groupAPrefix) {
 				groupAPlaceholderCount++
 				Ω(podPhase).To(gomega.Equal(v1.PodRunning))
 				continue
 			}
-			if strings.HasPrefix(podName, taskGroupBprefix) {
+			if strings.HasPrefix(podName, groupBPrefix) {
 				groupBPlaceholderCount++
 				Ω(podPhase).To(gomega.Equal(v1.PodPending))
 				continue
@@ -313,19 +315,15 @@ var _ = ginkgo.Describe("", func() {
 			selectedNode, memoryGiB, placeholderCount)
 
 		ginkgo.By("Tainting all nodes except " + selectedNode)
-		for _, nodeName := range nodesToTaint {
-			err = kClient.TaintNode(nodeName, taintKey, "value", v1.TaintEffectNoSchedule)
-			Ω(err).NotTo(gomega.HaveOccurred())
-		}
+		err = kClient.TaintNodes(nodesToTaint, taintKey, "value", v1.TaintEffectNoSchedule)
+		Ω(err).NotTo(gomega.HaveOccurred())
 
 		removeTaint := true
 		defer func() {
 			if removeTaint {
 				ginkgo.By("Untainting nodes (defer)")
-				for _, nodeName := range nodesToTaint {
-					err = kClient.UntaintNode(nodeName, taintKey)
-					Ω(err).NotTo(gomega.HaveOccurred(), "Could not remove taint from node "+nodeName)
-				}
+				err = kClient.UntaintNodes(nodesToTaint, taintKey)
+				Ω(err).NotTo(gomega.HaveOccurred(), "Could not remove taint from nodes "+strings.Join(nodesToTaint, ","))
 			}
 		}()
 
@@ -353,10 +351,8 @@ var _ = ginkgo.Describe("", func() {
 
 		ginkgo.By("Untainting nodes")
 		removeTaint = false
-		for _, nodeName := range nodesToTaint {
-			err = kClient.UntaintNode(nodeName, taintKey)
-			Ω(err).NotTo(gomega.HaveOccurred(), "Could not remove taint from node "+nodeName)
-		}
+		err = kClient.UntaintNodes(nodesToTaint, taintKey)
+		Ω(err).NotTo(gomega.HaveOccurred(), "Could not remove taint from nodes "+strings.Join(nodesToTaint, ","))
 
 		ginkgo.By("Waiting for placeholder replacement & sleep pods to finish")
 		err = kClient.WaitForJobPodsSucceeded(dev, job.Name, 1, 60*time.Second)
