@@ -61,6 +61,8 @@ export class AppsViewComponent implements OnInit {
   partitionSelected = '';
   leafQueueList: DropdownItem[] = [];
   leafQueueSelected = '';
+  
+  detailToggle: boolean = false;
 
   constructor(
     private scheduler: SchedulerService,
@@ -77,7 +79,6 @@ export class AppsViewComponent implements OnInit {
 
     this.appColumnDef = [
       { colId: 'applicationId', colName: 'Application ID' },
-      { colId: 'queueName', colName: 'Queue Name' },
       { colId: 'applicationState', colName: 'Application State' },
       {
         colId: 'lastStateChangeTime',
@@ -89,7 +90,11 @@ export class AppsViewComponent implements OnInit {
         colName: 'Used Resource',
         colFormatter: CommonUtil.resourceColumnFormatter,
       },
-      { colId: 'partition', colName: 'Partition' },
+      {
+        colId: 'pendingResource',
+        colName: 'Pending Resource',
+        colFormatter: CommonUtil.resourceColumnFormatter,
+      },
       {
         colId: 'submissionTime',
         colName: 'Submission Time',
@@ -301,4 +306,28 @@ export class AppsViewComponent implements OnInit {
       this.removeRowSelection();
     }
   }
+
+  formatResources(colValue:string):string[]{
+    const arr:string[]=colValue.split("<br/>")
+    // Check if there are "cpu" or "Memory" elements in the array
+    const hasCpu = arr.some((item) => item.toLowerCase().includes("cpu"));
+    const hasMemory = arr.some((item) => item.toLowerCase().includes("memory"));
+    if (!hasCpu) {
+      arr.unshift("CPU: n/a");
+    }
+    if (!hasMemory) {
+      arr.unshift("Memory: n/a");
+    }
+
+    // Concatenate the two arrays, with "cpu" and "Memory" elements first
+    const cpuAndMemoryElements = arr.filter((item) => item.toLowerCase().includes("CPU") || item.toLowerCase().includes("Memory"));
+    const otherElements = arr.filter((item) => !item.toLowerCase().includes("CPU") && !item.toLowerCase().includes("Memory"));
+    const result = cpuAndMemoryElements.concat(otherElements);
+
+    return result;
+  }
+
+  toggle(){
+    this.detailToggle = !this.detailToggle;
+  }
 }
