@@ -22,7 +22,7 @@ import { MatSelectChange } from '@angular/material/select';
 import { MatSort } from '@angular/material/sort';
 import { MatTableDataSource } from '@angular/material/table';
 import { NgxSpinnerService } from 'ngx-spinner';
-import { finalize } from 'rxjs/operators';
+import { finalize, flatMap } from 'rxjs/operators';
 
 import { SchedulerService } from '@app/services/scheduler/scheduler.service';
 import { NodeInfo } from '@app/models/node-info.model';
@@ -53,6 +53,8 @@ export class NodesViewComponent implements OnInit {
   partitionList: PartitionInfo[] = [];
   partitionSelected = '';
 
+  detailToggle: boolean = false;
+
   constructor(private scheduler: SchedulerService, private spinner: NgxSpinnerService) {}
 
   ngOnInit() {
@@ -135,6 +137,7 @@ export class NodesViewComponent implements OnInit {
       )
       .subscribe((data) => {
         this.nodeDataSource.data = data;
+        console.log(data);
       });
   }
 
@@ -190,4 +193,28 @@ export class NodesViewComponent implements OnInit {
     this.clearRowSelection();
     this.fetchNodeListForPartition(this.partitionSelected);
   }
+
+  formatResources(colValue:string):string[]{
+    const arr:string[]=colValue.split("<br/>")
+    // Check if there are "cpu" or "Memory" elements in the array
+    const hasCpu = arr.some((item) => item.toLowerCase().includes("cpu"));
+    const hasMemory = arr.some((item) => item.toLowerCase().includes("memory"));
+    if (!hasCpu) {
+      arr.unshift("CPU: n/a");
+    }
+    if (!hasMemory) {
+      arr.unshift("Memory: n/a");
+    }
+
+    // Concatenate the two arrays, with "cpu" and "Memory" elements first
+    const cpuAndMemoryElements = arr.filter((item) => item.toLowerCase().includes("CPU") || item.toLowerCase().includes("Memory"));
+    const otherElements = arr.filter((item) => !item.toLowerCase().includes("CPU") && !item.toLowerCase().includes("Memory"));
+    const result = cpuAndMemoryElements.concat(otherElements);
+
+    return result;
+  }
+
+  toggle(){
+    this.detailToggle = !this.detailToggle;
+  }
 }
