@@ -16,23 +16,24 @@
  * limitations under the License.
  */
 
-import { Injectable } from '@angular/core';
-import { HttpClient } from '@angular/common/http';
-import { Observable, queueScheduler } from 'rxjs';
-import { map } from 'rxjs/operators';
+import {HttpClient} from '@angular/common/http';
+import {Injectable} from '@angular/core';
+import {AllocationInfo} from '@app/models/alloc-info.model';
+import {AppInfo} from '@app/models/app-info.model';
+import {ClusterInfo} from '@app/models/cluster-info.model';
+import {HistoryInfo} from '@app/models/history-info.model';
+import {NodeInfo} from '@app/models/node-info.model';
+import {NodeUtilization} from '@app/models/node-utilization.model';
+import {Partition} from '@app/models/partition-info.model';
 
-import { QueueInfo, QueuePropertyItem } from '@app/models/queue-info.model';
-import { EnvconfigService } from '../envconfig/envconfig.service';
-import { ClusterInfo } from '@app/models/cluster-info.model';
-import { CommonUtil } from '@app/utils/common.util';
-import { SchedulerResourceInfo } from '@app/models/resource-info.model';
-import { AppInfo } from '@app/models/app-info.model';
-import { AllocationInfo } from '@app/models/alloc-info.model';
-import { HistoryInfo } from '@app/models/history-info.model';
-import { NodeInfo } from '@app/models/node-info.model';
-import { NOT_AVAILABLE } from '@app/utils/constants';
-import { Partition } from '@app/models/partition-info.model';
-import { SchedulerHealthInfo } from "@app/models/scheduler-health-info.model";
+import {QueueInfo, QueuePropertyItem} from '@app/models/queue-info.model';
+import {SchedulerResourceInfo} from '@app/models/resource-info.model';
+import {SchedulerHealthInfo} from "@app/models/scheduler-health-info.model";
+import {CommonUtil} from '@app/utils/common.util';
+import {NOT_AVAILABLE} from '@app/utils/constants';
+import {Observable} from 'rxjs';
+import {map} from 'rxjs/operators';
+import {EnvconfigService} from '../envconfig/envconfig.service';
 
 @Injectable({
   providedIn: 'root',
@@ -91,15 +92,17 @@ export class SchedulerService {
             const appInfo = new AppInfo(
               app['applicationID'],
               this.formatResource(app['usedResource'] as SchedulerResourceInfo),
+              this.formatResource(app['pendingResource'] as SchedulerResourceInfo),
               this.formatResource(app['maxUsedResource'] as SchedulerResourceInfo),
-              app['partition'],
-              app['queueName'],
               app['submissionTime'],
               app['lastStateChangeTime'],
+              app['stateLog'],
               app['finishedTime'],
               app['applicationState'],
               []
             );
+            appInfo.setLastStateChangeTime()
+            
             const allocations = app['allocations'];
             if (allocations && allocations.length > 0) {
               const appAllocations: AllocationInfo[] = [];
@@ -251,6 +254,11 @@ export class SchedulerService {
     );
   }
 
+  fetchNodeUtilization(): Observable<NodeUtilization>{
+    const nodeUtilizationUrl = `${this.envConfig.getSchedulerWebAddress()}/ws/v1/scheduler/node-utilization`;
+    return this.httpClient.get(nodeUtilizationUrl).pipe(map((data: any) => data as NodeUtilization));
+  }
+
   fecthHealthchecks(): Observable<SchedulerHealthInfo> {
     const healthCheckUrl = `${this.envConfig.getSchedulerWebAddress()}/ws/v1/scheduler/healthcheck`;
     return this.httpClient.get(healthCheckUrl).pipe(map((data: any) => data as SchedulerHealthInfo));
@@ -285,10 +293,12 @@ export class SchedulerService {
     const maxResource = data['maxResource'] as SchedulerResourceInfo;
     const guaranteedResource = data['guaranteedResource'] as SchedulerResourceInfo;
     const allocatedResource = data['allocatedResource'] as SchedulerResourceInfo;
+    const pendingResource = data['pendingResource'] as SchedulerResourceInfo;
     const absUsedCapacity = data['absUsedCapacity'] as SchedulerResourceInfo;
     queue.maxResource = this.formatResource(maxResource);
     queue.guaranteedResource = this.formatResource(guaranteedResource);
     queue.allocatedResource = this.formatResource(allocatedResource);
+    queue.pendingResource = this.formatResource(pendingResource);
     queue.absoluteUsedCapacity = this.formatPercent(absUsedCapacity);
     queue.absoluteUsedPercent = this.absUsagePercent(absUsedCapacity);
   }
@@ -318,7 +328,7 @@ export class SchedulerService {
     const formatted = [];
 
     if (resource && resource.memory !== undefined) {
-      formatted.push(`Memory: ${CommonUtil.formatMemory(resource.memory)}`);
+      formatted.push(`Memory: ${CommonUtil.formatBytes(resource.memory)}`);
     } else {
       formatted.push(`Memory: ${NOT_AVAILABLE}`);
     }
@@ -329,6 +339,33 @@ export class SchedulerService {
       formatted.push(`CPU: ${NOT_AVAILABLE}`);
     }
 
+    if (resource){
+      Object.keys(resource).forEach((key) => {
+        switch(key){
+          case "memory":
+          case "vcore":{
+            break;
+          }
+          case "ephemeral-storage":{
+            if (resource[`ephemeral-storage`] == 0) {
+              formatted.push(`ephemeral-storage: ${NOT_AVAILABLE}`);
+            }else{
+              formatted.push(`ephemeral-storage: ${CommonUtil.formatBytes(resource[key])}`);
+            }
+            break;
+          }
+          default:{
+            if (resource[key] == 0) {
+              formatted.push(`${key}: ${NOT_AVAILABLE}`);
+            }else{
+              formatted.push(`${key}: ${CommonUtil.formatOtherResource(resource[key])}`);
+            }
+            break;
+          }
+        }
+      });
+    }
+    
     return formatted.join(', ');
   }
 
