@@ -30,11 +30,11 @@ export class CommonUtil {
     return uniqid;
   }
 
-  static formatMemory(value: number | string): string {
-    const units: readonly string[] = ['KB', 'MB', 'GB', 'TB', 'PB'];
-    var unit: string = 'bytes';
+  static formatBytes(value: number | string): string {
+    const units: readonly string[] = ['kB', 'MB', 'GB', 'TB', 'PB'];
+    let unit: string = 'bytes';
     let toValue = +value
-    for (var i: number = 0, unitslen = units.length; toValue / 1000 >= 1 && i < unitslen;i = i + 1) {
+    for (let i = 0, unitslen = units.length; toValue / 1000 >= 1 && i < unitslen;i = i + 1) {
       toValue = toValue / 1000;
       unit = units[i];
     }
@@ -48,7 +48,6 @@ export class CommonUtil {
   static formatCount(value: number | string): string {
     const unit = 'K';
     const toValue = +value;
-
     if (toValue >= 10000) {
       return `${(toValue / 1000).toLocaleString(undefined, { maximumFractionDigits: 1 })} ${unit}`;
     }
@@ -56,6 +55,11 @@ export class CommonUtil {
     return toValue.toLocaleString();
   }
 
+  static formatOtherResource(value: number | string): string {
+    const toValue = +value;
+    return toValue.toLocaleString();
+  }
+
   static resourceColumnFormatter(value: string): string {
     return value.split(', ').join('<br/>');
   }
